/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;

import com.adobe.creativesdk.foundation.adobeinternal.auth.AdobeUXAuthManagerRestricted;
import com.adobe.creativesdk.foundation.auth.AdobeAuthUserProfile;
import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.asynctask.listeners.IBehanceSDKEditProfileAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserProfileAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKAddProjectModuleTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKCheckWIPConversionStatusAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKCreateProjectDraftTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKEditProfileAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserProfileTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKPublishProjectTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKSearchProjectsTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKAddProjectModuleAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKCheckWIPConversionStatusAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKCreateProjectDraftAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKEditProfileAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetUserProfileAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKPublishProjectAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKSearchProjectsAsyncTask;
import com.behance.sdk.asynctasks.result.BehanceSDKEditProfileTaskResult;
import com.behance.sdk.asynctasks.result.BehanceSDKGetUserProfileTaskResult;
import com.behance.sdk.dto.BehanceSDKPublishProjectStatusDTO;
import com.behance.sdk.dto.BehanceSDKPublishWIPStatusDTO;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.search.BehanceSDKProjectFiltersSelected;
import com.behance.sdk.enums.BehanceSDKCuratedGalleryType;
import com.behance.sdk.enums.BehanceSDKPublishProjectProgressState;
import com.behance.sdk.enums.BehanceSDKPublishWIPProgressState;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.exception.BehanceSDKFileUtilsInitializationException;
import com.behance.sdk.exception.BehanceSDKInvalidArgumentException;
import com.behance.sdk.exception.BehanceSDKInvalidImageException;
import com.behance.sdk.exception.BehanceSDKInvalidImagesException;
import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.exception.BehanceSDKUserNotEntitledException;
import com.behance.sdk.managers.AddWIPManager;
import com.behance.sdk.managers.BehanceSDKEditProfileManager;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.services.BehanceSDKPublishProjectService;
import com.behance.sdk.services.BehanceSDKPublishWIPService;
import com.behance.sdk.services.serviceconnections.BSDKCancelPublishProjectServiceConnection;
import com.behance.sdk.services.serviceconnections.BSDKCancelPublishWIPServiceConnection;
import com.behance.sdk.ui.activities.BehanceSDKCreateProjectWFActivity;
import com.behance.sdk.ui.activities.BehanceSDKCreateWIPWorkflowActivity;
import com.behance.sdk.ui.activities.BehanceSDKEditProfileActivity;
import com.behance.sdk.ui.activities.BehanceSDKProjectDetailActivity;
import com.behance.sdk.ui.activities.BehanceSDKPublishProjectActivity;
import com.behance.sdk.ui.activities.BehanceSDKPublishResultsActivity;
import com.behance.sdk.ui.activities.BehanceSDKPublishWIPActivity;
import com.behance.sdk.ui.fragments.BehanceSDKProjectDetailFragment;
import com.behance.sdk.ui.fragments.BehanceSDKProjectsFeedFragment;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFileUtils;
import com.behance.sdk.util.BehanceSDKImageLoaderUtils;
import com.behance.sdk.util.BehanceSDKPublishImageValidationResult;
import com.behance.sdk.util.BehanceSDKUtils;
import com.facebook.FacebookSdk;
import com.facebook.drawee.backends.pipeline.Fresco;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ugs
 * 
 */
public class BehanceSDK {

    private static final BehanceSDK INSTANCE = new BehanceSDK();
    private static  boolean behanceTheme = true;
    private static BehanceSDKCustomResourcesOptions behanceSDKCustomResourcesOptions;


    private static final String INVALID_PROJECT_TITLE_MESSAGE = "Behance project title cannot be more than "+ BehanceSDKConstants.PROJECT_TITLE_MAX_LENGTH +" characters in length";
    private static final String INVALID_PROJECT_DESCRIPTION_MESSAGE = "Behance project description cannot be more than " + BehanceSDKConstants.PROJECT_DESC_MAX_LENGTH + "characters in length";

    private IBehanceSDKUserCredentials credentials;

	private BehanceSDK() {

	}

	public static BehanceSDK getInstance() {
		return INSTANCE;
	}

    public  boolean useBehanceTheme() {
        return behanceTheme;
    }

    private  void setBehanceTheme(boolean useBehanceTheme) {
        behanceTheme = useBehanceTheme;
    }

    private void setBehanceSDKCustomResourcesOptions(BehanceSDKCustomResourcesOptions options){
        behanceSDKCustomResourcesOptions = options;
    }
    public BehanceSDKCustomResourcesOptions getBehanceSDKCustomResourcesOptions(){
        if(behanceSDKCustomResourcesOptions ==null)
            return new BehanceSDKCustomResourcesOptions();
        return behanceSDKCustomResourcesOptions;
    }




    public void initialize(IBehanceSDKUserCredentials credentials,Context applicationContext) {
		this.credentials = credentials;
        FacebookSdk.sdkInitialize(applicationContext);
	}

    public  void setBehanceSDKThemeResources(BehanceSDKCustomResourcesOptions options){
        setBehanceSDKCustomResourcesOptions(options);
        setBehanceTheme(false);
    }

    public void initializeProjectViewWF(Context context){
        initializeFileUtils(context);
        Fresco.initialize(context);
        if (!ImageLoader.getInstance().isInited()) {
            ImageLoaderConfiguration loaderConfiguration = BehanceSDKImageLoaderUtils.getImageLoaderConfig(context);
            ImageLoader.getInstance().init(loaderConfiguration);
        }
    }

    private void initializeFileUtils(Context context){
        try {
            BehanceSDKFileUtils.initialize(context);
        } catch (BehanceSDKFileUtilsInitializationException e) {

        }
    }

    private void validateIfUserIsEntitledForBehanceWorkflow() throws BehanceSDKUserNotEntitledException, BehanceSDKUserNotAuthenticatedException {
        BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
        if(!sdkUserManager.isUserAuthenticatedWithAdobe()){
            throw new BehanceSDKUserNotAuthenticatedException("No logged in user found.");
        }

        if(!sdkUserManager.isUserEntitledToUseBehance())
            throw new BehanceSDKUserNotEntitledException("User is not entitled to use Behance services");
    }

    public void launchProjectDetailsActivity(Context context,String projectID) throws BehanceSDKUserNotEntitledException,BehanceSDKUserNotAuthenticatedException{
        if(projectID!= null) {
            initializeProjectViewWF(context);
            validateIfUserIsEntitledForBehanceWorkflow();
            Intent intent = new Intent(context, BehanceSDKProjectDetailActivity.class);
            intent.putExtra(BehanceSDKProjectDetailFragment.ARG_PROJECT_ID, projectID);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            context.startActivity(intent);
        }

    }

    public void launchProjectDetailsActivity(Context context,BehanceSDKProjectDTO projectDTO) throws BehanceSDKUserNotEntitledException,BehanceSDKUserNotAuthenticatedException{
        if(projectDTO!= null) {
            initializeProjectViewWF(context);
            validateIfUserIsEntitledForBehanceWorkflow();
            Intent intent = new Intent(context, BehanceSDKProjectDetailActivity.class);
            intent.putExtra(BehanceSDKProjectDetailFragment.ARG_PROJECT_ID, projectDTO.getId());
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            context.startActivity(intent);
        }

    }

	public void launchAddProjectWorkFlow(Context context, Class<? extends Activity> notificationHandlerActivityClass) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        BehanceSDKAddProjectWorkflowOptions options = new BehanceSDKAddProjectWorkflowOptions();
		options.setNotificationHandlerActivityClass(notificationHandlerActivityClass);
		options.setFacebookShareEnabled(false);
		options.setTwitterShareEnabled(false);
		launchAddProjectWorkFlow(context, options);
	}

	public void launchAddProjectWorkFlow(Context context, BehanceSDKAddProjectWorkflowOptions options) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeForPublishProjectWorkflow(context, options);
		Intent detailsIntent = new Intent(context, BehanceSDKCreateProjectWFActivity.class);
		detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXStart,
                AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);

		context.startActivity(detailsIntent);
	}

    public BehanceSDKProjectsFeedFragment getProjectFeedFragment(Context context,BehanceSDKProjectFiltersSelected filtersSelected) throws BehanceSDKUserNotAuthenticatedException,BehanceSDKUserNotEntitledException{
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeProjectViewWF(context);
        Bundle arguments = new Bundle();
        arguments.putSerializable(BehanceSDKProjectsFeedFragment.DISCOVER_PROJECTS_KEY_SELECTED_FILTERS, filtersSelected);
        arguments.putBoolean(BehanceSDKProjectsFeedFragment.PROJECT_FEED_TYPE, false);
        BehanceSDKProjectsFeedFragment behanceSDKProjectsFeedFragment= new BehanceSDKProjectsFeedFragment();
        behanceSDKProjectsFeedFragment.setArguments(arguments);
        return behanceSDKProjectsFeedFragment;
    }

    public BehanceSDKProjectsFeedFragment getProjectFeedFragment(Context context,BehanceSDKCuratedGalleryType galleryType , int galleryID) throws BehanceSDKUserNotAuthenticatedException,BehanceSDKUserNotEntitledException{
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeProjectViewWF(context);
        Bundle arguments = new Bundle();
        arguments.putSerializable(BehanceSDKProjectsFeedFragment.PROJECT_FEED_GALLERY_TYPE, galleryType);
        arguments.putInt(BehanceSDKProjectsFeedFragment.PROJECT_FEED_GALLERY_ID, galleryID);
        arguments.putBoolean(BehanceSDKProjectsFeedFragment.PROJECT_FEED_TYPE, true);
        BehanceSDKProjectsFeedFragment behanceSDKProjectsFeedFragment= new BehanceSDKProjectsFeedFragment();
        behanceSDKProjectsFeedFragment.setArguments(arguments);
        return behanceSDKProjectsFeedFragment;
    }

    public void launchPublishProjectView(Context context, BehanceSDKPublishProjectOptions options) throws BehanceSDKInvalidImagesException,
            BehanceSDKInvalidArgumentException, BehanceSDKUserNotEntitledException, BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeForPublishProjectWorkflow(context, options);
        List<ImageModule> images = options.getProjectImageModules();
        if(null != images && false  == images.isEmpty()) {
            List<File> invalidImages = new ArrayList<File>(images.size());
            for (ImageModule imageModule : images) {
                if (BehanceSDKPublishImageValidationResult.VALID != validateImageForProject(imageModule)) {
                    //add all the invalid Images.
                    invalidImages.add(imageModule.getFile());
                }
            }
            if(false == invalidImages.isEmpty()){
                throw new BehanceSDKInvalidImagesException(invalidImages);
            }
        }else{
            throw new IllegalArgumentException("List of images cannot be null or empty.");
        }

        String title = options.getProjectTitle();
        if(false == TextUtils.isEmpty(title) && title.length() > BehanceSDKConstants.PROJECT_TITLE_MAX_LENGTH){
            throw new BehanceSDKInvalidArgumentException(INVALID_PROJECT_TITLE_MESSAGE);
        }

        String description = options.getProjectDescription();
        if(false == TextUtils.isEmpty(description) && description.length() > BehanceSDKConstants.PROJECT_DESC_MAX_LENGTH){
            throw new BehanceSDKInvalidArgumentException(INVALID_PROJECT_DESCRIPTION_MESSAGE);
        }
        //if any one image is invalid throw exception.
        Intent detailsIntent = new Intent(context, BehanceSDKPublishProjectActivity.class);
        detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_IMAGE_MODULES, (Serializable) options.getProjectImageModules());

        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_TITLE, options.getProjectTitle());
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_DESC, options.getProjectDescription());
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_TAGS, options.getProjectTags());
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_CONTAINS_ADULT_CONTENT, options.isProjectContainsAdultContent());
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_PROJECT_COPYRIGHT_SETTINGS, options.getProjectCopyrightSettings());
        detailsIntent.putExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED,options.isCCAssetBrowserDisabled());

        AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXStart,
                AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);

        context.startActivity(detailsIntent);
    }

    public void createProjectDraft(BehanceSDKProjectDraftOptions options, IBehanceSDKCreateProjectDraftListener listener) throws BehanceSDKUserNotAuthenticatedException,
            BehanceSDKUserNotEntitledException, BehanceSDKInvalidArgumentException {

        //validate user entitlements for behance
        validateIfUserIsEntitledForBehanceWorkflow();

        if(null != options) {
            String title = options.getProjectTitle();
            if (false == TextUtils.isEmpty(title) && title.length() > BehanceSDKConstants.PROJECT_TITLE_MAX_LENGTH) {
                throw new BehanceSDKInvalidArgumentException(INVALID_PROJECT_TITLE_MESSAGE);
            }

            String description = options.getProjectDescription();
            if (false == TextUtils.isEmpty(description) && description.length() > BehanceSDKConstants.PROJECT_DESC_MAX_LENGTH) {
                throw new BehanceSDKInvalidArgumentException(INVALID_PROJECT_DESCRIPTION_MESSAGE);
            }
        }

        if(null == listener){
            throw new BehanceSDKInvalidArgumentException("Instance of IBehanceSDKCreateProjectDraftListener cannot be null.");
        }

        BehanceSDKCreateProjectDraftAsyncTask createProjectDraftAsyncTask = new BehanceSDKCreateProjectDraftAsyncTask(listener);
        BehanceSDKCreateProjectDraftTaskParams params = new BehanceSDKCreateProjectDraftTaskParams();
        params.setClientId(credentials.getClientId());
        params.setBehanceSDKProjectDraftOptions(options);
        createProjectDraftAsyncTask.execute(params);
    }

    public void publishProject(String projectID, IBehanceSDKProjectPublishListener listener) throws BehanceSDKInvalidArgumentException,
            BehanceSDKUserNotAuthenticatedException, BehanceSDKUserNotEntitledException {

        //validate user entitlements for behance
        validateIfUserIsEntitledForBehanceWorkflow();
        if(TextUtils.isEmpty(projectID)){
            throw new BehanceSDKInvalidArgumentException("Project ID cannot be null or empty.");
        }
        BehanceSDKPublishProjectAsyncTask asyncTask = new BehanceSDKPublishProjectAsyncTask(listener);
        BehanceSDKPublishProjectTaskParams param = new BehanceSDKPublishProjectTaskParams();
        param.setClientId(credentials.getClientId());
        param.setProjectId(projectID);
        asyncTask.execute(param);
    }

    public void addImageToProject(String projectID, File image, IBehanceSDKADDProjectModuleListener listener) throws BehanceSDKUserNotAuthenticatedException,
            BehanceSDKUserNotEntitledException, BehanceSDKInvalidArgumentException {

        //validate user entitlements for behance
        validateIfUserIsEntitledForBehanceWorkflow();
        if(TextUtils.isEmpty(projectID)){
            throw new BehanceSDKInvalidArgumentException("Project ID cannot be null or empty.");
        }
        if(null == image || false == image.exists()){
            throw new BehanceSDKInvalidArgumentException("Image file cannot be null or not existing file.");
        }
        BehanceSDKAddProjectModuleTaskParams params = new BehanceSDKAddProjectModuleTaskParams();
        params.setClientId(credentials.getClientId());
        params.setProjectId(projectID);
        params.setImage(image);

        BehanceSDKAddProjectModuleAsyncTask addProjectModuleAsyncTask = new BehanceSDKAddProjectModuleAsyncTask(listener);
        addProjectModuleAsyncTask.execute(params);
    }


    public void cancelActivePublishProject(Context context, String publishRequestId) {
        Intent intent = new Intent(context, BehanceSDKPublishProjectService.class);
        BSDKCancelPublishProjectServiceConnection serviceConnection = new BSDKCancelPublishProjectServiceConnection(publishRequestId);
        context.bindService(intent, serviceConnection, 0);
    }

    public void displayPublishProjectSuccessView(Context context, BehanceSDKPublishProjectStatusDTO publishProjectStatusDTO) {
        if (publishProjectStatusDTO != null && publishProjectStatusDTO.getProgressState() == BehanceSDKPublishProjectProgressState.PUBLISH_SUCCESSFUL) {
            Intent detailsIntent = new Intent(context, BehanceSDKPublishResultsActivity.class);
            detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            detailsIntent.putExtra(BehanceSDKPublishResultsActivity.INTENT_STRING_EXTRA_WORK_URL, publishProjectStatusDTO.getProjectUrl());
            detailsIntent.putExtra(BehanceSDKPublishResultsActivity.INTENT_STRING_EXTRA_IMAGE_URL, publishProjectStatusDTO.getProjectCoverImageUrl());
            context.startActivity(detailsIntent);
        }
    }

    private void initializeForPublishProjectWorkflow(Context context, BehanceSDKAbstractPublishOptions options) {
        BehanceSDKProjectPublishWFManager wfManager = BehanceSDKProjectPublishWFManager.getInstance();
        BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(context);
        if (options.isFacebookShareEnabled() && options.getFacebookClientId() != null && options.getFacebookClientId().length() > 0) {
            BehanceSDKSocialAccountDTO facebookAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.FACEBOOK);
            if (facebookAccount != null) {
                String fbClientId = facebookAccount.getAccountClientId();
                if (!options.getFacebookClientId().equals(fbClientId)) {
                    socialAccountManager.removeAccount(BehanceSDKSocialAccountType.FACEBOOK);
                    facebookAccount = null;
                }
            }
            if (facebookAccount == null) {
                facebookAccount = new BehanceSDKSocialAccountDTO();
                facebookAccount.setAccountType(BehanceSDKSocialAccountType.FACEBOOK);
                facebookAccount.setAccountClientId(options.getFacebookClientId());
                socialAccountManager.addAccount(facebookAccount);
            }
        } else {
            wfManager.setHideFacebookSharing(true);
        }

        if (options.isTwitterShareEnabled() && options.getTwitterConsumerKey() != null && options.getTwitterConsumerKey().length() > 0
                && options.getTwitterConsumerSecretKey() != null && options.getTwitterConsumerSecretKey().length() > 0) {

            BehanceSDKSocialAccountDTO twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
            if (twitterAccount != null) {
                String twitterClientId = twitterAccount.getAccountClientId();
                String twitterClientSecret = twitterAccount.getAccountClientSecret();
                if (!options.getTwitterConsumerKey().equals(twitterClientId) || !options.getTwitterConsumerSecretKey().equals(twitterClientSecret)) {
                    socialAccountManager.removeAccount(BehanceSDKSocialAccountType.TWITTER);
                    twitterAccount = null;
                }
            }
            if (twitterAccount == null) {
                twitterAccount = new BehanceSDKSocialAccountDTO();
                twitterAccount.setAccountType(BehanceSDKSocialAccountType.TWITTER);
                twitterAccount.setAccountClientId(options.getTwitterConsumerKey());
                twitterAccount.setAccountClientSecret(options.getTwitterConsumerSecretKey());
                socialAccountManager.addAccount(twitterAccount);
            }

        } else {
            wfManager.setHideTwitterSharing(true);
        }

        wfManager.setUserCredentials(credentials);
        wfManager.setNotificationHandlerActivityClass(options.getNotificationHandlerActivityClass());
    }

    @Deprecated
	public void launchAddWIPWorkFlow(Context context, Class<? extends Activity> notificationHandlerActivityClass) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
		BehanceSDKAddWIPWorkflowOptions options = new BehanceSDKAddWIPWorkflowOptions();
		options.setNotificationHandlerActivityClass(notificationHandlerActivityClass);
		options.setFacebookShareEnabled(false);
		options.setTwitterShareEnabled(false);
		launchAddWIPWorkFlow(context, options);
	}

    @Deprecated
	public void launchAddWIPWorkFlow(Context context, BehanceSDKAddWIPWorkflowOptions options) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeForPublishWIPWorkflow(context, options);
		Intent detailsIntent = new Intent(context, BehanceSDKCreateWIPWorkflowActivity.class);
		detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXStart,
                AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceWIP);

		context.startActivity(detailsIntent);
	}

    public void launchPublishWIPView(Context context, BehanceSDKPublishWIPOptions options) throws BehanceSDKInvalidImageException, BehanceSDKInvalidImageException,
            BehanceSDKInvalidArgumentException, BehanceSDKUserNotEntitledException, BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        initializeForPublishWIPWorkflow(context, options);
        ImageModule imageModule = options.getWipRevisionImageModule();
        if (BehanceSDKPublishImageValidationResult.VALID != validateImageForWIP(imageModule)) {
            throw new BehanceSDKInvalidImageException(imageModule.getFile());
        }
        String title = options.getWipTitle();
        if(false == TextUtils.isEmpty(title) && title.length() > BehanceSDKConstants.WIP_TITLE_MAX_LENGTH){
            throw new BehanceSDKInvalidArgumentException("Behance WIP title cannot be more than "+BehanceSDKConstants.WIP_TITLE_MAX_LENGTH+" characters in length");
        }
        Intent detailsIntent = new Intent(context, BehanceSDKPublishWIPActivity.class);
        detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        detailsIntent.putExtra(BehanceSDKPublishWIPActivity.INTENT_INT_EXTRA_EXISTING_WIP_ID, options.getExistingWIPId());
        detailsIntent.putExtra(BehanceSDKPublishWIPActivity.INTENT_STRING_EXTRA_WIP_DESCRIPTION, options.getWipDescription());
        detailsIntent.putExtra(BehanceSDKPublishWIPActivity.INTENT_STRING_EXTRA_WIP_TAGS, options.getWipTags());
        detailsIntent.putExtra(BehanceSDKPublishWIPActivity.INTENT_STRING_EXTRA_WIP_TITLE, options.getWipTitle());
        detailsIntent.putExtra(BehanceSDKPublishWIPActivity.INTENT_SERIALIZABLE_EXTRA_WIP_REVISION_IMAGE_MODULE, options.getWipRevisionImageModule());

        AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXStart,
                AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceWIP);

        context.startActivity(detailsIntent);
    }

    public void cancelActivePublishWIP(Context context, String publishRequestId) {
        Intent intent = new Intent(context, BehanceSDKPublishWIPService.class);
        BSDKCancelPublishWIPServiceConnection serviceConnection = new BSDKCancelPublishWIPServiceConnection(publishRequestId);
        context.bindService(intent, serviceConnection, 0);
    }

    public void displayPublishWIPSuccessView(Context context, BehanceSDKPublishWIPStatusDTO publishWIPStatusDTO) {
        if (publishWIPStatusDTO != null && publishWIPStatusDTO.getProgressState() == BehanceSDKPublishWIPProgressState.PUBLISH_SUCCESSFUL) {
            Intent detailsIntent = new Intent(context, BehanceSDKPublishResultsActivity.class);
            detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            detailsIntent.putExtra(BehanceSDKPublishResultsActivity.INTENT_STRING_EXTRA_WORK_URL, publishWIPStatusDTO.getWipRevisionUrl());
            detailsIntent.putExtra(BehanceSDKPublishResultsActivity.INTENT_STRING_EXTRA_IMAGE_URL, publishWIPStatusDTO.getWipRevisionImageUrl());
            context.startActivity(detailsIntent);
        }
    }

    private void initializeForPublishWIPWorkflow(Context context, BehanceSDKAbstractPublishOptions options) {
        AddWIPManager wipManager = AddWIPManager.getInstance();
        BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(context);
        if (options.isFacebookShareEnabled() && options.getFacebookClientId() != null && options.getFacebookClientId().length() > 0) {
            BehanceSDKSocialAccountDTO facebookAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.FACEBOOK);
            if (facebookAccount != null) {
                String fbClientId = facebookAccount.getAccountClientId();
                if (!options.getFacebookClientId().equals(fbClientId)) {
                    socialAccountManager.removeAccount(BehanceSDKSocialAccountType.FACEBOOK);
                    facebookAccount = null;
                }
            }
            if (facebookAccount == null) {
                facebookAccount = new BehanceSDKSocialAccountDTO();
                facebookAccount.setAccountType(BehanceSDKSocialAccountType.FACEBOOK);
                facebookAccount.setAccountClientId(options.getFacebookClientId());
                socialAccountManager.addAccount(facebookAccount);
            }
        } else {
            wipManager.setHideFacebookSharing(true);
        }

        if (options.isTwitterShareEnabled() && options.getTwitterConsumerKey() != null && options.getTwitterConsumerKey().length() > 0
                && options.getTwitterConsumerSecretKey() != null && options.getTwitterConsumerSecretKey().length() > 0) {

            BehanceSDKSocialAccountDTO twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
            if (twitterAccount != null) {
                String twitterClientId = twitterAccount.getAccountClientId();
                String twitterClientSecret = twitterAccount.getAccountClientSecret();
                if (!options.getTwitterConsumerKey().equals(twitterClientId) || !options.getTwitterConsumerSecretKey().equals(twitterClientSecret)) {
                    socialAccountManager.removeAccount(BehanceSDKSocialAccountType.TWITTER);
                    twitterAccount = null;
                }
            }
            if (twitterAccount == null) {
                twitterAccount = new BehanceSDKSocialAccountDTO();
                twitterAccount.setAccountType(BehanceSDKSocialAccountType.TWITTER);
                twitterAccount.setAccountClientId(options.getTwitterConsumerKey());
                twitterAccount.setAccountClientSecret(options.getTwitterConsumerSecretKey());
                socialAccountManager.addAccount(twitterAccount);
            }

        } else {
            wipManager.setHideTwitterSharing(true);
        }
        wipManager.setUserCredentials(credentials);
        wipManager.setNotificationHandlerActivityClass(options.getNotificationHandlerActivityClass());
    }

    public void updateBehanceProfile(BehanceSDKUpdateProfileOptions options, IBehanceSDKUpdateProfileListener listener) throws BehanceSDKUserNotAuthenticatedException,
            BehanceSDKUserNotEntitledException, BehanceSDKInvalidArgumentException {

        validateIfUserIsEntitledForBehanceWorkflow();
        if(null == options){
            throw new BehanceSDKInvalidArgumentException("Profile options object cannot be null.");
        }

        BehanceSDKEditProfileAsyncTaskParams params = new BehanceSDKEditProfileAsyncTaskParams();
        params.setFirstName(options.getFirstName());
        params.setLastName(options.getLastName());
        params.setCountry(options.getCountry());
        params.setState(options.getState());
        params.setCity(options.getCity());
        params.setCompany(options.getCompany());
        params.setOccupation(options.getOccupation());
        params.setWebsite(options.getWebsite());
        ImageModule module = new ImageModule(options.getImage());
        params.setImage(module);

        params.setClientId(credentials.getClientId());

        final IBehanceSDKUpdateProfileListener finalListener =listener;
        BehanceSDKEditProfileAsyncTask editProfileAsyncTask = new BehanceSDKEditProfileAsyncTask(new IBehanceSDKEditProfileAsyncTaskListener() {
            @Override
            public void onEditProfileTaskSuccess(BehanceSDKEditProfileTaskResult result) {
                if(null != finalListener)
                    finalListener.onProfileUpdateSuccess();

            }

            @Override
            public void onEditProfileTaskFailure(BehanceSDKEditProfileTaskResult result) {
                if(null != finalListener)
                    finalListener.onProfileUpdateFailure(result.getExceptions());
            }
        });
        editProfileAsyncTask.execute(params);
    }

    public void launchEditProfileWorkFlow(Context context, IBehanceSDKEditProfileListener listener) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        launchEditProfileWorkFlow(context, null, listener);
    }

    public void launchEditProfileWorkFlow(Context context, BehanceSDKEditProfileOptions options, IBehanceSDKEditProfileListener listener) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        BehanceSDKEditProfileManager editProfileManager = BehanceSDKEditProfileManager.getInstance();
        editProfileManager.setUserCredentials(credentials);
        editProfileManager.setEditProfileListener(listener);
        Intent detailsIntent = new Intent(context, BehanceSDKEditProfileActivity.class);
        if (options != null) {
            detailsIntent.putExtra(BehanceSDKEditProfileActivity.INTENT_EXTRA_BOOLEAN_ENABLE_ALTERNATE_IMAGE_SELECTION_SOURCES, options.isEnableAlternateImageSelectionSources());
        }
        detailsIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(detailsIntent);
    }

    public BehanceSDKPublishImageValidationResult validateImageForProject(File imageFile){
        return BehanceSDKUtils.validateImageForProject(imageFile);
    }

    public BehanceSDKPublishImageValidationResult validateImageForProject(ImageModule imageModule){
        return BehanceSDKUtils.validateImageForProject(imageModule);
    }

    public BehanceSDKPublishImageValidationResult validateImageForWIP(File imageFile){
        return BehanceSDKUtils.validateImageForWIP(imageFile);
    }

    public BehanceSDKPublishImageValidationResult validateImageForWIP(ImageModule imageModule){
        return BehanceSDKUtils.validateImageForWIP(imageModule);
    }

    public void getBehanceUserProfile(final IBehanceSDKGetUserProfileListener listener) throws BehanceSDKUserNotEntitledException,
            BehanceSDKUserNotAuthenticatedException {
        validateIfUserIsEntitledForBehanceWorkflow();
        BehanceSDKGetUserProfileTaskParams params = new BehanceSDKGetUserProfileTaskParams();
        if(credentials.getUserAdobeAccountId() == null) {
            listener.onEditProfileFailure(new Exception("Adobe account id not provided"));
        } else {
            params.setUserId(credentials.getUserAdobeAccountId());
            params.setClientId(credentials.getClientId());
            IBehanceSDKGetUserProfileAsyncTaskListener asyncTaskListener = new IBehanceSDKGetUserProfileAsyncTaskListener() {
                @Override
                public void onGetUserProfileSuccess(BehanceSDKGetUserProfileTaskResult result, BehanceSDKGetUserProfileTaskParams params) {
                    BehanceSDKUserProfile userProfile = result.getBehanceUserProfile();
                    AdobeAuthUserProfile adobeUserProfile = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted().getUserProfile();
                    if (userProfile.getFirstName() == null || userProfile.getFirstName().isEmpty())
                        userProfile.setFirstName(adobeUserProfile.getFirstName());
                    if (userProfile.getLastName() == null || userProfile.getLastName().isEmpty())
                        userProfile.setFirstName(adobeUserProfile.getLastName());
                    listener.onGetUserProfileSuccess(userProfile);
                }

                @Override
                public void onGetUserProfileFailure(Exception e, BehanceSDKGetUserProfileTaskParams params) {
                    listener.onEditProfileFailure(e);
                }
            };

            BehanceSDKGetUserProfileAsyncTask getUserProfileAsyncTask = new BehanceSDKGetUserProfileAsyncTask(asyncTaskListener);
            getUserProfileAsyncTask.execute(params);
        }
    }

    public void searchProjects(BehanceSDKSearchProjectOptions options, IBehanceSDKSearchProjectListener listener) throws BehanceSDKUserNotAuthenticatedException,
            BehanceSDKUserNotEntitledException {

        validateIfUserIsEntitledForBehanceWorkflow();

        BehanceSDKSearchProjectsTaskParams params = new BehanceSDKSearchProjectsTaskParams();
        params.setClientId(credentials.getClientId());
        if(null != options) {
            params.setSearchString(options.getSearchString());
            params.setPageNumber(options.getPageNumber());
            params.setField(options.getField());
            params.setCountry(options.getCountry());
            params.setState(options.getState());
            params.setCity(options.getCity());
            params.setTags(options.getTags());
            params.setTimeSpan(options.getTimeSpan());
            params.setSortOption(options.getSortOption());
        }
        BehanceSDKSearchProjectsAsyncTask asyncTask = new BehanceSDKSearchProjectsAsyncTask(listener);
        asyncTask.execute(params);


    }

    public void checkWIPConversionStatus(String wipId, IBehanceSDKCheckWIPConversionStatusListener listener) {
        BehanceSDKCheckWIPConversionStatusAsyncTask asyncTask = new BehanceSDKCheckWIPConversionStatusAsyncTask(listener);
        BehanceSDKCheckWIPConversionStatusAsyncTaskParams params = new BehanceSDKCheckWIPConversionStatusAsyncTaskParams();
        params.setWipId(wipId);
        params.setClientId(credentials.getClientId());
        asyncTask.execute(params);
    }
}