/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2015 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk;

import android.app.Activity;

/**
 * Created by Sujit Reddy G
 */

public abstract class BehanceSDKAbstractPublishOptions {

    private Class<? extends Activity> notificationHandlerActivityClass;
    private boolean facebookShareEnabled = false;
    private String facebookClientId;
    private boolean twitterShareEnabled = false;
    private String twitterConsumerKey;
    private String twitterConsumerSecretKey;

    /**
     * Returns the Notification Handler Activity specified by the application.
     * This will be invoked when the user clicks on the project publish notification if the project publish is successful.
     *
     * @return Class<? extends Activity>
     */
    public Class<? extends Activity> getNotificationHandlerActivityClass() {
        return notificationHandlerActivityClass;
    }

    /**
     * Set the Notification Handler Activity. The activity set using this method will be
     * invoked using {@link Activity#startActivity(android.content.Intent)} method when the user clicks on the publish project notification.
     *
     * @param notificationHandlerActivityClass
     */
    public void setNotificationHandlerActivityClass(Class<? extends Activity> notificationHandlerActivityClass) {
        this.notificationHandlerActivityClass = notificationHandlerActivityClass;
    }

    /**
     * Returns true if the Facebook sharing is enabled.
     *
     * @return boolean
     */
    public boolean isFacebookShareEnabled() {
        return facebookShareEnabled;
    }

    /**
     * Set true if Facebook sharing is enabled for the publish project.
     *
     * @param facebookShareEnabled
     */
    public void setFacebookShareEnabled(boolean facebookShareEnabled) {
        this.facebookShareEnabled = facebookShareEnabled;
    }

    /**
     * Returns true if the Twitter sharing is enabled.
     *
     * @return
     */
    public boolean isTwitterShareEnabled() {
        return twitterShareEnabled;
    }

    /**
     * Set true if Twitter sharing is enabled for the publish project.
     *
     * @param twitterShareEnabled
     */
    public void setTwitterShareEnabled(boolean twitterShareEnabled) {
        this.twitterShareEnabled = twitterShareEnabled;
    }

    /**
     * Returns the client ID for Facebook
     *
     * @return
     */
    public String getFacebookClientId() {
        return facebookClientId;
    }

    /**
     * Set the client ID for Facebook.
     *
     * @param facebookClientId
     */
    public void setFacebookClientId(String facebookClientId) {
        this.facebookClientId = facebookClientId;
    }

    /**
     * Returns the Twitter Consumer Key.
     *
     * @return
     */
    public String getTwitterConsumerKey() {
        return twitterConsumerKey;
    }

    /**
     * Set the Twitter Consumer Key.
     *
     * @param twitterConsumerKey
     */
    public void setTwitterConsumerKey(String twitterConsumerKey) {
        this.twitterConsumerKey = twitterConsumerKey;
    }

    /**
     * Returns the Twitter Consumer Secret.
     *
     * @return
     */
    public String getTwitterConsumerSecretKey() {
        return twitterConsumerSecretKey;
    }

    /**
     * Set the Twitter Consumer Secret.
     *
     * @param twitterConsumerSecretKey
     */
    public void setTwitterConsumerSecretKey(String twitterConsumerSecretKey) {
        this.twitterConsumerSecretKey = twitterConsumerSecretKey;
    }
}