/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk;

import java.util.ArrayList;

/**
 * @author Sujit Reddy G
 *
 */

public class BehanceSDKImageSelectorOptions {

	public static int UNLIMITED_NUM_OF_IMAGES = -1;

	private int maxNumberOfImages;
	private BehanceSDKImageDimensions minImageDimensions;
	private BehanceSDKImageDimensions maxImageDimensions;
	private boolean showImageSelectionSources = true;
	private long minImageSizeInBytes = -1;
	private long maxImageSizeInBytes = -1;
	private ArrayList<String> allowedFileExtensions;
	private ArrayList<String> prohibitedFileExtensions;
	private String imageValidatorType;
	private boolean ccAssetBrowserDisabled = false;

	public String getImageValidatorType() {
		return imageValidatorType;
	}

	public void setImageValidatorType(String imageValidatorType) {
		this.imageValidatorType = imageValidatorType;
	}

	public static BehanceSDKImageSelectorOptions getSingleImageSelectionOptions() {
		return getSingleImageSelectionOptions(null, null);
	}

	public static BehanceSDKImageSelectorOptions getSingleImageSelectionOptions(BehanceSDKImageDimensions minImageDimensions,
																				BehanceSDKImageDimensions maxImageDimensions) {
		BehanceSDKImageSelectorOptions options = new BehanceSDKImageSelectorOptions();
		options.setMaxNumberOfImages(1);
		options.setMinImageDimensions(minImageDimensions);
		options.setMaxImageDimensions(maxImageDimensions);
		return options;
	}

	public static BehanceSDKImageSelectorOptions getUnlimitedImageSelectionOptions() {
		return getUnlimitedImageSelectionOptions(null, null);
	}

	public static BehanceSDKImageSelectorOptions getUnlimitedImageSelectionOptionsNoImageSource() {
		BehanceSDKImageSelectorOptions unlimitedImageSelectionOptions = getUnlimitedImageSelectionOptions(null, null);
		unlimitedImageSelectionOptions.hideImageSelectionSources();
		return unlimitedImageSelectionOptions;
	}

	public static BehanceSDKImageSelectorOptions getUnlimitedImageSelectionOptions(BehanceSDKImageDimensions minImageDimensions,
																				   BehanceSDKImageDimensions maxImageDimensions) {
		BehanceSDKImageSelectorOptions options = new BehanceSDKImageSelectorOptions();
		options.setMaxNumberOfImages(UNLIMITED_NUM_OF_IMAGES);
		options.setMinImageDimensions(minImageDimensions);
		options.setMaxImageDimensions(maxImageDimensions);
		return options;
	}

	public int getMaxNumberOfImages() {
		return maxNumberOfImages;
	}

	public void setMaxNumberOfImages(int maxNumberOfImages) {
		if (maxNumberOfImages == 0) {
			maxNumberOfImages = UNLIMITED_NUM_OF_IMAGES;
		}
		this.maxNumberOfImages = maxNumberOfImages;
	}

	public BehanceSDKImageDimensions getMinImageDimensions() {
		return minImageDimensions;
	}

	public void setMinImageDimensions(BehanceSDKImageDimensions minImageDimensions) {
		this.minImageDimensions = minImageDimensions;
	}

	public BehanceSDKImageDimensions getMaxImageDimensions() {
		return maxImageDimensions;
	}

	public void setMaxImageDimensions(BehanceSDKImageDimensions maxImageDimensions) {
		this.maxImageDimensions = maxImageDimensions;
	}

	public boolean isShowImageImageSelectionSources() {
		return showImageSelectionSources;
	}

	public void hideImageSelectionSources() {
		showImageSelectionSources = false;
	}

	public long getMinImageSizeInBytes() {
		return minImageSizeInBytes;
	}

	public void setMinImageSizeInBytes(long minImageSizeInBytes) {
		this.minImageSizeInBytes = minImageSizeInBytes;
	}

	public long getMaxImageSizeInBytes() {
		return maxImageSizeInBytes;
	}

	public void setMaxImageSizeInBytes(long maxImageSizeInBytes) {
		this.maxImageSizeInBytes = maxImageSizeInBytes;
	}

	public ArrayList<String> getAllowedFileExtensions() {
		return allowedFileExtensions;
	}

	public void setAllowedFileExtensions(ArrayList<String> allowedFileExtensions) {
		this.allowedFileExtensions = allowedFileExtensions;
	}

	public void addAllowedFileExtension(String fileExtension) {
		if (fileExtension != null && fileExtension.length() > 0) {
			if (allowedFileExtensions == null) {
				allowedFileExtensions = new ArrayList<String>();
			}
			allowedFileExtensions.add(fileExtension);
		}
	}

	public ArrayList<String> getProhibitedFileExtensions() {
		return prohibitedFileExtensions;
	}

	public void setProhibitedFileExtensions(ArrayList<String> prohibitedFileExtensions) {
		this.prohibitedFileExtensions = prohibitedFileExtensions;
	}

	public void addProhibitedFileExtension(String fileExtension) {
		if (fileExtension != null && fileExtension.length() > 0) {
			if (prohibitedFileExtensions == null) {
				prohibitedFileExtensions = new ArrayList<String>();
			}
			prohibitedFileExtensions.add(fileExtension);
		}
	}

	public void setCCAssetBrowserDisabled(boolean ccAssetBrowserDisabled){
		this.ccAssetBrowserDisabled = ccAssetBrowserDisabled;
	}

	public boolean isCCAssetBrowserDisabled(){
		return ccAssetBrowserDisabled;
	}
}