package com.behance.sdk;

import com.behance.sdk.enums.BehanceSDKCopyrightOption;

import java.io.File;

/**
 * Created by ugs on 02/07/15.
 */
public class BehanceSDKProjectDraftOptions {

    private String projectTitle;

    private String projectDescription;

    private String projectTags;

    private BehanceSDKCopyrightOption projectCopyrightSettings;

    private boolean projectContainsAdultContent;

    private String creativeFields;

    private File coverImage;

    /**
     * Returns Title of the Project set by user.
     *
     * @return String
     */
    public String getProjectTitle() {
        return projectTitle;
    }

    /**
     * Optional Title for new project
     *
     *
     * @param projectTitle Title for new project
     */
    public void setProjectTitle(String projectTitle) {
        this.projectTitle = projectTitle;
    }

    /**
     * Description for new project
     *
     * @return Description for new project
     */
    public String getProjectDescription() {
        return projectDescription;
    }

    /**
     * Optional Description for new project
     *
     * @param projectDescription Description for new project
     */
    public void setProjectDescription(String projectDescription) {
        this.projectDescription = projectDescription;
    }


    /**
     * Comma separated tags
     *
     * @return Comma separated tags
     */
    public String getProjectTags() {
        return projectTags;
    }

    /**
     * Comma separated tags to be set for project
     *
     * @param projectTags Comma separated tags to be set for project
     */
    public void setProjectTags(String projectTags) {
        this.projectTags = projectTags;
    }

    /**
     * Return CopyrightSettings of the project set by user
     *
     * @return BehanceSDKCopyrightOption
     *                  CopyrightSettings of the project
     */
    public BehanceSDKCopyrightOption getProjectCopyrightSettings() {
        return projectCopyrightSettings;
    }

    /**
     * Optional CopyrightSettings for new project.
     *
     * @param projectCopyrightSettings
     *                     CopyrightSettings for new project
     */
    public void setProjectCopyrightSettings(BehanceSDKCopyrightOption projectCopyrightSettings) {
        this.projectCopyrightSettings = projectCopyrightSettings;
    }


    /**
     * Return true if user marked the project content as adult content.
     *
     * @return true if project contains adult content.
     */
    public boolean isProjectContainsAdultContent() {
        return projectContainsAdultContent;
    }

    /**
     * Optional Adult content marker for new project.
     *
     * @param projectContainsAdultContent
     *                     set true if project contains adult content.
     */
    public void setProjectContainsAdultContent(boolean projectContainsAdultContent) {
        this.projectContainsAdultContent = projectContainsAdultContent;
    }

    /**
     * Returns the creative fields string set by the user.
     *
     * @return  creative fields string set by the user.
     */
    public String getCreativeFields() {
        return creativeFields;
    }

    /**
     * Creative fields strings pipe-delimited for the new project
     *
     * Valid creative field strings can be found from: http://www.behance.net/v2/fields,
     * ex "calligraphy|advertising|fashion"
     *
     * @param creativeFields pipe-delimited creative field strings.
     */
    public void setCreativeFields(String creativeFields) {
        this.creativeFields = creativeFields;
    }

    /**
     *  Returns the cover image set by the user for this project draft.
     *
     *  @return cover image file for the project draft.
     */
    public File getCoverImage() {
        return coverImage;
    }

    /**
     * Set cover image for the project draft.
     *
     * @param coverImage cover image file for the project draft
     */
    public void setCoverImage(File coverImage) {
        this.coverImage = coverImage;
    }
}
