/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.project.modules.ImageModule;

public class BehanceSDKPublishProjectOptions extends BehanceSDKAbstractPublishOptions {


    private List<ImageModule> projectImageModules;

    private String projectTitle;
    private String projectDescription;
    private String projectTags;
    private BehanceSDKCopyrightOption projectCopyrightSettings;
    private boolean projectContainsAdultContent;
    private boolean ccAssetBrowserDisabled = false;


    public BehanceSDKPublishProjectOptions(List<File> projectImageModules){
        setProjectImageModules(projectImageModules);
    }

    /**
     * Returns List of ImageModules representing images to be published as a project.
     *
     * @return java.util.List<com.behance.sdk.project.modules.ImageModule>
     */
    public List<ImageModule> getProjectImageModules() {
        return projectImageModules;
    }

    /**
     * Set List of ImageModules representing images to be published as a project.
     *
     * @param projectImageModules List of images which are getting published as a Behance Project.
     */
    private void setProjectImageModules(List<File> projectImageModules) {
        if(null != projectImageModules && false == projectImageModules.isEmpty()) {
            this.projectImageModules = new ArrayList<ImageModule>(projectImageModules.size());
            for (File image : projectImageModules) {
                ImageModule imageModule = new ImageModule(image);
                this.projectImageModules.add(imageModule);
            }
        }
    }


    /**
     * Returns Title of the Project set by user.
     *
     * @return String
     */
    public String getProjectTitle() {
        return projectTitle;
    }

    /**
     * Optional Title for new project
     *
     *
     * @param projectTitle Title for new project
     */
    public void setProjectTitle(String projectTitle) {
        this.projectTitle = projectTitle;
    }

    /**
     * Description for new project
     *
     * @return Description for new project
     */
    public String getProjectDescription() {
        return projectDescription;
    }

    /**
     * Optional Description for new project
     *
     * @param projectDescription Description for new project
     */
    public void setProjectDescription(String projectDescription) {
        this.projectDescription = projectDescription;
    }


    /**
     * Comma separated tags
     *
     * @return Comma separated tags
     */
    public String getProjectTags() {
        return projectTags;
    }

    /**
     * Comma separated tags to be set for project
     *
     * @param projectTags Comma separated tags to be set for project
     */
    public void setProjectTags(String projectTags) {
        this.projectTags = projectTags;
    }

    /**
     * Return CopyrightSettings of the project set by user
     *
     * @return BehanceSDKCopyrightOption
     *                  CopyrightSettings of the project
     */
    public BehanceSDKCopyrightOption getProjectCopyrightSettings() {
        return projectCopyrightSettings;
    }

    /**
     * Optional CopyrightSettings for new project.
     *
     * @param projectCopyrightSettings
     *                     CopyrightSettings for new project
     */
    public void setProjectCopyrightSettings(BehanceSDKCopyrightOption projectCopyrightSettings) {
        this.projectCopyrightSettings = projectCopyrightSettings;
    }


    /**
     * Return true if user marked the project content as adult content.
     *
     * @return true if project contains adult content.
     */
    public boolean isProjectContainsAdultContent() {
        return projectContainsAdultContent;
    }

    /**
     * Optional Adult content marker for new project.
     *
     * @param projectContainsAdultContent
     *                     set true if project contains adult content.
     */
    public void setProjectContainsAdultContent(boolean projectContainsAdultContent) {
        this.projectContainsAdultContent = projectContainsAdultContent;
    }

    /**
     * Set whether CC Asset Browser albums is to be shown or not
     *
     * @param ccAssetBrowserDisabled
     */
    public void setCCAssetBrowserDisabled(Boolean ccAssetBrowserDisabled){
        this.ccAssetBrowserDisabled = ccAssetBrowserDisabled;
    }

    /**
     * Returns whether Asset browser albums is disabled or not
     * @return
     */
    public boolean isCCAssetBrowserDisabled(){
        return ccAssetBrowserDisabled;
    }

}