/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk;

import com.behance.sdk.project.modules.ImageModule;

import java.io.File;

/**
 * Options for Publishing WIP on Behance
 *
 * @author Sujit Reddy G
 */

public class BehanceSDKPublishWIPOptions extends BehanceSDKAbstractPublishOptions {

    private ImageModule wipRevisionImageModule;
    private String wipTitle;
    private String wipDescription;
    private int existingWIPId;
    private String wipTags;

    /**
     * @param wipRevisionImageFile File of image to be published as WIP
     */
    public BehanceSDKPublishWIPOptions(File wipRevisionImageFile) {
        ImageModule imageModule = new ImageModule(wipRevisionImageFile);
        setWipRevisionImageModule(imageModule);
    }

    /**
     * @param wipRevisionImageModule ImageModule class representing image to be published as WIP
     */
    public BehanceSDKPublishWIPOptions(ImageModule wipRevisionImageModule) {
        setWipRevisionImageModule(wipRevisionImageModule);
    }

    /**
     * Returns ImageModule representing image to be published as WIP
     *
     * @return com.behance.sdk.project.modules.ImageModule
     */
    public ImageModule getWipRevisionImageModule() {
        return wipRevisionImageModule;
    }

    /**
     * Set ImageModule representing image to be published as WIP
     *
     * @param wipRevisionImageModule ImageModule class representing image to be published as WIP
     */
    private void setWipRevisionImageModule(ImageModule wipRevisionImageModule) {
        this.wipRevisionImageModule = wipRevisionImageModule;
    }

    /**
     * Returns Title of the WIP
     *
     * @return String
     */
    public String getWipTitle() {
        return wipTitle;
    }

    /**
     * Optional Title for new WIP
     * If this value is set, a new WIP is created with this name and the image will be published in the new WIP
     *
     * @param wipTitle Title for new WIP
     */
    public void setWipTitle(String wipTitle) {
        this.wipTitle = wipTitle;
    }

    /**
     * Description for new WIP
     *
     * @return Description for new WIP
     */
    public String getWipDescription() {
        return wipDescription;
    }

    /**
     * Optional Description for new WIP
     *
     * @param wipDescription Description for new WIP
     */
    public void setWipDescription(String wipDescription) {
        this.wipDescription = wipDescription;
    }

    /**
     * @return Id of WIP
     */
    public int getExistingWIPId() {
        return existingWIPId;
    }

    /**
     * Set this value if you want to publish revision to an existing WIP. If
     * this value is set, value set using setTitle() is ignored
     *
     * @param existingWIPId Id of WIP
     */
    public void setExistingWIPId(int existingWIPId) {
        this.existingWIPId = existingWIPId;
    }

    /**
     * Comma separated tags
     *
     * @return
     */
    public String getWipTags() {
        return wipTags;
    }

    /**
     * Comma separated tags to be set for WIP
     *
     * @param wipTags Comma separated tags to be set for WIP
     */
    public void setWipTags(String wipTags) {
        this.wipTags = wipTags;
    }
}