/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;

import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.facebook.login.LoginManager;
//import com.facebook.Session;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKSocialAccountManager {

	private static final String SHARED_PREFERENCES_FILE_NAME = "BEHANCE_SDK_SOCIAL_ACCOUNTS_SHARED_PREFERENES_KEY";

	private static final String SP_KEY_ACCOUNT_CLIENT_ID = "SP_KEY_ACCOUNT_CLIENT_ID";
	private static final String SP_KEY_ACCOUNT_CLIENT_SECRET = "SP_KEY_ACCOUNT_CLIENT_SECRET";
	private static final String SP_KEY_USER_ID = "SP_KEY_USER_ID";
	private static final String SP_KEY_ACCOUNT_DISPLAY_NAME_RES_ID = "SP_KEY_ACCOUNT_DISPLAY_NAME_RES_ID";
	private static final String SP_KEY_USER_AUTH_TOKEN = "SP_KEY_USER_AUTH_TOKEN";
	private static final String SP_KEY_USER_AUTH_TOKEN_EXPIRY_TIME = "SP_KEY_USER_AUTH_TOKEN_EXPIRY_TIME";
	private static final String SP_KEY_ADDITIONAL_DATA = "SP_KEY_ADDITIONAL_DATA";
	private static final String SP_KEY_SHARE_ENABLED_LAST_TIME = "SP_KEY_SHARE_ENABLED_LAST_TIME";
	private static final String SP_KEY_USER_AUTHENTICATED = "SP_KEY_USER_AUTHENTICATED";

	private static BehanceSDKSocialAccountManager instance;

	private Context context;

	public static BehanceSDKSocialAccountManager getInstance(Context context) {
		if (instance == null) {
			instance = new BehanceSDKSocialAccountManager(context);
		}
		return instance;
	}

	private BehanceSDKSocialAccountManager(Context context) {
		this.context = context;
	}

	public boolean addAccount(BehanceSDKSocialAccountDTO account) {
		if (account != null) {
			BehanceSDKSocialAccountDTO existingAccount = getAccount(account.getAccountType());
			if (existingAccount != null) {
				if (!areAccountsSame(existingAccount, account)) {
					return saveAccountToStorage(account);
				} else {
					return true;
				}
			} else {
				return saveAccountToStorage(account);
			}
		}
		return false;
	}

	public boolean updateAccount(BehanceSDKSocialAccountDTO account) {
		return addAccount(account);
	}

	private boolean areAccountsSame(BehanceSDKSocialAccountDTO firstAccount, BehanceSDKSocialAccountDTO secondAccount) {

		if ((firstAccount.getAccountClientId() != null && secondAccount.getAccountClientId() == null)
				|| (firstAccount.getAccountClientId() == null && secondAccount.getAccountClientId() != null)
				|| (firstAccount.getAccountClientId() != null && !firstAccount.getAccountClientId().equals(secondAccount.getAccountClientId()))) {
			return false;
		}
		if ((firstAccount.getAccountClientSecret() != null && secondAccount.getAccountClientSecret() == null)
				|| (firstAccount.getAccountClientSecret() == null && secondAccount.getAccountClientSecret() != null)
				|| (firstAccount.getAccountClientSecret() != null && !firstAccount.getAccountClientSecret().equals(
						secondAccount.getAccountClientSecret()))) {
			return false;
		}
		if (firstAccount.getAccountDisplayNameResourceId() != secondAccount.getAccountDisplayNameResourceId()) {
			return false;
		}
		if ((firstAccount.getAdditionalData() != null && secondAccount.getAdditionalData() == null)
				|| (firstAccount.getAdditionalData() == null && secondAccount.getAdditionalData() != null)
				|| (firstAccount.getAdditionalData() != null && !firstAccount.getAdditionalData().equals(secondAccount.getAdditionalData()))) {
			return false;
		}
		if ((firstAccount.getUserAuthToken() != null && secondAccount.getUserAuthToken() == null)
				|| (firstAccount.getUserAuthToken() == null && secondAccount.getUserAuthToken() != null)
				|| (firstAccount.getUserAuthToken() != null && !firstAccount.getUserAuthToken().equals(secondAccount.getUserAuthToken()))) {
			return false;
		}
		if (firstAccount.getUserAuthTokenExpiryTime() != secondAccount.getUserAuthTokenExpiryTime()) {
			return false;
		}
		if ((firstAccount.getUserId() != null && secondAccount.getUserId() == null)
				|| (firstAccount.getUserId() == null && secondAccount.getUserId() != null)
				|| (firstAccount.getUserId() != null && !firstAccount.getUserId().equals(secondAccount.getUserId()))) {
			return false;
		}
		if (firstAccount.isSharingEnabledLastTime() != secondAccount.isSharingEnabledLastTime()) {
			return false;
		}
		if (firstAccount.isUserAuthenticated() != secondAccount.isUserAuthenticated()) {
			return false;
		}
		return true;
	}

	public BehanceSDKSocialAccountDTO getAccount(BehanceSDKSocialAccountType accountType) {
		return readAccountFromStorage(accountType);
	}

	public boolean removeAccount(BehanceSDKSocialAccountType accountType) {
		if (accountType == BehanceSDKSocialAccountType.FACEBOOK) {
			BehanceSDKSocialAccountDTO facebookAccount = getAccount(BehanceSDKSocialAccountType.FACEBOOK);
			if (facebookAccount != null) {
				/*Session session = new Session.Builder(context).setApplicationId(facebookAccount.getAccountClientId()).build();
				session.closeAndClearTokenInformation();
				Session.setActiveSession(null);*/
				LoginManager.getInstance().logOut();
			}
		}
		return removeAccountFromStorage(accountType);
	}

	public boolean revokeAccountAccess(BehanceSDKSocialAccountType accountType) {
		return removeAccount(accountType);
	}

	private SharedPreferences getSharedPreference(BehanceSDKSocialAccountType accountType) {
		String accountFileName = SHARED_PREFERENCES_FILE_NAME + "_" + accountType.name();
		SharedPreferences sharedPreferences = context.getSharedPreferences(accountFileName, Context.MODE_PRIVATE);
		return sharedPreferences;
	}

	private SharedPreferences getExistingAccountSharedPreference(BehanceSDKSocialAccountType accountType) {
		SharedPreferences sharedPreferences = getSharedPreference(accountType);
		if (sharedPreferences.contains(SP_KEY_ACCOUNT_CLIENT_ID)) {
			return sharedPreferences;
		} else {
			return null;
		}
	}

	private BehanceSDKSocialAccountDTO readAccountFromStorage(BehanceSDKSocialAccountType accountType) {
		SharedPreferences sharedPreference = getExistingAccountSharedPreference(accountType);
		if (sharedPreference != null) {
			BehanceSDKSocialAccountDTO account = new BehanceSDKSocialAccountDTO();
			account.setAccountClientId(sharedPreference.getString(SP_KEY_ACCOUNT_CLIENT_ID, null));
			account.setAccountClientSecret(sharedPreference.getString(SP_KEY_ACCOUNT_CLIENT_SECRET, null));
			account.setAccountDisplayNameResourceId(sharedPreference.getInt(SP_KEY_ACCOUNT_DISPLAY_NAME_RES_ID, -1));
			account.setAccountType(accountType);
			account.setUserAuthToken(sharedPreference.getString(SP_KEY_USER_AUTH_TOKEN, null));
			account.setUserAuthTokenExpiryTime(sharedPreference.getLong(SP_KEY_USER_AUTH_TOKEN_EXPIRY_TIME, 0));
			account.setUserId(sharedPreference.getString(SP_KEY_USER_ID, null));
			account.setAdditionalData(sharedPreference.getString(SP_KEY_ADDITIONAL_DATA, null));
			account.setSharingEnabledLastTime(sharedPreference.getBoolean(SP_KEY_SHARE_ENABLED_LAST_TIME, false));
			account.setUserAuthenticated(sharedPreference.getBoolean(SP_KEY_USER_AUTHENTICATED, false));
			return account;
		} else {
			return null;
		}
	}

	private boolean saveAccountToStorage(BehanceSDKSocialAccountDTO account) {
		SharedPreferences sharedPreference = getSharedPreference(account.getAccountType());
		Editor editor = sharedPreference.edit();
		editor.putString(SP_KEY_ACCOUNT_CLIENT_ID, account.getAccountClientId());
		editor.putString(SP_KEY_ACCOUNT_CLIENT_SECRET, account.getAccountClientSecret());
		editor.putInt(SP_KEY_ACCOUNT_DISPLAY_NAME_RES_ID, account.getAccountDisplayNameResourceId());
		editor.putString(SP_KEY_USER_AUTH_TOKEN, account.getUserAuthToken());
		editor.putLong(SP_KEY_USER_AUTH_TOKEN_EXPIRY_TIME, account.getUserAuthTokenExpiryTime());
		editor.putString(SP_KEY_USER_ID, account.getUserId());
		editor.putString(SP_KEY_ADDITIONAL_DATA, account.getAdditionalData());
		editor.putBoolean(SP_KEY_SHARE_ENABLED_LAST_TIME, account.isSharingEnabledLastTime());
		editor.putBoolean(SP_KEY_USER_AUTHENTICATED, account.isUserAuthenticated());
		return editor.commit();
	}

	private boolean removeAccountFromStorage(BehanceSDKSocialAccountType accountType) {
		SharedPreferences sharedPreference = getSharedPreference(accountType);
		Editor edit = sharedPreference.edit();
		edit.clear();
		return edit.commit();
	}
}