package com.behance.sdk.OAuth;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.basic.DefaultOAuthProvider;

public class TwitterClient {

	// Twitter Constants
	public static final String TWITTER_OAUTH_CALLBACK_URI = "com.behance.behance://oauth.callback";

	public static final String TWITTER_REQUEST_TOKEN_URL = "https://api.twitter.com/oauth/request_token";

	public static final String TWITTER_AUTHORIZE_TOKEN_URL = "https://api.twitter.com/oauth/authorize";

	public static final String TWITTER_ACCESS_TOKEN_URL = "https://api.twitter.com/oauth/access_token";

	private String oAuthURL;
	private String oAuthVerifier;
	private String userName;
	private static TwitterClient instance;

	// The reason we are assigning a provider and consumer to a client is
	// the OAuth works only if the retrieveAccessToken and retrieveRequestToken
	// are called using the same instance of provider and consumer. Otherwise
	// you will get the following error:-
	// Authorized request token or token secret not set. Did you retrieve an
	// authorized request token before?

	private OAuthProvider mProvider;
	private OAuthConsumer mConsumer;

	public String getoAuthURL() {
		return oAuthURL;
	}

	public void setoAuthURL(String oAuthURL) {
		this.oAuthURL = oAuthURL;
	}

	public String getoAuthVerifier() {
		return oAuthVerifier;
	}

	public void setoAuthVerifier(String oAuthVerifier) {
		this.oAuthVerifier = oAuthVerifier;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public static TwitterClient getInstance(String consumerToken, String consumerSecret) {
		if (instance == null) {
			instance = new TwitterClient(consumerToken,consumerSecret);
		}
		return instance;
	}

	private TwitterClient(String consumerToken, String consumerSecret) {
		initialize(consumerToken,consumerSecret);
	}

	private void initialize(String consumerToken, String consumerSecret) {
		mProvider = new DefaultOAuthProvider(TWITTER_REQUEST_TOKEN_URL, TWITTER_ACCESS_TOKEN_URL, TWITTER_AUTHORIZE_TOKEN_URL);
		mConsumer = new DefaultOAuthConsumer(consumerToken,consumerSecret);
	}

	public OAuthProvider getOAuthProvider() {
		return mProvider;
	}

	public OAuthConsumer getOAuthConsumer() {
		return mConsumer;
	}
}
