/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctask.params;

import com.behance.sdk.R;
import com.behance.sdk.enums.BehanceSDKProjectsSortOption;
import com.behance.sdk.enums.BehanceSDKProjectsTimeSpan;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKGetProjectsAsyncTaskParams extends BehanceSDKSearchProjectsTaskParams {

	private String field;
	private BehanceSDKProjectsSortOption sortOption;
	private BehanceSDKProjectsTimeSpan timeSpan;
	private String country;
	private String state;
	private String city;

	public BehanceSDKProjectsSortOption getSortOption() {
		return sortOption;
	}

	public void setSortOption(BehanceSDKProjectsSortOption sortOption) {
		this.sortOption = sortOption;
	}

	public BehanceSDKProjectsTimeSpan getTimeSpan() {
		return timeSpan;
	}

	public void setTimeSpan(BehanceSDKProjectsTimeSpan timeSpan) {
		this.timeSpan = timeSpan;
	}

	public String getField() {
		return field;
	}

	public void setField(String field) {
		this.field = field;
	}

	/**
	 * @return the country
	 */
	public String getCountry() {
		return country;
	}

	/**
	 * @param country
	 *            the country to set
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	/**
	 * @return the state
	 */
	public String getState() {
		return state;
	}

	/**
	 * @param state
	 *            the state to set
	 */
	public void setState(String state) {
		this.state = state;
	}

	/**
	 * @return the city
	 */
	public String getCity() {
		return city;
	}

	/**
	 * @param city
	 *            the city to set
	 */
	public void setCity(String city) {
		this.city = city;
	}

	public boolean compareSearchFilters(BehanceSDKGetProjectsAsyncTaskParams params) {

		if (null == params)
			return false;

		return sortOption == params.sortOption && timeSpan == params.timeSpan
				&& (field == params.field || (field != null && field.equals(params.getField())))
				&& (country == params.country || (country != null && country.equals(params.getCountry())))
				&& (state == params.state || (state != null && state.equals(params.getState())))
				&& (city == params.city || (city != null && city.equals(params.getCity()))) && super.compareSearchFilters(params);
	}

	public enum ProjectsTimeSpan {
		ALL("all"), WEEK("week"), MONTH("month"), TODAY("today");

		private String actualValue;

		ProjectsTimeSpan(String actualValue) {
			this.actualValue = actualValue;
		}

		public static ProjectsTimeSpan fromName(String name) {
			for (ProjectsTimeSpan value : values()) {
				if (value.name().equalsIgnoreCase(name)) {
					return value;
				}
			}
			return null;
		}

		public static ProjectsTimeSpan fromName(String name, ProjectsTimeSpan defaultValue) {
			ProjectsTimeSpan fromName = fromName(name);
			if (null != fromName)
				return fromName;
			return defaultValue;
		}

		@Override
		public String toString() {
			return actualValue;
		}
	}

	public enum ProjectsSortOption {

		FEATURED_DATE("featured_date", R.string.bsdk_sort_option_featured, 0), APPRECIATIONS("appreciations", R.string.bsdk_sort_option_most_appreciated, 1), VIEWS(
				"views", R.string.bsdk_sort_option_most_viewed, 2), COMMENTS("comments", R.string.bsdk_sort_option_most_discussed, 3), PUBLISHED_DATE(
				"published_date", R.string.bsdk_sort_option_most_recent, 4), RELEVANCE("relevance", R.string.bsdk_sort_option_most_relevant, 5);

		String actualValue;
		int index;
		private int displayNameResourceId;

		ProjectsSortOption(String actualValue, int displayNameResourceId, int index) {
			this.actualValue = actualValue;
			this.displayNameResourceId = displayNameResourceId;
			this.index = index;
		}

		public static ProjectsSortOption fromName(String name) {
			for (ProjectsSortOption value : values()) {
				if (value.name().equalsIgnoreCase(name)) {
					return value;
				}
			}
			return null;
		}

		public static ProjectsSortOption fromName(String name, ProjectsSortOption defaultValue) {
			ProjectsSortOption fromName = fromName(name);
			if (null != fromName)
				return fromName;
			return defaultValue;
		}

		public static ProjectsSortOption fromIndex(int index) {
			for (ProjectsSortOption value : values()) {
				if (value.getIndex() == index) {
					return value;
				}
			}
			return FEATURED_DATE;
		}

		public int getDisplayNameResourceId() {
			return displayNameResourceId;
		}

		public int getIndex() {
			return index;
		}

		@Override
		public String toString() {
			return actualValue;
		}
	}
}
