package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.IBehanceSDKADDProjectModuleListener;
import com.behance.sdk.asynctask.params.BehanceSDKAddProjectModuleTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by ugs on 07/07/15.
 */
public class BehanceSDKAddProjectModuleAsyncTask  extends AsyncTask<BehanceSDKAddProjectModuleTaskParams, Void, BehanceSDKAsyncTaskResultWrapper>{

    private static final String MODULE_TYPE_KEY = "type";
    private static final String MODULE_TYPE_IMAGE_VALUE = "image";
    private static final String MODULE_IMAGE_KEY = MODULE_TYPE_IMAGE_VALUE;
    private static final String PROJECTID_KEY = "{PROJECTID}";

    private IBehanceSDKADDProjectModuleListener listener;

    public BehanceSDKAddProjectModuleAsyncTask(IBehanceSDKADDProjectModuleListener listener){

        this.listener = listener;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper doInBackground(BehanceSDKAddProjectModuleTaskParams... params) {
        BehanceSDKAddProjectModuleTaskParams serviceParams = params[0];
        BehanceSDKAsyncTaskResultWrapper result = new BehanceSDKAsyncTaskResultWrapper();
        try {
            Map<String, String> urlParams = new HashMap<String, String>(1);
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParams.getClientId());
            String finalAddModuleUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.ADD_PROJECT_MODULE_URL, urlParams);
            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                finalAddModuleUrl = BehanceSDKUrlUtil.appendQueryStringParam(finalAddModuleUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN,
                        userAccessToken);
            }
            finalAddModuleUrl = finalAddModuleUrl.replace(PROJECTID_KEY, serviceParams.getProjectId());

            BehanceMultipartEntity uploadModuleEntity = new BehanceMultipartEntity();
            uploadModuleEntity.addTextPart(MODULE_TYPE_KEY,BehanceMultipartEntity.CONTENT_TYPE_TEXT_ISO_8859,
                    BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,MODULE_TYPE_IMAGE_VALUE.getBytes(BehanceMultipartEntity.CHARSET_ISO_8859));
            uploadModuleEntity.addImagePart(MODULE_IMAGE_KEY, serviceParams.getImage().getName(), BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM,
                    BehanceMultipartEntity.CONTENT_TRANSFER_BINARY, serviceParams.getImage().getAbsolutePath());
            uploadModuleEntity.setChunked(true);

            BehanceConnectionResponse<String> uploadModuleHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(finalAddModuleUrl, uploadModuleEntity, null);

            if (uploadModuleHttpResponse.getResponseCode() != BehanceHttpsConnection.SC_CREATED) {
                result.setExceptionOccurred(true);
                result.setException(new BehanceSDKException(uploadModuleHttpResponse.getResponseObject()));
            }else{
                result.setExceptionOccurred(false);
            }

        } catch (Throwable throwable){
            result.setExceptionOccurred(true);
            result.setException(new BehanceSDKException(throwable));
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper result) {
        if(null != listener){
            if(result.isExceptionOccurred()){
                listener.onFailure((BehanceSDKException) result.getException());
            }else{
                listener.onSuccess();
            }
        }

    }
}
