/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKAppreciateProjectAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKAppreciateProjectAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.Logger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class BehanceSDKAppreciateProjectAsyncTask extends AsyncTask<BehanceSDKAppreciateProjectAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKAppreciateProjectAsyncTask.class);

	private IBehanceSDKAppreciateProjectAsyncTaskListener taskHandler;
	private BehanceSDKAppreciateProjectAsyncTaskParams appreciateProjectParams;

	public BehanceSDKAppreciateProjectAsyncTask(IBehanceSDKAppreciateProjectAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKAppreciateProjectAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
		result.setResult(false);

		appreciateProjectParams = params[0];

		String projectId = appreciateProjectParams.getProjectId();

		try {
			if (projectId != null && projectId.length() > 0) {

				Map<String, String> urlParams = new HashMap<String, String>();
				urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
				urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectId);

				String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.APPRECIATE_PROJECT_API_URL, urlParams);

				String userAccessToken = appreciateProjectParams.getUserAccessToken();
				if (userAccessToken != null) {
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
				}

				logger.debug("Appreciate Project url - %s", url);

				BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url);

				String jsonResponseString = connectionResponse.getResponseObject();
				logger.debug("Appreciate Project response for [project id - %s ] : %s", projectId, jsonResponseString);
				int httpStatusCode = connectionResponse.getResponseCode();
				if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
					JSONObject jsonObject = new JSONObject(jsonResponseString);
					httpStatusCode = jsonObject.getInt("http_code");
					if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
						result.setResult(true);
					} else if (httpStatusCode == IBehanceHttpConnection.SC_NOT_FOUND) {
						logger.error("HTTP Response code 404 when trying to appreciate project [Project ID - %s]", projectId);
						result.setException(new BehanceSDKException("Project not found"));
						result.setExceptionOccurred(true);
						result.setResult(false);
					} else {
						logger.error("Unexpected HTTP Response code when trying to appreciate project [Project ID - %s] [Response code - %d]", projectId,httpStatusCode);
						result.setException(new BehanceSDKException("Invalid server response code " + httpStatusCode));
						result.setExceptionOccurred(true);
						result.setResult(false);
					}
				}
				logger.debug("Appreciate Project http response status code - %s", httpStatusCode);
			}

		} catch (Exception e) {
			logger.error(e, "Problem trying to appreciate project [projectId - %s]", projectId);
			result.setException(e);
			result.setExceptionOccurred(true);
			result.setResult(false);
		} catch (Throwable t) {
			logger.error(t, "Problem trying to appreciate project [projectId - %s]", projectId);
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
			result.setResult(false);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onAppreciateProjectAsyncTaskFailure(resultWrapper.getException(), appreciateProjectParams);
		} else {
			taskHandler.onAppreciateProjectAsyncTaskSuccess(resultWrapper.getResult(), appreciateProjectParams);
		}
	}
}