package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.IBehanceSDKCheckWIPConversionStatusListener;
import com.behance.sdk.asynctask.params.BehanceSDKCheckWIPConversionStatusAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.asynctasks.result.BehanceSDKCheckWIPConversionStatusTaskResult;
import com.behance.sdk.enums.BehanceSDKWIPConversionStatus;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by forta on 4/11/16.
 */
public class BehanceSDKCheckWIPConversionStatusAsyncTask extends AsyncTask<BehanceSDKCheckWIPConversionStatusAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<BehanceSDKCheckWIPConversionStatusTaskResult>> {

    private BehanceSDKCheckWIPConversionStatusAsyncTaskParams taskParams;

    private IBehanceSDKCheckWIPConversionStatusListener taskHandler;

    public BehanceSDKCheckWIPConversionStatusAsyncTask(IBehanceSDKCheckWIPConversionStatusListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<BehanceSDKCheckWIPConversionStatusTaskResult> doInBackground(BehanceSDKCheckWIPConversionStatusAsyncTaskParams... params) {
        BehanceSDKAsyncTaskResultWrapper<BehanceSDKCheckWIPConversionStatusTaskResult> resultWrapper = new BehanceSDKAsyncTaskResultWrapper<>();

        if (params.length != 1) {
            resultWrapper.setException(new BehanceSDKException("WIP Id is required"));
            resultWrapper.setExceptionOccurred(true);
            return resultWrapper;
        }

        try {
            taskParams = params[0];
            String wipId = taskParams.getWipId();

            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
            urlParams.put(BehanceSDKUrlUtil.KEY_WIP_ID, wipId);
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.CHECK_WIP_CONVERSION_STATUS_API_URL, urlParams);

            String userAccessToken = taskParams.getUserAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
            String jsonResponseString = response.getResponseObject();

            BehanceSDKCheckWIPConversionStatusTaskResult result = new BehanceSDKCheckWIPConversionStatusTaskResult();

            switch (response.getResponseCode()) {
                case 400:
                    result.setConversionStatus(BehanceSDKWIPConversionStatus.WIPConversionStatusNotConverted);
                    break;
                case 404:
                    result.setConversionStatus(BehanceSDKWIPConversionStatus.WIPConversionStatusDoesNotExist);
                    break;
                case 403:
                    result.setConversionStatus(BehanceSDKWIPConversionStatus.WIPConversionStatusUnowned);
                    break;
                case 200:
                    JSONObject responseOb = new JSONObject(jsonResponseString);
                    JSONObject projectOb = responseOb.optJSONObject("project");
                    result.setProjectId(projectOb.optInt("id"));
                    result.setConversionStatus(projectOb.optLong("published_on", 0) == 0 ?
                            BehanceSDKWIPConversionStatus.WIPConversionStatusConvertedNotPublished :
                            BehanceSDKWIPConversionStatus.WIPConversionStatusConvertedPublished);
                    break;
                default:
                    result.setConversionStatus(BehanceSDKWIPConversionStatus.WIPConversionStatusUnknown);
            }

            resultWrapper.setResult(result);


        } catch (IOException | BehanceSDKUserNotAuthenticatedException e) {
            e.printStackTrace();
            resultWrapper.setException(e);
            resultWrapper.setExceptionOccurred(true);
        } catch (JSONException e) {
            e.printStackTrace();
            resultWrapper.setException(e);
            resultWrapper.setExceptionOccurred(true);
        }

        return resultWrapper;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<BehanceSDKCheckWIPConversionStatusTaskResult> resultWrapper) {
        if (resultWrapper.isExceptionOccurred()) {
            taskHandler.onCheckWIPConversionStatusFailure(resultWrapper.getException());
        } else {
            taskHandler.onCheckWIPConversionStatusSuccess(resultWrapper.getResult());
        }
    }

}
