package com.behance.sdk.asynctasks;

import android.os.AsyncTask;
import android.text.TextUtils;

import com.behance.sdk.BehanceSDKProjectDraftOptions;
import com.behance.sdk.IBehanceSDKCreateProjectDraftListener;
import com.behance.sdk.asynctask.params.BehanceSDKCreateProjectDraftTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.util.BehanceSDKUrlUtil;
import org.json.JSONObject;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by ugs on 06/07/15.
 */
public class BehanceSDKCreateProjectDraftAsyncTask extends AsyncTask<BehanceSDKCreateProjectDraftTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<String>>{


    private static final String TITLE_KEY = "title";
    private static final String DESCRIPTION = "description";
    private static final String TAGS_KEY = "tags";
    private static final String FIELDS_KEY = "fields";
    private static final String LICENSE_KEY = "license";
    private static final String MATURE_CONTENT_KEY = "mature_content";
    private static final String MODULE_IMAGE_KEY = "image";
    public static final String PROJECT = "project";
    public static final String PROJECT_ID = "id";

    private IBehanceSDKCreateProjectDraftListener listener;


    public BehanceSDKCreateProjectDraftAsyncTask(IBehanceSDKCreateProjectDraftListener listener){

        this.listener = listener;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<String> doInBackground(BehanceSDKCreateProjectDraftTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper result = new BehanceSDKAsyncTaskResultWrapper();
        try {
            BehanceSDKCreateProjectDraftTaskParams param = params[0];
            Map<String, String> urlParams = new HashMap<String, String>(1);
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, param.getClientId());
            String createProjectDraftUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PROJECT_CREATE_URL, urlParams);

            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                createProjectDraftUrl = BehanceSDKUrlUtil.appendQueryStringParam(createProjectDraftUrl,
                        BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            BehanceSDKProjectDraftOptions draftOptions = param.getBehanceSDKProjectDraftOptions();
            BehanceMultipartEntity createProjectDraftEntity = null;
            if(null != draftOptions) {
                createProjectDraftEntity = new BehanceMultipartEntity();
                createProjectDraftEntity.setChunked(true);

                String title = draftOptions.getProjectTitle();
                if(false == TextUtils.isEmpty(title)){
                    createProjectDraftEntity.addTextPart(TITLE_KEY,BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                            BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,title.getBytes(BehanceMultipartEntity.CHARSET_UTF8));
                }

                String description = draftOptions.getProjectDescription();
                if(false == TextUtils.isEmpty(description)){
                    createProjectDraftEntity.addTextPart(DESCRIPTION,BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                            BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,description.getBytes(BehanceMultipartEntity.CHARSET_UTF8));
                }

                String tags = draftOptions.getProjectTags();
                if(false == TextUtils.isEmpty(tags)){
                    createProjectDraftEntity.addTextPart(TAGS_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                            BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, tags.getBytes(BehanceMultipartEntity.CHARSET_UTF8));

                }

                String creativeFields = draftOptions.getCreativeFields();
                if(false == TextUtils.isEmpty(creativeFields)){
                    createProjectDraftEntity.addTextPart(FIELDS_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                            BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, creativeFields.getBytes(BehanceMultipartEntity.CHARSET_UTF8));

                }
                String containsAdultContentStr = "";
                if (draftOptions.isProjectContainsAdultContent()) {
                    containsAdultContentStr = "1";
                } else {
                    containsAdultContentStr = "0";
                }
                createProjectDraftEntity.addTextPart(MATURE_CONTENT_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                        BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, containsAdultContentStr.getBytes(BehanceMultipartEntity.CHARSET_UTF8));

                BehanceSDKCopyrightOption copyrightSettings = draftOptions.getProjectCopyrightSettings();
                if(null != copyrightSettings){
                    createProjectDraftEntity.addTextPart(LICENSE_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                            BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, copyrightSettings.getValue().getBytes(BehanceMultipartEntity.CHARSET_UTF8));
                }

                File coverImage  =  draftOptions.getCoverImage();
                if (null != coverImage) {
                    createProjectDraftEntity.addImagePart(MODULE_IMAGE_KEY,coverImage.getName(),
                            BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM,BehanceMultipartEntity.CONTENT_TRANSFER_BINARY,coverImage.getAbsolutePath());
                }
            }
            BehanceConnectionResponse<String> createProjectDraftHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(createProjectDraftUrl, createProjectDraftEntity, null);
            if (createProjectDraftHttpResponse.getResponseCode() != BehanceHttpsConnection.SC_CREATED) {
                result.setExceptionOccurred(true);
                result.setException(new BehanceSDKException(createProjectDraftHttpResponse.getResponseObject()));
            } else {
                result.setExceptionOccurred(false);
                String draftProjectResponseJsonString = createProjectDraftHttpResponse.getResponseObject();
                JSONObject draftProjectResponseRootJsonObject = new JSONObject(draftProjectResponseJsonString);
                JSONObject draftProjectJsonObject = (JSONObject) draftProjectResponseRootJsonObject.get(PROJECT);
                result.setResult(draftProjectJsonObject.optString(PROJECT_ID));
            }
        }catch (Throwable ex){
            result.setExceptionOccurred(true);
            result.setException(new BehanceSDKException(ex));
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<String> result) {
        if(null != listener){
            if(result.isExceptionOccurred()){
                listener.onFailure((BehanceSDKException) result.getException());
            } else{
                listener.onSuccess(result.getResult());
            }
        }
    }
}
