/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKDeleteProjectCommentAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKDeleteProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class BehanceSDKDeleteProjectCommentAsyncTask extends AsyncTask<BehanceSDKDeleteProjectCommentAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKDeleteProjectCommentAsyncTask.class);

	private IBehanceSDKDeleteProjectCommentAsyncTaskListener taskHandler;
	private BehanceSDKDeleteProjectCommentAsyncTaskParams taskParams;

	public BehanceSDKDeleteProjectCommentAsyncTask(IBehanceSDKDeleteProjectCommentAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKDeleteProjectCommentAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
		result.setResult(false);

		taskParams = params[0];

		String commentId = String.valueOf(taskParams.getCommentId());
		String projectId = String.valueOf(taskParams.getProjectId());

		try {

			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			urlParams.put(BehanceSDKUrlUtil.KEY_COMMENT_ID, commentId);
			urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectId);

			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.DELETE_PROJECT_COMMENT_API_URL, urlParams);

			String userAccessToken = taskParams.getUserAccessToken();
			if (userAccessToken != null) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}

			logger.debug("Delete Project Comment url - %s", url);

			BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpDeleteRequest(url);

			String jsonResponseString = connectionResponse.getResponseObject();

			logger.debug("Delete Project Comment response: %s", jsonResponseString);

			int httpStatusCode = connectionResponse.getResponseCode();
			if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
				JSONObject jsonObject = new JSONObject(jsonResponseString);
				httpStatusCode = jsonObject.getInt("http_code");
				if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
					result.setResult(true);
				} else {
					result.setResult(false);
				}
			}
			logger.debug("Delete Project Comment http response status code - %s", httpStatusCode);
		} catch (Exception e) {
			logger.error(e, "Problem trying to Delete Project Comment");
			result.setException(e);
			result.setExceptionOccurred(true);
			result.setResult(false);
		} catch (Throwable t) {
			logger.error(t, "Problem trying to Delete Project Comment");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
			result.setResult(false);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onDeleteProjectCommentAsyncTaskFailure(resultWrapper.getException(), taskParams);
		} else {
			taskHandler.onDeleteProjectCommentAsyncTaskSuccess(resultWrapper.getResult(), taskParams);
		}
	}
}