package com.behance.sdk.asynctasks;

import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.text.TextUtils;

import com.behance.sdk.asynctask.listeners.IBehanceSDKEditProfileAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKEditProfileAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKEditProfileTaskResult;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.network.BehanceNameValuePair;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by forta on 9/15/14.
 */
public class BehanceSDKEditProfileAsyncTask extends AsyncTask<BehanceSDKEditProfileAsyncTaskParams, Void, BehanceSDKEditProfileTaskResult> {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKEditProfileAsyncTask.class);

    private IBehanceSDKEditProfileAsyncTaskListener taskHandler;

    public BehanceSDKEditProfileAsyncTask(IBehanceSDKEditProfileAsyncTaskListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKEditProfileTaskResult doInBackground(BehanceSDKEditProfileAsyncTaskParams... params) {
        BehanceSDKEditProfileTaskResult result = new BehanceSDKEditProfileTaskResult();

        BehanceSDKEditProfileAsyncTaskParams editProfileAsyncTaskParams = params[0];


        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, editProfileAsyncTaskParams.getClientId());
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.EDIT_USER_PROFILE_API_URL, urlParams);

            ArrayList<BehanceNameValuePair> httpPostParams = new ArrayList<>();
            httpPostParams.add(new BehanceNameValuePair("token_type", "device_token"));

            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }
            String firstName = editProfileAsyncTaskParams.getFirstName();
            if (!TextUtils.isEmpty(firstName)) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_FIRST_NAME, firstName));
            }
            String lastName = editProfileAsyncTaskParams.getLastName();
            if (!TextUtils.isEmpty(lastName)) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_LAST_NAME, lastName));
            }
            String occupation = editProfileAsyncTaskParams.getOccupation();
            if (null != occupation) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_OCCUPATION, occupation));
            }
            String company = editProfileAsyncTaskParams.getCompany();
            if (null != company) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_COMPANY, company));
            }
            String website = editProfileAsyncTaskParams.getWebsite();
            if (null != website) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_WEBSITE, website));
            }
            String country = editProfileAsyncTaskParams.getCountry();
            if (!TextUtils.isEmpty(country)) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_COUNTRY, country));
            }
            String state = editProfileAsyncTaskParams.getState();
            if (!TextUtils.isEmpty(state)) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_STATE, state));
            }
            String city = editProfileAsyncTaskParams.getCity();
            if (!TextUtils.isEmpty(city)) {
                httpPostParams.add(new BehanceNameValuePair(BehanceSDKUrlUtil.PARAM_KEY_CITY, city));
            }
            BehanceConnectionResponse<String>  httpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url, httpPostParams, null);

            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Update Profile server response - %s", jsonResponseString);
            JSONObject jsonObject = new JSONObject(jsonResponseString);

            int httpResponseCode = jsonObject.optInt("http_code");
            int updatedVerification = jsonObject.optInt("updated");
            logger.debug("Update Profile [HTTP Response code - %s] [Updated - %s]", httpResponseCode, updatedVerification);

            if (httpResponseCode == BehanceHttpsConnection.SC_OK) {
                if (updatedVerification == 1) {
                    result.setProfileUpdateFailed(false);
                }
            } else {
                result.setExceptionOccurred(true);
                result.setProfileUpdateFailed(true);
                result.addException(new Exception("Invalid server response saving user profile"));
            }

            Bitmap profileImageBitmap = editProfileAsyncTaskParams.getProfileImageBitmap();
            ImageModule profileImageModule = editProfileAsyncTaskParams.getImage();
            if(profileImageBitmap != null && profileImageModule != null) {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                profileImageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
                byte[] imageBytes = baos.toByteArray();
                ByteArrayInputStream coverImageInputStream = new ByteArrayInputStream(imageBytes);
                BehanceMultipartEntity behanceMultipartEntity = new BehanceMultipartEntity();
                behanceMultipartEntity.addImagePart("image", editProfileAsyncTaskParams.getImage().getName(), BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM, BehanceMultipartEntity.CONTENT_TRANSFER_BINARY, coverImageInputStream);
                BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpPatchRequest(url, behanceMultipartEntity, null);
                coverImageInputStream.close();
                jsonObject = new JSONObject(response.getResponseObject());
                httpResponseCode = jsonObject.optInt("http_code");
                updatedVerification = jsonObject.optInt("updated");
                if (httpResponseCode == BehanceHttpsConnection.SC_OK) {
                    if (updatedVerification == 1) {
                        result.setProfileAvatarUpdateFailed(false);
                    }
                } else {
                    result.setProfileAvatarUpdateFailed(true);
                    result.setExceptionOccurred(true);
                    result.addException(new Exception("Invalid server response saving user profile avatar"));
                }
            } else {
                result.setProfileAvatarUpdateFailed(false);
            }

        } catch (Exception e) {
            logger.error(e, "Unknown problem saving user profile");
            result.setExceptionOccurred(true);
            result.addException(e);
        }

        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKEditProfileTaskResult result) {
        if (result.isExceptionOccurred())
            taskHandler.onEditProfileTaskFailure(result);
        else
            taskHandler.onEditProfileTaskSuccess(result);
    }
}

