/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKFollowUserAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKFollowUnFollowUserAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class BehanceSDKFollowUnFollowUserAsyncTask extends AsyncTask<BehanceSDKFollowUnFollowUserAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKFollowUnFollowUserAsyncTask.class);

	private IBehanceSDKFollowUserAsyncTaskListener taskHandler;
	private BehanceSDKFollowUnFollowUserAsyncTaskParams followUserParams;

	public BehanceSDKFollowUnFollowUserAsyncTask(IBehanceSDKFollowUserAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKFollowUnFollowUserAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
		result.setResult(false);

		followUserParams = params[0];

		BehanceSDKUserDTO userDTO = followUserParams.getUserDTO();
		String userId = String.valueOf(userDTO.getId());

		try {
			if (userId != null && userId.length() > 0) {
				Map<String, String> urlParams = new HashMap<String, String>();
				urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
				urlParams.put(BehanceSDKUrlUtil.KEY_USER_ID, userId);

				String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.FOLLOW_USER_API_URL, urlParams);

				String userAccessToken = followUserParams.getUserAccessToken();
				if (userAccessToken != null) {
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
				}

				logger.debug("Follow/Unfollow user url - %s", url);

				BehanceConnectionResponse<String> connectionResponse;
				if (followUserParams.isFollowUser()) {
					connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url);
				} else {
					connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpDeleteRequest(url);
				}

				String jsonResponseString = connectionResponse.getResponseObject();
				logger.debug("Follow/UnFollow user response for [userId - %s ] [Follow - %s] : %s", userId, followUserParams.isFollowUser(),
						jsonResponseString);

				int httpStatusCode = connectionResponse.getResponseCode();
				if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
					JSONObject jsonObject = new JSONObject(jsonResponseString);
					httpStatusCode = jsonObject.getInt("http_code");
					if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
						result.setResult(true);
					} else {
						result.setResult(false);
					}
				}
				logger.debug("Follow/Unfollow user http response status code - %s", httpStatusCode);
			}

		} catch (Exception e) {
			logger.error(e, "Problem trying to follow/unfollow user [userId - %s] [Follow user - %s]", userId, followUserParams.isFollowUser());
			result.setException(e);
			result.setExceptionOccurred(true);
			result.setResult(false);
		} catch (Throwable t) {
			logger.error(t, "Problem trying to follow/unfollow user [userId - %s] [Follow user - %s]", userId, followUserParams.isFollowUser());
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
			result.setResult(false);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onFollowUserAsyncTaskFailure(resultWrapper.getException(), followUserParams);
		} else {
			taskHandler.onFollowUserAsyncTaskSuccess(resultWrapper.getResult(), followUserParams);
		}
	}
}