package com.behance.sdk.asynctasks;

import java.lang.ref.WeakReference;
import java.util.List;

import android.content.Context;
import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKGetAlbumsTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetAlbumsTaskParams;
import com.behance.sdk.dto.BehanceSDKAlbumDTO;
import com.behance.sdk.files.LocalImageLoader;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKGetAlbumsAsyncTask extends AsyncTask<BehanceSDKGetAlbumsTaskParams, Void, BehanceSDKGetAlbumsTaskResult> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetAlbumsAsyncTask.class);

	private IBehanceSDKGetAlbumsTaskListener listener;

	public BehanceSDKGetAlbumsAsyncTask(IBehanceSDKGetAlbumsTaskListener listener) {
		this.listener = listener;
	}

	@Override
	protected BehanceSDKGetAlbumsTaskResult doInBackground(BehanceSDKGetAlbumsTaskParams... arg0) {
		BehanceSDKGetAlbumsTaskResult result = new BehanceSDKGetAlbumsTaskResult();
		try {
			BehanceSDKGetAlbumsTaskParams taskParams = arg0[0];
			WeakReference<Context> contextWeakReference = taskParams.getContextWeakReference();
			Context context = contextWeakReference.get();
			if (context != null) {
				List<BehanceSDKAlbumDTO> albumsList = LocalImageLoader.getAlbumsList(context, false, taskParams.isShowImageSelectionSources());
				result.setAlbumsList(albumsList);
			} else {
				result.setExceptionOccurred(true);
				result.setException(new Exception("Context is null"));
			}
		} catch (Exception e) {
			result.setExceptionOccurred(true);
			result.setException(e);
			logger.error(e, "Problem loading albums");
		} catch (Error e) {
			result.setExceptionOccurred(true);
			result.setException(new Exception(e));
			logger.error(e, "Problem loading albums");
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKGetAlbumsTaskResult result) {
		if (result.isExceptionOccurred()) {
			listener.onGetAlbumsTaskFailure(result.getException());
		} else {
			listener.onGetAlbumsTaskSuccess(result.getAlbumsList());
		}
	}
}
