package com.behance.sdk.asynctasks;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.exception.OAuthCommunicationException;
import oauth.signpost.exception.OAuthExpectationFailedException;
import oauth.signpost.exception.OAuthMessageSignerException;
import oauth.signpost.exception.OAuthNotAuthorizedException;
import android.os.AsyncTask;

import com.behance.sdk.OAuth.TwitterClient;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetAuthURLFromTwitterTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetAuthURLFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKGetAuthUrlFromTwitterTaskResult;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

/*************************************************************************
 * ADOBE CONFIDENTIAL ___________________
 * 
 * Copyright 2013 Adobe Systems Incorporated All Rights Reserved.
 * 
 * NOTICE: All information contained herein is, and remains the property of
 * Adobe Systems Incorporated and its suppliers, if any. The intellectual and
 * technical concepts contained herein are proprietary to Adobe Systems
 * Incorporated and its suppliers and are protected by all applicable
 * intellectual property laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material is
 * strictly forbidden unless prior written permission is obtained from Adobe
 * Systems Incorporated.
 **************************************************************************/

public class BehanceSDKGetAuthURLFromTwitterAsyncTask extends AsyncTask<BehanceSDKGetAuthURLFromTwitterAsyncTaskParams, Void, BehanceSDKGetAuthUrlFromTwitterTaskResult> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetAuthURLFromTwitterAsyncTask.class);

	private IBehanceSDKGetAuthURLFromTwitterTaskListener listener;

	public BehanceSDKGetAuthURLFromTwitterAsyncTask(IBehanceSDKGetAuthURLFromTwitterTaskListener listener) {
		this.listener = listener;
	}

	@Override
	protected BehanceSDKGetAuthUrlFromTwitterTaskResult doInBackground(BehanceSDKGetAuthURLFromTwitterAsyncTaskParams... params) {
		BehanceSDKGetAuthUrlFromTwitterTaskResult result = new BehanceSDKGetAuthUrlFromTwitterTaskResult();
		String authUrl = null;
		BehanceSDKGetAuthURLFromTwitterAsyncTaskParams taskParams = params[0];
		String consumerToken = taskParams.getConsumerKey();
		String consumerSecret = taskParams.getConsumerSecret();

		OAuthProvider provider = TwitterClient.getInstance(consumerToken,consumerSecret).getOAuthProvider();
		OAuthConsumer consumer = TwitterClient.getInstance(consumerToken,consumerSecret).getOAuthConsumer();
		try {
			authUrl = provider.retrieveRequestToken(consumer, TwitterClient.TWITTER_OAUTH_CALLBACK_URI);
		} catch (OAuthMessageSignerException e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (OAuthNotAuthorizedException e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (OAuthExpectationFailedException e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (OAuthCommunicationException e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Exception e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		}
		logger.debug("Auth URL returned from twitter is %s", authUrl);
		result.setAuthURL(authUrl);
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKGetAuthUrlFromTwitterTaskResult result) {
		if (result.isExceptionOccurred()) {
			listener.onGetAuthURLFromTwitterFailure(result);
		} else {
			listener.onGetAuthURLFromTwitterSuccess(result);
		}

	}
}
