package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.params.BehanceSDKGetCitiesAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCitiesAsyncTaskListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by forta on 9/17/14.
 */
public class BehanceSDKGetCitiesAsyncTask extends AsyncTask<BehanceSDKGetCitiesAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCityDTO>>> {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetCitiesAsyncTask.class);

    private IBehanceSDKGetCitiesAsyncTaskListener taskHandler;

    private BehanceSDKGetCitiesAsyncTaskParams taskParams;

    public BehanceSDKGetCitiesAsyncTask(IBehanceSDKGetCitiesAsyncTaskListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCityDTO>> doInBackground(BehanceSDKGetCitiesAsyncTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCityDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCityDTO>>();

        try {
            taskParams = params[0];

            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, taskParams.getClientId());
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_CITIES_API_URL, urlParams);

            if (taskParams.getCountryId() != null && taskParams.getCountryId().length() > 0) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_COUNTRY, taskParams.getCountryId());
            }

            if (taskParams.getStateId() != null && taskParams.getStateId().length() > 0) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_STATE, taskParams.getStateId());
            }

            if (taskParams.getCitySearchStr() != null && taskParams.getCitySearchStr().length() > 0) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_CITY, taskParams.getCitySearchStr());
            }

            logger.debug("Get Cities URL - %s", url);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);


            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get cities response: %s", jsonResponseString);

            ArrayList<BehanceSDKCityDTO> statesList = new ArrayList<BehanceSDKCityDTO>();

            JSONArray rootJsonArray = new JSONArray(jsonResponseString);
            for (int counter = 0; counter < rootJsonArray.length(); counter++) {
                Object cityObject = rootJsonArray.opt(counter);
                if (cityObject != null) {
                    JSONObject cityJsonObject = (JSONObject) cityObject;
                    BehanceSDKCityDTO CityDTO = new BehanceSDKCityDTO();
                    CityDTO.setId(cityJsonObject.optString("id"));
                    CityDTO.setDisplayName(cityJsonObject.optString("n"));
                    statesList.add(CityDTO);
                }
            }
            Collections.sort(statesList);
            result.setResult(statesList);
        } catch (Exception e) {
            logger.error(e, "Problem getting Cities from server");
            result.setExceptionOccurred(true);
            result.setException(e);
        } catch (Throwable t) {
            logger.error(t, "Problem getting Cities from server");
            result.setException(new Exception(t.getMessage()));
            result.setExceptionOccurred(true);
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCityDTO>> resultWrapper) {
        if (resultWrapper.isExceptionOccurred()) {
            taskHandler.onGetCitiesFailure(taskParams, resultWrapper.getException());
        } else {
            taskHandler.onGetCitiesSuccess(taskParams, resultWrapper.getResult());
        }
    }
}