package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.params.BehanceSDKAbstractTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCountriesAsyncTaskListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by forta on 9/17/14.
 */
public class BehanceSDKGetCountriesAsyncTask extends AsyncTask<BehanceSDKAbstractTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCountryDTO>>> {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetCountriesAsyncTask.class);

    private IBehanceSDKGetCountriesAsyncTaskListener taskHandler;

    public BehanceSDKGetCountriesAsyncTask(IBehanceSDKGetCountriesAsyncTaskListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCountryDTO>> doInBackground(BehanceSDKAbstractTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCountryDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCountryDTO>>();

        BehanceSDKAbstractTaskParams taskParams = params[0];

        try {

            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, taskParams.getClientId());
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_COUNTRIES_API_URL, urlParams);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get Countries response: %s", jsonResponseString);

            ArrayList<BehanceSDKCountryDTO> countriesList = new ArrayList<BehanceSDKCountryDTO>();

            JSONObject rootJsonObject = new JSONObject(jsonResponseString);
            Iterator<?> keysIterator = rootJsonObject.keys();
            while (keysIterator.hasNext()) {
                String keyStr = (String) keysIterator.next();
                Object countryObject = rootJsonObject.optJSONObject(keyStr);
                if (countryObject != null) {
                    JSONObject countryJsonObject = (JSONObject) countryObject;
                    BehanceSDKCountryDTO countryDTO = new BehanceSDKCountryDTO();
                    countryDTO.setId(countryJsonObject.optString("id"));
                    countryDTO.setDisplayName(countryJsonObject.optString("n"));
                    countriesList.add(countryDTO);
                }
            }
            Collections.sort(countriesList);
            result.setResult(countriesList);
        } catch (Exception e) {
            logger.error(e, "Problem getting Countries from server");
            result.setExceptionOccurred(true);
            result.setException(e);
        } catch (Throwable t) {
            logger.error(t, "Problem getting Countries from server");
            result.setException(new Exception(t.getMessage()));
            result.setExceptionOccurred(true);
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKCountryDTO>> resultWrapper) {
        if (resultWrapper.isExceptionOccurred()) {
            taskHandler.onGetCountriesFailure(resultWrapper.getException());
        } else {
            taskHandler.onGetCountriesSuccess(resultWrapper.getResult());
        }
    }
}