package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.params.BehanceSDKGetStatesOfCountryAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetStatesOfCountryAsyncTaskListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by forta on 9/17/14.
 */
public class BehanceSDKGetStatesOfCountryAsyncTask extends AsyncTask<BehanceSDKGetStatesOfCountryAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKStateDTO>>> {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetStatesOfCountryAsyncTask.class);

    private IBehanceSDKGetStatesOfCountryAsyncTaskListener taskHandler;

    private String countryId;

    public BehanceSDKGetStatesOfCountryAsyncTask(IBehanceSDKGetStatesOfCountryAsyncTaskListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKStateDTO>> doInBackground(BehanceSDKGetStatesOfCountryAsyncTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKStateDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKStateDTO>>();

        BehanceSDKGetStatesOfCountryAsyncTaskParams taskParams = params[0];

        if (params.length != 1) {
            result.setException(new Exception("Missing country ID"));
            result.setExceptionOccurred(true);
            return result;
        }

        try {
            countryId = taskParams.getCountryId();
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_COUNTRY_ID, countryId);
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, taskParams.getClientId());
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_STATES_API_URL, urlParams);

            logger.debug("Get States of Country URL - %s", url);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);


            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get States of Country [Country id - %s ] response: %s", countryId, jsonResponseString);

            ArrayList<BehanceSDKStateDTO> statesList = new ArrayList<BehanceSDKStateDTO>();

            JSONObject rootJsonObject = new JSONObject(jsonResponseString);
            Iterator<?> keysIterator = rootJsonObject.keys();
            while (keysIterator.hasNext()) {
                String keyStr = (String) keysIterator.next();
                Object stateObject = rootJsonObject.optJSONObject(keyStr);
                if (stateObject != null) {
                    JSONObject stateJsonObject = (JSONObject) stateObject;
                    BehanceSDKStateDTO stateDTO = new BehanceSDKStateDTO();
                    stateDTO.setId(stateJsonObject.optString("id"));
                    stateDTO.setDisplayName(stateJsonObject.optString("n"));
                    statesList.add(stateDTO);
                }
            }
            Collections.sort(statesList);
            result.setResult(statesList);
        } catch (Exception e) {
            logger.error(e, "Problem getting States of Country from server");
            result.setExceptionOccurred(true);
            result.setException(e);
        } catch (Throwable t) {
            logger.error(t, "Problem getting States of Country from server");
            result.setException(new Exception(t.getMessage()));
            result.setExceptionOccurred(true);
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKStateDTO>> resultWrapper) {
        if (resultWrapper.isExceptionOccurred()) {
            taskHandler.onGetStatesFailure(countryId, resultWrapper.getException());
        } else {
            taskHandler.onGetStatesSuccess(countryId, resultWrapper.getResult());
        }
    }
}