package com.behance.sdk.asynctasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.managers.BehanceSDKUserManager;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserWIPsTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserWIPsTaskParams;
import com.behance.sdk.dto.BehanceSDKWIPDTO;
import com.behance.sdk.dto.BehanceSDKWIPDTO.WIPRevisionImageType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKGetUserWIPsAsyncTask extends AsyncTask<BehanceSDKGetUserWIPsTaskParams, Void, BehanceSDKGetUserWIPsTaskResult> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetUserWIPsAsyncTask.class);

	private static final int DEFAULT_REQUEST_PAGE_SIZE = 24;

	private IBehanceSDKGetUserWIPsTaskListener listener;
	private BehanceSDKGetUserWIPsTaskParams taskParams;

	public BehanceSDKGetUserWIPsAsyncTask(IBehanceSDKGetUserWIPsTaskListener listener) {
		this.listener = listener;
	}

	@Override
	protected BehanceSDKGetUserWIPsTaskResult doInBackground(BehanceSDKGetUserWIPsTaskParams... arg0) {
		BehanceSDKGetUserWIPsTaskResult result = new BehanceSDKGetUserWIPsTaskResult();
		try {
			taskParams = arg0[0];
			int wipsPageNumber = 1;
			List<BehanceSDKWIPDTO> totalUserWIPsList = new ArrayList<BehanceSDKWIPDTO>();
			do {
				List<BehanceSDKWIPDTO> wipsFromServer = getUserWips(taskParams, wipsPageNumber, result);
				if (wipsFromServer != null) {
					totalUserWIPsList.addAll(wipsFromServer);
				}

				if (wipsFromServer == null || wipsFromServer.size() < DEFAULT_REQUEST_PAGE_SIZE || result.isExceptionOccurred()) {
					break;
				}
				wipsPageNumber++;
			} while (true);
			result.setWipsList(totalUserWIPsList);

		} catch (Exception e) {
			result.setExceptionOccurred(true);
			result.setException(e);
			logger.error(e, "Problem Getting user WIPs");
		} catch (Error e) {
			result.setExceptionOccurred(true);
			result.setException(new Exception(e));
			logger.error(e, "Problem Getting user WIPs");
		}
		return result;
	}

	private List<BehanceSDKWIPDTO> getUserWips(BehanceSDKGetUserWIPsTaskParams taskParams, int pageNumber,
			BehanceSDKGetUserWIPsTaskResult result) throws IOException,JSONException, BehanceSDKUserNotAuthenticatedException {
		Map<String, String> urlParams = new HashMap<String, String>();
		urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, taskParams.getClientId());
		urlParams.put(BehanceSDKUrlUtil.KEY_USER_ID, taskParams.getUserId());
		String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_USER_WIPS_API_URL, urlParams);

		url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, pageNumber);
		url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE, DEFAULT_REQUEST_PAGE_SIZE);

        BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
        String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
		if (userAccessToken != null) {
			url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
		}

		BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
		String jsonResponseString = httpResponse.getResponseObject();
		logger.debug("Get user WIPs response: %s", jsonResponseString);
		JSONObject jsonObject = new JSONObject(jsonResponseString);
		int httpResponseCode = jsonObject.optInt("http_code");

		if (httpResponseCode == BehanceHttpsConnection.SC_OK) {
			List<BehanceSDKWIPDTO> response = new ArrayList<BehanceSDKWIPDTO>();
			try {
				JSONArray wipsJsonArray = jsonObject.optJSONArray("wips");
				for (int counter = 0; counter < wipsJsonArray.length(); counter++) {
					JSONObject wipJsonObject = wipsJsonArray.getJSONObject(counter);

					BehanceSDKWIPDTO wipDTO = new BehanceSDKWIPDTO();
					wipDTO.setId(wipJsonObject.optInt("id"));
					wipDTO.setTitle(wipJsonObject.optString("title"));

					// get the first revision image
					JSONObject revisionsJsonObject = wipJsonObject.optJSONObject("revisions");
					if (revisionsJsonObject != null) {
						JSONArray revisionIDsJsonArray = revisionsJsonObject.names();
						if (revisionIDsJsonArray != null) {
							for (int counter2 = 0; counter2 < revisionIDsJsonArray.length(); counter2++) {
								JSONObject revisionJsonObject = revisionsJsonObject.optJSONObject(revisionIDsJsonArray.getString(counter2));
								if (revisionJsonObject != null) {
									JSONObject imagesJSONObject = revisionJsonObject.optJSONObject("images");
									if (imagesJSONObject != null) {
										JSONArray imageTypesJsonArray = imagesJSONObject.names();
										for (int counter3 = 0; counter3 < imageTypesJsonArray.length(); counter3++) {
											String imageType = imageTypesJsonArray.getString(counter3);
											JSONObject imageJsonObject = imagesJSONObject.getJSONObject(imageType);
											String imageUrl = imageJsonObject.optString("url");
											if ("thumbnail_sm".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMBNAIL_SM, imageUrl);
											} else if ("thumbnail".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMBNAIL, imageUrl);
											} else if ("normal_resolution".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.NORMAL_RESOLUTION, imageUrl);
											} else if ("high_definition".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.HIGH_DEFINITION, imageUrl);
											} else if ("thumb_40".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMB_40, imageUrl);
											} else if ("thumb_80".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMB_80, imageUrl);
											} else if ("thumb_120".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMB_120, imageUrl);
											} else if ("thumb_240".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.THUMB_240, imageUrl);
											} else if ("limit_320".equals(imageType)) {
												wipDTO.setLatestRevisionImageUrl(WIPRevisionImageType.LIMIT_320, imageUrl);
											}
										}
										// we just need the first revision image
										break;
									}
								}
							}
						}
					}
					response.add(wipDTO);
				}
			} catch (JSONException e) {
				logger.error(e, "Problem parsing WIP JSON response");
				throw e;
			}
			return response;
		} else {
			logger.error("Unexpected HTTP Response code when trying to fetch User WIPs. [User ID - %s] [Response code - %d]", taskParams.getUserId(),
					httpResponseCode);
			result.setException(new Exception("Invalid server response code " + httpResponseCode));
			result.setExceptionOccurred(true);
		}
		return null;
	}

	@Override
	protected void onPostExecute(BehanceSDKGetUserWIPsTaskResult result) {
		if (result.isExceptionOccurred()) {
			listener.onGetUserWIPsTaskFailure(result.getException(), taskParams);
		} else {
			listener.onGetUserWIPsTaskSuccess(result.getWipsList(), taskParams);
		}
	}
}
