/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.asynctasks;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;

import android.os.AsyncTask;

import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.asynctasks.result.BehanceSDKCreativeFieldsTaskResult;
import com.behance.sdk.datamanager.listeners.interfaces.IGetCreativeFieldsAsyncTaskListener;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.enums.BehanceSDKCreativeFieldCategory;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

/**
 * @author ugs
 *
 */
public class BehanceSDKLoadCreativeFieldsAsyncTask extends AsyncTask<Void, Void, BehanceSDKCreativeFieldsTaskResult<List<BehanceSDKCreativeFieldDTO>>> {

	/**
	 * 
	 */
	private static final String POPULAR_JSON_TAG = "popular";

	private static final String NAME = "name";

	private static final String ID = "id";

	private static final String FIELDS_JSON_TAG = "fields";

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKLoadCreativeFieldsAsyncTask.class);

	private IGetCreativeFieldsAsyncTaskListener taskHandler;

	private IBehanceSDKUserCredentials iUserCredentials;

	public BehanceSDKLoadCreativeFieldsAsyncTask(IGetCreativeFieldsAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKCreativeFieldsTaskResult<List<BehanceSDKCreativeFieldDTO>> doInBackground(Void... params) {

		BehanceSDKCreativeFieldsTaskResult<List<BehanceSDKCreativeFieldDTO>> result = new BehanceSDKCreativeFieldsTaskResult<List<BehanceSDKCreativeFieldDTO>>();

		try {
			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, iUserCredentials.getClientId());
			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_CREATIVE_FIELDS_API_URL, urlParams);
			BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

			String jsonResponseString = httpResponse.getResponseObject();
			logger.debug("Get creative fields response: %s", jsonResponseString);

			ArrayList<BehanceSDKCreativeFieldDTO> creativeFields = new ArrayList<BehanceSDKCreativeFieldDTO>();
			Map<String,BehanceSDKCreativeFieldDTO> popularcreativeFields = new HashMap<String,BehanceSDKCreativeFieldDTO>();

			JSONObject rootJsonObject = new JSONObject(jsonResponseString);
			JSONArray popular = rootJsonObject.optJSONArray(POPULAR_JSON_TAG);
			for (int i = 0; i < popular.length(); i++) {
				JSONObject field = popular.getJSONObject(i);
				BehanceSDKCreativeFieldDTO creativeFieldDTO = new BehanceSDKCreativeFieldDTO();
				String id = field.optString(ID);
				creativeFieldDTO.setId(id);
				creativeFieldDTO.setName(field.optString(NAME));
				creativeFieldDTO.setCategory(BehanceSDKCreativeFieldCategory.POPULAR);
				popularcreativeFields.put(id,creativeFieldDTO);
			}
			JSONArray fields = rootJsonObject.optJSONArray(FIELDS_JSON_TAG);
			for (int i = 0; i < fields.length(); i++) {
				JSONObject field = fields.getJSONObject(i);
				String fieldID = field.optString(ID);
				BehanceSDKCreativeFieldDTO creativeFieldDTO;
				if(popularcreativeFields.containsKey(fieldID)){
					creativeFieldDTO = popularcreativeFields.get(fieldID);
				}else{
					creativeFieldDTO = new BehanceSDKCreativeFieldDTO();
					creativeFieldDTO.setId(fieldID);
					creativeFieldDTO.setName(field.optString(NAME));
				}
				creativeFields.add(creativeFieldDTO);
			}
			result.setResult(creativeFields);
		} catch (Exception e) {
			result.setExceptionOccurred(true);
			result.setException(e);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKCreativeFieldsTaskResult<List<BehanceSDKCreativeFieldDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onLoadCreativeFieldsFailure(resultWrapper.getException());
		} else {
			taskHandler.onLoadCreativeFieldsSuccess(resultWrapper.getResult());
		}
	}

	/**
	 * @param iUserCredentials
	 */
	public void setUserCredentials(IBehanceSDKUserCredentials iUserCredentials) {
		this.iUserCredentials = iUserCredentials;
		
	}
}