/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKPostProjectCommentAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKPostProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceNameValuePair;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKPostProjectCommentAsyncTask extends AsyncTask<BehanceSDKPostProjectCommentAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Integer>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPostProjectCommentAsyncTask.class);

	private IBehanceSDKPostProjectCommentAsyncTaskListener taskHandler;
	private BehanceSDKPostProjectCommentAsyncTaskParams postCommentParams;

	public BehanceSDKPostProjectCommentAsyncTask(IBehanceSDKPostProjectCommentAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<Integer> doInBackground(BehanceSDKPostProjectCommentAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<Integer> result = new BehanceSDKAsyncTaskResultWrapper<>();
		result.setResult(-1);

		postCommentParams = params[0];

		String commentContent = postCommentParams.getCommentContent();
		String projectId = String.valueOf(postCommentParams.getProjectId());

		try {
			if (commentContent != null && commentContent.length() > 0) {

				Map<String, String> urlParams = new HashMap<>();
				urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
				urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectId);

				String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.POST_PROJECT_COMMENTS_API_URL, urlParams);

				String userAccessToken = postCommentParams.getUserAccessToken();
				if (userAccessToken != null) {
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
				}

				logger.debug("Post Project comment url - %s", url);

				List<BehanceNameValuePair> nameValuePairs = new ArrayList<>(1);
				nameValuePairs.add(new BehanceNameValuePair("comment", commentContent));

				BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url, nameValuePairs);

				String jsonResponseString = connectionResponse.getResponseObject();
				if (jsonResponseString.contains("User must be verified"))
					postCommentParams.setUserVerified(false);
				logger.debug("Post Project comment response: %s", jsonResponseString);

				int httpStatusCode = connectionResponse.getResponseCode();
				if (httpStatusCode == IBehanceHttpConnection.SC_CREATED) {
					JSONObject jsonObject = new JSONObject(jsonResponseString);
					httpStatusCode = jsonObject.getInt("http_code");
					if (httpStatusCode == IBehanceHttpConnection.SC_CREATED) {
						result.setResult(jsonObject.optInt("comment_id"));
					} else {
						logger.error("Unexpected HTTP Response code when trying to comment on project [Project ID - %s] [Response code - %d]", projectId,httpStatusCode);
						result.setException(new BehanceSDKException("Invalid server response code " + httpStatusCode));
						result.setExceptionOccurred(true);
						result.setResult(-1);
					}
				}
				logger.debug("Post WIP commenet http response status code - %s", httpStatusCode);
			}
		} catch (Exception e) {
			logger.error(e, "Problem trying to post WIP comment");
			result.setException(e);
			result.setExceptionOccurred(true);
			result.setResult(-1);
		} catch (Throwable t) {
			logger.error(t, "Problem trying to post WIP comment");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
			result.setResult(-1);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Integer> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onPostProjectCommentAsyncTaskFailure(resultWrapper.getException(), postCommentParams);
		} else {
			taskHandler.onPostProjectCommentAsyncTaskSuccess(resultWrapper.getResult(), postCommentParams);
		}
	}
}