/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKPostUserSettingsOnServerAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKPostUserSettingsOnServerAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.search.BehanceSDKUserSettingsOnServerDTO;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceNameValuePair;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKPostUserSettingsOnServerAsyncTask extends AsyncTask<BehanceSDKPostUserSettingsOnServerAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPostUserSettingsOnServerAsyncTask.class);

	private IBehanceSDKPostUserSettingsOnServerAsyncTaskListener taskHandler;

	private BehanceSDKPostUserSettingsOnServerAsyncTaskParams saveSettingsParams;

	public BehanceSDKPostUserSettingsOnServerAsyncTask(IBehanceSDKPostUserSettingsOnServerAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKPostUserSettingsOnServerAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
		result.setResult(false);

		try {

			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_USER_SETTINGS_ON_SERVER_API_URL, urlParams);

			if (params.length == 1) {
				saveSettingsParams = params[0];
				String userAccessToken = saveSettingsParams.getUserAccessToken();
				if (userAccessToken != null) {
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
				}
			}

			BehanceSDKUserSettingsOnServerDTO settingsDTO = saveSettingsParams.getSettings();

			List<BehanceNameValuePair> nameValuePairs = new ArrayList<BehanceNameValuePair>(1);
			nameValuePairs.add(new BehanceNameValuePair("safe", settingsDTO.isSafeBrowsingOn() ? "1" : "0"));

			BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url, nameValuePairs);

			String jsonResponseString = connectionResponse.getResponseObject();
			logger.debug("Save user settings async task response: %s", jsonResponseString);

			JSONObject rootJsonObject = new JSONObject(jsonResponseString);
			int httpStatusCode = rootJsonObject.optInt("http_code");
			if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
				result.setResult(true);
			} else {
				result.setResult(false);
			}
		} catch (Exception e) {
			logger.error(e, "Problem saving user settings on server");
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem saving user settings on server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onPostUserSettingsOnServerAsyncTaskFailure(resultWrapper.getException(), saveSettingsParams);
		} else {
			taskHandler.onPostUserSettingsOnServerAsyncTaskSuccess(resultWrapper.getResult(), saveSettingsParams);
		}
	}
}