package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.IBehanceSDKProjectPublishListener;
import com.behance.sdk.asynctask.params.BehanceSDKPublishProjectTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by ugs on 06/07/15.
 */
public class BehanceSDKPublishProjectAsyncTask extends AsyncTask<BehanceSDKPublishProjectTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

    private static final String PROJECTID_KEY = "{PROJECTID}";
    private static final String PUBLISHED_KEY = "published";
    private static final String PUBLISHED = "1";

    private IBehanceSDKProjectPublishListener listener;

    public BehanceSDKPublishProjectAsyncTask(IBehanceSDKProjectPublishListener listener){

        this.listener = listener;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKPublishProjectTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
        BehanceSDKPublishProjectTaskParams serviceParams =params[0];
        try
        {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParams.getClientId());
            // publish project details
            String finalPublishUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PUBLISH_PROJECT_URL, urlParams);
            finalPublishUrl = finalPublishUrl.replace(PROJECTID_KEY, serviceParams.getProjectId());
            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                finalPublishUrl = BehanceSDKUrlUtil
                    .appendQueryStringParam(finalPublishUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }
            BehanceMultipartEntity publishProjectEntity = new BehanceMultipartEntity();
            publishProjectEntity.setBrowserCompatible(true);
            publishProjectEntity.addTextPart(PUBLISHED_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
                    BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, PUBLISHED.getBytes(BehanceMultipartEntity.CHARSET_UTF8));
            BehanceConnectionResponse<String>  publishProjectDetailsHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPutRequest(finalPublishUrl,publishProjectEntity,null);

            if (publishProjectDetailsHttpResponse.getResponseCode() == BehanceHttpsConnection.SC_OK) {
                result.setExceptionOccurred(false);
            }else{
                result.setExceptionOccurred(true);
                result.setException(new BehanceSDKException(publishProjectDetailsHttpResponse.getResponseObject()));
            }
        } catch(Throwable ex){
            result.setExceptionOccurred(true);
            result.setException(new BehanceSDKException(ex));
        }

        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> result) {

        if(null != listener){
            if(result.isExceptionOccurred()){
                listener.onFailure((BehanceSDKException) result.getException());
            }else{
                listener.onSuccess();
            }
        }
    }
}
