package com.behance.sdk.asynctasks;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.exception.OAuthCommunicationException;
import oauth.signpost.exception.OAuthExpectationFailedException;
import oauth.signpost.exception.OAuthMessageSignerException;
import oauth.signpost.exception.OAuthNotAuthorizedException;

import org.json.JSONObject;

import android.content.Context;
import android.net.Uri;
import android.os.AsyncTask;

import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.OAuth.TwitterClient;
import com.behance.sdk.asynctask.listeners.IBehanceSDKRetrieveAccessTokenFromTwitterTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKRetrieveAccessTokenFromTwitterTaskResult;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.services.BehanceSDKPublishWIPService;

/*************************************************************************
 * ADOBE CONFIDENTIAL ___________________
 * 
 * Copyright 2013 Adobe Systems Incorporated All Rights Reserved.
 * 
 * NOTICE: All information contained herein is, and remains the property of
 * Adobe Systems Incorporated and its suppliers, if any. The intellectual and
 * technical concepts contained herein are proprietary to Adobe Systems
 * Incorporated and its suppliers and are protected by all applicable
 * intellectual property laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material is
 * strictly forbidden unless prior written permission is obtained from Adobe
 * Systems Incorporated.
 **************************************************************************/

public class BehanceSDKRetrieveAccessTokenFromTwitterAsyncTask extends
		AsyncTask<BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams, Void, BehanceSDKRetrieveAccessTokenFromTwitterTaskResult> {

	private static final String OAUTH_VERIFIER_FROM_BROWSER = "oauth_verifier";
	private static final String GET_TWITTER_USERNAME_API = "https://api.twitter.com/1.1/account/settings.json";
	private IBehanceSDKRetrieveAccessTokenFromTwitterTaskListener listener;
	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishWIPService.class);

	private BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result;
	private BehanceSDKSocialAccountManager socialAccountManager;
	private BehanceSDKSocialAccountDTO socialAccountDTO;
	private OAuthProvider provider;
	private OAuthConsumer consumer;
	private String consumerToken;
	private String consumerSecret;

	public BehanceSDKRetrieveAccessTokenFromTwitterAsyncTask(IBehanceSDKRetrieveAccessTokenFromTwitterTaskListener listener, Context context) {
		this.listener = listener;
		socialAccountManager = BehanceSDKSocialAccountManager.getInstance(context);
		socialAccountDTO = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);

	}

	@Override
	protected BehanceSDKRetrieveAccessTokenFromTwitterTaskResult doInBackground(BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams... params) {
		result = new BehanceSDKRetrieveAccessTokenFromTwitterTaskResult();
		BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams taskParams = params[0];
		String url = taskParams.getAuthURL();
		consumerToken = taskParams.getConsumerKey();
		consumerSecret = taskParams.getConsumerSecret();
		provider = TwitterClient.getInstance(consumerToken, consumerSecret).getOAuthProvider();
		consumer = TwitterClient.getInstance(consumerToken, consumerSecret).getOAuthConsumer();
		retrieveAccessTokenFromTwitter(url, consumerToken, consumerSecret);
		if (consumer.getToken() == null || consumer.getToken().isEmpty() || consumer.getTokenSecret() == null || consumer.getTokenSecret().isEmpty()) {
			result.setExceptionOccurred(true);
			result.setException(new Exception("Twitter Access token or access secret is empty"));
			return result;
		} else {
			if (socialAccountDTO != null) {
				socialAccountDTO.setUserAuthToken(consumer.getToken());
				socialAccountDTO.setAdditionalData(consumer.getTokenSecret());
				String userName = retrieveUserNameFromTwitter();
				socialAccountDTO.setUserId(userName);
				socialAccountManager.updateAccount(socialAccountDTO);
			}
		}
		return result;
	}

	private String retrieveUserNameFromTwitter() {
		String userName = null;
		try {
			// Create a new consumer using the commons implementation
			BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(GET_TWITTER_USERNAME_API, null, null, consumer);
			String jsonResponseString = httpResponse.getResponseObject();

			int statusCode = httpResponse.getResponseCode();
			if (statusCode == BehanceHttpsConnection.SC_OK) {
				JSONObject jsonObject = new JSONObject(jsonResponseString);
				userName = jsonObject.optString("screen_name");
			} else {
				logger.error("Unexpected return code while retrieving user name from twitter. [return code - %s]", statusCode);
			}
		} catch (Throwable e) {
			logger.error(e, "Problem retrieving user name");
		}
		return userName;
	}

	private void retrieveAccessTokenFromTwitter(String url, String consumerToken, String consumerSecret) {
		if (url != null && url.length() > 0) {
			Uri authUri = Uri.parse(url);
			String oauth_verifier = authUri.getQueryParameter(OAUTH_VERIFIER_FROM_BROWSER);
			if (oauth_verifier != null && oauth_verifier.length() > 0) {

				try {
					provider.retrieveAccessToken(consumer, oauth_verifier);
				} catch (OAuthMessageSignerException e) {
					result.setExceptionOccurred(true);
					result.setException(e);
				} catch (OAuthNotAuthorizedException e) {
					result.setExceptionOccurred(true);
					result.setException(e);
				} catch (OAuthExpectationFailedException e) {
					result.setExceptionOccurred(true);
					result.setException(e);
				} catch (OAuthCommunicationException e) {
					result.setExceptionOccurred(true);
					result.setException(e);
				} catch (Exception e) {
					result.setExceptionOccurred(true);
					result.setException(e);
				}
			}else{
				result.setExceptionOccurred(true);
				Exception e = new OAuthNotAuthorizedException(null);
				result.setException(e);
			}
		}
	}

	@Override
	protected void onPostExecute(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result) {
		if (result.isExceptionOccurred()) {
			listener.onRetrieveAccessTokenFromTwitterFailure(result);
		} else {
			listener.onRetrieveAccessTokenFromTwitterSuccess(result);
		}

	}

}
