/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2015 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKUnpublishProjectTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKUnpublishProjectTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceNameValuePair;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKUnpublishProjectAsyncTask extends AsyncTask<BehanceSDKUnpublishProjectTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<Boolean>> {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKUnpublishProjectAsyncTask.class);

    private IBehanceSDKUnpublishProjectTaskListener taskHandler;
    private BehanceSDKUnpublishProjectTaskParams taskParams;

    public BehanceSDKUnpublishProjectAsyncTask(IBehanceSDKUnpublishProjectTaskListener taskHandler) {
        this.taskHandler = taskHandler;
    }

    @Override
    protected BehanceSDKAsyncTaskResultWrapper<Boolean> doInBackground(BehanceSDKUnpublishProjectTaskParams... params) {

        BehanceSDKAsyncTaskResultWrapper<Boolean> result = new BehanceSDKAsyncTaskResultWrapper<Boolean>();
        result.setResult(false);

        try {
            taskParams = params[0];

            String projectId = String.valueOf(taskParams.getProjectId());

            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
            urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectId);

            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.UNPUBLISH_PROJECT_API_URL, urlParams);

            String userAccessToken = taskParams.getUserAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            logger.debug("Unpublish Project url - %s", url);

            List<BehanceNameValuePair> nameValuePairs = new ArrayList<BehanceNameValuePair>(1);
            nameValuePairs.add(new BehanceNameValuePair("published", "0"));

            BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpPutRequest(url, nameValuePairs);
            String jsonResponseString = connectionResponse.getResponseObject();

            logger.debug("Unpublish Project response: %s", jsonResponseString);

            int httpStatusCode = connectionResponse.getResponseCode();
            if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
                JSONObject jsonObject = new JSONObject(jsonResponseString);
                httpStatusCode = jsonObject.getInt("http_code");
                if (httpStatusCode == IBehanceHttpConnection.SC_OK) {
                    int updated = jsonObject.getInt("updated");
                    if (updated == 1) {
                        result.setResult(true);
                    } else {
                        result.setResult(false);
                    }
                } else {
                    result.setResult(false);
                }
            }
            logger.debug("Unpublish Project http response status code - %s", httpStatusCode);
            result.setResult(true);
        } catch (Exception e) {
            logger.error(e, "Problem trying to Unpublish Project");
            result.setException(e);
            result.setExceptionOccurred(true);
            result.setResult(false);
        } catch (Throwable t) {
            logger.error(t, "Problem trying to Unpublish Project");
            BehanceSDKException ex = new BehanceSDKException(t);
            result.setException(ex);
            result.setExceptionOccurred(true);
            result.setResult(false);
        }
        return result;
    }

    @Override
    protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<Boolean> resultWrapper) {
        if (taskHandler != null) {
            if (resultWrapper.isExceptionOccurred()) {
                taskHandler.onUnpublishProjectTaskFailure(resultWrapper.getException(), taskParams);
            } else {
                taskHandler.onUnpublishProjectTaskSuccess(resultWrapper.getResult(), taskParams);
            }
        }
    }
}