package com.behance.sdk.datamanager;

import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.asynctask.params.BehanceSDKAbstractTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetCitiesAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetStatesOfCountryAsyncTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKGetCitiesAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetCountriesAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetStatesOfCountryAsyncTask;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCitiesAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCountriesAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetStatesOfCountryAsyncTaskListener;
import com.behance.sdk.listeners.IBehanceSDKLocationFiltersDataManagerListener;
import com.behance.sdk.managers.BehanceSDKEditProfileManager;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArraySet;

public class BehanceSDKProfileLocationsDataManager implements IBehanceSDKGetCountriesAsyncTaskListener, IBehanceSDKGetStatesOfCountryAsyncTaskListener, IBehanceSDKGetCitiesAsyncTaskListener {

    private static BehanceSDKProfileLocationsDataManager instance;

    private List<BehanceSDKCountryDTO> countriesList;
    private BehanceSDKGetCountriesAsyncTask getCountriesAsyncTask;
    private boolean getCountriesTaskInProgress;
    private CopyOnWriteArraySet<IBehanceSDKLocationFiltersDataManagerListener> listenersSet;

    private List<BehanceSDKStateDTO> statesList;
    private String activeCountryId;
    private BehanceSDKGetStatesOfCountryAsyncTask getStatesAsyncTask;
    private boolean getStatesAsyncTaskInProgress;

    private List<BehanceSDKCityDTO> citiesList;
    private BehanceSDKGetCitiesAsyncTaskParams activeGetCitiesParams;
    private BehanceSDKGetCitiesAsyncTask getCitiesAsyncTask;
    private boolean getCitiesAsyncTaskInProgress;

    private String clientId;

    private BehanceSDKProfileLocationsDataManager() {
        countriesList = new ArrayList<BehanceSDKCountryDTO>();
        statesList = new ArrayList<BehanceSDKStateDTO>();
        listenersSet = new CopyOnWriteArraySet<IBehanceSDKLocationFiltersDataManagerListener>();

        setCredentials();
    }

    private void setCredentials() {
        BehanceSDKEditProfileManager manager = BehanceSDKEditProfileManager.getInstance();
        IBehanceSDKUserCredentials userCredentials = manager.getUserCredentials();
        clientId = userCredentials.getClientId();
    }

    public static BehanceSDKProfileLocationsDataManager getInstance() {
        if (instance == null) {
            instance = new BehanceSDKProfileLocationsDataManager();
        }
        return instance;
    }

    public void loadCountries() {
        if (countriesList == null || countriesList.isEmpty()) {
            loadCountriesFromServer();
        } else {
            notifyListenersOnGetCountriesSuccess();
        }
    }

    private void loadCountriesFromServer() {
        if (getCountriesAsyncTask == null) {
            setGetCountriesTaskInProgress(true);
            getCountriesAsyncTask = new BehanceSDKGetCountriesAsyncTask(this);
            BehanceSDKAbstractTaskParams params = new BehanceSDKAbstractTaskParams();

            if(clientId == null)
                setCredentials();
            params.setClientId(clientId);

            getCountriesAsyncTask.execute(params);
        }
    }

    private void notifyListenersOnGetCountriesSuccess() {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onCountriesLoadingSuccess(countriesList);
            }
        }
    }

    @Override
    public void onGetCountriesSuccess(List<BehanceSDKCountryDTO> result) {
        countriesList.clear();
        countriesList.addAll(result);
        notifyListenersOnGetCountriesSuccess();
        getCountriesAsyncTask = null;
        setGetCountriesTaskInProgress(false);
    }

    @Override
    public void onGetCountriesFailure(Exception e) {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onCountriesLoadingFailure(e);
            }
        }
        getCountriesAsyncTask = null;
        setGetCountriesTaskInProgress(false);
    }

    public void addListener(IBehanceSDKLocationFiltersDataManagerListener listener) {
        if (listener != null && !listenersSet.contains(listener)) {
            listenersSet.add(listener);
        }
    }

    public void removeListener(IBehanceSDKLocationFiltersDataManagerListener listener) {
        if (listener != null) {
            listenersSet.remove(listener);
        }
    }

    public boolean isGetCountriesTaskInProgress() {
        return getCountriesTaskInProgress;
    }

    private void setGetCountriesTaskInProgress(boolean getCountriesTaskInProgress) {
        this.getCountriesTaskInProgress = getCountriesTaskInProgress;
    }

    public boolean isGetStatesAsyncTaskInProgress() {
        return getStatesAsyncTaskInProgress;
    }

    private void setGetStatesAsyncTaskInProgress(boolean getStatesAsyncTaskInProgress) {
        this.getStatesAsyncTaskInProgress = getStatesAsyncTaskInProgress;
    }

    public void loadStates(String countryId) {
        if (activeCountryId != null && activeCountryId.equals(countryId) && statesList != null) {
            notifyListenersOnGetStatesSuccess(activeCountryId, statesList);
        } else {
            activeCountryId = countryId;
            loadStatesFromServer(countryId);
        }
    }

    private void loadStatesFromServer(String countryId) {
        if (!isGetStatesAsyncTaskInProgress() && getStatesAsyncTask == null) {
            setGetStatesAsyncTaskInProgress(true);
            getStatesAsyncTask = new BehanceSDKGetStatesOfCountryAsyncTask(this);
            BehanceSDKGetStatesOfCountryAsyncTaskParams params = new BehanceSDKGetStatesOfCountryAsyncTaskParams();
            params.setCountryId(countryId);

            if(clientId == null)
                setCredentials();
            params.setClientId(clientId);

            getStatesAsyncTask.execute(params);
        }
    }

    private void notifyListenersOnGetStatesSuccess(String countryId, List<BehanceSDKStateDTO> result) {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onStatesLoadingSuccess(countryId, result);
            }
        }
    }

    @Override
    public void onGetStatesSuccess(String countryId, List<BehanceSDKStateDTO> result) {
        statesList.clear();
        statesList.addAll(result);
        notifyListenersOnGetStatesSuccess(countryId, result);
        getStatesAsyncTask = null;
        setGetStatesAsyncTaskInProgress(false);
    }

    @Override
    public void onGetStatesFailure(String countryId, Exception e) {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onStatesLoadingFailure(countryId, e);
            }
        }
        getStatesAsyncTask = null;
        setGetStatesAsyncTaskInProgress(false);
    }

    public boolean isGetCitiesAsyncTaskInProgress() {
        return getCitiesAsyncTaskInProgress;
    }

    private void setGetCitiesAsyncTaskInProgress(boolean getCitiesAsyncTaskInProgress) {
        this.getCitiesAsyncTaskInProgress = getCitiesAsyncTaskInProgress;
    }

    public void searchForCities(String countryId, String stateId, String citySearchStr) {
        BehanceSDKGetCitiesAsyncTaskParams params = new BehanceSDKGetCitiesAsyncTaskParams();
        params.setCountryId(countryId);
        params.setStateId(stateId);
        params.setCitySearchStr(citySearchStr);

        if (citiesList != null && activeGetCitiesParams != null && activeGetCitiesParams.equals(params)) {
            notifyOnGetCitiesSuccessListeners(activeGetCitiesParams, citiesList);
        } else {
            searchCitiesOnServer(params);
        }
    }

    private void searchCitiesOnServer(BehanceSDKGetCitiesAsyncTaskParams params) {
        activeGetCitiesParams = params;
        setGetCitiesAsyncTaskInProgress(true);

        if (getCitiesAsyncTask != null) {
            getCitiesAsyncTask.cancel(true);
            getCitiesAsyncTask = null;
        }

        if(clientId == null)
            setCredentials();
        params.setClientId(clientId);

        getCitiesAsyncTask = new BehanceSDKGetCitiesAsyncTask(this);
        getCitiesAsyncTask.execute(params);
    }

    @Override
    public void onGetCitiesSuccess(BehanceSDKGetCitiesAsyncTaskParams params, List<BehanceSDKCityDTO> result) {
        if (params.equals(activeGetCitiesParams)) {
            notifyOnGetCitiesSuccessListeners(params, result);
        }
    }

    private void notifyOnGetCitiesSuccessListeners(BehanceSDKGetCitiesAsyncTaskParams params, List<BehanceSDKCityDTO> result) {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onCitiesLoadingSuccess(params, result);
            }
        }
        getCitiesAsyncTask = null;
        setGetCitiesAsyncTaskInProgress(false);
    }

    @Override
    public void onGetCitiesFailure(BehanceSDKGetCitiesAsyncTaskParams params, Exception e) {
        if (!listenersSet.isEmpty()) {
            for (IBehanceSDKLocationFiltersDataManagerListener listener : listenersSet) {
                listener.onCitiesLoadingFailure(params, e);
            }
        }
        getCitiesAsyncTask = null;
        setGetCitiesAsyncTaskInProgress(false);
    }
}