/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.datamanager;


import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCuratedGalleryProjectsAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectsAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectsAsyncTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKGetCuratedGalleryProjectsAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetProjectsAsyncTask;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.executors.BehanceSDKSerialExecutor;
import com.behance.sdk.listeners.IBehanceSDKProjectsDataManagerGetProjectsListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKProjectsDataManager implements IBehanceSDKGetProjectsAsyncTaskListener , IBehanceSDKGetCuratedGalleryProjectsAsyncTaskListener {

	private static BehanceSDKProjectsDataManager instance;
	private List<BehanceSDKProjectDTO> projectsList;
	private BehanceSDKGetProjectsAsyncTask getProjectsAsyncTask;
	private BehanceSDKGetCuratedGalleryProjectsAsyncTask getCuratedGalleryProjectsAsyncTask;
	private BehanceSDKSerialExecutor getProjectsExecutor;
	private Set<IBehanceSDKProjectsDataManagerGetProjectsListener> getProjectsListenersSet;
	private boolean getProjectsAsyncTaskInProgress = false;
	private final Object getProjectsSyncLockObject;

	private BehanceSDKProjectsDataManager() {
		getProjectsSyncLockObject = new Object();
		getProjectsExecutor = new BehanceSDKSerialExecutor(BehanceSDKGetProjectsAsyncTask.class.getSimpleName());
	}

	public static BehanceSDKProjectsDataManager getInstance() {
		if (instance == null) {
			instance = new BehanceSDKProjectsDataManager();
		}
		return instance;
	}

	public List<BehanceSDKProjectDTO> getProjects() {
		if (projectsList == null) {
			projectsList = new ArrayList<BehanceSDKProjectDTO>();
		}
		return projectsList;
	}

	private void setProjects(List<BehanceSDKProjectDTO> projects) {
		getProjects().clear();
		projectsList.addAll(projects);
	}

	public void loadProjectsFromServer(BehanceSDKGetProjectsAsyncTaskParams params) {
		if (getProjectsAsyncTask != null) {
			getProjectsAsyncTask.cancel(true);
			getProjectsAsyncTask = null;
		}
		setGetProjectsAsyncTaskInProgress(true);
		getProjectsAsyncTask = new BehanceSDKGetProjectsAsyncTask(this);
		getProjectsAsyncTask.executeOnExecutor(getProjectsExecutor, params);
	}

	public void loadGalleryProjectsFromServer(BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params) {
		if (getCuratedGalleryProjectsAsyncTask != null) {
			getCuratedGalleryProjectsAsyncTask.cancel(true);
			getCuratedGalleryProjectsAsyncTask = null;
		}
		setGetProjectsAsyncTaskInProgress(true);
		getCuratedGalleryProjectsAsyncTask = new BehanceSDKGetCuratedGalleryProjectsAsyncTask(this);
		getCuratedGalleryProjectsAsyncTask.executeOnExecutor(getProjectsExecutor, params);
	}

	@Override
	public void onGetCuratedGalleryProjectsSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params){
		synchronized (getProjectsSyncLockObject) {
			if (params.getPageNumber() > 1) {
				getProjects().addAll(result);
			} else {
				setProjects(result);
			}
			if (getProjectsListenersSet != null) {
				for (IBehanceSDKProjectsDataManagerGetProjectsListener listener : getProjectsListenersSet) {
					if (listener != null) {
						listener.onProjectsDataManagerGetProjectsSuccess(result, params);
					}
				}
			}
			setGetProjectsAsyncTaskInProgress(false);
			getCuratedGalleryProjectsAsyncTask = null;
		}
	}

	@Override
	public void onGetCuratedGalleryProjectsFailure(Exception e, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params){
		HashMap<String,String> hashMap = new HashMap<>();
		//AnalyticsAgent.logError(AnalyticsErrorType.PROJECTS_LOAD_ERROR,e,hashMap);
		synchronized (getProjectsSyncLockObject) {
			if (getProjectsListenersSet != null) {
				for (IBehanceSDKProjectsDataManagerGetProjectsListener listener : getProjectsListenersSet) {
					if (listener != null) {
						listener.onProjectsDataManagerGetProjectsFailure(e, params);
					}
				}
			}
			setGetProjectsAsyncTaskInProgress(false);
			getCuratedGalleryProjectsAsyncTask = null;
		}
	}


	@Override
	public void onGetProjectsAsyncTaskSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetProjectsAsyncTaskParams params) {
		synchronized (getProjectsSyncLockObject) {
			if (params.getPageNumber() > 1) {
				getProjects().addAll(result);
			} else {
				setProjects(result);
			}
			if (getProjectsListenersSet != null) {
				for (IBehanceSDKProjectsDataManagerGetProjectsListener listener : getProjectsListenersSet) {
					if (listener != null) {
						listener.onProjectsDataManagerGetProjectsSuccess(result, params);
					}
				}
			}
			setGetProjectsAsyncTaskInProgress(false);
			getProjectsAsyncTask = null;
		}
	}

	@Override
	public void onGetProjectsAsyncTaskFailure(Exception e, BehanceSDKGetProjectsAsyncTaskParams params) {
		HashMap<String,String> hashMap = new HashMap<>();
		//AnalyticsAgent.logError(AnalyticsErrorType.PROJECTS_LOAD_ERROR,e,hashMap);
		synchronized (getProjectsSyncLockObject) {
			if (getProjectsListenersSet != null) {
				for (IBehanceSDKProjectsDataManagerGetProjectsListener listener : getProjectsListenersSet) {
					if (listener != null) {
						listener.onProjectsDataManagerGetProjectsFailure(e, params);
					}
				}
			}
			setGetProjectsAsyncTaskInProgress(false);
			getProjectsAsyncTask = null;
		}
	}

	public void addGetProjectsListener(IBehanceSDKProjectsDataManagerGetProjectsListener listener) {
		synchronized (getProjectsSyncLockObject) {
			if (getProjectsListenersSet == null) {
				getProjectsListenersSet = new HashSet<IBehanceSDKProjectsDataManagerGetProjectsListener>();
			}
			if (!getProjectsListenersSet.contains(listener)) {
				getProjectsListenersSet.add(listener);
			}
		}
	}

	public void removeGetProjectsListener(IBehanceSDKProjectsDataManagerGetProjectsListener listener) {
		synchronized (getProjectsSyncLockObject) {
			if (getProjectsListenersSet != null) {
				getProjectsListenersSet.remove(listener);
			}
		}
	}

	public boolean isGetProjectsAsyncTaskInProgress() {
		return getProjectsAsyncTaskInProgress;
	}

	public void setGetProjectsAsyncTaskInProgress(boolean getProjectsAsyncTaskInProgress) {
		this.getProjectsAsyncTaskInProgress = getProjectsAsyncTaskInProgress;
	}

	public void clearProjectsDataManagerCache() {
		synchronized (getProjectsSyncLockObject) {
			if (projectsList != null && !projectsList.isEmpty()) {
				projectsList.clear();
			}
		}
	}
}