/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.datamanager;

import java.util.ArrayList;
import java.util.List;

import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.asynctasks.BehanceSDKLoadCreativeFieldsAsyncTask;
import com.behance.sdk.datamanager.listeners.interfaces.IFiltersDataManagerListener;
import com.behance.sdk.datamanager.listeners.interfaces.IGetCreativeFieldsAsyncTaskListener;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;

/**
 * @author ugs
 *
 */
public class FiltersDataManager implements IGetCreativeFieldsAsyncTaskListener {
	
	private static final FiltersDataManager FILTERS_DATA_MANAGER = new FiltersDataManager();
	private List<BehanceSDKCreativeFieldDTO> creativeFields;
	
//	private List<WIPTagDTO> wipTags;
	
	private List<IFiltersDataManagerListener> listeners;
	private BehanceSDKLoadCreativeFieldsAsyncTask loadCreativeFieldsAsyncTask;
	private boolean loadCreativeFieldsTaskInProgress;
//	private LoadWIPTagsAsyncTask loadWIPTagsAsyncTask;
//	private boolean loadWIPTagsTaskInProgress;

	private FiltersDataManager(){
		listeners = new ArrayList<IFiltersDataManagerListener>();
	}
	
	public static FiltersDataManager getInstance(){
		return FILTERS_DATA_MANAGER;
	}
	
	public void addListener(IFiltersDataManagerListener listener) {
		if (listener != null && !listeners.contains(listener)) {
			listeners.add(listener);
		}
	}

	public void removeListener(IFiltersDataManagerListener listener) {
		if (listener != null) {
			listeners.remove(listener);
		}
	}
	
	public void loadCreativeFields(IBehanceSDKUserCredentials iUserCredentials){
		if(null == creativeFields || creativeFields.isEmpty()){
			loadCreativeFieldsFromServer(iUserCredentials);
		}else{
			notifyListenersOnLoadCreativeFieldSuccess();
		}
	}

	private void notifyListenersOnLoadCreativeFieldSuccess() {
		if (false == listeners.isEmpty()) {
			for (IFiltersDataManagerListener listener : listeners) {
				listener.onCreativeFieldsLoadingSuccess(creativeFields);
			}
		}
	}

	private void loadCreativeFieldsFromServer(IBehanceSDKUserCredentials iUserCredentials) {
		if (loadCreativeFieldsAsyncTask == null) {
			setLoadCreativeFieldsTaskInProgress(true);
			loadCreativeFieldsAsyncTask = new BehanceSDKLoadCreativeFieldsAsyncTask(this);
			loadCreativeFieldsAsyncTask.setUserCredentials(iUserCredentials);
			loadCreativeFieldsAsyncTask.execute();
		}
	}
	
	private void setLoadCreativeFieldsTaskInProgress(boolean loadCreativeFieldsTaskInProgress) {
		this.loadCreativeFieldsTaskInProgress = loadCreativeFieldsTaskInProgress;
	}

	
	public boolean isLoadCreativeFieldsTaskInProgress() {
		return loadCreativeFieldsTaskInProgress;
	}
	

	/* (non-Javadoc)
	 * @see com.behance.network.interfaces.listeners.IGetCreativeFieldsAsyncTaskListener#onGetCreativeFieldsSuccess(java.util.List)
	 */
	@Override
	public void onLoadCreativeFieldsSuccess(List<BehanceSDKCreativeFieldDTO> fields) {
		if(null == creativeFields)
			creativeFields = new ArrayList<BehanceSDKCreativeFieldDTO>(fields.size());
		creativeFields.clear();
		creativeFields.addAll(fields);
		notifyListenersOnLoadCreativeFieldSuccess();
		loadCreativeFieldsAsyncTask = null;
		setLoadCreativeFieldsTaskInProgress(false);
	}

	/* (non-Javadoc)
	 * @see com.behance.network.interfaces.listeners.IGetCreativeFieldsAsyncTaskListener#onGetCreativeFieldsFailure(java.lang.Exception)
	 */
	@Override
	public void onLoadCreativeFieldsFailure(Exception e) {
		if (false == listeners.isEmpty()) {
			for (IFiltersDataManagerListener listener : listeners) {
				listener.onCreativeFieldsLoadingFailure(e);
			}
		}
		loadCreativeFieldsAsyncTask = null;
		setLoadCreativeFieldsTaskInProgress(false);
	}

	 
//	public void loadWIPTags(){
//		if(null == wipTags || wipTags.isEmpty()){
//			loadWIPTagsFromServer();
//		}else{
//			notifyListenersOnLoadWIPTagsSuccess();
//		}
//	}
//
//	private void loadWIPTagsFromServer() {
//		if (loadWIPTagsAsyncTask == null) {
//			setLoadWIPTagsTaskInProgress(true);
//			loadWIPTagsAsyncTask = new LoadWIPTagsAsyncTask(this);
//			loadWIPTagsAsyncTask.execute();
//		}
//	}
//	
//	private void notifyListenersOnLoadWIPTagsSuccess() {
//		if (false == listeners.isEmpty()) {
//			for (IFiltersDataManagerListener listener : listeners) {
//				listener.onWIPTagsLoadingSuccess(wipTags);
//			}
//		}
//	}
//	
//	private void setLoadWIPTagsTaskInProgress(boolean loadWIPTagsTaskInProgress) {
//		this.loadWIPTagsTaskInProgress = loadCreativeFieldsTaskInProgress;
//	}
//
//	
//	public boolean isLoadWIPTagsTaskInProgress() {
//		return loadWIPTagsTaskInProgress;
//	}
	
	
//	/* (non-Javadoc)
//	 * @see com.behance.network.interfaces.listeners.IGetWIPTagsAsyncTaskListener#onLoadWIPTagsSuccess(java.util.List)
//	 */
//	@Override
//	public void onLoadWIPTagsSuccess(List<WIPTagDTO> tagsList) {
//		if(null == wipTags)
//			wipTags = new ArrayList<WIPTagDTO>(tagsList.size());
//		wipTags.clear();
//		wipTags.addAll(tagsList);
//		notifyListenersOnLoadWIPTagsSuccess();
//		loadWIPTagsAsyncTask = null;
//		setLoadWIPTagsTaskInProgress(false);		
//	}
//
//	/* (non-Javadoc)
//	 * @see com.behance.network.interfaces.listeners.IGetWIPTagsAsyncTaskListener#onLoadWIPTagsFailure(java.lang.Exception)
//	 */
//	@Override
//	public void onLoadWIPTagsFailure(Exception e) {
//		if (false == listeners.isEmpty()) {
//			for (IFiltersDataManagerListener listener : listeners) {
//				listener.onWIPTagsLoadingFailure(e);
//			}
//		}
//		loadWIPTagsAsyncTask = null;
//		setLoadWIPTagsTaskInProgress(false);
//	}
}
