/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.parser;


import com.behance.sdk.dto.project.BehanceSDKAbstractProjectModuleDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleAudioDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleEmbedDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleImageDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleTextDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleVideoDTO;
import com.behance.sdk.enums.BehanceSDKProjectModuleType;

import org.json.JSONException;
import org.json.JSONObject;

public class BehanceSDKProjectModuleFactory {


	public static BehanceSDKAbstractProjectModuleDTO getProjectModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleType moduleType = BehanceSDKProjectModuleType.fromString(module.getString("type"));
		if (moduleType == BehanceSDKProjectModuleType.IMAGE) {
			return getImageModuleInstance(module);
		} else if (moduleType == BehanceSDKProjectModuleType.EMBED) {
			return getEmbedModuleInstance(module);
		} else if (moduleType == BehanceSDKProjectModuleType.VIDEO) {
			return getVideoModuleInstance(module);
		} else if (moduleType == BehanceSDKProjectModuleType.TEXT) {
			return getTextModuleInstance(module);
		} else if (moduleType == BehanceSDKProjectModuleType.AUDIO) {
			return getAudioModuleInstance(module);
		} else {
			return null;
		}
	}

	private static BehanceSDKAbstractProjectModuleDTO getTextModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleTextDTO moduleDTO = new BehanceSDKProjectModuleTextDTO();
		moduleDTO.setId(module.optString("id"));
		moduleDTO.setFormattedText(module.optString("text"));
		moduleDTO.setPlainText(module.optString("text_plain"));
		moduleDTO.setAlignment(module.optString("alignment"));
		return moduleDTO;
	}

	private static BehanceSDKAbstractProjectModuleDTO getAudioModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleAudioDTO audioModule = new BehanceSDKProjectModuleAudioDTO();
		audioModule.setId(module.optString("id"));
		audioModule.setEmbedHTML(module.optString("embed"));
		audioModule.setSrcURL(module.optString("src"));
		audioModule.setAlignment(module.optString("alignment"));
		return audioModule;
	}

	private static BehanceSDKAbstractProjectModuleDTO getVideoModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleVideoDTO moduleDTO = new BehanceSDKProjectModuleVideoDTO();
		moduleDTO.setId(module.optString("id"));
		moduleDTO.setEmbedHTML(module.optString("embed"));
		moduleDTO.setHeight(module.optInt("height"));
		moduleDTO.setWidth(module.optInt("width"));
		moduleDTO.setImageSrc(module.optString("image_src"));
		moduleDTO.setPlayerSkin(module.optString("player_skin"));
		moduleDTO.setPlayerUrl(module.optString("player"));
		moduleDTO.setVideoSrc(module.optString("src"));
		moduleDTO.setAlignment(module.optString("alignment"));
		return moduleDTO;
	}

	private static BehanceSDKAbstractProjectModuleDTO getEmbedModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleEmbedDTO moduleDTO = new BehanceSDKProjectModuleEmbedDTO();
		moduleDTO.setId(module.optString("id"));
		moduleDTO.setEmbedHTML(module.optString("embed"));
		moduleDTO.setAlignment(module.optString("alignment"));
		moduleDTO.setCaptionFormattedText(module.optString("caption"));
		moduleDTO.setCaptionPlainText(module.optString("caption_plain"));
		moduleDTO.setCaptionAlignment(module.optString("caption_alignment"));
		return moduleDTO;
	}

	private static BehanceSDKAbstractProjectModuleDTO getImageModuleInstance(JSONObject module) throws JSONException {
		BehanceSDKProjectModuleImageDTO moduleDTO = new BehanceSDKProjectModuleImageDTO();
		JSONObject dimenOb = module.optJSONObject("dimensions");
		if(dimenOb != null && dimenOb.optJSONObject("disp") != null) {
			JSONObject dispDimenOb = dimenOb.optJSONObject("disp");
			moduleDTO.setHeight(dispDimenOb.optInt("height"));
			moduleDTO.setWidth(dispDimenOb.optInt("width"));
		} else {
			moduleDTO.setHeight(module.optInt("height"));
			moduleDTO.setWidth(module.optInt("width"));
		}
		moduleDTO.setId(module.optString("id"));
		moduleDTO.setSrcUrl(module.optString("src"));
		moduleDTO.setFullBleed(module.optInt("full_bleed") == 1);
		Object sizesObject = module.opt("sizes");
		if (sizesObject instanceof JSONObject) {
			JSONObject moduleSizes = (JSONObject) sizesObject;
			moduleDTO.setHDSrcUrl(moduleSizes.optString("max_1240"));
		}
		moduleDTO.setAlignment(module.optString("alignment"));
		moduleDTO.setCaptionFormattedText(module.optString("caption"));
		moduleDTO.setCaptionPlainText(module.optString("caption_plain"));
		moduleDTO.setCaptionAlignment(module.optString("caption_alignment"));
		return moduleDTO;
	}

}
