/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.parser;


import com.behance.sdk.dto.search.BehanceSDKFeaturedDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDetailsSectionDTO;
import com.behance.sdk.dto.search.BehanceSDKWebLinkDTO;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.Iterator;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceUserDTOParser {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceUserDTOParser.class);

	public BehanceSDKUserDTO parse(JSONObject userJsonObject) throws JSONException {
		try {
			if (userJsonObject != null) {
				BehanceSDKUserDTO behanceUserDTO = new BehanceSDKUserDTO();
				behanceUserDTO.setId(userJsonObject.optInt("id"));
				behanceUserDTO.setFirstName(userJsonObject.optString("first_name"));
				behanceUserDTO.setLastName(userJsonObject.optString("last_name"));
				behanceUserDTO.setUserName(userJsonObject.optString("username"));
				behanceUserDTO.setProfileUrl(userJsonObject.optString("url"));
				behanceUserDTO.setDisplayName(userJsonObject.optString("display_name"));
				behanceUserDTO.setCity(userJsonObject.optString("city"));
				behanceUserDTO.setState(userJsonObject.optString("state"));
				behanceUserDTO.setCountry(userJsonObject.optString("country"));
				behanceUserDTO.setCompany(userJsonObject.optString("company"));
				behanceUserDTO.setOccupation(userJsonObject.optString("occupation"));
				behanceUserDTO.setCreatedOn(userJsonObject.optLong("created_on"));
				behanceUserDTO.setTwitterHandle(userJsonObject.optString("twitter"));

				JSONArray linksJSONArray = userJsonObject.optJSONArray("links");
				if (linksJSONArray != null) {
					for (int linksCounter = 0; linksCounter < linksJSONArray.length(); linksCounter++) {
						JSONObject linkJsonObject = linksJSONArray.optJSONObject(linksCounter);
						if (linkJsonObject != null) {
							BehanceSDKWebLinkDTO userLink = new BehanceSDKWebLinkDTO();
							userLink.setTitle(linkJsonObject.optString("title"));
							userLink.setUrl(linkJsonObject.optString("url"));
							behanceUserDTO.addWebLink(userLink);
						}
					}
				}

				JSONObject sectionsJSONObject = userJsonObject.optJSONObject("sections");
				if (sectionsJSONObject != null) {
					Iterator<?> sectionKeysIterator = sectionsJSONObject.keys();
					while (sectionKeysIterator.hasNext()) {
						String sectionsTitle = String.valueOf(sectionKeysIterator.next());
						if (sectionsTitle != null) {
							String sectionDescription = sectionsJSONObject.optString(sectionsTitle);
							if (sectionDescription != null) {
								BehanceSDKUserDetailsSectionDTO sectionDTO = new BehanceSDKUserDetailsSectionDTO();
								sectionDTO.setTitle(sectionsTitle);
								sectionDTO.setDescription(sectionDescription);
								behanceUserDTO.addDetailsSection(sectionDTO);
							}
						}
					}
				}

				int currentUserFollowingValue = userJsonObject.optInt("is_following", 0);
				if (currentUserFollowingValue == 1) {
					behanceUserDTO.setCurrentUserFollowing(true);
				} else {
					behanceUserDTO.setCurrentUserFollowing(false);
				}

				// get user stats
				JSONObject statsJSONObject = userJsonObject.optJSONObject("stats");
				if (statsJSONObject != null) {
					behanceUserDTO.setFollowingCount(statsJSONObject.optInt("following"));
					behanceUserDTO.setFollowersCount(statsJSONObject.optInt("followers"));
					behanceUserDTO.setAppreciationsCount(statsJSONObject.optInt("appreciations"));
					behanceUserDTO.setCommentsCount(statsJSONObject.optInt("comments"));
					behanceUserDTO.setViewsCount(statsJSONObject.optInt("views"));
				}

				// get user profile images
				JSONObject imagesJSONObject = userJsonObject.optJSONObject("images");
				if (null != imagesJSONObject) {
					behanceUserDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_50, imagesJSONObject.optString("50"));
					behanceUserDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_115, imagesJSONObject.optString("115"));
					behanceUserDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_138, imagesJSONObject.optString("138"));
                    behanceUserDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_276, imagesJSONObject.optString("276"));
				}

				// get user fields
				JSONArray ownerFields = userJsonObject.optJSONArray("fields");
				if (null != ownerFields) {
					for (int counter = 0; counter < ownerFields.length(); counter++) {
						behanceUserDTO.addField(ownerFields.optString(counter));
					}
				}

				// get user featured details
				JSONArray featuresJSONArray = userJsonObject.optJSONArray("features");
				if (featuresJSONArray != null) {
					for (int featureCount = 0; featureCount < featuresJSONArray.length(); featureCount++) {
						JSONObject feature = featuresJSONArray.getJSONObject(featureCount);

						JSONObject featureSite = feature.optJSONObject("site");
						if (featureSite != null) {
							BehanceSDKFeaturedDTO featureDTO = new BehanceSDKFeaturedDTO();
							featureDTO.setSiteDomain(featureSite.optString("domain"));
							featureDTO.setSiteIcon(featureSite.optString("icon"));
							featureDTO.setSiteId(featureSite.optString("id"));
							featureDTO.setSiteKey(featureSite.optString("key"));
							featureDTO.setSiteName(featureSite.optString("name"));
							featureDTO.setSiteUrl(featureSite.optString("url"));

							JSONObject featureSiteRibbon = featureSite.optJSONObject("ribbon");
							if (featureSiteRibbon != null) {
								featureDTO.setSiteRibbonImage(featureSiteRibbon.optString("image"));
								featureDTO.setSiteRibbonBiggerImage(featureSiteRibbon.optString("image_2x"));
							}
							behanceUserDTO.addFeaturedDetail(featureDTO);
						}
					}
				}
				return behanceUserDTO;
			}
		} catch (JSONException e) {
			logger.error(e, "Problem parsing User JSON");
			throw e;
		}
		return null;
	}
}
