/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.project;

import android.util.SparseArray;

import com.behance.sdk.dto.search.BehanceSDKImageDTO;

import java.io.Serializable;

/**
 * 
 * @author ugs
 * 
 */
public class BehanceSDKProjectCoversDTO implements Serializable {

	private static final long serialVersionUID = -7479303355724199312L;

	public static final int SIZE_115 = 115;
	public static final int SIZE_202 = 202;
	public static final int SIZE_230 = 230;
	public static final int SIZE_404 = 404;

	private static final int NUMBER_OF_COVERIMAGES = 4;

	private SparseArray<BehanceSDKImageDTO> coverImages = new SparseArray<BehanceSDKImageDTO>(NUMBER_OF_COVERIMAGES);

	public void addCoverImages(int res, String url) {
		if (url != null && url.length() > 0) {
			this.coverImages.put(res, BehanceSDKImageDTO.getNewInstance(url));
		}
	}

	private BehanceSDKImageDTO findCoverImageInIncreasingSizeOrder(int imageSize) {
		BehanceSDKImageDTO result = coverImages.get(imageSize);
		if (result == null && imageSize <= SIZE_115) {
			result = coverImages.get(SIZE_115);
		}
		else if (result == null && imageSize <= SIZE_202) {
			result = coverImages.get(SIZE_202);
		}
        else if (result == null && imageSize <= SIZE_230) {
			result = coverImages.get(SIZE_230);
		}
        else if (result == null) {
			result = coverImages.get(SIZE_404);
		}
		return result;
	}

	private BehanceSDKImageDTO findCoverImageInDecreasingSizeOrder(int imageSize) {
		BehanceSDKImageDTO result = coverImages.get(imageSize);
		if (result == null && imageSize > SIZE_230) {
			result = coverImages.get(SIZE_404);
		}
		if (result == null && imageSize > SIZE_202) {
			result = coverImages.get(SIZE_230);
		}
		if (result == null && imageSize > SIZE_115) {
			result = coverImages.get(SIZE_202);
		}
		if (result == null) {
			result = coverImages.get(SIZE_115);
		}
		return result;
	}

	public BehanceSDKImageDTO findCoverImageInIncreasingSizeOrderAndFallback(int imageSize) {
		BehanceSDKImageDTO imageDTO = findCoverImageInIncreasingSizeOrder(imageSize);
		if (imageDTO == null) {
			imageDTO = findCoverImageInDecreasingSizeOrder(imageSize);
		}
		return imageDTO;
	}
}
