/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.project;


import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.util.BehanceSDKFileUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author ugs
 *
 */

public class BehanceSDKProjectDTO implements Serializable {

	public static final String MATURE_CONTENT_ACCESS_RESTRICTED_AGE = "restricted-age";
	public static final String MATURE_CONTENT_ACCESS_RESTRICTED_SAFE_BROWSING = "restricted-safe";
	public static final String MATURE_CONTENT_ACCESS_RESTRICTED_LOGGED_OUT = "logged-out";
	public static final String MATURE_CONTENT_ACCESS_ALLOWED = "allowed";

	public static final String PUBLIC_PROJECT = "public";
	public static final String LAYOUT_HTML_FILE_NAME = "layout.html";

	private static final long serialVersionUID = 401439769574614616L;

	private String id;
	private String name;
	private String projectUrl;
	private String privacy;
	private String description;
	private String matureAccess;
	private String custom_css;
	private String html;
	private String shortUrl;
	private String layoutHTMLPath;
	private int canvasWidth;
	private BehanceSDKProjectStylesDTO projectStyles;

	private long publishedDate;
	private long createdDate;
	private long modifiedDate;
	private long appreciatedByUserOn;

	private boolean matureContent;
	private boolean isVisible = true;
	private boolean publicProject;
	private boolean appreciatedByUser;

	private List<String> fields;
	private List<BehanceSDKUserDTO> owners;
	private List<BehanceSDKAbstractProjectModuleDTO> modules;
	private List<BehanceSDKProjectToolDTO> tools;
	private List<BehanceSDKProjectFeaturedDTO> features;
	private List<String> tags;
	private boolean allowComments;


	private BehanceSDKProjectCoversDTO covers;
	private BehanceSDKProjectStatsDTO stats;
	private BehanceSDKProjectCopyrightDTO copyright;

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<BehanceSDKUserDTO> getOwners() {
		if (owners != null) {
			return owners;
		} else {
			return Collections.emptyList();
		}
	}

	public void setOwners(List<BehanceSDKUserDTO> owners) {
		this.owners = owners;
	}

	public void addOwner(BehanceSDKUserDTO owner) {
		if (owners == null) {
			owners = new ArrayList<BehanceSDKUserDTO>();
		}
		owners.add(owner);
	}

	public List<String> getFields() {
		if (fields != null) {
			return fields;
		} else {
			return Collections.emptyList();
		}
	}

	public void setFields(List<String> fields) {
		this.fields = fields;
	}

	public void addField(String field) {
		if (fields == null) {
			fields = new ArrayList<String>();
		}
		fields.add(field);
	}

	public String getProjectUrl() {
		return projectUrl;
	}

	public void setUrl(String projectUrl) {
		this.projectUrl = projectUrl;
	}

	public String getPrivacy() {
		return privacy;
	}

	public void setPrivacy(String privacy) {
		this.privacy = privacy;
		setPublicProject(PUBLIC_PROJECT.equalsIgnoreCase(privacy));
	}

	public boolean isMatureContent() {
		return matureContent;
	}

	public void setMatureContent(boolean matureContent) {
		this.matureContent = matureContent;
	}

	public String getMatureAccess() {
		if (matureAccess == null) {
			matureAccess = MATURE_CONTENT_ACCESS_RESTRICTED_SAFE_BROWSING;
		}
		return matureAccess;
	}

	public void setMatureAccess(String matureAccess) {
		this.matureAccess = matureAccess;
	}

	public long getCreatedDate() {
		return createdDate;
	}

	public void setCreatedDate(long createdDate) {
		this.createdDate = createdDate;
	}

	public long getModifiedDate() {
		return modifiedDate;
	}

	public void setModifiedDate(long modifiedDate) {
		this.modifiedDate = modifiedDate;
	}

	public long getPublishedDate() {
		return publishedDate;
	}

	public void setPublishedDate(long publishedDate) {
		this.publishedDate = publishedDate;
	}

	public BehanceSDKProjectStatsDTO getStats() {
		if (stats == null) {
			stats = new BehanceSDKProjectStatsDTO();
		}
		return stats;
	}

	public void setStats(BehanceSDKProjectStatsDTO stats) {
		this.stats = stats;
	}

	public BehanceSDKProjectCoversDTO getCovers() {
		if (covers == null) {
			covers = new BehanceSDKProjectCoversDTO();
		}
		return covers;
	}

	public void setCovers(BehanceSDKProjectCoversDTO covers) {
		this.covers = covers;
	}

	public List<BehanceSDKAbstractProjectModuleDTO> getModules() {
		if (modules != null) {
			return modules;
		} else {
			return Collections.emptyList();
		}
	}

	public void setModules(List<BehanceSDKAbstractProjectModuleDTO> modules) {
		this.modules = modules;
	}

	public void addModule(BehanceSDKAbstractProjectModuleDTO module) {
		if (modules == null) {
			modules = new ArrayList<BehanceSDKAbstractProjectModuleDTO>();
		}
		modules.add(module);
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getCustomCSS() {
		return custom_css;
	}

	public void setCustomCSS(String custom_css) {
		this.custom_css = custom_css;
	}

	public String getHTML() {
		return html;
	}

	public void setHTML(String html) {
		this.html = html;
	}

	public boolean isVisible() {
		return isVisible;
	}

	public void setVisible(boolean isVisible) {
		this.isVisible = isVisible;
	}

	public String getShortUrl() {
		return shortUrl;
	}

	public void setShortUrl(String shortUrl) {
		this.shortUrl = shortUrl;
	}

	public boolean isPublicProject() {
		if (getPrivacy() != null) {
			return publicProject;
		} else {
			return true;
		}
	}

	public void setPublicProject(boolean publicProject) {
		this.publicProject = publicProject;
	}

	public List<BehanceSDKProjectFeaturedDTO> getFeatures() {
		if (features != null) {
			return features;
		} else {
			return Collections.emptyList();
		}
	}

	public void setFeatures(List<BehanceSDKProjectFeaturedDTO> features) {
		this.features = features;
	}

	public void addFeature(BehanceSDKProjectFeaturedDTO feature) {
		if (features == null) {
			features = new ArrayList<BehanceSDKProjectFeaturedDTO>();
		}
		features.add(feature);
	}

	public List<String> getTags() {
		if (tags == null) {
			tags = Collections.emptyList();
		}
		return tags;
	}

	public void setTags(List<String> tags) {
		this.tags = tags;
	}

	public void addTag(String tag) {
		if (tags == null) {
			tags = new ArrayList<String>();
		}
		tags.add(tag);
	}

	public BehanceSDKProjectCopyrightDTO getCopyright() {
		if (copyright == null) {
			copyright = new BehanceSDKProjectCopyrightDTO();
		}
		return copyright;
	}

	public void setCopyright(BehanceSDKProjectCopyrightDTO copyright) {
		this.copyright = copyright;
	}

	public String getLayoutHTMLPath() {
		if (null == layoutHTMLPath || layoutHTMLPath.length() == 0) {
			layoutHTMLPath = BehanceSDKFileUtils.getNetworkAppProjectHTMLStoragePath(id);
		}
		return layoutHTMLPath;
	}

	public long getAppreciatedByUserOn() {
		return appreciatedByUserOn;
	}

	public void setAppreciatedByUserOn(long appreciatedByUserOn) {
		this.appreciatedByUserOn = appreciatedByUserOn;
	}

	public boolean isAppreciatedByUser() {
		return appreciatedByUser;
	}

	public void setAppreciatedByUser(boolean appreciatedByUser) {
		this.appreciatedByUser = appreciatedByUser;
	}

	public BehanceSDKProjectStylesDTO getProjectStyles() {
		return projectStyles;
	}

	public void setProjectStyles(BehanceSDKProjectStylesDTO projectStyles) {
		this.projectStyles = projectStyles;
	}

	public int getCanvasWidth() {
		return canvasWidth;
	}

	public void setCanvasWidth(int canvasWidth) {
		this.canvasWidth = canvasWidth;
	}

	public boolean isAllowComments() {
		return allowComments;
	}

	public void setAllowComments(boolean allowComments) {
		this.allowComments = allowComments;
	}

	public List<BehanceSDKProjectToolDTO> getTools() {
		return tools;
	}

	public void setTools(List<BehanceSDKProjectToolDTO> tools) {
		this.tools = tools;
	}
}
