/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.search;

import android.util.SparseArray;

import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.util.BehanceSDKSerializableSparseArray;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKUserDTO implements Serializable	{

	public static final int PROFILE_IMAGE_SIZE_50 = 50;
    public static final int PROFILE_IMAGE_SIZE_100 = 100;
	public static final int PROFILE_IMAGE_SIZE_115 = 115;
	public static final int PROFILE_IMAGE_SIZE_138 = 138;
    public static final int PROFILE_IMAGE_SIZE_230 = 230;
    public static final int PROFILE_IMAGE_SIZE_276 = 276;
	private static final int INITIAL_SIZE_FOR_COLLECTION = 3;

	private int id;
	private String firstName;
	private String lastName;
	private String displayName;
	private String emailId;
	private String profileUrl;
	private String userName;
	private String city;
	private String state;
	private String country;
	private String company;
	private String occupation;
	private long createdOn;

	private int followingCount;
	private int followersCount;
	private int appreciationsCount;
	private int commentsCount;
	private int viewsCount;

	private boolean isCurrentUserFollowing;

	private BehanceSDKSerializableSparseArray<BehanceSDKImageDTO> profileImagesArray;
	private List<String> fields;
	private List<BehanceSDKFeaturedDTO> featuredDetails;
	private List<BehanceSDKWebLinkDTO> webLinks;
	private List<BehanceSDKUserDetailsSectionDTO> detailsSections;
	private List<BehanceSDKUserWorkExpDTO> workExperiences;

	private String fieldsDisplayString;
	private String locationDisplayString;

	private List<BehanceSDKProjectDTO> projects;
	private List<BehanceSDKWorkInProgressDTO> wips;
	private List<BehanceSDKCollectionDTO> createdCollections;
	private List<BehanceSDKCollectionDTO> followedCollections;

	private String twitterHandle;

    private boolean isEnterprise = false;

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getProfileUrl() {
		return profileUrl;
	}

	public void setProfileUrl(String profileUrl) {
		this.profileUrl = profileUrl;
	}

	public String getDisplayName() {
		if (displayName != null && displayName.length() > 0) {
			return displayName;
		} else {
			return getFirstName() + " " + getLastName();
		}
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getEmailId() {
		return emailId;
	}

	public void setEmailId(String emailId) {
		this.emailId = emailId;
	}

	public List<String> getFields() {
		if (fields != null) {
			return fields;
		} else {
			return Collections.emptyList();
		}
	}

	public String getFieldsDisplayString() {
		if (fieldsDisplayString == null) {
			String result = "";
			if (fields != null) {
				for (String field : fields) {
					result = result + ", " + field;
				}
				result = result.substring(2);
			}
			fieldsDisplayString = result;
		}
		return fieldsDisplayString;
	}

	public void addField(String fieldStr) {
		if (null == fields) {
			fields = new ArrayList<String>(INITIAL_SIZE_FOR_COLLECTION);
		}
		fields.add(fieldStr);
	}

	private SparseArray<BehanceSDKImageDTO> getProfileImages() {
		if (profileImagesArray == null) {
			profileImagesArray = new BehanceSDKSerializableSparseArray<>(INITIAL_SIZE_FOR_COLLECTION);
		}
		return profileImagesArray;
	}

	public void addImage(int key, String url) {
		if (null == profileImagesArray) {
			profileImagesArray = new BehanceSDKSerializableSparseArray<BehanceSDKImageDTO>(INITIAL_SIZE_FOR_COLLECTION);
		}
		profileImagesArray.put(key, BehanceSDKImageDTO.getNewInstance(url));
	}

	public BehanceSDKImageDTO findProfileImageInIncreasingSizeOrder(int imageSize) {
		BehanceSDKImageDTO result = getProfileImages().get(imageSize);
		if (result == null && imageSize <= PROFILE_IMAGE_SIZE_50) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_50);
		}
		if (result == null && imageSize <= PROFILE_IMAGE_SIZE_115) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_115);
		}
		if (result == null) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_138);
		}
		return result;
	}

	public BehanceSDKImageDTO findProfileImageInDecreasingSizeOrder(int imageSize) {
		BehanceSDKImageDTO result = getProfileImages().get(imageSize);
		if (result == null && imageSize > PROFILE_IMAGE_SIZE_138) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_138);
		}
		if (result == null && imageSize > PROFILE_IMAGE_SIZE_115) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_115);
		}
		if (result == null) {
			result = getProfileImages().get(PROFILE_IMAGE_SIZE_138);
		}
		return result;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getCompany() {
		return company;
	}

	public void setCompany(String company) {
		this.company = company;
	}

	public String getOccupation() {
		return occupation;
	}

	public void setOccupation(String occupation) {
		this.occupation = occupation;
	}

	public long getCreatedOn() {
		return createdOn;
	}

	public void setCreatedOn(long createdOn) {
		this.createdOn = createdOn;
	}

	public String getLocationDisplayString() {
		if (locationDisplayString == null) {
			locationDisplayString = getCity() + ", " + getCountry();
		}
		return locationDisplayString;
	}

	public List<BehanceSDKFeaturedDTO> getFeaturedDetails() {
		if (featuredDetails != null) {
			return featuredDetails;
		} else {
			return Collections.emptyList();
		}
	}

	public void addFeaturedDetail(BehanceSDKFeaturedDTO feature) {
		if (featuredDetails == null) {
			featuredDetails = new ArrayList<BehanceSDKFeaturedDTO>();
		}
		featuredDetails.add(feature);
	}

	public List<BehanceSDKProjectDTO> getProjects() {
		if (projects != null) {
			return projects;
		} else {
			return Collections.emptyList();
		}
	}

	public void setProjects(List<BehanceSDKProjectDTO> projects) {
		this.projects = projects;
	}

	public List<BehanceSDKWorkInProgressDTO> getWIPs() {
		if (wips != null) {
			return wips;
		} else {
			return Collections.emptyList();
		}
	}

	public void setWIPs(List<BehanceSDKWorkInProgressDTO> wips) {
		this.wips = wips;
	}

	public List<BehanceSDKCollectionDTO> getCreatedCollections() {
		if (createdCollections != null) {
			return createdCollections;
		} else {
			return Collections.emptyList();
		}
	}

	public void setCreatedCollections(List<BehanceSDKCollectionDTO> collections) {
		this.createdCollections = collections;
	}

	public int getFollowingCount() {
		return followingCount;
	}

	public void setFollowingCount(int followingCount) {
		this.followingCount = followingCount;
	}

	public int getFollowersCount() {
		return followersCount;
	}

	public void setFollowersCount(int followersCount) {
		this.followersCount = followersCount;
	}

	public int getAppreciationsCount() {
		return appreciationsCount;
	}

	public void setAppreciationsCount(int appreciationsCount) {
		this.appreciationsCount = appreciationsCount;
	}

	public int getCommentsCount() {
		return commentsCount;
	}

	public void setCommentsCount(int commentsCount) {
		this.commentsCount = commentsCount;
	}

	public int getViewsCount() {
		return viewsCount;
	}

	public void setViewsCount(int viewsCount) {
		this.viewsCount = viewsCount;
	}

	public boolean isCurrentUserFollowing() {
		return isCurrentUserFollowing;
	}

	public void setCurrentUserFollowing(boolean isCurrentUserFollowing) {
		this.isCurrentUserFollowing = isCurrentUserFollowing;
	}

	public String getTwitterHandle() {
		return twitterHandle;
	}

	public void setTwitterHandle(String twitterHandle) {
		this.twitterHandle = twitterHandle;
	}

	public List<BehanceSDKWebLinkDTO> getWebLinks() {
		if (webLinks != null) {
			return webLinks;
		} else {
			return Collections.emptyList();
		}
	}

	public void setWebLinks(List<BehanceSDKWebLinkDTO> webLinks) {
		this.webLinks = webLinks;
	}

	public void addWebLink(BehanceSDKWebLinkDTO webLink) {
		if (webLinks == null) {
			webLinks = new ArrayList<BehanceSDKWebLinkDTO>();
		}
		webLinks.add(webLink);
	}

	public List<BehanceSDKUserDetailsSectionDTO> getDetailsSections() {
		if (detailsSections != null) {
			return detailsSections;
		} else {
			return Collections.emptyList();
		}
	}

	public void setDetailsSections(List<BehanceSDKUserDetailsSectionDTO> detailsSections) {
		this.detailsSections = detailsSections;
	}

	public void addDetailsSection(BehanceSDKUserDetailsSectionDTO section) {
		if (detailsSections == null) {
			detailsSections = new ArrayList<BehanceSDKUserDetailsSectionDTO>();
		}
		detailsSections.add(section);
	}

	public List<BehanceSDKUserWorkExpDTO> getWorkExperiences() {
		if (workExperiences != null) {
			return workExperiences;
		} else {
			return Collections.emptyList();
		}
	}

	public void setWorkExperiences(List<BehanceSDKUserWorkExpDTO> workExperiencesList) {
		this.workExperiences = workExperiencesList;
	}

	public void addWorkExperience(BehanceSDKUserWorkExpDTO workEx) {
		if (workExperiences == null) {
			workExperiences = new ArrayList<BehanceSDKUserWorkExpDTO>();
		}
		workExperiences.add(workEx);
	}

	public List<BehanceSDKCollectionDTO> getFollowedCollections() {
		if (followedCollections != null) {
			return followedCollections;
		} else {
			return Collections.emptyList();
		}
	}

	public void setFollowedCollections(List<BehanceSDKCollectionDTO> followedCollections) {
		this.followedCollections = followedCollections;
	}

    public boolean isEnterpriseUser() {
        return isEnterprise;
    }

    public void setUserAsEnterprise(boolean isEnterprise) {
        this.isEnterprise = isEnterprise;
    }
}