/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.search;


import com.behance.sdk.enums.BehanceSDKWIPRevisionImageType;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKWIPRevisionDTO implements Serializable {

	private static final long serialVersionUID = -5881125985196541707L;

	private int id;
	private String url;
	private String shortUrl;
	private int number;
	private long createdTime;
	private String description;
	private Map<String, BehanceSDKImageDTO> imagesMap;
	private List<BehanceSDKWIPRevisionCommentDTO> comments;
	private List<String> tags;

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getShortUrl() {
		return shortUrl;
	}

	public void setShortUrl(String shortUrl) {
		this.shortUrl = shortUrl;
	}

	public int getNumber() {
		return number;
	}

	public void setNumber(int number) {
		this.number = number;
	}

	public long getCreatedTime() {
		return createdTime;
	}

	public void setCreatedTime(long createdTime) {
		this.createdTime = createdTime;
	}

	public void setImage(BehanceSDKWIPRevisionImageType imageType, BehanceSDKImageDTO imageDTO) {
		if (imagesMap == null) {
			imagesMap = new HashMap<String, BehanceSDKImageDTO>();
		}
		imagesMap.put(imageType.name(), imageDTO);
	}

	private BehanceSDKImageDTO getImage(BehanceSDKWIPRevisionImageType imageType) {
		return imagesMap.get(imageType.name());
	}

	public BehanceSDKImageDTO findImageToDisplay(BehanceSDKWIPRevisionImageType imageType) {
		BehanceSDKImageDTO finalImageDTO = getImage(imageType);
		if (finalImageDTO == null) {
			if (imageType == BehanceSDKWIPRevisionImageType.THUMBNAIL_SM
					|| imageType == BehanceSDKWIPRevisionImageType.THUMBNAIL) {
				finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.THUMBNAIL_SM);
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.THUMBNAIL);
				}
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.NORMAL_RESOLUTION);
				}
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.HIGH_DEFINITION);
				}
			}

			if (imageType == BehanceSDKWIPRevisionImageType.HIGH_DEFINITION || imageType == BehanceSDKWIPRevisionImageType.NORMAL_RESOLUTION) {
				finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.HIGH_DEFINITION);
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.NORMAL_RESOLUTION);
				}
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.THUMBNAIL);
				}
				if (finalImageDTO == null) {
					finalImageDTO = getImage(BehanceSDKWIPRevisionImageType.THUMBNAIL_SM);
				}
			}
		}
		return finalImageDTO;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<BehanceSDKWIPRevisionCommentDTO> getComments() {
		if (comments != null) {
			return comments;
		} else {
			return Collections.emptyList();
		}
	}

	public void setComments(List<BehanceSDKWIPRevisionCommentDTO> comments) {
		this.comments = comments;
	}

	public List<String> getTags() {
		return tags;
	}

	public void setTags(List<String> tags) {
		this.tags = tags;
	}

	public void addTag(String tag) {
		if (tags == null) {
			tags = new ArrayList<String>();
		}
		tags.add(tag);
	}
}
