/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.enums;

import android.content.Context;

import com.behance.sdk.R;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ugs
 * 
 */
public enum BehanceSDKCopyrightOption {

	BY("by"), 
	BY_SA("by-sa"), 
	BY_ND("by-nd"), 
	BY_NC("by-nc"), 
	BY_NC_SA("by-nc-sa"), 
	BY_NC_ND("by-nc-nd"), 
	NO_USE("no-use");


	private final String value;

	BehanceSDKCopyrightOption(String value) {
		this.value = value;
	}

	public String getDescription(Context context) {
		int response = -1;
		if (value != null) {
			if (value.equalsIgnoreCase("no-use")) {
				response = R.string.bsdk_copyright_no_use_at_all;
			} else if (value.equalsIgnoreCase("by")) {
				response = R.string.bsdk_copyright_cc_by;
			} else if (value.equalsIgnoreCase("by-sa")) {
				response = R.string.bsdk_copyright_cc_by_sa;
			} else if (value.equalsIgnoreCase("by-nd")) {
				response = R.string.bsdk_copyright_cc_by_nd;
			} else if (value.equalsIgnoreCase("by-nc")) {
				response = R.string.bsdk_copyright_cc_by_nc;
			} else if (value.equalsIgnoreCase("by-nc-sa")) {
				response = R.string.bsdk_copyright_cc_by_nc_sa;
			} else if (value.equalsIgnoreCase("by-nc-nd")) {
				response = R.string.bsdk_copyright_cc_by_nc_nd;
			}
		}
		return context.getString(response);
	}

	public String getValue() {
		return value;
	}

	public String getCategory(Context context) {
		int response = -1;
		if (value.equalsIgnoreCase("no-use")) {
			response = R.string.bsdk_copyright_no_license_header;
		} else if (value.equalsIgnoreCase("by")
				|| value.equalsIgnoreCase("by-sa")
				|| value.equalsIgnoreCase("by-nc")
				|| value.equalsIgnoreCase("by-nd")
				|| value.equalsIgnoreCase("by-nc-sa")
				|| value.equalsIgnoreCase("by-nc-nd")) {
			response = R.string.bsdk_copyright_creative_commons_header;
		}
		return context.getString(response);
	}

	public static BehanceSDKCopyrightOption getCopyrightOptionFromValue(String copyrightValue) {
		BehanceSDKCopyrightOption[] values = BehanceSDKCopyrightOption.values();
		for (BehanceSDKCopyrightOption behanceCopyrightOption : values) {
			if (behanceCopyrightOption.getValue().equals(copyrightValue)) {
				return behanceCopyrightOption;
			}
		}
		return null;
	}

	public static BehanceSDKCopyrightOption getDefaultValue() {
		return BY_NC_ND;
	}

	public static List<BehanceSDKCopyrightOption> getCopyrightOptions(String category, Context context) {
		BehanceSDKCopyrightOption[] values = BehanceSDKCopyrightOption.values();
		List<BehanceSDKCopyrightOption> copyrights = new ArrayList<BehanceSDKCopyrightOption>();
		for (BehanceSDKCopyrightOption behanceCopyrightOption : values) {
			if (behanceCopyrightOption.getCategory(context).equals(category)) {
				copyrights.add(behanceCopyrightOption);
			}
		}
		return copyrights;
	}

	public static Map<String, List<BehanceSDKCopyrightOption>> getAllCopyrightOptions(Context context) {
		Map<String, List<BehanceSDKCopyrightOption>> all = new LinkedHashMap<String, List<BehanceSDKCopyrightOption>>();
		BehanceSDKCopyrightOption[] values = BehanceSDKCopyrightOption.values();
		for (BehanceSDKCopyrightOption behanceCopyrightOption : values) {
			String category = behanceCopyrightOption.getCategory(context);
			List<BehanceSDKCopyrightOption> list;
			if(all.containsKey(category)){
				list = all.get(category);
			}else{
				list = new ArrayList<BehanceSDKCopyrightOption>();
				all.put(category, list);
			}
			list.add(behanceCopyrightOption);
		}
		return all;
	}
}
