/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.files;

import android.content.Context;
import android.database.Cursor;
import android.provider.MediaStore;

import com.behance.sdk.dto.BehanceSDKAlbumDTO;
import com.behance.sdk.enums.BehanceSDKAlbumType;
import com.behance.sdk.project.modules.ImageModule;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ugs
 * 
 */
public class LocalImageLoader {

	private static final String ASC = " ASC";
	private static final String EMPTY_STRING = "";

	private static List<BehanceSDKAlbumDTO> albumsAndImageSourcesList;
    private static Object lockObject = new Object();
    private static volatile boolean cacheCleared = false;
    private static volatile boolean inProgress = false;
	public static List<BehanceSDKAlbumDTO> getAlbumsList(Context context, boolean forceRefresh, boolean showImageSelectionSources) {
		if (forceRefresh || albumsAndImageSourcesList == null || albumsAndImageSourcesList.isEmpty()) {
            synchronized (lockObject){
                inProgress = true;
                cacheCleared = false;
            }
            List<ImageAlbum> deviceAlbums = getListOfAlbums(context);
            synchronized (lockObject) {
                albumsAndImageSourcesList = new ArrayList<BehanceSDKAlbumDTO>();
                if(cacheCleared) {
                    cacheCleared = false;
                    return albumsAndImageSourcesList;
                }
                if (showImageSelectionSources) {
                    // add camera to albums list
                    BehanceSDKAlbumDTO cameraAlbumDTO = new BehanceSDKAlbumDTO(EMPTY_STRING, BehanceSDKAlbumType.CAMERA);
                    albumsAndImageSourcesList.add(cameraAlbumDTO);
                    BehanceSDKAlbumDTO ccAlbumDTO = new BehanceSDKAlbumDTO(EMPTY_STRING, BehanceSDKAlbumType.CREATIVE_CLOUD);
                    albumsAndImageSourcesList.add(ccAlbumDTO);
                }
                if (deviceAlbums != null && !deviceAlbums.isEmpty()) {
                    for (ImageAlbum imageAlbum : deviceAlbums) {
                        BehanceSDKAlbumDTO albumDTO = new BehanceSDKAlbumDTO(imageAlbum.getName(), BehanceSDKAlbumType.DEVICE_ALBUM);
                        albumDTO.setImages(imageAlbum.getImages());
                        albumsAndImageSourcesList.add(albumDTO);
                    }
                }
                inProgress = false;
            }
		}
		return albumsAndImageSourcesList;
	}

	public static BehanceSDKAlbumDTO getFromAlbumWithImageSelectionSources(Context context, int albumIndex) {
		List<BehanceSDKAlbumDTO> albumsList = getAlbumsList(context, false, false);
		if (albumsList != null && albumsList.size() > albumIndex) {
			return albumsList.get(albumIndex);
		} else {
			return null;
		}
	}

	public static void clearCachedAlbumAndImagesData() {
        synchronized (lockObject) {
            if(null != albumsAndImageSourcesList){
                for (BehanceSDKAlbumDTO albums : albumsAndImageSourcesList){
                    List<ImageModule> imageModules =  albums.getImages();
                    if (imageModules != null && imageModules.isEmpty() == false) {
                        for (ImageModule localImage : imageModules) {
                            localImage.recycleBitmaps();
                        }
                      imageModules = null;
                    }
                }
                albumsAndImageSourcesList.clear();
                albumsAndImageSourcesList = null;
            }
            if(inProgress)
                cacheCleared = true;
        }
	}

	private static List<ImageAlbum> getListOfAlbums(Context c) {

			List<ImageModule> listOfImages = getListOfImages(c);
			Map<String, ImageAlbum> imageAlbums = new HashMap<String, ImageAlbum>();
			for (ImageModule localImage : listOfImages) {
				String albumName = localImage.getAlbumName();
				ImageAlbum imageAlbum;
				if (imageAlbums.containsKey(albumName)) {
					imageAlbum = imageAlbums.get(albumName);
				} else {
					imageAlbum = new ImageAlbum(albumName);
					imageAlbums.put(albumName, imageAlbum);
				}
				imageAlbum.addImage(localImage);
			}
                   List<ImageAlbum> albumsList = new ArrayList<ImageAlbum>(imageAlbums.size());
                    albumsList.addAll(imageAlbums.values());
		return albumsList;
	}

	private  static List<ImageModule> getListOfImages(Context c) {
        List<ImageModule> imagesTempList = new ArrayList<ImageModule>();
			String[] projection = new String[] { MediaStore.Images.ImageColumns._ID, MediaStore.Images.ImageColumns.DATA,
					MediaStore.Images.ImageColumns.BUCKET_DISPLAY_NAME };
			Cursor[] cursors = new Cursor[1];
			cursors[0] = c.getContentResolver().query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, projection, EMPTY_STRING, (String[]) null,
					MediaStore.Images.ImageColumns.DATA + ASC);

            for (Cursor cursor : cursors) {
                while (cursor.moveToNext()) {
                    String id = cursor.getString(cursor.getColumnIndex(MediaStore.Images.ImageColumns._ID));
                    String file = cursor.getString(cursor.getColumnIndex(MediaStore.Images.ImageColumns.DATA));
                    String albumName = cursor.getString(cursor.getColumnIndex(MediaStore.Images.ImageColumns.BUCKET_DISPLAY_NAME));
                    LocalImage image = new LocalImage(Long.parseLong(id), file);
                    image.setAlbumName(albumName);
                    //Only add valid images
                    if (image.getWidth() > 0)
                        imagesTempList.add(image);
                }
            }
        return imagesTempList;
	}
}