package com.behance.sdk.fragments.headless;

import android.app.Activity;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.support.v4.app.Fragment;
import android.widget.Toast;

import com.adobe.creativesdk.foundation.IAdobeGenericErrorCallback;
import com.adobe.creativesdk.foundation.internal.storage.AdobePhotoAssetsDataSource;
import com.adobe.creativesdk.foundation.internal.storage.asset.AdobeAssetCompPage;
import com.adobe.creativesdk.foundation.internal.storage.asset.AdobeAssetDrawPage;
import com.adobe.creativesdk.foundation.internal.storage.asset.AdobeAssetPSMixPage;
import com.adobe.creativesdk.foundation.internal.storage.asset.AdobeAssetSketchbookPage;
import com.adobe.creativesdk.foundation.internal.utils.AdobeCSDKException;
import com.adobe.creativesdk.foundation.storage.AdobeAssetCompFile;
import com.adobe.creativesdk.foundation.storage.AdobeAssetDesignLibraryItemFilter;
import com.adobe.creativesdk.foundation.storage.AdobeAssetDesignLibraryItemFilterType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetDesignLibraryItemType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetDrawFile;
import com.adobe.creativesdk.foundation.storage.AdobeAssetException;
import com.adobe.creativesdk.foundation.storage.AdobeAssetFile;
import com.adobe.creativesdk.foundation.storage.AdobeAssetFileRenditionType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetImageDimensions;
import com.adobe.creativesdk.foundation.storage.AdobeAssetLibrary;
import com.adobe.creativesdk.foundation.storage.AdobeAssetLibraryItemImage;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMIMETypeFilter;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMIMETypeFilterType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMimeTypes;
import com.adobe.creativesdk.foundation.storage.AdobeAssetPSMixFile;
import com.adobe.creativesdk.foundation.storage.AdobeAssetPackagePages;
import com.adobe.creativesdk.foundation.storage.AdobeAssetSketchbook;
import com.adobe.creativesdk.foundation.storage.AdobePhotoAsset;
import com.adobe.creativesdk.foundation.storage.AdobePhotoException;
import com.adobe.creativesdk.foundation.storage.AdobeSelection;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionAssetFile;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionCompFile;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionDrawAsset;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionLibraryAsset;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionPSMixFile;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionPhotoAsset;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionSketchAsset;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowser;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserConfiguration;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserOption;
import com.adobe.creativesdk.foundation.storage.IAdobeGenericRequestCallback;
import com.adobe.creativesdk.foundation.storage.IAdobeRequestCompletionCallback;
import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageProperties;
import com.behance.sdk.factory.BehanceSDKImageValidatorFactory;
import com.behance.sdk.factory.IBehanceSDKImageValidator;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.activities.BehanceSDKCCLauncherActivity;
import com.behance.sdk.util.BehanceSDKUtils;

import junit.framework.TestCase;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

/**
 * Created by ugs on 29/06/15.
 */
public class BehanceSDKCCBrowserHeadlessFragment extends Fragment {

    public static final String HEIGHT = "height";
    public static final String WIDTH = "width";

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKCCBrowserHeadlessFragment.class);

    private static final int BEHANCE_SDK_ASSET_BROWSER_LAUNCH_REQUEST_CODE = 9786;

    private static final int MAX_WIDTH_HEIGHT = 2560;
    public static final String IMAGE_GIF = "image/gif";
    public static final String IMAGE_PNG = "image/png";
    public static final String IMAGE_JPEG = "image/jpeg";



    private ArrayList<AdobeSelection> mSelectedAssetsList;

    private List<File> files;

    private List<File> downloadErrors;

    private List<File> invalidFile;

    private File tempStorageFolder;

    private String imageValidatorType;

    private ICreativeCloudAssetDownloadListener listener;



    private volatile boolean isDownloadInProgress = false;

    public boolean isDownloadInProgress() {
        return isDownloadInProgress;
    }

    public interface ICreativeCloudAssetDownloadListener {
        void onDownloadComplete(List<File> downloadedFiles, List<File> downloadError, List<File> invalidFiles);

        void onDownloadError(AdobeAssetException error);
    }

    public void setDownloadListener(ICreativeCloudAssetDownloadListener listener) {
        this.listener = listener;
    }

    public void removeDownloadListener() {
        this.listener = null;
    }

    public BehanceSDKCCBrowserHeadlessFragment() {
        setRetainInstance(true);
    }

    public void launchCCAssetBrowser(Activity activity, Intent intent) throws AdobeCSDKException {



        imageValidatorType = intent.getStringExtra(BehanceSDKCCLauncherActivity.ARGS_IMAGE_VALIDATOR_TYPE);


        AdobeUXAssetBrowserConfiguration browserConfiguration = new AdobeUXAssetBrowserConfiguration();
        Serializable assetBrowserOptions = intent.getSerializableExtra(BehanceSDKCCLauncherActivity.ARGS_ASSET_BROWSER_OPTIONS);
        if (null != assetBrowserOptions)
            browserConfiguration.options = (EnumSet<AdobeUXAssetBrowserOption>) assetBrowserOptions;
        browserConfiguration.designLibraryItemFilter = AdobeAssetDesignLibraryItemFilter.createFromDesignLibraryItems(EnumSet.of(AdobeAssetDesignLibraryItemType.AdobeAssetDesignLibraryItemImages) ,
                AdobeAssetDesignLibraryItemFilterType.ADOBE_ASSET_DESIGNLIBRARYITEM_FILTER_INCLUSION);
                Serializable allowedMimeTypes = intent.getSerializableExtra(BehanceSDKCCLauncherActivity.ARGS_ALLOWED_MIME_TYPES);
        if (null != allowedMimeTypes) {
            browserConfiguration.mimeTypeFilter = AdobeAssetMIMETypeFilter.createFromMimeTypes((EnumSet<AdobeAssetMimeTypes>) allowedMimeTypes,
                    AdobeAssetMIMETypeFilterType.ADOBE_ASSET_MIMETYPE_FILTERTYPE_INCLUSION);
        }
   

        AdobeUXAssetBrowser sharedAssetBrowserInstance = AdobeUXAssetBrowser.getSharedInstance();
        sharedAssetBrowserInstance.popupFileBrowser(activity, BEHANCE_SDK_ASSET_BROWSER_LAUNCH_REQUEST_CODE, browserConfiguration);

    }


    public synchronized void cancelDownload() {
        if (null != mSelectedAssetsList && false == mSelectedAssetsList.isEmpty()) {
            for (AdobeSelection selection : mSelectedAssetsList) {
                if (selection instanceof AdobeSelectionAssetFile) {
                    AdobeAssetFile aFile = ((AdobeSelectionAssetFile) selection).getSelectedItem();
                    aFile.cancelDataRequest();
                    BehanceSDKImageProperties properties = getImageProperties(aFile);
                    if (properties.getHeight() > MAX_WIDTH_HEIGHT || properties.getWidth() > MAX_WIDTH_HEIGHT) {
                        aFile.cancelRenditionRequest();
                    } else {
                        aFile.cancelDataRequest();
                    }
                }
            }
            if (null != files && !files.isEmpty()) {
                for (File file : files) {
                    if (file.exists()) {
                        file.delete();
                    }
                }
            }
            BehanceSDKUtils.deleteRecursive(tempStorageFolder);
            mSelectedAssetsList = null;
            files = null;
            downloadErrors = null;
            invalidFile = null;
        }
        isDownloadInProgress = false;
    }

    private String getUpdatedName(String name) {
        String type = BehanceSDKUtils.getFileExtension(name);
        // return name;
        if (type.equals(""))
            return (name + ".png");
        else
            return name;
    }

    public synchronized void downloadSelectedFiles(Intent data) {
        isDownloadInProgress = true;
        AdobeUXAssetBrowser.ResultProvider mAssetBrowserResult = new AdobeUXAssetBrowser.ResultProvider(data);
        mSelectedAssetsList = mAssetBrowserResult.getSelectionAssetArray();
        if (null != mSelectedAssetsList && !mSelectedAssetsList.isEmpty()) {
            final int totalSize = mSelectedAssetsList.size();
            files = new ArrayList<File>();
            downloadErrors = new ArrayList<File>();
            invalidFile = new ArrayList<File>();
            tempStorageFolder = getStorageDirectoryForDownload();

            for (AdobeSelection selection : mSelectedAssetsList) {
                if (selection instanceof AdobeSelectionAssetFile) {
                    processImage(totalSize, (AdobeSelectionAssetFile) selection);
                } else if (selection instanceof AdobeSelectionPhotoAsset) {
                    processPhoto(totalSize, (AdobeSelectionPhotoAsset) selection);
                } else if (selection instanceof AdobeSelectionLibraryAsset) {
                    processLibraryItem(totalSize, (AdobeSelectionLibraryAsset) selection);
                } else if (selection instanceof AdobeSelectionDrawAsset) {
                    processDraw(totalSize, (AdobeSelectionDrawAsset) selection);
                }  else if (selection instanceof AdobeSelectionSketchAsset) {
                    processSketch(totalSize, (AdobeSelectionSketchAsset) selection);
                } else if (selection instanceof AdobeSelectionCompFile) {
                    processComp(totalSize, (AdobeSelectionCompFile) selection);
                } else if (selection instanceof AdobeSelectionPSMixFile) {
                    processMix(totalSize, (AdobeSelectionPSMixFile) selection);
                }
            }
        } else {
            isDownloadInProgress = false;
        }

    }


    private void processLibraryItem(final int totalSize, AdobeSelectionLibraryAsset selection) {
        AdobeAssetFile file = null;
        if (selection.getSelectedImageIDs() != null) {
            final AdobeAssetLibrary library = (AdobeAssetLibrary) selection.getSelectedItem();
            AdobeAssetLibraryItemImage imageItem = library.getImages().get(selection.getSelectedImageIDs().get(0));
            final File targetFile = new File(tempStorageFolder, getUpdatedName(imageItem.getName()));

            if (imageItem.getPrimaryComponentType().equals("image/vnd.adobe.shape+svg")) {
                file = imageItem.getRendition();
            } else {
                file = imageItem.getImage();
            }
            if (file != null) {
                downloadRendition(totalSize, file, targetFile);
            }
        }
    }


    private void processDraw(final int totalSize, AdobeSelectionDrawAsset selection) {
        final int index = selection.getSelectedPageIndex();
        final AdobeAssetDrawFile drawFile = (AdobeAssetDrawFile) selection.getSelectedItem();
        final File imageFile = new File(tempStorageFolder, getUpdatedName(drawFile.getName()));

        drawFile.loadMetadata(new IAdobeRequestCompletionCallback() {
            @Override
            public void onCompletion() {
                TestCase.assertTrue(drawFile.getPages().size() > 0);
                AdobeAssetDrawPage assetDrawPage = null;
                assetDrawPage = (AdobeAssetDrawPage) drawFile.getPages().get(index);
                TestCase.assertEquals(index, assetDrawPage.pageNumber);
                AdobeAssetPackagePages packagePages = assetDrawPage._package;
                TestCase.assertEquals(drawFile.getPages(), packagePages.getPages());
                assetDrawPage.getRenditionWithType(AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_PNG, new AdobeAssetImageDimensions(MAX_WIDTH_HEIGHT, MAX_WIDTH_HEIGHT), new IAdobeGenericRequestCallback<byte[], AdobeAssetException>() {
                    @Override
                    public void onCancellation() {
                    }

                    @Override
                    public void onCompletion(byte[] data) {
                        saveDatatoFile(data, imageFile);
                        IBehanceSDKImageValidator imageValidator = getImageValidator();
                        if (null != imageValidator) {
                            BehanceSDKImageProperties properties = getImageProperties(imageFile);
                            if (imageValidator.validate(getActivity(), properties)) {
                                files.add(imageFile);
                            } else {
                                invalidFile.add(imageFile);
                            }
                        } else {
                            files.add(imageFile);
                        }
                        checkForDownloadComplete(totalSize);
                    }

                    @Override
                    public void onError(AdobeAssetException error) {
                        downloadErrors.add(imageFile);
                        showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, imageFile.getName()));
                        if (listener != null)
                            listener.onDownloadError(error);
                    }

                    @Override
                    public void onProgress(double progress) {

                    }
                });

            }
        }, new IAdobeGenericErrorCallback<AdobeCSDKException>() {
            @Override
            public void onError(AdobeCSDKException error) {
                TestCase.fail("loadMetadata Error Block :" + error.getDescription());
            }
        });
    }



    private void processComp(final int totalSize, AdobeSelectionCompFile selection) {
        final int index = selection.getSelectedPageIndex();
        final AdobeAssetCompFile compFile = (AdobeAssetCompFile) selection.getSelectedItem();
        final File imageFile = new File(tempStorageFolder, getUpdatedName(compFile.getName()));

        compFile.loadMetadata(new IAdobeRequestCompletionCallback() {
            @Override
            public void onCompletion() {
                TestCase.assertTrue(compFile.getPages().size() > 0);
                AdobeAssetCompPage assetCompPage = null;
                assetCompPage = (AdobeAssetCompPage) compFile.getPages().get(index);
                assetCompPage.getRenditionWithType(AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_PNG, new AdobeAssetImageDimensions(MAX_WIDTH_HEIGHT, MAX_WIDTH_HEIGHT), new IAdobeGenericRequestCallback<byte[], AdobeAssetException>() {
                    @Override
                    public void onCancellation() {

                    }

                    @Override
                    public void onCompletion(byte[] data) {
                        saveDatatoFile(data, imageFile);
                        IBehanceSDKImageValidator imageValidator = getImageValidator();
                        if (null != imageValidator) {
                            BehanceSDKImageProperties properties = getImageProperties(imageFile);
                            if (imageValidator.validate(getActivity(), properties)) {
                                files.add(imageFile);
                            } else {
                                invalidFile.add(imageFile);
                            }
                        } else {
                            files.add(imageFile);
                        }
                        checkForDownloadComplete(totalSize);

                    }

                    @Override
                    public void onError(AdobeAssetException error) {
                        downloadErrors.add(imageFile);
                        showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, imageFile.getName()));
                        if (listener != null)
                            listener.onDownloadError(error);
                    }

                    @Override
                    public void onProgress(double progress) {

                    }
                });
                //}

            }
        }, new IAdobeGenericErrorCallback<AdobeCSDKException>() {
            @Override
            public void onError(AdobeCSDKException error) {
                TestCase.fail("loadMetadata Error Block :" + error.getDescription());
            }
        });

    }

    private void processMix(final int totalSize, AdobeSelectionPSMixFile selection) {
        final int index = selection.getSelectedPageIndex();
        final AdobeAssetPSMixFile mixFile = (AdobeAssetPSMixFile) selection.getSelectedItem();
        final File imageFile = new File(tempStorageFolder, getUpdatedName(mixFile.getName()));
        mixFile.loadMetadata(new IAdobeRequestCompletionCallback() {
            @Override
            public void onCompletion() {
                TestCase.assertTrue(mixFile.getPages().size() > 0);
                AdobeAssetPSMixPage mixPage = null;
                mixPage = (AdobeAssetPSMixPage) mixFile.getPages().get(index);
                mixPage.getRenditionWithType(AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_PNG, new AdobeAssetImageDimensions(MAX_WIDTH_HEIGHT, MAX_WIDTH_HEIGHT), new IAdobeGenericRequestCallback<byte[], AdobeAssetException>() {
                    @Override
                    public void onCancellation() {

                    }

                    @Override
                    public void onCompletion(byte[] data) {
                        saveDatatoFile(data, imageFile);
                        IBehanceSDKImageValidator imageValidator = getImageValidator();
                        if (null != imageValidator) {
                            BehanceSDKImageProperties properties = getImageProperties(imageFile);
                            if (imageValidator.validate(getActivity(), properties)) {
                                files.add(imageFile);
                            } else {
                                invalidFile.add(imageFile);
                            }
                        } else {
                            files.add(imageFile);
                        }
                        checkForDownloadComplete(totalSize);
                    }

                    @Override
                    public void onError(AdobeAssetException error) {
                        downloadErrors.add(imageFile);
                        showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, imageFile.getName()));
                        if (listener != null)
                            listener.onDownloadError(error);
                    }

                    @Override
                    public void onProgress(double progress) {

                    }
                });
                //}

            }
        }, new IAdobeGenericErrorCallback<AdobeCSDKException>() {
            @Override
            public void onError(AdobeCSDKException error) {
                TestCase.fail("loadMetadata Error Block :" + error.getDescription());
            }
        });
    }


    private void processSketch(final int totalSize, AdobeSelectionSketchAsset selection) {
        final int index = selection.getSelectedPageIndex();
        final AdobeAssetSketchbook sketchBook = (AdobeAssetSketchbook) selection.getSelectedItem();
        final File imageFile = new File(tempStorageFolder, getUpdatedName(sketchBook.getName()));


        sketchBook.loadMetadata(new IAdobeRequestCompletionCallback() {
            @Override
            public void onCompletion() {
                TestCase.assertTrue(sketchBook.getPages().size() > 0);
                AdobeAssetSketchbookPage assetSketchbookPage = null;
                assetSketchbookPage = (AdobeAssetSketchbookPage) sketchBook.getPages().get(index);
                TestCase.assertEquals(index, assetSketchbookPage.pageNumber);
                AdobeAssetPackagePages packagePages = assetSketchbookPage._package;
                TestCase.assertEquals(sketchBook.getPages(), packagePages.getPages());
                assetSketchbookPage.getRenditionWithType(AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_PNG, new AdobeAssetImageDimensions(MAX_WIDTH_HEIGHT, MAX_WIDTH_HEIGHT), new IAdobeGenericRequestCallback<byte[], AdobeAssetException>() {
                    @Override
                    public void onCancellation() {

                    }

                    @Override
                    public void onCompletion(byte[] data) {
                        saveDatatoFile(data, imageFile);
                        IBehanceSDKImageValidator imageValidator = getImageValidator();
                        if (null != imageValidator) {
                            BehanceSDKImageProperties properties = getImageProperties(imageFile);
                            if (imageValidator.validate(getActivity(), properties)) {
                                files.add(imageFile);
                            } else {
                                invalidFile.add(imageFile);
                            }
                        } else {
                            files.add(imageFile);
                        }
                        checkForDownloadComplete(totalSize);
                    }

                    @Override
                    public void onError(AdobeAssetException error) {
                        downloadErrors.add(imageFile);
                        showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, imageFile.getName()));
                        if (listener != null)
                            listener.onDownloadError(error);
                    }

                    @Override
                    public void onProgress(double progress) {

                    }
                });
                //}

            }
        }, new IAdobeGenericErrorCallback<AdobeCSDKException>() {
            @Override
            public void onError(AdobeCSDKException error) {
                TestCase.fail("loadMetadata Error Block :" + error.getDescription());
            }
        });
    }


    private void processPhoto(int totalSize, AdobeSelectionPhotoAsset selection) {
        IBehanceSDKImageValidator imageValidator = getImageValidator();
        final AdobePhotoAsset photoAsset = selection.getSelectedItem();
        final File imageFile = new File(tempStorageFolder, getUpdatedName(photoAsset.getName()));
        BehanceSDKImageProperties properties = getImageProperties(photoAsset);
        if (imageValidator == null || imageValidator.validate(getActivity(), properties)) {
            downloadRendition(totalSize, photoAsset, imageFile);
        } else {
            invalidFile.add(imageFile);
        }

    }

    private void downloadRendition(final int totalSize, AdobePhotoAsset photoAsset, final File targetFile) {
        IAdobeGenericRequestCallback<byte[], AdobePhotoException> renditionCallback = new IAdobeGenericRequestCallback<byte[], AdobePhotoException>() {
            @Override
            public void onProgress(double progress) {
            }

            @Override
            public void onError(AdobePhotoException e) {
                downloadErrors.add(targetFile);
                showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, targetFile.getName()));
                if(totalSize == (downloadErrors.size()+invalidFile.size())) {
                    if (null != listener)
                        listener.onDownloadError(null);
                }else
                    checkForDownloadComplete(totalSize);
            }

            @Override
            public void onCompletion(byte[] data) {
                saveDatatoFile(data, targetFile);
                IBehanceSDKImageValidator imageValidator = getImageValidator();
                if (null != imageValidator) {
                    BehanceSDKImageProperties properties = getImageProperties(targetFile);
                    if (imageValidator.validate(getActivity(), properties)) {
                        files.add(targetFile);
                    } else {
                        invalidFile.add(targetFile);
                    }
                } else {
                    files.add(targetFile);
                }
                checkForDownloadComplete(totalSize);
            }

            @Override
            public void onCancellation() {
            }
        };
        AdobePhotoAssetsDataSource.getRenditionForAsset(photoAsset, renditionCallback);
    }

    private void processImage(int totalSize, AdobeSelectionAssetFile selection) {

        IBehanceSDKImageValidator imageValidator = getImageValidator();
        final AdobeAssetFile aFile = selection.getSelectedItem();
        BehanceSDKImageProperties properties = getImageProperties(aFile);
        final File imageFile = new File(tempStorageFolder, getUpdatedName(aFile.getName()));
        if (properties.getHeight() > MAX_WIDTH_HEIGHT || properties.getWidth() > MAX_WIDTH_HEIGHT) {
            downloadRendition(totalSize, aFile, imageFile);
        } else if (null == imageValidator || imageValidator.validate(getActivity(), properties)) {
            downloadOriginalFile(totalSize, aFile, imageFile);
        } else {
            invalidFile.add(imageFile);
        }

    }


    private void downloadRendition(final int totalSize, AdobeAssetFile aFile, final File targetFile) {

        AdobeAssetImageDimensions dimension = new AdobeAssetImageDimensions(MAX_WIDTH_HEIGHT, MAX_WIDTH_HEIGHT);
        AdobeAssetFileRenditionType renditionType = getRenditionType(aFile.getType());
        IAdobeGenericRequestCallback<byte[], AdobeAssetException> genericRequestCallback = new IAdobeGenericRequestCallback<byte[], AdobeAssetException>() {
            @Override
            public void onProgress(double progress) {

            }

            @Override
            public void onError(AdobeAssetException o) {
                downloadErrors.add(targetFile);
                showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, targetFile.getName()));
                if(totalSize == (downloadErrors.size()+invalidFile.size())) {
                    if (null != listener)
                        listener.onDownloadError(o);
                }else
                    checkForDownloadComplete(totalSize);


            }

            @Override
            public void onCompletion(byte[] data) {
                saveDatatoFile(data, targetFile);
                IBehanceSDKImageValidator imageValidator = getImageValidator();
                if (null != imageValidator) {
                    BehanceSDKImageProperties properties = getImageProperties(targetFile);
                    if (imageValidator.validate(getActivity(), properties)) {
                        files.add(targetFile);
                    } else {
                        invalidFile.add(targetFile);
                    }
                } else {
                    files.add(targetFile);
                }
                checkForDownloadComplete(totalSize);

            }

            @Override
            public void onCancellation() {

            }
        };

        aFile.getRenditionWithType(renditionType, dimension, genericRequestCallback);

    }

    private void downloadOriginalFile(final int totalSize, final AdobeAssetFile aFile, final File targetFile) {

        IAdobeGenericRequestCallback<Boolean, AdobeAssetException> genericRequestCallback = new IAdobeGenericRequestCallback<Boolean, AdobeAssetException>() {
            @Override
            public void onCancellation() {
                if (targetFile.exists()) {
                    targetFile.delete();
                }
            }

            @Override
            public void onCompletion(Boolean aBoolean) {

                files.add(targetFile);
                checkForDownloadComplete(totalSize);
            }

            @Override
            public void onError(AdobeAssetException e) {
                downloadErrors.add(targetFile);
                showMessage(getString(R.string.bsdk_cc_asset_browser_file_download_error, targetFile.getName()));
                if(totalSize == (downloadErrors.size()+invalidFile.size())) {
                    if (null != listener)
                       listener.onDownloadError(e);
                }else
                   checkForDownloadComplete(totalSize);

            }

            @Override
            public void onProgress(double progress) {

            }
        };

        aFile.downloadAssetFile(targetFile.toURI(), genericRequestCallback);

    }


    private void checkForDownloadComplete(int totalSize) {

        int total = files.size() + downloadErrors.size() + invalidFile.size();
        if (total == totalSize) {
            files = moveFileToPicturesDirectory();
            if (null != listener ) {
                listener.onDownloadComplete(files, downloadErrors, invalidFile);
            }
            isDownloadInProgress = false;
        }

    }

    private List<File> moveFileToPicturesDirectory() {

        List<File> newFiles = new ArrayList<File>();
        File storageDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES);
        for (File file : files) {
            File imageFile = new File(storageDir, file.getName());
            if (false == imageFile.exists())
                file.renameTo(imageFile);
            newFiles.add(imageFile);
        }
        BehanceSDKUtils.deleteRecursive(tempStorageFolder);
        return newFiles;

    }


    private IBehanceSDKImageValidator getImageValidator() {

        return BehanceSDKImageValidatorFactory.getInstance().getImageValidator(imageValidatorType);

    }

    private void computeImageDimensions(File file, BehanceSDKImageProperties properties) {

        if (file != null && file.exists() && file.isFile()) {
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeFile(file.getAbsolutePath(), options);
            properties.setWidth(options.outWidth);
            properties.setHeight(options.outHeight);
            properties.setSize(file.length());
        }

    }

    private BehanceSDKImageProperties getImageProperties(File file) {

        BehanceSDKImageProperties properties = new BehanceSDKImageProperties();
        String type = BehanceSDKUtils.getFileExtension(file.getName());
        if ("".equals(type))
            properties.setType("png");
        else
            properties.setType(type);
        computeImageDimensions(file, properties);
        return properties;

    }

    private BehanceSDKImageProperties getImageProperties(AdobePhotoAsset adobePhotoAsset) {
        BehanceSDKImageProperties properties = new BehanceSDKImageProperties();
        properties.setSize(adobePhotoAsset.getSize());
        String type = BehanceSDKUtils.getFileExtension(adobePhotoAsset.getName());
        if ("".equals(type))
            properties.setType("png");
        else
            properties.setType(type);
        JSONObject metaData = adobePhotoAsset.getMetadata();
        if (metaData != null)
            if (metaData.has(HEIGHT) && metaData.has(WIDTH)) {
                try {
                    int height = metaData.getInt(HEIGHT);
                    int width = metaData.getInt(WIDTH);
                    properties.setHeight(height);
                    properties.setWidth(width);
                } catch (JSONException e) {
                    logger.error("Exception parsing image metadata of CC photo asset " + adobePhotoAsset.getName(), e);
                }
            }
        return properties;

    }


    private BehanceSDKImageProperties getImageProperties(AdobeAssetFile adobeAssetFile) {

        BehanceSDKImageProperties properties = new BehanceSDKImageProperties();
        properties.setSize(adobeAssetFile.getFileSize());
        String type = BehanceSDKUtils.getFileExtension(adobeAssetFile.getName());
        if (type.equals(""))
            properties.setType("png");
        else
            properties.setType(type);
        JSONObject optionalMetaData = adobeAssetFile.getOptionalMetadata();
        if (optionalMetaData != null) {
            if (optionalMetaData.has(HEIGHT) && optionalMetaData.has("width")) {
                try {
                    int height = optionalMetaData.getInt(HEIGHT);
                    int width = optionalMetaData.getInt("width");
                    properties.setHeight(height);
                    properties.setWidth(width);
                } catch (JSONException e) {
                    logger.error("Exception parsing image metadata of CC asset file " + adobeAssetFile.getName(), e);
                }
            }
        }
        return properties;

    }

    private AdobeAssetFileRenditionType getRenditionType(String fileType) {

        if (IMAGE_GIF.equals(fileType))
            return AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_GIF;
        else if (IMAGE_PNG.equals(fileType))
            return AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_PNG;
        else if (IMAGE_JPEG.equals(fileType))
            return AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_JPEG;
        else
            return AdobeAssetFileRenditionType.ADOBE_ASSET_FILE_RENDITION_TYPE_JPEG;

    }


    private void saveDatatoFile(byte[] data, File imageFile) {
        try {
            FileOutputStream outputStream = new FileOutputStream(imageFile);
            outputStream.write(data);
            outputStream.close();
        } catch (Exception ex) {
            String errorMessage = getString(R.string.bsdk_cc_asset_browser_file_download_error, imageFile.getName());
            logger.error(errorMessage, ex);
            downloadErrors.add(imageFile);
            showMessage(errorMessage);
            if(listener != null)
                listener.onDownloadError(null);
        }
    }

    private File getStorageDirectoryForDownload() {
        File temp = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES) + "/CC" + System.currentTimeMillis());
        if (temp.exists()) {
            BehanceSDKUtils.deleteRecursive(temp);
        }
        temp.mkdir();
        return temp;
    }


    private void showMessage(String message) {
        Toast.makeText(getActivity(), message, Toast.LENGTH_LONG).show();
    }

}

