package com.behance.sdk.fragments.headless;

import android.graphics.Bitmap;
import android.support.v4.app.Fragment;

import com.adobe.creativesdk.foundation.adobeinternal.auth.AdobeUXAuthManagerRestricted;
import com.adobe.creativesdk.foundation.auth.AdobeAuthUserProfile;
import com.behance.sdk.BehanceSDKUserProfile;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.asynctask.listeners.IBehanceSDKEditProfileAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserProfileAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKEditProfileAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserProfileTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKEditProfileAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetUserProfileAsyncTask;
import com.behance.sdk.asynctasks.result.BehanceSDKEditProfileTaskResult;
import com.behance.sdk.asynctasks.result.BehanceSDKGetUserProfileTaskResult;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.managers.BehanceSDKEditProfileManager;
import com.behance.sdk.project.modules.ImageModule;

/**
 * Created by forta on 9/15/14.
 */
public class BehanceSDKEditProfileHeadlessFragment extends Fragment implements IBehanceSDKGetUserProfileAsyncTaskListener, IBehanceSDKEditProfileAsyncTaskListener {

    private boolean getUserProfileTaskInProgress = false;
    private BehanceSDKGetUserProfileAsyncTask getUserProfileAsyncTask;
    private boolean editProfileTaskInProgress = false;
    private BehanceSDKEditProfileAsyncTask editProfileAsyncTask;
    private Callbacks callbacks;

    private String firstName;
    private String lastName;
    private String occupation;
    private String company;
    private String website;
    private BehanceSDKCountryDTO country;
    private BehanceSDKStateDTO state;
    private BehanceSDKCityDTO city;
    private String originalImageUrl;
    private ImageModule imageModule;
    private Bitmap imageBitmap;

    @Override
    public void onGetUserProfileSuccess(BehanceSDKGetUserProfileTaskResult result, BehanceSDKGetUserProfileTaskParams params) {
        BehanceSDKUserProfile userProfile = result.getBehanceUserProfile();
        if(userProfile.getFirstName() == null || userProfile.getFirstName().isEmpty() || userProfile.getLastName() == null || userProfile.getLastName().isEmpty()) {
            AdobeAuthUserProfile adobeUserProfile = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted().getUserProfile();
            setFirstName(adobeUserProfile.getFirstName());
            setLastName(adobeUserProfile.getLastName());
            if(callbacks != null)
                callbacks.setProfileEdited(true);
        } else {
            setFirstName(userProfile.getFirstName());
            setLastName(userProfile.getLastName());
        }
        setOccupation(userProfile.getOccupation());
        setCompany(userProfile.getCompany());
        setWebsite(userProfile.getWebsite());
        setCountry(userProfile.getCountry());
        setState(userProfile.getState());
        setCity(userProfile.getCity());
        setGetUserProfileTaskInProgress(false);
        setOriginalImageUrl(userProfile.getProfileImageUrl());
        callbacks.onGetUserProfileSuccess();
    }

    @Override
    public void onGetUserProfileFailure(Exception e, BehanceSDKGetUserProfileTaskParams params) {
        setGetUserProfileTaskInProgress(false);
        callbacks.onGetUserProfileFailure(e, params);
    }

    @Override
    public void onEditProfileTaskSuccess(BehanceSDKEditProfileTaskResult result) {
        setEditProfileTaskInProgress(false);
        callbacks.onEditProfileSuccess(result);
    }

    @Override
    public void onEditProfileTaskFailure(BehanceSDKEditProfileTaskResult result) {
        setEditProfileTaskInProgress(false);
        callbacks.onEditProfileFailure(result);
    }

    public String getOriginalImageUrl() {
        return originalImageUrl;
    }

    public void setOriginalImageUrl(String originalImageUrl) {
        this.originalImageUrl = originalImageUrl;
    }

    public void setImageBitmap(Bitmap imageBitmap) {
        this.imageBitmap = imageBitmap;
    }

    public Bitmap getImageBitmap() {
        return imageBitmap;
    }

    public interface Callbacks {
        public void onGetUserProfileSuccess();
        public void onGetUserProfileFailure(Exception e, BehanceSDKGetUserProfileTaskParams params);
        public void onEditProfileSuccess(BehanceSDKEditProfileTaskResult result);
        public void onEditProfileFailure(BehanceSDKEditProfileTaskResult result);
        public void setProfileEdited(boolean edited);
    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    public BehanceSDKEditProfileHeadlessFragment() {
        setRetainInstance(true);
    }

    public boolean isGetUserProfileTaskInProgress() {
        return getUserProfileTaskInProgress;
    }

    public void setGetUserProfileTaskInProgress(boolean getUserProfileTaskInProgress) {
        this.getUserProfileTaskInProgress = getUserProfileTaskInProgress;
    }

    public boolean isEditProfileTaskInProgress() {
        return editProfileTaskInProgress;
    }

    public void setEditProfileTaskInProgress(boolean editProfileTaskInProgress) {
        this.editProfileTaskInProgress = editProfileTaskInProgress;
    }

    public void saveUserPortfolioChanges() {
        if(!editProfileTaskInProgress && editProfileAsyncTask == null) {
            setEditProfileTaskInProgress(true);
            editProfileAsyncTask = new BehanceSDKEditProfileAsyncTask(this);
            BehanceSDKEditProfileAsyncTaskParams params = applyParamChanges();

            BehanceSDKEditProfileManager manager = BehanceSDKEditProfileManager.getInstance();
            IBehanceSDKUserCredentials userCredentials = manager.getUserCredentials();
            params.setClientId(userCredentials.getClientId());

            editProfileAsyncTask.execute(params);
        }
    }

    private BehanceSDKEditProfileAsyncTaskParams applyParamChanges() {
        BehanceSDKEditProfileAsyncTaskParams params = new BehanceSDKEditProfileAsyncTaskParams();
        if(firstName != null)
            params.setFirstName(firstName);
        if(lastName != null)
            params.setLastName(lastName);
        if(occupation != null)
            params.setOccupation(occupation);
        if(company != null)
            params.setCompany(company);
        if(website != null)
            params.setWebsite(website);
        if(country != null)
            params.setCountry(country.getDisplayName());
        if(state != null)
            params.setState(state.getDisplayName());
        if(city != null)
            params.setCity(city.getDisplayName());
        if(imageModule != null)
            params.setImage(imageModule);
        if(imageBitmap != null)
            params.setProfileImageBitmap(imageBitmap);
        return params;
    }

    public void loadUserDetailsFromServer() {
        if (!isGetUserProfileTaskInProgress() && getUserProfileAsyncTask == null) {
            setGetUserProfileTaskInProgress(true);

            BehanceSDKGetUserProfileTaskParams params = new BehanceSDKGetUserProfileTaskParams();
            BehanceSDKEditProfileManager manager = BehanceSDKEditProfileManager.getInstance();
            IBehanceSDKUserCredentials userCredentials = manager.getUserCredentials();
            if(userCredentials != null){
                params.setUserId(userCredentials.getUserAdobeAccountId());
                params.setClientId(userCredentials.getClientId());
            }
            else{
                params.setUserId("");
                params.setClientId("");
            }

            getUserProfileAsyncTask = new BehanceSDKGetUserProfileAsyncTask(this);
            getUserProfileAsyncTask.execute(params);
        }
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setOccupation(String occupation) {
        this.occupation = occupation;
    }

    public String getOccupation() {
        return occupation;
    }

    public void setCompany(String company) {
        this.company = company;
    }

    public String getCompany() {
        return company;
    }

    public void setWebsite(String website) {
        this.website = website;
    }

    public String getWebsite() {
        return website;
    }

    public void setCountry(BehanceSDKCountryDTO country) {
        this.country = country;
    }

    public BehanceSDKCountryDTO getCountry() {
        return country;
    }

    public void setState(BehanceSDKStateDTO state) {
        this.state = state;
    }

    public BehanceSDKStateDTO getState() {
        return state;
    }

    public void setCity(BehanceSDKCityDTO city) {
        this.city = city;
    }

    public BehanceSDKCityDTO getCity() {
        return city;
    }

    public void setImage(ImageModule image) {
        this.imageModule = image;
    }

    public ImageModule getImage() {
        return imageModule;
    }

    public boolean isValidLocation(){
        if(null == country || BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID.equals(country.getId())){
            return false;
        }
        if( (BehanceSDKCountryDTO.COUNTRY_CODES_FOR_LOAD_STATES.contains(country.getId()) && null == state) ||
                null != state && BehanceSDKStateDTO.ALL_STATES_ID.equals(state.getId()) ){
            return false;
        }
        if( null == city  || BehanceSDKCityDTO.ALL_CITIES_ID.equals(city.getId())){
            return false;
        }
        return true;
    }
}
