/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.fragments.headless;

import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;

import com.behance.sdk.asynctask.listeners.IBehanceSDKAppreciateProjectAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKDeleteProjectCommentAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKFollowUserAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectCommentsAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectDetailsAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKPostProjectCommentAsyncTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKPostUserSettingsOnServerAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKAppreciateProjectAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKDeleteProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKFollowUnFollowUserAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectCommentsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectDetailsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKPostProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKPostUserSettingsOnServerAsyncTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKAppreciateProjectAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKDeleteProjectCommentAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKFollowUnFollowUserAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetProjectCommentsAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetProjectDetailsAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKGetUserProjectsListAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKPostProjectCommentAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKPostUserSettingsOnServerAsyncTask;
import com.behance.sdk.dto.project.BehanceSDKAbstractProjectModuleDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCommentDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleImageDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;

import java.util.ArrayList;
import java.util.List;

public class BehanceSDKGetProjectDetailsHeadlessFragment extends Fragment implements IBehanceSDKGetProjectCommentsAsyncTaskListener,
		IBehanceSDKGetProjectDetailsAsyncTaskListener, IBehanceSDKPostProjectCommentAsyncTaskListener,
		IBehanceSDKFollowUserAsyncTaskListener, IBehanceSDKPostUserSettingsOnServerAsyncTaskListener, IBehanceSDKAppreciateProjectAsyncTaskListener,
		IBehanceSDKDeleteProjectCommentAsyncTaskListener {

	private static final String MATURE_CONTENT_ACCESS_ALLOWED = "allowed";
	private boolean getProjectsDetailsAsyncTaskInProgress = false;
	private boolean getProjectCommentsAsyncTaskInProgress = false;
	private boolean getUserProjectListAsyncTaskInProgress = false;
	private boolean postProjectCommentTaskInProgress = false;
	private boolean followUnFollowTaskInProgress = false;
	private boolean postSettingTaskInProgress = false;
	private boolean appreciateProjectTaskInProgress = false;
	private boolean deleteCommentTaskInProgress = false;
	private int lastLoadedProjectCommentsPageNumber;

	private Callbacks callbacks;

	private BehanceSDKGetProjectDetailsAsyncTask getProjectDetailsTask;
	private BehanceSDKGetProjectCommentsAsyncTask getProjectCommentsTask;
	private BehanceSDKGetUserProjectsListAsyncTask getUserProjectListTask;
	private BehanceSDKPostProjectCommentAsyncTask postProjectCommentTask;
	private BehanceSDKFollowUnFollowUserAsyncTask followUnFollowUserAsyncTask;
	private BehanceSDKPostUserSettingsOnServerAsyncTask postSettingsTask;
	private BehanceSDKAppreciateProjectAsyncTask appreciateProjectTask;
	private BehanceSDKDeleteProjectCommentAsyncTask deleteCommentTask;

	private BehanceSDKProjectDTO mActiveProject;
	private List<BehanceSDKProjectDTO> mUserProjectList;
	private List<BehanceSDKProjectCommentDTO> activeProjectCommentsList = new ArrayList<BehanceSDKProjectCommentDTO>();
	private boolean moreCommentsAvailable = false;

	private List<String> imageURLs;

	public String[] getImageURLsArray() {
		String[] imageURLsArray = new String[getImageURLs().size()];
		for(int i = 0; i < imageURLsArray.length; i++) {
			imageURLsArray[i] = imageURLs.get(i);
		}
		return imageURLsArray;
	}


	public List<String> getImageURLs() {
		if(imageURLs == null) {
			imageURLs = new ArrayList<>();
			for(BehanceSDKAbstractProjectModuleDTO module : mActiveProject.getModules()) {
				if(module instanceof BehanceSDKProjectModuleImageDTO) {
					imageURLs.add(((BehanceSDKProjectModuleImageDTO)module).getHDSrcUrl());
				}
			}
		}
		return imageURLs;
	}

	public interface Callbacks {
		public void onGetProjectDetailsSuccess(BehanceSDKProjectDTO activeProject);

		public void onGetProjectDetailsFailure(Exception e);

		public void onGetProjectCommentsSuccess(List<BehanceSDKProjectCommentDTO> commentList, boolean moreCommentsAvailable,
												BehanceSDKGetProjectCommentsAsyncTaskParams taskParams);

		public void onGetProjectCommentsFailure(Exception e, BehanceSDKGetProjectCommentsAsyncTaskParams taskParams);

		public void onPostCommentSuccess(int commentId, BehanceSDKPostProjectCommentAsyncTaskParams taskParams);

		public void onPostCommentFailure(Exception e);

		public void onFollowUnFollowUserSuccess(boolean result, BehanceSDKFollowUnFollowUserAsyncTaskParams params);

		public void onFollowUnFollowUserFailure(Exception e, BehanceSDKFollowUnFollowUserAsyncTaskParams params);

		public void onPostSettingsTaskSuccess(boolean result, BehanceSDKPostUserSettingsOnServerAsyncTaskParams params);

		public void onPostSettingsTaskFailure(Exception e);

		public void onDeleteCommentSuccess(boolean result, BehanceSDKDeleteProjectCommentAsyncTaskParams params);

		public void onDeleteCommentFailure(Exception e, BehanceSDKDeleteProjectCommentAsyncTaskParams params);

		public void onAppreciateProjectAsyncTaskSuccess(boolean result);

		public void onAppreciateProjectAsyncTaskFailure(Exception e);

	}

	public BehanceSDKGetProjectDetailsHeadlessFragment() {
		setRetainInstance(true);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}

	public BehanceSDKProjectDTO getActiveProject() {
		return mActiveProject;
	}

	public void setActiveProject(BehanceSDKProjectDTO mActiveProject) {
		this.mActiveProject = mActiveProject;
	}

	public boolean isGetProjectDetailsAsyncTaskInProgress() {
		return getProjectsDetailsAsyncTaskInProgress;
	}

	private void setGetProjectDetailsAsyncTaskInProgress(boolean getProjectsDetailsAsyncTaskInProgress) {
		this.getProjectsDetailsAsyncTaskInProgress = getProjectsDetailsAsyncTaskInProgress;
	}

	public boolean isGetProjectCommentsAsyncTaskInProgress() {
		return getProjectCommentsAsyncTaskInProgress;
	}

	private void setGetProjectCommentsAsyncTaskInProgress(boolean getProjectCommentsAsyncTaskInProgress) {
		this.getProjectCommentsAsyncTaskInProgress = getProjectCommentsAsyncTaskInProgress;
	}

	public boolean isGetUserProjectListAsyncTaskInProgress() {
		return getUserProjectListAsyncTaskInProgress;
	}

	private void setGetUserProjectListAsyncTaskInProgress(boolean getUserProjectListAsyncTaskInProgress) {
		this.getUserProjectListAsyncTaskInProgress = getUserProjectListAsyncTaskInProgress;
	}

	public boolean isPostProjectCommentTaskInProgress() {
		return postProjectCommentTaskInProgress;
	}

	private void setPostProjectCommentTaskInProgress(boolean postProjectCommentTaskInProgress) {
		this.postProjectCommentTaskInProgress = postProjectCommentTaskInProgress;
	}

	public boolean isFollowUnFollowTaskInProgress() {
		return followUnFollowTaskInProgress;
	}

	private void setFollowUnFollowTaskInProgress(boolean followUnFollowTaskInProgress) {
		this.followUnFollowTaskInProgress = followUnFollowTaskInProgress;
	}

	public boolean isPostSettingTaskInProgress() {
		return postSettingTaskInProgress;
	}

	public void setPostSettingTaskInProgress(boolean postSettingTaskInProgress) {
		this.postSettingTaskInProgress = postSettingTaskInProgress;
	}

	public List<BehanceSDKProjectDTO> getUserProjectList() {
		return mUserProjectList;
	}

	private void setUserProjectList(List<BehanceSDKProjectDTO> mUserProjectList) {
		this.mUserProjectList = mUserProjectList;
	}

	public void setActiveProjectComments(List<BehanceSDKProjectCommentDTO> activeProjectComments) {
		setActiveProjectCommentsList(new ArrayList<BehanceSDKProjectCommentDTO>());
		getActiveProjectCommentsList().addAll(activeProjectComments);
	}

	public List<BehanceSDKProjectCommentDTO> getActiveProjectCommentsList() {
		return activeProjectCommentsList;
	}

	private void setActiveProjectCommentsList(List<BehanceSDKProjectCommentDTO> activeProjectCommentsList) {
		this.activeProjectCommentsList = activeProjectCommentsList;
	}

	public void loadProjectDetailsFromServer(BehanceSDKGetProjectDetailsAsyncTaskParams params) {
		if (!isGetProjectDetailsAsyncTaskInProgress()) {
			if (getProjectDetailsTask == null) {
				getProjectDetailsTask = new BehanceSDKGetProjectDetailsAsyncTask(this);
				getProjectDetailsTask.execute(params);
				setGetProjectDetailsAsyncTaskInProgress(true);
			}
		}
	}

	public void loadProjectCommentsFromServer(BehanceSDKGetProjectCommentsAsyncTaskParams params) {
		if (!isGetProjectCommentsAsyncTaskInProgress() && getProjectCommentsTask == null) {
			setMoreCommentsAvailable(false);
			setGetProjectCommentsAsyncTaskInProgress(true);
			getProjectCommentsTask = new BehanceSDKGetProjectCommentsAsyncTask(this);
			lastLoadedProjectCommentsPageNumber = 1;
			params.setPageNumber(lastLoadedProjectCommentsPageNumber);
			params.setPageSize(BehanceSDKGetProjectCommentsAsyncTaskParams.PROJECT_COMMENTS_PAGE_SIZE);
			getProjectCommentsTask.execute(params);
		}
	}

	public void loadProjectCommentsNextPageFromServer(BehanceSDKGetProjectCommentsAsyncTaskParams params) {
		if (!isGetProjectCommentsAsyncTaskInProgress() && getProjectCommentsTask == null && isMoreCommentsAvailable()) {
			setMoreCommentsAvailable(false);
			setGetProjectCommentsAsyncTaskInProgress(true);
			getProjectCommentsTask = new BehanceSDKGetProjectCommentsAsyncTask(this);
			lastLoadedProjectCommentsPageNumber++;
			params.setPageNumber(lastLoadedProjectCommentsPageNumber);
			params.setPageSize(BehanceSDKGetProjectCommentsAsyncTaskParams.PROJECT_COMMENTS_PAGE_SIZE);
			getProjectCommentsTask.execute(params);
		}
	}


	public void postComment(String commentContent, String projectId) {
		if (!isPostProjectCommentTaskInProgress() && postProjectCommentTask == null) {
			setPostProjectCommentTaskInProgress(true);
			postProjectCommentTask = new BehanceSDKPostProjectCommentAsyncTask(this);
			BehanceSDKPostProjectCommentAsyncTaskParams params = new BehanceSDKPostProjectCommentAsyncTaskParams();
			params.setCommentContent(commentContent);
			params.setProjectId(projectId);
			postProjectCommentTask.execute(params);
		}
	}

	public void followUser(BehanceSDKUserDTO userDTO) {
		followUnFollowUserAsyncTask = null;
		setFollowUnFollowTaskInProgress(true);
		followUnFollowUserAsyncTask = new BehanceSDKFollowUnFollowUserAsyncTask(this);
		BehanceSDKFollowUnFollowUserAsyncTaskParams params = new BehanceSDKFollowUnFollowUserAsyncTaskParams();
		params.setUserDTO(userDTO);
		params.setFollowUser(true);
		followUnFollowUserAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, params);
	}

	public void unFollowUser(BehanceSDKUserDTO userDTO) {
		followUnFollowUserAsyncTask = null;
		setFollowUnFollowTaskInProgress(true);
		followUnFollowUserAsyncTask = new BehanceSDKFollowUnFollowUserAsyncTask(this);
		BehanceSDKFollowUnFollowUserAsyncTaskParams params = new BehanceSDKFollowUnFollowUserAsyncTaskParams();
		params.setUserDTO(userDTO);
		params.setFollowUser(false);
		followUnFollowUserAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, params);
	}

	public void setMatureContentAccessForUser(BehanceSDKPostUserSettingsOnServerAsyncTaskParams params) {
		if (!isPostProjectCommentTaskInProgress() && postSettingsTask == null) {
			setPostProjectCommentTaskInProgress(true);
			postSettingsTask = new BehanceSDKPostUserSettingsOnServerAsyncTask(this);
			postSettingsTask.execute(params);
		}
	}

	public void appreciateProject(String projectId) {
		if (!isAppreciateProjectTaskInProgress() && appreciateProjectTask == null) {
			setAppreciateProjectTaskInProgress(true);
			appreciateProjectTask = new BehanceSDKAppreciateProjectAsyncTask(this);
			BehanceSDKAppreciateProjectAsyncTaskParams params = new BehanceSDKAppreciateProjectAsyncTaskParams();
			params.setProjectId(projectId);
			appreciateProjectTask.execute(params);
		}
	}

	public void deleteComment(String projectId, String commentId) {
		if (!isDeleteCommentTaskInProgress() && deleteCommentTask == null) {
			setDeleteCommentTaskInProgress(true);
			BehanceSDKDeleteProjectCommentAsyncTaskParams params = new BehanceSDKDeleteProjectCommentAsyncTaskParams();
			params.setCommentId(commentId);
			params.setProjectId(projectId);
			deleteCommentTask = new BehanceSDKDeleteProjectCommentAsyncTask(this);
			deleteCommentTask.execute(params);
		}
	}

	@Override
	public void onGetProjectDetailsAsyncTaskSuccess(BehanceSDKProjectDTO activeProject) {
		setActiveProject(activeProject);
		setGetProjectDetailsAsyncTaskInProgress(false);
		getProjectDetailsTask = null;
		if (this.callbacks != null) {
			callbacks.onGetProjectDetailsSuccess(activeProject);
		}
	}

	@Override
	public void onGetProjectDetailsAsyncTaskFailure(Exception e) {
		getProjectDetailsTask = null;
		setGetProjectDetailsAsyncTaskInProgress(false);
		if (this.callbacks != null) {
			callbacks.onGetProjectDetailsFailure(e);
		}
	}

	@Override
	public void onGetProjectCommentsSuccess(List<BehanceSDKProjectCommentDTO> commentList, BehanceSDKGetProjectCommentsAsyncTaskParams taskParams) {
		if (taskParams.getPageNumber() > 1 && getActiveProjectCommentsList() != null) {
			getActiveProjectCommentsList().addAll(commentList);
		} else {
			setActiveProjectComments(commentList);
		}
		getProjectCommentsTask = null;
		setGetProjectCommentsAsyncTaskInProgress(false);

		if (commentList.size() >= BehanceSDKGetProjectCommentsAsyncTaskParams.PROJECT_COMMENTS_PAGE_SIZE) {
			setMoreCommentsAvailable(true);
		} else {
			setMoreCommentsAvailable(false);
		}
		if (this.callbacks != null) {
			callbacks.onGetProjectCommentsSuccess(commentList, isMoreCommentsAvailable(), taskParams);
		}
	}

	@Override
	public void onGetProjectCommentsFailure(Exception e, BehanceSDKGetProjectCommentsAsyncTaskParams taskParams) {
		getProjectCommentsTask = null;
		setMoreCommentsAvailable(false);
		setGetProjectCommentsAsyncTaskInProgress(false);
		if (this.callbacks != null) {
			callbacks.onGetProjectCommentsFailure(e, taskParams);
		}
	}


	@Override
	public void onPostProjectCommentAsyncTaskSuccess(int commentId, BehanceSDKPostProjectCommentAsyncTaskParams params) {
		if (callbacks != null) {
			callbacks.onPostCommentSuccess(commentId,params);
		}
		setPostProjectCommentTaskInProgress(false);
		postProjectCommentTask = null;

	}

	@Override
	public void onPostProjectCommentAsyncTaskFailure(Exception e, BehanceSDKPostProjectCommentAsyncTaskParams params) {
		if (callbacks != null) {
			callbacks.onPostCommentFailure(e);
		}
		setPostProjectCommentTaskInProgress(false);
		postProjectCommentTask = null;

	}

	@Override
	public void onFollowUserAsyncTaskSuccess(boolean result, BehanceSDKFollowUnFollowUserAsyncTaskParams params) {
		if (callbacks != null) {
			callbacks.onFollowUnFollowUserSuccess(result, params);
		}
		setFollowUnFollowTaskInProgress(false);
		followUnFollowUserAsyncTask = null;
	}

	@Override
	public void onFollowUserAsyncTaskFailure(Exception e, BehanceSDKFollowUnFollowUserAsyncTaskParams params) {
		if (callbacks != null) {
			callbacks.onFollowUnFollowUserFailure(e, params);
		}
		setFollowUnFollowTaskInProgress(false);
		followUnFollowUserAsyncTask = null;
	}

	@Override
	public void onPostUserSettingsOnServerAsyncTaskSuccess(Boolean result, BehanceSDKPostUserSettingsOnServerAsyncTaskParams params) {
		setPostProjectCommentTaskInProgress(false);
		mActiveProject.setMatureAccess(MATURE_CONTENT_ACCESS_ALLOWED);
		postSettingsTask = null;
		if (callbacks != null) {
			callbacks.onPostSettingsTaskSuccess(result, params);
		}
	}

	@Override
	public void onPostUserSettingsOnServerAsyncTaskFailure(Exception e, BehanceSDKPostUserSettingsOnServerAsyncTaskParams params) {
		setPostProjectCommentTaskInProgress(false);
		postSettingsTask = null;
		if (callbacks != null) {
			callbacks.onPostSettingsTaskFailure(e);
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		setActiveProject(null);
	}

	public boolean isAppreciateProjectTaskInProgress() {
		return appreciateProjectTaskInProgress;
	}

	public void setAppreciateProjectTaskInProgress(boolean appreciateProjectTaskInProgress) {
		this.appreciateProjectTaskInProgress = appreciateProjectTaskInProgress;
	}

	@Override
	public void onAppreciateProjectAsyncTaskSuccess(boolean result, BehanceSDKAppreciateProjectAsyncTaskParams params) {
		setAppreciateProjectTaskInProgress(false);
		appreciateProjectTask = null;
		if(callbacks != null)
			callbacks.onAppreciateProjectAsyncTaskSuccess(result);
		if(mActiveProject != null) {
			mActiveProject.setAppreciatedByUser(result);
			mActiveProject.getStats().setAppreciationsCount(mActiveProject.getStats().getAppreciationsCount() + 1);
		}
	}

	@Override
	public void onAppreciateProjectAsyncTaskFailure(Exception e, BehanceSDKAppreciateProjectAsyncTaskParams params) {
		setAppreciateProjectTaskInProgress(false);
		appreciateProjectTask = null;
		if(callbacks != null)
			callbacks.onAppreciateProjectAsyncTaskFailure(e);
	}

	public boolean isDeleteCommentTaskInProgress() {
		return deleteCommentTaskInProgress;
	}

	private void setDeleteCommentTaskInProgress(boolean deleteCommentTaskInProgress) {
		this.deleteCommentTaskInProgress = deleteCommentTaskInProgress;
	}

	@Override
	public void onDeleteProjectCommentAsyncTaskSuccess(boolean result, BehanceSDKDeleteProjectCommentAsyncTaskParams params) {
		setDeleteCommentTaskInProgress(false);
		deleteCommentTask = null;
		if (callbacks != null) {
			callbacks.onDeleteCommentSuccess(result, params);
		}
	}

	@Override
	public void onDeleteProjectCommentAsyncTaskFailure(Exception e, BehanceSDKDeleteProjectCommentAsyncTaskParams params) {
		setDeleteCommentTaskInProgress(false);
		deleteCommentTask = null;
		if (callbacks != null) {
			callbacks.onDeleteCommentFailure(e, params);
		}
	}

	public boolean isMoreCommentsAvailable() {
		return moreCommentsAvailable;
	}

	private void setMoreCommentsAvailable(boolean moreCommentsAvailable) {
		this.moreCommentsAvailable = moreCommentsAvailable;
	}

}
