/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.fragments.headless;

import android.os.Bundle;
import android.support.v4.app.Fragment;


import com.behance.sdk.asynctask.params.BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectsAsyncTaskParams;
import com.behance.sdk.datamanager.BehanceSDKProjectsDataManager;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.listeners.IBehanceSDKProjectsDataManagerGetProjectsListener;

import java.util.List;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKGetProjectsHeadlessFragment extends Fragment implements IBehanceSDKProjectsDataManagerGetProjectsListener {

	private Callbacks callbacks;
	private BehanceSDKProjectsDataManager projectsDataManager;
	protected BehanceSDKGetProjectsAsyncTaskParams latestParams;
	protected BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams curateGalleryParams;

	public interface Callbacks {
		public void onGetProjectsSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params);

		public void onGetProjectsFailure(Exception e, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params);

		public void onGetProjectsSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetProjectsAsyncTaskParams params);

		public void onGetProjectsFailure(Exception e, BehanceSDKGetProjectsAsyncTaskParams params);


	}

	public BehanceSDKGetProjectsHeadlessFragment() {
		projectsDataManager = BehanceSDKProjectsDataManager.getInstance();
		setRetainInstance(true);
		addProjectManagerListener();
	}

	protected void addProjectManagerListener() {
		projectsDataManager.addGetProjectsListener(this);
	}

	protected void removeProjectManagerListener() {
		if (projectsDataManager != null)
			projectsDataManager.removeGetProjectsListener(this);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	public void loadProjectsFromServer(BehanceSDKGetProjectsAsyncTaskParams params) {
		this.latestParams = params;
		projectsDataManager.loadProjectsFromServer(params);
	}

	public void loadGalleryProjectsFromServer(BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params){
		this.curateGalleryParams = params;
		projectsDataManager.loadGalleryProjectsFromServer(params);
	}

	public boolean isGetProjectsAsyncTaskInProgress() {
		return projectsDataManager.isGetProjectsAsyncTaskInProgress();
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}

	@Override
	public void onProjectsDataManagerGetProjectsSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params) {
		if (callbacks != null) {
			if (params != null) {
				callbacks.onGetProjectsSuccess(result, params);
			}
		}
	}

	@Override
	public void onProjectsDataManagerGetProjectsFailure(Exception e, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params) {
		if (callbacks != null) {
			if (params != null) {
				callbacks.onGetProjectsFailure(e, params);
			}
		}
	}


	@Override
	public void onProjectsDataManagerGetProjectsSuccess(List<BehanceSDKProjectDTO> result, BehanceSDKGetProjectsAsyncTaskParams params) {
		if (callbacks != null) {
			if (params != null && params.compareSearchFilters(latestParams)) {
				callbacks.onGetProjectsSuccess(result, params);
			}
		}
	}

	@Override
	public void onProjectsDataManagerGetProjectsFailure(Exception e, BehanceSDKGetProjectsAsyncTaskParams params) {
		if (callbacks != null) {
			if (params != null && params.compareSearchFilters(latestParams)) {
				callbacks.onGetProjectsFailure(e, params);
			}
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		removeProjectManagerListener();
		clearProjectsDataManagerCache();
	}

	protected void clearProjectsDataManagerCache() {
		if (projectsDataManager != null) {
			projectsDataManager.clearProjectsDataManagerCache();
		}
	}
}