/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.fragments.headless;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.os.AsyncTask;
import android.support.v4.app.Fragment;

import com.behance.sdk.asynctask.listeners.IBehanceSDKGetAlbumsTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetAlbumsTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKGetAlbumsAsyncTask;
import com.behance.sdk.dto.BehanceSDKAlbumDTO;

public class BehanceSDKImageSelectorHeadlessFragment extends Fragment implements IBehanceSDKGetAlbumsTaskListener {

	private boolean albumsLoadingInProgress = false;
	private BehanceSDKGetAlbumsAsyncTask getAlbumsTask;

	private List<BehanceSDKAlbumDTO> albumsList;
	private Callbacks callbacks;

	public interface Callbacks {
		public void onGetAlbumsSuccess();

		public void onGetAlbumsFailure(Exception e);
	}

	public BehanceSDKImageSelectorHeadlessFragment() {
		setRetainInstance(true);
	}

	public void setCallbacks(Callbacks listener) {
		this.callbacks = listener;
	}

	public List<BehanceSDKAlbumDTO> getAlbumsList() {
		return albumsList;
	}
	
	public void clearAlbumsCache() {
		if (albumsList != null) {
			albumsList.clear();
			albumsList = null;
		}
	}

	public boolean isAlbumsLoadingInProgress() {
		return albumsLoadingInProgress;
	}

	private void setAlbumsLoadingInProgress(boolean albumsLoadingInProgress) {
		this.albumsLoadingInProgress = albumsLoadingInProgress;
	}

	public void loadAlbums(Context context, boolean withImageSelectionSources) {
		if (!isAlbumsLoadingInProgress()) {
			setAlbumsLoadingInProgress(true);
			BehanceSDKGetAlbumsTaskParams params = new BehanceSDKGetAlbumsTaskParams();
			WeakReference<Context> contextRef = new WeakReference<Context>(context);
			params.setContextWeakReference(contextRef);
			params.setShowImageSelectionSources(withImageSelectionSources);
			getAlbumsTask = new BehanceSDKGetAlbumsAsyncTask(this);
			getAlbumsTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, params);
		}
	}

	@Override
	public void onGetAlbumsTaskSuccess(List<BehanceSDKAlbumDTO> result) {
		if (result != null) {
			if (albumsList == null) {
				albumsList = new ArrayList<BehanceSDKAlbumDTO>();
			}
			albumsList.clear();
			albumsList.addAll(result);
		}
		setAlbumsLoadingInProgress(false);
		getAlbumsTask = null;
		if (callbacks != null) {
			callbacks.onGetAlbumsSuccess();
		}
	}

	@Override
	public void onGetAlbumsTaskFailure(Exception e) {
		setAlbumsLoadingInProgress(false);
		getAlbumsTask = null;
		if (callbacks != null) {
			callbacks.onGetAlbumsFailure(e);
		}
	}
}