package com.behance.sdk.fragments.headless;

import android.os.AsyncTask;
import android.support.v4.app.Fragment;

import com.behance.sdk.asynctask.listeners.IBehanceSDKGetAuthURLFromTwitterTaskListener;
import com.behance.sdk.asynctask.listeners.IBehanceSDKRetrieveAccessTokenFromTwitterTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetAuthURLFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKGetAuthURLFromTwitterAsyncTask;
import com.behance.sdk.asynctasks.BehanceSDKRetrieveAccessTokenFromTwitterAsyncTask;
import com.behance.sdk.asynctasks.result.BehanceSDKGetAuthUrlFromTwitterTaskResult;
import com.behance.sdk.asynctasks.result.BehanceSDKRetrieveAccessTokenFromTwitterTaskResult;

public class BehanceSDKLoginToTwitterHeadlessFragment extends Fragment implements IBehanceSDKGetAuthURLFromTwitterTaskListener, IBehanceSDKRetrieveAccessTokenFromTwitterTaskListener {

	private Callbacks callbacks;
	private boolean getAuthURLFromTwitterTaskInProgress;
	private boolean retrieveAccessTokenFromTwitterTaskInProgress;
	
	private BehanceSDKGetAuthURLFromTwitterAsyncTask getAuthURLFromTwitterAsyncTask;
	private BehanceSDKRetrieveAccessTokenFromTwitterAsyncTask retrieveAccessTokenFromTwitterAsyncTask;

	public interface Callbacks {
		public void onGetAuthURLFromTwitterSuccess(BehanceSDKGetAuthUrlFromTwitterTaskResult result);

		public void onGetAuthURLFromTwitterFailure(Exception e);

		public void onRetrieveAccessTokenFromTwitterSuccess(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result);

		public void onRetrieveAccessTokenFromTwitterFailure(Exception e);
	}

	public void setCallbacks(Callbacks listener) {
		this.callbacks = listener;
	}

	public BehanceSDKLoginToTwitterHeadlessFragment() {
		setRetainInstance(true);
	}

	private boolean isGetAuthURLFromTwitterTaskInProgress() {
		return getAuthURLFromTwitterTaskInProgress;
	}

	private void setGetAuthURLFromTwitterTaskInProgress(boolean getAuthURLFromTwitterTaskInProgress) {
		this.getAuthURLFromTwitterTaskInProgress = getAuthURLFromTwitterTaskInProgress;
	}

	public void getAuthURLFromTwitter(BehanceSDKGetAuthURLFromTwitterAsyncTaskParams params) {
		if (!isGetAuthURLFromTwitterTaskInProgress() && getAuthURLFromTwitterAsyncTask == null) {
			getAuthURLFromTwitterAsyncTask = new BehanceSDKGetAuthURLFromTwitterAsyncTask(this);
			getAuthURLFromTwitterAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,params);
			setGetAuthURLFromTwitterTaskInProgress(true);
		}
	}

	public void retrieveAccessTokenFromTwitter(BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams params) {
		if (!isRetrieveAccessTokenFromTwitterTaskInProgress() && retrieveAccessTokenFromTwitterAsyncTask == null) {
			retrieveAccessTokenFromTwitterAsyncTask = new BehanceSDKRetrieveAccessTokenFromTwitterAsyncTask(this, getActivity());
			retrieveAccessTokenFromTwitterAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,params);
			setRetrieveAccessTokenFromTwitterTaskInProgress(true);
		}
	}

	@Override
	public void onGetAuthURLFromTwitterSuccess(BehanceSDKGetAuthUrlFromTwitterTaskResult result) {
		if (callbacks != null) {
			callbacks.onGetAuthURLFromTwitterSuccess(result);
		}
		getAuthURLFromTwitterAsyncTask = null;
		setGetAuthURLFromTwitterTaskInProgress(false);
	}

	@Override
	public void onGetAuthURLFromTwitterFailure(BehanceSDKGetAuthUrlFromTwitterTaskResult result) {
		if (callbacks != null) {
			callbacks.onGetAuthURLFromTwitterFailure(result.getException());
		}
		getAuthURLFromTwitterAsyncTask = null;
		setGetAuthURLFromTwitterTaskInProgress(false);
	}

	private boolean isRetrieveAccessTokenFromTwitterTaskInProgress() {
		return retrieveAccessTokenFromTwitterTaskInProgress;
	}

	private void setRetrieveAccessTokenFromTwitterTaskInProgress(boolean retrieveAccessTokenFromTwitterTaskInProgress) {
		this.retrieveAccessTokenFromTwitterTaskInProgress = retrieveAccessTokenFromTwitterTaskInProgress;
	}

	@Override
	public void onRetrieveAccessTokenFromTwitterSuccess(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result) {
		if (callbacks != null) {
			callbacks.onRetrieveAccessTokenFromTwitterSuccess(result);
		}
		setRetrieveAccessTokenFromTwitterTaskInProgress(false);
		retrieveAccessTokenFromTwitterAsyncTask = null;
	}

	@Override
	public void onRetrieveAccessTokenFromTwitterFailure(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result) {
		if (callbacks != null) {
			callbacks.onRetrieveAccessTokenFromTwitterFailure(result.getException());
		}
		setRetrieveAccessTokenFromTwitterTaskInProgress(false);
		retrieveAccessTokenFromTwitterAsyncTask = null;
	}

}
