package com.behance.sdk.fragments.headless;

import java.util.List;

import android.support.v4.app.Fragment;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.ui.fragments.BehanceSDKCopyrightSettingsDialog;
import com.behance.sdk.ui.fragments.BehanceSDKCreativeFieldsFilterDialog;

import com.facebook.GraphRequest;
import com.facebook.GraphRequestAsyncTask;
import com.facebook.GraphResponse;
import com.facebook.login.LoginResult;

import org.json.JSONObject;

public class BehanceSDKPublishProjectHeadlessFragment extends Fragment implements BehanceSDKCreativeFieldsFilterDialog.Callbacks,
		BehanceSDKCopyrightSettingsDialog.Callbacks{

	private Callbacks callbacks;

	private boolean getFacebookUserIdInProgress = false;
	private LoginResult loginResult =null;

	public interface Callbacks {

		public void onCreativeFieldsSelected(List<BehanceSDKCreativeFieldDTO> selectedFields);

		public void onCopyrightSettingSelected(BehanceSDKCopyrightOption copyRightSetting);

		public void onGetFacebookUserIdComplete(String userName);
	}

	public void setCallbacks(Callbacks listener) {
		this.callbacks = listener;
	}

	public BehanceSDKPublishProjectHeadlessFragment() {
		setRetainInstance(true);
	}

	@Override
	public void onCreativeFieldsSelected(List<BehanceSDKCreativeFieldDTO> selectedFields) {
		if (null != callbacks)
			callbacks.onCreativeFieldsSelected(selectedFields);
	}

	@Override
	public void onCopyrightSettingSelected(BehanceSDKCopyrightOption copyRightSetting) {
		if (null != callbacks)
			callbacks.onCopyrightSettingSelected(copyRightSetting);
	}

	public void loadFacebookUserId(LoginResult activeSession) {
		setGetFacebookUserIdInProgress(true);
		GraphRequestAsyncTask meRequest = GraphRequest.newMeRequest(activeSession.getAccessToken(),new GraphRequest.GraphJSONObjectCallback() {
			@Override
			public void onCompleted(JSONObject user, GraphResponse response) {
				setGetFacebookUserIdInProgress(false);
				if (callbacks != null) {
					if (user != null) {
						callbacks.onGetFacebookUserIdComplete(user.optString("name"));
					} else {
						callbacks.onGetFacebookUserIdComplete(null);
					}
				}
			}
		}).executeAsync();

	}

	public boolean isGetFacebookUserIdInProgress() {
		return getFacebookUserIdInProgress;
	}

	public void setGetFacebookUserIdInProgress(boolean getFacebookUserIdInProgress) {
		this.getFacebookUserIdInProgress = getFacebookUserIdInProgress;
	}

	public LoginResult getLoginResult(){
		return loginResult;
	}

	public void setLoginResult(LoginResult loginResult){
		this.loginResult =loginResult;
	}

}