/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.fragments.headless;

import java.util.List;

import android.os.AsyncTask;
import android.support.v4.app.Fragment;

import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserWIPsTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKAbstractTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserWIPsTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKGetUserWIPsAsyncTask;
import com.behance.sdk.dto.BehanceSDKWIPDTO;
import com.behance.sdk.managers.AddWIPManager;
import com.facebook.GraphRequest;
import com.facebook.GraphRequestAsyncTask;
import com.facebook.GraphResponse;
import com.facebook.login.LoginResult;

import org.json.JSONObject;


public class BehanceSDKPublishWIPHeadlessFragment extends Fragment implements IBehanceSDKGetUserWIPsTaskListener {

	private boolean getUserWIPsTaskInProgress = false;
	private BehanceSDKGetUserWIPsAsyncTask getUserWIPsTask;
	private List<BehanceSDKWIPDTO> userWIPsList;

	private boolean getFacebookUserIdInProgress = false;

	private Callbacks callbacks;

	public interface Callbacks {

		public void onGetUserWIPsSuccess();

		public void onGetUserWIPsFailure(Exception e);

		public void onGetFacebookUserIdComplete(String userName);
	}

	public BehanceSDKPublishWIPHeadlessFragment() {
		setRetainInstance(true);
	}

	public void setCallbacks(Callbacks listener) {
		this.callbacks = listener;
	}

	private void populateAuthParams(BehanceSDKAbstractTaskParams params) {
		AddWIPManager wipManager = AddWIPManager.getInstance();
		IBehanceSDKUserCredentials userCredentials = wipManager.getUserCredentials();
		params.setClientId(userCredentials.getClientId());
	}

	public boolean isGetUserWIPsTaskInProgress() {
		return getUserWIPsTaskInProgress;
	}

	private void setGetUserWIPsTaskInProgress(boolean getUserWIPsTaskInProgress) {
		this.getUserWIPsTaskInProgress = getUserWIPsTaskInProgress;
	}

	public void loadUserWIPsFromServer() {
		if (!isGetUserWIPsTaskInProgress() && getUserWIPsTask == null) {
			setGetUserWIPsTaskInProgress(true);
			BehanceSDKGetUserWIPsTaskParams params = new BehanceSDKGetUserWIPsTaskParams();
			AddWIPManager wipManager = AddWIPManager.getInstance();
			IBehanceSDKUserCredentials userCredentials = wipManager.getUserCredentials();
			params.setUserId(userCredentials.getUserBehanceAccountId());
			populateAuthParams(params);
			getUserWIPsTask = new BehanceSDKGetUserWIPsAsyncTask(this);
			getUserWIPsTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, params);
		}
	}

	@Override
	public void onGetUserWIPsTaskSuccess(List<BehanceSDKWIPDTO> result, BehanceSDKGetUserWIPsTaskParams params) {
		setUserWIPsList(result);
		if (callbacks != null) {
			callbacks.onGetUserWIPsSuccess();
		}
		getUserWIPsTask = null;
		setGetUserWIPsTaskInProgress(false);
	}

	@Override
	public void onGetUserWIPsTaskFailure(Exception e, BehanceSDKGetUserWIPsTaskParams params) {
		if (callbacks != null) {
			callbacks.onGetUserWIPsFailure(e);
		}
		getUserWIPsTask = null;
		setGetUserWIPsTaskInProgress(false);
	}

	public List<BehanceSDKWIPDTO> getUserWIPsList() {
		return userWIPsList;
	}

	private void setUserWIPsList(List<BehanceSDKWIPDTO> userWIPsList) {
		this.userWIPsList = userWIPsList;
	}

	public void loadFacebookUserId(LoginResult activeSession) {
		setGetFacebookUserIdInProgress(true);
		GraphRequestAsyncTask meRequest = GraphRequest.newMeRequest(activeSession.getAccessToken(), new GraphRequest.GraphJSONObjectCallback() {
			@Override
			public void onCompleted(JSONObject user, GraphResponse response) {
				setGetFacebookUserIdInProgress(false);
				if (callbacks != null) {
					if (user != null) {
						callbacks.onGetFacebookUserIdComplete(user.optString("name"));
					} else {
						callbacks.onGetFacebookUserIdComplete(null);
					}
				}
			}
		}).executeAsync();
	}

	public boolean isGetFacebookUserIdInProgress() {
		return getFacebookUserIdInProgress;
	}

	public void setGetFacebookUserIdInProgress(boolean getFacebookUserIdInProgress) {
		this.getFacebookUserIdInProgress = getFacebookUserIdInProgress;
	}
}