package com.behance.sdk.fragments.headless;

import android.support.v4.app.Fragment;
import com.behance.sdk.asynctask.listeners.IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;

public class BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment extends Fragment implements IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler {

    private boolean retrieveBehanceUserDetailTaskInProgress;

    private BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask retrieveUserDetailsUsingAdobeIDTask;
    private Callbacks callbacks;

    public BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment() {
        setRetainInstance(true);
    }

    public interface Callbacks {
        public void onRetrieveBehanceUserDetailsSuccess(BehanceSDKUserDTO userDTO);

        public void onRetrieveBehanceUserDetailsFailure(Exception e);
    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    public boolean isRetrieveUserDetailTaskInProgress() {
        return retrieveBehanceUserDetailTaskInProgress;
    }

    public void setRetrieveUserDetailTaskInProgress(boolean retrieveUserDetailTaskInProgress) {
        this.retrieveBehanceUserDetailTaskInProgress = retrieveUserDetailTaskInProgress;
    }

    public void retrieveBehanceUserDetailsUsingAdobeID(BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams params) {
        if (!isRetrieveUserDetailTaskInProgress() && retrieveUserDetailsUsingAdobeIDTask == null) {
            setRetrieveUserDetailTaskInProgress(true);
            retrieveUserDetailsUsingAdobeIDTask = new BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask(this);
            retrieveUserDetailsUsingAdobeIDTask.execute(params);
        }
    }

    @Override
    public void onRetrieveBehanceUserDetailsSuccess(BehanceSDKUserDTO userDTO) {
        if (this.callbacks != null) {
            callbacks.onRetrieveBehanceUserDetailsSuccess(userDTO);
        }
        retrieveUserDetailsUsingAdobeIDTask = null;
        setRetrieveUserDetailTaskInProgress(false);
    }

    @Override
    public void onRetrieveBehanceUserDetailsFailure(Exception e) {
        if (this.callbacks != null) {
            callbacks.onRetrieveBehanceUserDetailsFailure(e);
        }
        retrieveUserDetailsUsingAdobeIDTask = null;
        setRetrieveUserDetailTaskInProgress(false);
    }

    @Override
    public void onDetach() {
        super.onDetach();
        if (retrieveUserDetailsUsingAdobeIDTask != null) {
            retrieveUserDetailsUsingAdobeIDTask.cancel(true);
        }
    }
}