/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.logger;

import android.util.Log;

import com.behance.sdk.util.BehanceSDKConstants;

/**
 * @author Sujit Reddy G
 *
 */

public class Logger implements ILogger {

	private int LOG_LEVEL = BehanceSDKConstants.LOG_LEVEL;
	private boolean WTF = LOG_LEVEL > 0;
	private boolean ERROR = LOG_LEVEL > 1;
	private boolean WARN = LOG_LEVEL > 2;
	private boolean INFO = LOG_LEVEL > 3;
	private boolean DEBUG = LOG_LEVEL > 4;
	
	private String tag;
	
	public Logger(@SuppressWarnings("rawtypes") Class clazz) {
		tag = clazz.getSimpleName();
	}

	@Override
	public void debug(String msg, Object... args) {
		if(DEBUG) {
			Log.d(tag, getFormattedMessage(msg, args));
		}
	}

	@Override
	public void debug(Throwable tr) {
		if(DEBUG) {
			Log.d(tag, tr.getMessage(), tr);
		}
	}

	@Override
	public void debug(Throwable tr, String msg, Object... args) {
		if(DEBUG) {
			Log.d(tag, getFormattedMessage(msg, args), tr);
		}
	}

	@Override
	public void error(String msg, Object... args) {
		if(ERROR) {
			Log.e(tag, getFormattedMessage(msg, args));
		}
	}

	@Override
	public void error(Throwable tr) {
		if(ERROR) {
			Log.e(tag, tr.getMessage(), tr);
		}
	}

	@Override
	public void error(Throwable tr, String msg, Object... args) {
		if(ERROR) {
			Log.e(tag, getFormattedMessage(msg, args), tr);
		}
	}

	@Override
	public void info(String msg, Object... args) {
		if(INFO) {
			Log.i(tag, getFormattedMessage(msg, args));
		}
	}

	@Override
	public void info(Throwable tr) {
		if(INFO) {
			Log.i(tag, tr.getMessage(), tr);
		}
	}

	@Override
	public void info(Throwable tr, String msg, Object... args) {
		if(INFO) {
			Log.i(tag, getFormattedMessage(msg, args), tr);
		}
	}

	@Override
	public void warn(String msg, Object... args) {
		if(WARN) {
			Log.w(tag, getFormattedMessage(msg, args));
		}
	}

	@Override
	public void warn(Throwable tr) {
		if(WARN) {
			Log.w(tag, tr);
		}
	}

	@Override
	public void warn(Throwable tr, String msg, Object... args) {
		if(WARN) {
			Log.w(tag, getFormattedMessage(msg, args), tr);
		}
	}

	@Override
	public void wtf(String msg, Object... args) {
		if(WTF) {
			Log.wtf(tag, getFormattedMessage(msg, args));
		}
	}

	@Override
	public void wtf(Throwable tr) {
		if(WTF) {
			Log.wtf(tag, tr);
		}
	}

	@Override
	public void wtf(Throwable tr, String msg, Object... args) {
		if(WTF) {
			Log.wtf(tag, getFormattedMessage(msg, args), tr);
		}
	}

	private String getFormattedMessage(String msg, Object... args) {
		try {
			if(args.length > 0) {
				return String.format(msg, args);
			} else {
				return msg;
			}
		} catch (Exception e) {
			error(e);
			return "";
		}
	}
	
}
