/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2015 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.managers;

import android.accounts.Account;
import android.accounts.AccountManager;

import com.adobe.creativesdk.foundation.adobeinternal.auth.AdobeUXAuthManagerRestricted;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloud;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloudManager;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeEntitlementServices;
import com.adobe.creativesdk.foundation.auth.AdobeAuthUserProfile;
import com.adobe.creativesdk.foundation.internal.auth.AdobeAuthIdentityManagementService;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;
import com.behance.sdk.asynctask.listeners.IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.fragments.headless.BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.util.BehanceSDKConstants;

/**
 * Created by Sujit Reddy G
 */
public class BehanceSDKUserManager {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKUserManager.class);
    private BehanceSDKUserDTO userDTO;


    private static BehanceSDKUserManager instance;


    public static BehanceSDKUserManager getInstance() {
        if (instance == null) {
            instance = new BehanceSDKUserManager();
        }

        return instance;
    }

    private BehanceSDKUserManager() {
    }

    public boolean isUserAuthenticatedWithAdobe() {
        AdobeUXAuthManagerRestricted authManager = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted();
        return authManager.isAuthenticated();
    }

    public boolean isUserLoggedIn() {
        AdobeUXAuthManagerRestricted authManager = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted();
        AdobeAuthUserProfile userProfile = authManager.getUserProfile();
        return isUserAuthenticatedWithAdobe() && userProfile != null;
    }

    public BehanceSDKUserDTO getUserDTO(){
        return userDTO;
    }

    public void setUserDTO(BehanceSDKUserDTO userDTO){
        this.userDTO = userDTO;
    }

    public String getAdobeUserId() {
        AdobeUXAuthManagerRestricted authManager = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted();
        return authManager.getAdobeID();
    }

    public String getAccessToken() {
        AdobeAuthIdentityManagementService identityManagementService = AdobeAuthIdentityManagementService.getSharedInstance();
        return identityManagementService.getAccessToken();
    }

    public boolean isUserEntitledToUseBehance() {
        AdobeEntitlementServices entitlementServices = AdobeEntitlementServices.getSharedServices();
        if(entitlementServices != null)
        return entitlementServices.isEntitledToService(AdobeEntitlementServices.AdobeEntitlementServiceBehance, null);
        else return false;
    }

    public boolean isUserAnEnterpriseUser() {
        AdobeUXAuthManagerRestricted authManager = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted();
        AdobeAuthUserProfile userProfile = authManager.getUserProfile();
        if (userProfile != null) {
            return userProfile.isEnterpriseUser();
        }
        return false;
    }

    public AdobeCloud getUserDefaultCloud() {
        AdobeCloudManager sharedCloudManager = AdobeCloudManager.getSharedCloudManager();
        return sharedCloudManager.getDefaultCloud();
    }

    /**
     * Should be invoked this only from a AsyncTask
     *
     * @return
     * @throws BehanceSDKUserNotAuthenticatedException
     */
    public String checkExpiryAndGetAccessToken() throws BehanceSDKUserNotAuthenticatedException {
        AdobeUXAuthManagerRestricted authManager = AdobeUXAuthManagerRestricted.getSharedAuthManagerRestricted();
        AdobeAuthIdentityManagementService identityManagementService = AdobeAuthIdentityManagementService.getSharedInstance();
        if (!authManager.hasValidAccessToken()) {
            try {
                logger.debug("Access token expired. Trying to get new access token.");
                boolean reAuthenticateStatus = authManager.reAuthenticate();

                logger.debug("Refreshed Access token status [%s].", reAuthenticateStatus);
                if (!reAuthenticateStatus) {
                    BehanceSDKUserNotAuthenticatedException exception = new BehanceSDKUserNotAuthenticatedException("Problem refreshing access token");
                    throw exception;
                }
            } catch (Exception e) {
                // device token is invalidated from other clients
                logger.warn("Device token expired.");
                BehanceSDKUserNotAuthenticatedException exception = new BehanceSDKUserNotAuthenticatedException("Problem refreshing access token");
                throw exception;
            }
        }
        logger.debug("Access token is valid. Using the existing one.");
        return identityManagementService.getAccessToken();
    }






}
