package com.behance.sdk.network;

import com.adobe.creativesdk.foundation.AdobeCSDKFoundation;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.apache.commons.io.IOUtils;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


import javax.net.ssl.HttpsURLConnection;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.exception.OAuthCommunicationException;
import oauth.signpost.exception.OAuthExpectationFailedException;
import oauth.signpost.exception.OAuthMessageSignerException;
import oauth.signpost.http.HttpParameters;

/**
 * Created by Sujit Reddy G
 */
public class BehanceHttpsConnection implements IBehanceHttpConnection {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceHttpsConnection.class);

    private static final String LINE_ENDING = "\r\n";
    private static final int READ_TIMEOUT_MILLIS = 10000;
    private static final int CONNECT_TIMEOUT_MILLIS = 15000;
    public static final String CHARSET_NAME_UTF_8 = "UTF-8";

    private static BehanceHttpsConnection instance = null;
    private NoSSLv3Factory noSSLv3Factory;
    private static final int BUFFER_SIZE = 32 * 1024;

    public static BehanceHttpsConnection getInstance() {
        if (instance == null) {
            instance = new BehanceHttpsConnection();

        }
        return instance;
    }

    private BehanceHttpsConnection() {
        noSSLv3Factory = new NoSSLv3Factory();
    }

    public BehanceConnectionResponse<String> invokeHttpGetRequest(String requestUrl) throws IOException {
        return invokeHttpRequest(BehanceHttpMethod.GET, requestUrl);
    }

    public BehanceConnectionResponse<String> invokeHttpGetRequest(String requestUrl,List<BehanceNameValuePair> urlParamsList) throws IOException {
        return invokeHttpRequest(BehanceHttpMethod.GET, requestUrl, urlParamsList);
    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl) throws IOException {
        return invokeHttpPostRequest(requestUrl, null);
    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl, List<BehanceNameValuePair> urlParamsList) throws IOException {
        return invokeHttpPostRequest(requestUrl, urlParamsList, null);
    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl,
                                                                   List<BehanceNameValuePair> urlParamsList,
                                                                   List<BehanceNameValuePair> headersList,
                                                                   OAuthConsumer consumer) throws IOException, OAuthCommunicationException, OAuthExpectationFailedException, OAuthMessageSignerException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
    //    urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
    //    urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.POST.name());
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());
        urlConnection.setDoInput(true);

        consumer.sign(urlConnection);

        if (headersList != null) {
            for (BehanceNameValuePair headerNameValuePair : headersList) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        StringBuilder paramsStringBuilder = null;
        if (urlParamsList != null) {
            paramsStringBuilder = new StringBuilder();
            boolean first = true;
            for (BehanceNameValuePair paramNameValuePair : urlParamsList) {
                if (paramNameValuePair != null) {
                    if (first) {
                        first = false;
                    } else {
                        paramsStringBuilder.append("&");
                    }
                    paramsStringBuilder.append(URLEncoder.encode(paramNameValuePair.getName(), CHARSET_NAME_UTF_8));
                    paramsStringBuilder.append("=");
                    paramsStringBuilder.append(URLEncoder.encode(paramNameValuePair.getValue(), CHARSET_NAME_UTF_8));
                }
            }
        }

        if (paramsStringBuilder != null) {
            urlConnection.setDoOutput(true);
            OutputStream outputStream = urlConnection.getOutputStream();
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, CHARSET_NAME_UTF_8));
            bufferedWriter.write(paramsStringBuilder.toString());
            bufferedWriter.flush();
            bufferedWriter.close();
            outputStream.close();
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;

    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl, List<BehanceNameValuePair> urlParamsList,
                                                                   List<BehanceNameValuePair> headersList) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
   //     urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
   //     urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.POST.name());
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());
        urlConnection.setDoInput(true);

        if (headersList != null) {
            for (BehanceNameValuePair headerNameValuePair : headersList) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        StringBuilder paramsStringBuilder = null;
        if (urlParamsList != null) {
            paramsStringBuilder = new StringBuilder();
            boolean first = true;
            for (BehanceNameValuePair paramNameValuePair : urlParamsList) {
                if (paramNameValuePair != null) {
                    if (first) {
                        first = false;
                    } else {
                        paramsStringBuilder.append("&");
                    }
                    paramsStringBuilder.append(URLEncoder.encode(paramNameValuePair.getName(), CHARSET_NAME_UTF_8));
                    paramsStringBuilder.append("=");
                    paramsStringBuilder.append(URLEncoder.encode(paramNameValuePair.getValue(), CHARSET_NAME_UTF_8));
                }
            }
        }

        if (paramsStringBuilder != null) {
            urlConnection.setDoOutput(true);
            OutputStream outputStream = urlConnection.getOutputStream();
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, CHARSET_NAME_UTF_8));
            bufferedWriter.write(paramsStringBuilder.toString());
            bufferedWriter.flush();
            bufferedWriter.close();
            outputStream.close();
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl, String requestPayload, List<BehanceNameValuePair> headersList) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());
        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
  //      urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
  //      urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        urlConnection.setRequestMethod(BehanceHttpMethod.POST.name());
        urlConnection.setDoInput(true);

        if (headersList != null) {
            for (BehanceNameValuePair headerNameValuePair : headersList) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        if (requestPayload != null && !requestPayload.isEmpty()) {
            urlConnection.setDoOutput(true);
            OutputStream outputStream = urlConnection.getOutputStream();
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, CHARSET_NAME_UTF_8));
            bufferedWriter.write(requestPayload);
            bufferedWriter.flush();
            bufferedWriter.close();
            outputStream.close();
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpPostRequest(String requestUrl,
                                                                   BehanceMultipartEntity multipartEntity,
                                                                   BehanceRequestCancellationWrapper cancellationWrapper) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
    //    urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
    //    urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.POST.name());
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        urlConnection.setDoInput(true);

        if (multipartEntity.getMultipartHeaders() != null) {
            for (BehanceNameValuePair headerNameValuePair : multipartEntity.getMultipartHeaders()) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        if (multipartEntity.getMultiPartDataList() != null && !multipartEntity.getMultiPartDataList().isEmpty()) {
            writeToOutputStream(urlConnection, multipartEntity, cancellationWrapper);
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpPutRequest(String requestUrl) throws IOException {
        return invokeHttpPutRequest(requestUrl, null);
    }

    public BehanceConnectionResponse<String> invokeHttpPutRequest(String requestUrl, String requestPayload, List<BehanceNameValuePair> headersList) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
    //    urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
    //    urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.PUT.name());
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        urlConnection.setDoInput(true);

        if (headersList != null) {
            for (BehanceNameValuePair headerNameValuePair : headersList) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        if (requestPayload != null && !requestPayload.isEmpty()) {
            urlConnection.setDoOutput(true);
            OutputStream outputStream = urlConnection.getOutputStream();
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, CHARSET_NAME_UTF_8));
            bufferedWriter.write(requestPayload);
            bufferedWriter.flush();
            bufferedWriter.close();
            outputStream.close();
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);
        
        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpPutRequest(String requestUrl, List<BehanceNameValuePair> nameValuePairsList) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        //     urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
   //     urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.PUT.name());
        urlConnection.setDoInput(true);

        StringBuilder paramsStringBuilder = null;
        if (nameValuePairsList != null) {
            paramsStringBuilder = new StringBuilder();
            boolean first = true;
            for (BehanceNameValuePair nameValuePair : nameValuePairsList) {
                if (nameValuePair != null) {
                    if (first) {
                        first = false;
                    } else {
                        paramsStringBuilder.append("&");
                    }
                    paramsStringBuilder.append(URLEncoder.encode(nameValuePair.getName(), CHARSET_NAME_UTF_8));
                    paramsStringBuilder.append("=");
                    paramsStringBuilder.append(URLEncoder.encode(nameValuePair.getValue(), CHARSET_NAME_UTF_8));
                }
            }
        }

        if (paramsStringBuilder != null) {
            urlConnection.setDoOutput(true);
            OutputStream outputStream = urlConnection.getOutputStream();
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, CHARSET_NAME_UTF_8));
            bufferedWriter.write(paramsStringBuilder.toString());
            bufferedWriter.flush();
            bufferedWriter.close();
            outputStream.close();
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    private static final void setRequestMethodUsingWorkaroundForJREBug(
            final HttpURLConnection httpURLConnection, final String method) {
        try {
            httpURLConnection.setRequestMethod(method);
            // Check whether we are running on a buggy JRE
        } catch (final ProtocolException pe) {
            Class<?> connectionClass = httpURLConnection
                    .getClass();
            Field delegateField = null;
            try {
                delegateField = connectionClass.getDeclaredField("delegate");
                delegateField.setAccessible(true);
                HttpURLConnection delegateConnection = (HttpURLConnection) delegateField
                        .get(httpURLConnection);
                setRequestMethodUsingWorkaroundForJREBug(delegateConnection, method);
            } catch (NoSuchFieldException e) {
                // Ignore for now, keep going
            } catch (IllegalArgumentException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
            try {
                Field methodField;
                while (connectionClass != null) {
                    try {
                        methodField = connectionClass
                                .getDeclaredField("method");
                    } catch (NoSuchFieldException e) {
                        connectionClass = connectionClass.getSuperclass();
                        continue;
                    }
                    methodField.setAccessible(true);
                    methodField.set(httpURLConnection, method);
                    break;
                }
            } catch (final Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    public BehanceConnectionResponse<String> invokeHttpPatchRequest(String requestUrl, BehanceMultipartEntity multipartEntity,
                                                                  BehanceRequestCancellationWrapper cancellationWrapper) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
   //     urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
   //     urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod("PATCH");
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        //setRequestMethodUsingWorkaroundForJREBug(urlConnection,"PATCH");
        urlConnection.setDoOutput(true);

        if (multipartEntity.getMultipartHeaders() != null) {
            for (BehanceNameValuePair headerNameValuePair : multipartEntity.getMultipartHeaders()) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        if (multipartEntity.getMultiPartDataList() != null && !multipartEntity.getMultiPartDataList().isEmpty()) {
            writeToOutputStream(urlConnection, multipartEntity, cancellationWrapper);
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpPutRequest(String requestUrl, BehanceMultipartEntity multipartEntity,/*String boundaryID,
                                                                   ArrayList<BehanceMultiPartData> multiPartDataList,
                                                                   List<BehanceNameValuePair> headersList,*/
                                                                  BehanceRequestCancellationWrapper cancellationWrapper) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        //    urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
    //    urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(BehanceHttpMethod.PUT.name());
        urlConnection.setDoInput(true);

        if (multipartEntity.getMultipartHeaders() != null) {
            for (BehanceNameValuePair headerNameValuePair : multipartEntity.getMultipartHeaders()) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        if (multipartEntity.getMultiPartDataList() != null && !multipartEntity.getMultiPartDataList().isEmpty()) {
            writeToOutputStream(urlConnection,multipartEntity,cancellationWrapper);
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);

        return response;
    }

    public BehanceConnectionResponse<String> invokeHttpDeleteRequest(String requestUrl) throws IOException {
        return invokeHttpRequest(BehanceHttpMethod.DELETE, requestUrl);
    }

    public BehanceConnectionResponse<String> invokeHttpDeleteRequest(String requestUrl, List<BehanceNameValuePair> headersList) throws IOException {
        return invokeHttpRequest(BehanceHttpMethod.DELETE, requestUrl, headersList);
    }

    private BehanceConnectionResponse<String> invokeHttpRequest(BehanceHttpMethod httpMethod, String requestUrl) throws IOException {
        return invokeHttpRequest(httpMethod, requestUrl, null);
    }

    private BehanceConnectionResponse<String> invokeHttpRequest(BehanceHttpMethod httpMethod, String requestUrl, List<BehanceNameValuePair> headersList) throws IOException {
        BehanceConnectionResponse<String> response = new BehanceConnectionResponse<String>();
        requestUrl = BehanceSDKUrlUtil.appendQueryStringParam(requestUrl,BehanceSDKUrlUtil.PARAM_KEY_CLIENT_ID,AdobeCSDKFoundation.getClientId());

        URL urlClass = new URL(requestUrl);
        HttpsURLConnection urlConnection = (HttpsURLConnection) urlClass.openConnection();
        urlConnection.setSSLSocketFactory(noSSLv3Factory);
        urlConnection.setRequestProperty(BehanceSDKUrlUtil.KEY_X_IMS_CLIENT_ID, AdobeCSDKFoundation.getClientId());

        //    urlConnection.setReadTimeout(READ_TIMEOUT_MILLIS);
    //    urlConnection.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        urlConnection.setRequestMethod(httpMethod.name());
        urlConnection.setDoInput(true);

        if (headersList != null) {
            for (BehanceNameValuePair headerNameValuePair : headersList) {
                if (headerNameValuePair != null) {
                    urlConnection.setRequestProperty(headerNameValuePair.getName(), headerNameValuePair.getValue());
                }
            }
        }

        urlConnection.connect();
        readResponseFromUrlConnection(urlConnection, response);
        return response;
    }


    private void readResponseFromUrlConnection(HttpsURLConnection urlConnection, BehanceConnectionResponse<String> response) throws IOException {
        int responseCode;
        try {
            responseCode = urlConnection.getResponseCode();
        } catch (IOException e) {
            //Will return 401 the 2nd time called
            //http://stackoverflow.com/a/15973063
            responseCode = urlConnection.getResponseCode();
        }
        response.setResponseCode(responseCode);

        if(responseCode == HttpURLConnection.HTTP_OK){
            Map<String,List<String>> headers =  urlConnection.getHeaderFields();
            response.setHeaders(headers);
        }
        logger.debug("Reading response from [Response code - %s] [URL - %s] ", responseCode, urlConnection.getURL());

        InputStream inputStream = null;
        try {
            StringBuilder responseStringBuilder = new StringBuilder();

            inputStream = urlConnection.getErrorStream();
            if (inputStream == null) {
                inputStream = urlConnection.getInputStream();
            }

            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
            String lineStr;
            while ((lineStr = bufferedReader.readLine()) != null) {
                responseStringBuilder.append(lineStr + LINE_ENDING);
            }

            response.setResponseObject(responseStringBuilder.toString());
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (Throwable t) {
                logger.warn(t, "Problem closing input stream. [URL - %s]", urlConnection.getURL());
            }
            try {
                urlConnection.disconnect();
            } catch (Throwable t) {
                logger.warn(t, "Problem disconnecting url connection. [URL - %s]", urlConnection.getURL());
            }
        }
    }

    private void sendFileInputStream(BufferedInputStream fileInputStream,
                                     BehanceMultipartEntity multipartEntity,
                                     BehanceMultiPartData multiPartData,
                                     OutputStream dos,
                                     BehanceRequestCancellationWrapper cancellationWrapper) throws IOException {
        if (fileInputStream.available() > 0) {
            byte[] dataBuffer = BehanceMultipartEntity.generateMultiPartFormData(multiPartData, multipartEntity.getBoundary(), multipartEntity.isBrowserCompatible());
            dos.write(dataBuffer);
            multipartEntity.incrementTransferredBytesLength(dataBuffer.length);

            byte[] buffer = new byte[BUFFER_SIZE];

            // read file and write it into form...
            int bytesRead = fileInputStream.read(buffer, 0, BUFFER_SIZE);

            while (bytesRead > 0) {
                if(cancellationWrapper != null && cancellationWrapper.isCancelled())
                {
                    return;
                }

                dos.write(buffer, 0, bytesRead);
                dos.flush();
                multipartEntity.incrementTransferredBytesLength(bytesRead);
                bytesRead = fileInputStream.read(buffer, 0, BUFFER_SIZE);
            }
        }

    }

    protected void writeToOutputStream(HttpsURLConnection urlConnection,
                                       BehanceMultipartEntity multipartEntity,
                                       BehanceRequestCancellationWrapper cancellationWrapper) {
        String delimeter="\r\n";
        byte[] delbytes=delimeter.getBytes();
        OutputStream dos = null;

        try {

            urlConnection.setDoOutput(true);
            urlConnection.setDoInput(true);

            if(multipartEntity.isChunked())
            {
                urlConnection.setChunkedStreamingMode(BUFFER_SIZE);
            }

            dos = urlConnection.getOutputStream();

            for(BehanceMultiPartData multiPartData: multipartEntity.getMultiPartDataList()) {

                if(cancellationWrapper != null && cancellationWrapper.isCancelled())
                {
                    break;
                }

                if (multiPartData.getSourcePath()!=null) {
                    BufferedInputStream fileInputStream = null;
                    try {
                        fileInputStream = new BufferedInputStream(new FileInputStream(multiPartData.getSourcePath()), BUFFER_SIZE);
                        sendFileInputStream(fileInputStream,multipartEntity,multiPartData,dos,cancellationWrapper);
                    } catch (FileNotFoundException e) {
                        e.printStackTrace();
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        IOUtils.closeQuietly(fileInputStream);
                    }

                }
                else if(multiPartData.getInputStream() != null)
                {
                    BufferedInputStream fileInputStream = null;
                    fileInputStream = new BufferedInputStream(multiPartData.getInputStream(), BUFFER_SIZE);
                    try {
                        sendFileInputStream(fileInputStream,multipartEntity,multiPartData,dos,cancellationWrapper);
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        IOUtils.closeQuietly(fileInputStream);
                    }
                }

                else if(multiPartData.getData()!=null){
                    byte[] dataBuffer = BehanceMultipartEntity.generateMultiPartFormData(multiPartData, multipartEntity.getBoundary(), multipartEntity.isBrowserCompatible());
                    dos.write(dataBuffer);
                    multipartEntity.incrementTransferredBytesLength(dataBuffer.length);
                    dos.write(multiPartData.getData());
                    multipartEntity.incrementTransferredBytesLength(multiPartData.getData().length);
                }

                dos.write(delbytes);
                dos.flush();
            }
            byte[] endingBoundary = String.format("--%s--\r\n", multipartEntity.getBoundary()).getBytes(Charset.forName("UTF-8"));
            dos.write(endingBoundary);
            dos.flush();

        } catch (IOException e) {
            e.printStackTrace();
            logger.error(e, "Error during io operation");
        } finally {
            //IOUtils.closeQuietly(fileInputStream);
            IOUtils.closeQuietly(dos);
        }
    }
}
