package com.behance.sdk.network;

import com.behance.sdk.project.modules.EmbedModule;
import com.behance.sdk.project.modules.ImageModule;


import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by rkumarba on 12/8/2015.
 */
public class BehanceMultipartEntity {
    private final static char[] MULTIPART_CHARS =
            "-_1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
                    .toCharArray();

    public static final String CONTENT_TYPE_APPLICATION_OCTETSTREAM = "application/octet-stream";
    public static final String CONTENT_TRANSFER_BINARY = "binary";
    public static final String CONTENT_TRANSFER_8BIT = "8bit";
    public static final String CONTENT_TYPE_TEXT_UTF8 = "text/plain; charset=UTF-8";
    public static final String CONTENT_TYPE_TEXT_ISO_8859 = "text/plain; charset=ISO-8859-1";
    public static final String CONTENT_TYPE_APPLICATION_XHTML = "application/xhtml+xml; charset=ISO-8859-1";
    public static final String CHARSET_UTF8 = "UTF-8";
    public static final String CHARSET_ISO_8859 = "ISO-8859-1";

    ArrayList<BehanceNameValuePair> headers = null;

    private ArrayList<BehanceMultiPartData> multiPartDataArrayList = null;

    public String getBoundary() {
        return boundary;
    }

    public void setBoundary(String boundary) {
        this.boundary = boundary;
    }

    private String boundary;

    public boolean isChunked() {
        return chunked;
    }

    public void setChunked(boolean chunked) {
        this.chunked = chunked;
    }

    private boolean chunked;

    /** browser-compatible mode, i.e. only write Content-Disposition; use content charset */
    private boolean browserCompatible;

    public void incrementTransferredBytesLength(long transferredBytesLength) {
        if(this.transferredBytesLength == -1)
        {
            this.transferredBytesLength =  transferredBytesLength;
        }
        else
        {
            this.transferredBytesLength += transferredBytesLength;
        }
        notifyUploadProgress();
    }

    private long transferredBytesLength;

    public BehanceMultipartEntityWrapper getBehanceHttpEntityWrapperCallback() {
        return behanceHttpEntityWrapperCallback;
    }

    public void setBehanceHttpEntityWrapperCallback(BehanceMultipartEntityWrapper behanceHttpEntityWrapperCallback) {
        this.behanceHttpEntityWrapperCallback = behanceHttpEntityWrapperCallback;
    }

    private BehanceMultipartEntityWrapper behanceHttpEntityWrapperCallback;

    private static int getSize(ByteArrayInputStream is){
        int size = is.available();
        return size;
    }

    public long getTotalBytesLength() {
        if(totalBytesLength == -1)
        {
            totalBytesLength = 0;
            for(BehanceMultiPartData multiPartData: this.getMultiPartDataList()) {
                totalBytesLength += generateMultiPartFormData(multiPartData,this.getBoundary(),this.isBrowserCompatible()).length;
                if (multiPartData.getSourcePath()!=null) {
                    totalBytesLength += new File(multiPartData.getSourcePath()).length();
                }
                else if(multiPartData.getInputStream() != null)
                {
                    totalBytesLength += getSize(multiPartData.getInputStream());
                }
                else if(multiPartData.getData()!=null){
                    totalBytesLength += multiPartData.getData().length;
                }
            }
        }
        return totalBytesLength;
    }

    private long totalBytesLength;

    private String generateBoundary() {
        final StringBuilder buffer = new StringBuilder();
        final Random rand = new Random();
        final int count = rand.nextInt(11) + 30; // a random size from 30 to 40
        for (int i = 0; i < count; i++) {
            buffer.append(MULTIPART_CHARS[rand.nextInt(MULTIPART_CHARS.length)]);
        }
        return buffer.toString();
    }

    public BehanceMultipartEntity()
    {
        this.boundary = generateBoundary();
        this.multiPartDataArrayList = new ArrayList<>();
        this.chunked = false;
        this.browserCompatible = false;
        this.transferredBytesLength = -1;
        this.totalBytesLength = -1;
        this.behanceHttpEntityWrapperCallback = null;
    }

    public ArrayList<BehanceMultiPartData> getMultiPartDataList() {
        return multiPartDataArrayList;
    }

    public ArrayList<BehanceNameValuePair> getMultipartHeaders() {
        ArrayList<BehanceNameValuePair> headerList = new ArrayList<>();
        if(headers != null)
        {
            headerList.addAll(headers);
        }
        headerList.add(new BehanceNameValuePair("Content-Type", "multipart/form-data; boundary=" + getBoundary()));
        if(isChunked())
        {
            headerList.add(new BehanceNameValuePair("Transfer-Encoding","chunked"));
        }
        return headerList;
    }

    public void addCustomMultipartHeader(BehanceNameValuePair header)
    {
        if(headers == null)
        {
            headers = new ArrayList<>();
        }
        headers.add(header);
    }

    public static byte[]  generateMultiPartFormData(BehanceMultiPartData multiPartData,String boundary,boolean isBrowserCompatible) {

        StringBuffer multipartString=new StringBuffer("");
        multipartString.append(String.format("--%s\r\n", boundary));

        if(multiPartData.getFileName()!=null && multiPartData.getName()!= null)
        {
            multipartString.append(String.format("Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"\r\n",multiPartData.getName(),multiPartData.getFileName()));
        }
        else if(multiPartData.getFileName()!=null)
        {
            multipartString.append(String.format("Content-Disposition: form-data; filename=\"%s\"\r\n",multiPartData.getFileName()));
        }
        else if(multiPartData.getName()!= null)
        {
            multipartString.append(String.format("Content-Disposition: form-data; name=\"%s\"\r\n",multiPartData.getName()));
        }
        else
        {
            multipartString.append("Content-Disposition: form-data;\r\n");
        }

        if(multiPartData.getContentTransferEncoding() != null && !isBrowserCompatible) {
            multipartString.append(String.format("Content-Transfer-Encoding: %s\r\n", multiPartData.getContentTransferEncoding()));
        }
        String manifestContentId = multiPartData.getContentID();

        if(manifestContentId!=null && !isBrowserCompatible){
            if(manifestContentId.startsWith("cid:"))
                manifestContentId=manifestContentId.substring(4);
            multipartString.append(String.format("Content-ID: <%s>\r\n",manifestContentId));

        }

        if(multiPartData.getContentType() != null && !isBrowserCompatible) {
            multipartString.append(String.format("Content-Type: %s\r\n", multiPartData.getContentType()));
        }

        multipartString.append("\r\n");

        try {
            return  multipartString.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    public void addTextPart(String name,String contentType,String contentTransferEncoding,byte[] data)
    {
        BehanceMultiPartData multiPartData = new BehanceMultiPartData();
        multiPartData.setName(name);
        multiPartData.setContentTransferEncoding(contentTransferEncoding);
        multiPartData.setContentType(contentType);
        multiPartData.setData(data);
        this.getMultiPartDataList().add(multiPartData);
    }

    public void addImagePart(String name,String fileName,String contentType,String contentTransferEncoding,String imageSourcePath)
    {
        BehanceMultiPartData multiPartData = new BehanceMultiPartData();
        multiPartData.setName(name);
        multiPartData.setContentTransferEncoding(contentTransferEncoding);
        multiPartData.setContentType(contentType);
        multiPartData.setFileName(fileName);
        multiPartData.setSourcePath(imageSourcePath);
        this.getMultiPartDataList().add(multiPartData);
    }

    public void addImagePart(String name,String fileName,String contentType,String contentTransferEncoding,ByteArrayInputStream inputStream)
    {
        BehanceMultiPartData multiPartData = new BehanceMultiPartData();
        multiPartData.setName(name);
        multiPartData.setContentTransferEncoding(contentTransferEncoding);
        multiPartData.setContentType(contentType);
        multiPartData.setFileName(fileName);
        multiPartData.setInputStream(inputStream);
        this.getMultiPartDataList().add(multiPartData);
    }


    public boolean isBrowserCompatible() {
        return browserCompatible;
    }

    public void setBrowserCompatible(boolean browserCompatible) {
        this.browserCompatible = browserCompatible;
    }

    private void notifyUploadProgress() {
        float progressPercentage = ((float) this.transferredBytesLength / getTotalBytesLength()) * 100;
        if (this.behanceHttpEntityWrapperCallback != null) {
            this.behanceHttpEntityWrapperCallback.onUploadProgressUpdate(progressPercentage);
        }
    }
}
