/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.project;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.project.modules.CoverImage;
import com.behance.sdk.project.modules.EmbedModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;

/**
 * @author ugs
 * 
 */
public class NewBehanceProject implements Serializable {

	private static final int CONTAINS_ADULT_CONTENT = 1;

	private static final int NO_ADULT_CONTENT = 0;

	private static final long serialVersionUID = -1163691639211926852L;

	private static final int NUMBER_OF_PROJECT_MODULES = 4;
	
	private int embedModuleCount = 0;

	private String title;

	private String description;

	private List<BehanceSDKCreativeFieldDTO> fields;

	private String tags;

	private List<ProjectModule> projectModules;

	private CoverImage cover;

	private BehanceSDKCopyrightOption copyRight = BehanceSDKCopyrightOption.getDefaultValue();

	private int containAdultContent = NO_ADULT_CONTENT;

	private String projectURL;

	private String projectID;

	private boolean shareOnFaceBook;

	private boolean shareOnTwitter;

	public NewBehanceProject() {
		projectModules = new ArrayList<ProjectModule>(NUMBER_OF_PROJECT_MODULES);
	}

	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @param title
	 *            the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *            the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the fields
	 */
	public List<BehanceSDKCreativeFieldDTO> getFields() {
		return fields;
	}

	/**
	 * @param fields
	 *            the fields to set
	 */
	public void setFields(List<BehanceSDKCreativeFieldDTO> fields) {
		this.fields = fields;
	}

	/**
	 * @return the tags
	 */
	public String getTags() {
		return tags;
	}

	/**
	 * @param tags
	 *            the tags to set
	 */
	public void setTags(String tags) {
		this.tags = tags;
	}

	/**
	 * @return the covers
	 */
	public CoverImage getCoverImage() {
		return cover;
	}

	/**
	 * @param cover
	 *            the covers to set
	 */
	public void setCoverImage(CoverImage cover) {
		this.cover = cover;
	}

	/**
	 * @return the projectModules
	 */
	public List<ProjectModule> getProjectModules() {
		return projectModules;
	}

	/**
	 * @return the projectModules
	 */
	public int getNumberOfProjectModules() {
		return projectModules.size();
	}

	/**
	 * @param modules
	 *            the projectModules to set
	 */
	public void addProjectModules(List<? extends ProjectModule> modules) {
		if (null != modules && false == modules.isEmpty()) {
			projectModules.addAll(modules);
		}
	}

	/**
	 * @param module
	 *            the projectModules to set
	 */
	public void addProjectModule(ProjectModule module) {
		if (false == projectModules.contains(module)){
			projectModules.add(module);
			if(module.getType() == ProjectModuleTypes.EMBED){
				((EmbedModule)module).setOrder(++embedModuleCount);
			}
		}
	}

	/**
	 * @param module
	 */
	public void removeProjectModules(ProjectModule module) {
		if (null != projectModules) {
			projectModules.remove(module);
		}

	}

	/**
	 * @return
	 */
	public List<ImageModule> getImageModules() {
		List<ImageModule> imageModules = new ArrayList<ImageModule>();
		for (ProjectModule projectModule : projectModules) {
			ProjectModuleTypes moduleType = projectModule.getType();
			if (moduleType == ProjectModuleTypes.IMAGE || moduleType == ProjectModuleTypes.CREATIVECLOUD_ASSET) {
				imageModules.add((ImageModule) projectModule);
			}
		}
		return imageModules;
	}

	public boolean isEmpty() {
		return getNumberOfProjectModules() == 0 ? true : false;
	}

	/**
	 * @param fromPosition
	 * @param toPosition
	 */
	public void reorderSelectedModules(int fromPosition, int toPosition) {
		ProjectModule fromLocation = projectModules.remove(fromPosition);
		projectModules.add(toPosition, fromLocation);
	}

	public void clear() {
		if (null != cover)
			cover.remove();
		for (ProjectModule projectModule : projectModules) {
			projectModule.clear();
		}

	}
	
	public boolean isProjectContainsAdultContent() {
		if(CONTAINS_ADULT_CONTENT == containAdultContent)
			return true;
		else
			return false;
		
	}

	public int containsAdultContent() {
		return containAdultContent;
	}
	public void markProjectForAdultContent(boolean containsAdultContent) {
		if(containsAdultContent)
			containAdultContent = CONTAINS_ADULT_CONTENT;
		else
			containAdultContent = NO_ADULT_CONTENT;
	}

	public BehanceSDKCopyrightOption getCopyRight() {
		return copyRight;
	}

	public void setCopyRight(BehanceSDKCopyrightOption behanceCopyRightOption) {
		 copyRight = behanceCopyRightOption;
	}
	/**
	 * @param projectImageSelection
	 */
	public void addImagesToProject(List<? extends ImageModule> projectImageSelection) {
		for (ImageModule localImage : projectImageSelection) {
			if(false == projectModules.contains(localImage))
				addProjectModule(localImage);
		}
	}

	public void removeAllImageModules() {
		List<ImageModule> imageModules = getImageModules();
		projectModules.removeAll(imageModules);
	}

	public String getURL() {
		return projectURL;
	}
	
	public void setURL(String projectURL) {
		this.projectURL = projectURL;
	}

	public void setProjectID(String projectID) {
		this.projectID = projectID;
	}
	
	public String getProjectID() {
		return this.projectID;
	}

	public void setShareOnFaceBook(boolean shareOnFaceBook) {
		this.shareOnFaceBook = shareOnFaceBook;
	}
	
	public boolean getShareOnFaceBook() {
		return this.shareOnFaceBook;
	}
	
	public void setShareOnTwitter(boolean shareOnTwitter) {
		this.shareOnTwitter = shareOnTwitter;
	}
	
	public boolean getShareOnTwitter() {
		return this.shareOnTwitter;
	}
}
