/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.project.modules;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import android.graphics.Bitmap;

import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.util.BehanceSDKConstants;

/**
 * @author ugs
 * 
 */
public class CoverImage {

	private static final ILogger logger = LoggerFactory.getLogger(CoverImage.class);
	
	private ImageModule originalImage;
	
	private Bitmap coverImage;
	
	private static final int COVER_IMAGE_HEIGHT = 316;

	private static final int COVER_IMAGE_WIDTH = 404;


	public CoverImage(ImageModule originalImage, Bitmap covertedFile) {
		this.originalImage = originalImage;
		this.coverImage = covertedFile;
	}

	public String getName() {
		if(null != originalImage)
			return originalImage.getName();
		else
			return BehanceSDKConstants.EMPTY_STRING;
	}

	public Bitmap getBitMap() {
		return coverImage;
	}

	public void remove() {
		if(null != coverImage)
			coverImage.recycle();
	}

	public ImageModule getOriginalImage() {
		return originalImage;
	}

	public byte[] getByteArray() {
		if(null != coverImage){
			Bitmap covertedFile = Bitmap.createScaledBitmap(coverImage,COVER_IMAGE_WIDTH,COVER_IMAGE_HEIGHT, 
					 false);
			ByteArrayOutputStream baos = null;
			try {
				baos = new ByteArrayOutputStream();
				covertedFile.compress(Bitmap.CompressFormat.PNG, 100, baos);
				byte[] byteArray = baos.toByteArray();
				return byteArray;
			} finally {
				if (null != baos) {
					try {
						baos.close();
					} catch (IOException e) {
						logger.error(e);
					}
				}
			}
		}
		return new byte[0];
	}

}
