package com.behance.sdk.project.validator;

import android.content.Context;
import android.widget.Toast;

import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageProperties;
import com.behance.sdk.factory.IBehanceSDKImageValidator;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUtils;

import java.util.List;

/**
 * Created by ugs on 26/06/15.
 */
public class BehanceSDKProjectImageModuleValidator implements IBehanceSDKImageValidator {


    public static final int FACTOR_TO_CONVERT_BYTE_TO_MB = 1024 * 1024;

    @Override
    public boolean validate(Context context, BehanceSDKImageProperties params) {
        return imageMatchesRequirements(context, params);
    }

    private void displayToastMessage(Context context,String msgToDisplay) {
        if (msgToDisplay != null) {
            Toast.makeText(context, msgToDisplay, Toast.LENGTH_LONG).show();
        }
    }

    private boolean imageMatchesRequirements(Context context, BehanceSDKImageProperties params) {

        String fileExtension = params.getType();
        String fileName = params.getName();
        List<String> allowedFileExtensionsList = BehanceSDKUtils.getAllowedFileExtensionsForProjectImage();
        if (fileExtension != null && fileExtension.length() > 0) {
            if (allowedFileExtensionsList != null && !allowedFileExtensionsList.isEmpty()) {
                if (!allowedFileExtensionsList.contains(fileExtension)) {
                    String msgToDisplay = context.getString(R.string.bsdk_publish_project_from_cc_invalid_file_type, fileExtension,
                            allowedFileExtensionsList.toString());
                    displayToastMessage(context, msgToDisplay);
                    return false;
                }
            }
        } else {
            String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_file_type_invalid_msg, fileName);
            displayToastMessage(context, msgToDisplay);
            return false;
        }

        long imageSizeInBytes = params.getSize();

        long maxImageSizeBytes = BehanceSDKConstants.PROJECT_IMAGE_MODULE_MAX_SIZE_BYTES;
            if (imageSizeInBytes < 0 || maxImageSizeBytes > 0 && imageSizeInBytes > maxImageSizeBytes) {

                String msgToDisplay = context.getString(R.string.bsdk_publish_project_from_cc_invalid_image_size, fileName,
                        BehanceSDKConstants.PROJECT_IMAGE_MODULE_MAX_SIZE_BYTES_STR);
                displayToastMessage(context, msgToDisplay);
                return false;
            }
        return true;
    }
}
