package com.behance.sdk.project.validator;

import android.content.Context;
import android.widget.Toast;

import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageProperties;
import com.behance.sdk.factory.IBehanceSDKImageValidator;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUtils;

import java.util.List;

/**
 * Created by ugs on 26/06/15.
 */
public class BehanceSDKWIPImageModuleValidator implements IBehanceSDKImageValidator {
    @Override
    public boolean validate(Context context, BehanceSDKImageProperties params) {
        return imageMatchesRequirements(context, params);
    }

    private void displayToastMessage(Context context, String msgToDisplay) {
        if (null != context && msgToDisplay != null) {
            Toast.makeText(context, msgToDisplay, Toast.LENGTH_LONG).show();
        }
    }

    private boolean imageMatchesRequirements(Context context, BehanceSDKImageProperties params) {

        String fileName = params.getName();
        String fileExtension = params.getType();
        List<String> allowedFileExtensionsList = BehanceSDKUtils.getAllowedFileExtensionsForWIPImage();
        List<String> prohibitedFileExtensionsList = null;
        if (fileExtension != null && fileExtension.length() > 0) {
            if (prohibitedFileExtensionsList != null && !prohibitedFileExtensionsList.isEmpty()) {
                if (prohibitedFileExtensionsList.contains(fileExtension)) {
                    String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_file_type_not_allowed_msg, fileName,
                            prohibitedFileExtensionsList.toString());
                    displayToastMessage(context, msgToDisplay);
                    return false;
                }
            }

            if (allowedFileExtensionsList != null && !allowedFileExtensionsList.isEmpty()) {
                if (!allowedFileExtensionsList.contains(fileExtension)) {
                    String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_file_type_allowed_msg, fileName,
                            allowedFileExtensionsList.toString());
                    displayToastMessage(context, msgToDisplay);
                    return false;
                }
            }
        } else {
            String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_file_type_invalid_msg, fileName);
            displayToastMessage(context, msgToDisplay);
            return false;
        }

        int imageWidth = params.getWidth();
        int imageHeight = params.getHeight();
        long imageSizeInBytes = params.getSize();

        long minImageSizeBytes = -1;
        long maxImageSizeBytes = BehanceSDKConstants.WIP_IMAGE_MAX_SIZE_BYTES;
        if (imageSizeInBytes > 0) {
            if (minImageSizeBytes > 0 && imageSizeInBytes < minImageSizeBytes) {
                String sizeInMBStr = (minImageSizeBytes / (1024 * 1024)) + " MB";
                String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_size_less_than_min_required_msg, sizeInMBStr);
                displayToastMessage(context, msgToDisplay);
                return false;
            }

            if (maxImageSizeBytes > 0 && imageSizeInBytes > maxImageSizeBytes) {
                String sizeInMBStr = (maxImageSizeBytes / (1024 * 1024)) + " MB";
                String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_size_more_than_max_allowed_msg, sizeInMBStr);
                displayToastMessage(context, msgToDisplay);
                return false;
            }
        } else {
            String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_size_invalid_msg);
            displayToastMessage(context, msgToDisplay);
            return false;
        }

        int minImageWidth = BehanceSDKConstants.WIP_IMAGE_MIN_WIDTH;
        int minImageHeight = BehanceSDKConstants.WIP_IMAGE_MIN_HEIGHT;
        int maxImageWidth = -1;
        int maxImageHeight = -1;

        if (imageWidth > 0 && imageHeight > 0) {
            if (minImageWidth > 0 && minImageHeight > 0) {
                if (imageWidth < minImageWidth || imageHeight < minImageHeight) {
                    String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_dimensions_less_than_min_required_msg, minImageWidth,
                            minImageHeight, imageWidth, imageHeight);
                    displayToastMessage(context, msgToDisplay);
                    return false;
                }
            }

            if (maxImageWidth > 0 && maxImageHeight > 0) {
                if (imageWidth > maxImageWidth || imageHeight > maxImageHeight) {
                    String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_dimensions_more_than_max_allowed_msg, maxImageWidth,
                            maxImageHeight, imageWidth, imageHeight);
                    displayToastMessage(context, msgToDisplay);
                    return false;
                }
            }
            return true;
        } else {
            String msgToDisplay = context.getString(R.string.bsdk_image_selector_view_image_dimensions_invalid_msg);
            displayToastMessage(context, msgToDisplay);
            return false;
        }
    }

}
