/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2014 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.activities;

import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import com.adobe.creativesdk.foundation.IAdobeGenericCompletionCallback;
import com.adobe.creativesdk.foundation.IAdobeGenericErrorCallback;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloud;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloudManager;
import com.adobe.creativesdk.foundation.internal.utils.AdobeCSDKException;
import com.behance.sdk.R;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.adapters.BehanceSDKAdobeCloudSelectionListItemAdapter;

import java.util.ArrayList;

/**
 * @author Sujit Reddy G
 */

public class BehanceSDKAdobeCloudSelectionActivity extends FragmentActivity implements OnClickListener, IAdobeGenericCompletionCallback<ArrayList<AdobeCloud>>, IAdobeGenericErrorCallback<AdobeCSDKException>, AdapterView.OnItemClickListener {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKAdobeCloudSelectionActivity.class);

    public static final int ACTIVITY_RESULT_CODE_CLOUD_SELECTED = 1;
    public static final int ACTIVITY_RESULT_CODE_CANCELLED = 2;
    public static final int ACTIVITY_RESULT_CODE_ERROR = 3;

    private AdobeCloudManager sharedCloudManager;
    private ListView cloudsListView;
    private View progressSpinner;

    private boolean activityDestroyed = false;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        activityDestroyed = false;

        setContentView(R.layout.bsdk_activity_adobe_cloud_selection);

        sharedCloudManager = AdobeCloudManager.getSharedCloudManager();

        View closeViewBtn = findViewById(R.id.bsdkCloudSelectionViewBackBtnContainer);
        closeViewBtn.setOnClickListener(this);

        cloudsListView = (ListView) findViewById(R.id.bsdkCloudSelectionViewCloudsListView);
        cloudsListView.setOnItemClickListener(this);

        progressSpinner = findViewById(R.id.bsdkCloudSelectionViewProgressSpinner);
    }

    @Override
    protected void onResume() {
        super.onResume();
        loadAdobeClouds();
    }

    private void loadAdobeClouds() {
        showProgressSpinner();
        sharedCloudManager.refreshClouds(this, this, null);
    }

    private void showProgressSpinner() {
        if (progressSpinner != null) {
            cloudsListView.setVisibility(View.INVISIBLE);
            progressSpinner.setVisibility(View.VISIBLE);
        }
    }

    private void hideProgressSpinner() {
        if (progressSpinner != null) {
            cloudsListView.setVisibility(View.VISIBLE);
            progressSpinner.setVisibility(View.GONE);
        }
    }

    @Override
    public void onSaveInstanceState(Bundle arg0) {
        super.onSaveInstanceState(arg0);
    }

    @Override
    public void onClick(View clickedView) {
        if (clickedView.getId() == R.id.bsdkCloudSelectionViewBackBtnContainer) {
            setResult(ACTIVITY_RESULT_CODE_CANCELLED);
            closeThisView();
        }
    }

    @Override
    public void onBackPressed() {
        setResult(ACTIVITY_RESULT_CODE_CANCELLED);
        closeThisView();
    }

    private void closeThisView() {
        activityDestroyed = true;
        finish();
    }

    @Override
    public void onCompletion(final ArrayList<AdobeCloud> adobeClouds) {
        if (!activityDestroyed) {
            if (adobeClouds != null) {
                if (cloudsListView != null) {
                    cloudsListView.post(new Runnable() {
                        @Override
                        public void run() {
                            setCloudListAdapter(adobeClouds);
                        }
                    });
                }
            } else {
                handleCloudRetrievingError();
            }
        }
    }

    @Override
    public void onError(AdobeCSDKException e) {
        logger.error(e, "Problem retrieving Clouds");
        if (!activityDestroyed) {
            if (cloudsListView != null) {
                cloudsListView.post(new Runnable() {
                    @Override
                    public void run() {
                        handleCloudRetrievingError();
                    }
                });
            }
        }
    }

    private void setCloudListAdapter(ArrayList<AdobeCloud> adobeClouds) {
        if (adobeClouds != null) {
            AdobeCloud defaultCloud = sharedCloudManager.getDefaultCloud();
            BehanceSDKAdobeCloudSelectionListItemAdapter listItemArrayAdapter = new BehanceSDKAdobeCloudSelectionListItemAdapter(this, adobeClouds, defaultCloud);
            cloudsListView.setAdapter(listItemArrayAdapter);
            hideProgressSpinner();
        }
    }

    private void handleCloudRetrievingError() {
        Toast.makeText(this, R.string.bsdk_cloud_selection_view_loading_error_msg, Toast.LENGTH_LONG).show();
        setResult(ACTIVITY_RESULT_CODE_ERROR);
        closeThisView();
    }

    private void handleCloudOptionSelection(AdobeCloud selectedCloud) {
        if (selectedCloud != null) {
            sharedCloudManager.setDefaultCloud(selectedCloud);
        }
        setResult(ACTIVITY_RESULT_CODE_CLOUD_SELECTED);
        closeThisView();
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        AdobeCloud selectedCloud = (AdobeCloud) cloudsListView.getItemAtPosition(position);
        handleCloudOptionSelection(selectedCloud);
    }
}