package com.behance.sdk.ui.activities;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;

import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloud;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloudManager;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.ui.fragments.BehanceSDKELWFragment;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

/**
 * Created by ugs on 30/03/15.
 */
public abstract class BehanceSDKBasePublishActivity extends FragmentActivity
        implements BehanceSDKELWFragment.EnterpriseUserWarningDialogCallbacks {

    protected static final String SHARED_PREFERENCES_FILE_NAME = "BEHANCE_SDK_PUBLISH_ENTERPRISE_WARNING_SHARED_PREFERENES_KEY";

    protected static final String SP_KEY_ENTERPRISE_USER_WARNING_SHOWN = "SP_KEY_ENTERPRISE_USER_WARNING_SHOWN";

    private static final String KEY_ENTERPRISE_USER_WARNING_VIEW_VISIBLE = "KEY_ENTERPRISE_USER_WARNING_VIEW_VISIBLE";

    public static final int ACTIVITY_RESULT_REQUEST_CODE_CLOUD_SELECTION = 6778;

    private volatile boolean isWarningScreenVisible = false;

    private volatile boolean isCloudPickerLaunched = false;


    protected abstract IBehanceSDKUserCredentials getBehanceSDKUserCredentials();

    protected String getUserBehanceAccountId() {
        IBehanceSDKUserCredentials userCredentials = getBehanceSDKUserCredentials();
        return userCredentials.getUserBehanceAccountId();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putBoolean(KEY_ENTERPRISE_USER_WARNING_VIEW_VISIBLE, isWarningScreenVisible);
        outState.putBoolean(KEY_ENTERPRISE_USER_WARNING_VIEW_VISIBLE, isCloudPickerLaunched);
    }

    @Override
    protected void onCreate(Bundle arg0) {
        super.onCreate(arg0);
        initializeImageLoader();
        if(null != arg0){
            isWarningScreenVisible = arg0.getBoolean(KEY_ENTERPRISE_USER_WARNING_VIEW_VISIBLE, false);
            FragmentManager fragmentManager = getSupportFragmentManager();
            Fragment elwfragment = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ENTERPRISE_USER_WARNING_FRAGMENT);
            if (elwfragment instanceof BehanceSDKELWFragment) {
                ((BehanceSDKELWFragment) elwfragment).setCallbacks(this);
            }
        } else {
            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            if (sdkUserManager.isUserAnEnterpriseUser()) {
                SharedPreferences preferences = getSharedPreference();
                String preferenceKey = SP_KEY_ENTERPRISE_USER_WARNING_SHOWN + getUserBehanceAccountId();
                boolean alreadyShown = preferences.getBoolean(preferenceKey, false);
                if (false == alreadyShown) {
                    showEnterpriseUserWarningScreen(preferences, preferenceKey);
                }

                if(false == isWarningScreenVisible && false == isDefaultCloudSet()) {
                    launchCloudPicker();
                }
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == ACTIVITY_RESULT_REQUEST_CODE_CLOUD_SELECTION){
            isCloudPickerLaunched = false;
            continueWithWorkflow();
        }
    }

    private void launchCloudPicker(){
            isCloudPickerLaunched = true;
            Intent cloudSelectionActivityIntent = new Intent(this, BehanceSDKAdobeCloudSelectionActivity.class);
            startActivityForResult(cloudSelectionActivityIntent, ACTIVITY_RESULT_REQUEST_CODE_CLOUD_SELECTION);
    }

    private boolean isDefaultCloudSet(){
        AdobeCloudManager sharedCloudManager = AdobeCloudManager.getSharedCloudManager();
        AdobeCloud defaultCloud = sharedCloudManager.getDefaultCloud();
        if(null == defaultCloud)
            return false;
        return true;
    }

    public void continueWithWorkflow(){

    }

    private void showEnterpriseUserWarningScreen(SharedPreferences preferences, String preferenceKey) {
        isWarningScreenVisible = true;
        FragmentManager fragmentManager = getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        Fragment previousInstance = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ENTERPRISE_USER_WARNING_FRAGMENT);
        if (previousInstance != null) {
            fragmentTransaction.remove(previousInstance);
        }
        fragmentTransaction.addToBackStack(null);
        BehanceSDKELWFragment elwfragment = new BehanceSDKELWFragment();
        elwfragment.setCallbacks(this);
        elwfragment.show(fragmentTransaction, BehanceSDKFragmentTags.FRAGMENT_TAG_ENTERPRISE_USER_WARNING_FRAGMENT);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putBoolean(preferenceKey, true);
        editor.commit();
    }



    private SharedPreferences getSharedPreference() {
        ApplicationInfo application = getApplicationInfo();
        String accountFileName = SHARED_PREFERENCES_FILE_NAME + "_" + application.name;
        SharedPreferences sharedPreferences = getSharedPreferences(accountFileName, Context.MODE_PRIVATE);
        return sharedPreferences;
    }

    @Override
    public void onContinueButtonPressed() {
        isWarningScreenVisible = false;
        if(false == isDefaultCloudSet())
            launchCloudPicker();
        else
            continueWithWorkflow();
    }

    public boolean CanContinueWithWokflow(){
        return isWarningScreenVisible || isCloudPickerLaunched;
    }
    @Override
    public void onCancelButtonPressed() {
        finish();
    }

    @Override
    public void onBackButtonPressed() {
        finish();
    }

    private void initializeImageLoader() {
        // initialize ImageLoader
        if (!ImageLoader.getInstance().isInited()) {
            ImageLoaderConfiguration loaderConfiguration = ImageLoaderConfiguration.createDefault(this);
            ImageLoader.getInstance().init(loaderConfiguration);
        }
    }
}
