package com.behance.sdk.ui.activities;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Toast;

import com.adobe.creativesdk.foundation.internal.utils.AdobeCSDKException;
import com.adobe.creativesdk.foundation.storage.AdobeAssetException;
import com.adobe.creativesdk.foundation.storage.AdobeAssetFile;
import com.adobe.creativesdk.foundation.storage.AdobeAssetFileRenditionType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetImageDimensions;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMIMETypeFilter;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMIMETypeFilterType;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMimeTypes;
import com.adobe.creativesdk.foundation.storage.AdobeSelection;
import com.adobe.creativesdk.foundation.storage.AdobeSelectionAssetFile;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowser;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserConfiguration;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserOption;
import com.adobe.creativesdk.foundation.storage.IAdobeGenericRequestCallback;
import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageProperties;
import com.behance.sdk.factory.BehanceSDKImageValidatorFactory;
import com.behance.sdk.factory.IBehanceSDKImageValidator;
import com.behance.sdk.fragments.headless.BehanceSDKCCBrowserHeadlessFragment;
import com.behance.sdk.fragments.headless.BehanceSDKPublishWIPHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;
import com.behance.sdk.util.BehanceSDKUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

/**
 * Created by ugs on 18/06/15.
 */
public class BehanceSDKCCLauncherActivity extends FragmentActivity implements View.OnClickListener, BehanceSDKCCBrowserHeadlessFragment.ICreativeCloudAssetDownloadListener {

    public static final String ACTIVITY_RESULT_CC_SUCCESSFULLY_DOWNLOADED_FILES = "ACTIVITY_CC_SUCCESSFULLY_DOWNLOADED_FILES";
    public static final String ACTIVITY_RESULT_CC_DOWNLOAD_FAILED_FILES = "ACTIVITY_CC_DOWNLOAD_FAILED_FILES";
    public static final String ACTIVITY_RESULT_CC_INVALID_FILES = "ACTIVITY_CC_INVALID_FILES";

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKCCLauncherActivity.class);
    private static final String HEADLESS_FRAGMENT_TAG_CC_BROWSER = "HEADLESS_FRAGMENT_TAG_CC_BROWSER";

    private final int BEHANCE_SDK_ASSET_BROWSER_LAUNCH_REQUEST_CODE = 9786;

    public static final String ARGS_ALLOWED_MIME_TYPES = "ARGS_ALLOWED_MIME_TYPES";
    public static final String ARGS_ALLOWED_LIBRARY_TYPES = "ARGS_ALLOWED_LIBRARY_TYPES";

    public static final String ARGS_ASSET_BROWSER_OPTIONS = "ARGS_ASSET_BROWSER_OPTIONS";
    public static final String ARGS_IMAGE_VALIDATOR_TYPE = "ARGS_IMAGE_VALIDATOR_TYPE";


    private BehanceSDKGenericAlertDialog cancelDownloadAlertDialog;

    private BehanceSDKCCBrowserHeadlessFragment headlessFragment;




    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.bsdk_activity_cc_asset_browser_launcher);
        headlessFragment = (BehanceSDKCCBrowserHeadlessFragment) getSupportFragmentManager().findFragmentByTag(
                HEADLESS_FRAGMENT_TAG_CC_BROWSER);
        if (headlessFragment == null) {
            headlessFragment = new BehanceSDKCCBrowserHeadlessFragment();
            getSupportFragmentManager().beginTransaction().add(headlessFragment, HEADLESS_FRAGMENT_TAG_CC_BROWSER).commit();
        }
        headlessFragment.setDownloadListener(this);

        if(null == savedInstanceState){
            try {
                headlessFragment.launchCCAssetBrowser(this, getIntent());
            }catch (AdobeCSDKException e){
                //Need to add a toast message.
                logger.error(e, "Problem launching Creative Cloud Asset Browser");
            }
        }

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, final Intent data) {

            super.onActivityResult(requestCode, resultCode, data);
            if (requestCode == BEHANCE_SDK_ASSET_BROWSER_LAUNCH_REQUEST_CODE && resultCode == Activity.RESULT_OK && null != data) {
                headlessFragment.downloadSelectedFiles(data);
            } else {
                setResult(Activity.RESULT_CANCELED);
                closeActivity();
            }

    }

    private void closeActivity(){
        headlessFragment.removeDownloadListener();
        finish();

    }

    @Override
    public void onBackPressed() {
        if(null !=  headlessFragment && headlessFragment.isDownloadInProgress()) {
            cancelDownloadAlertDialog = BehanceSDKGenericAlertDialog.getInstance(this,
                    R.string.bsdk_cc_asset_browser_cancel_download_alert_dialog_title,
                    R.string.bsdk_cc_asset_browser_cancel_download_alert_dialog_text,
                    R.string.bsdk_generic_alert_dialog_ok_btn_label,
                    R.string.bsdk_generic_alert_dialog_cancel_btn_label);
            cancelDownloadAlertDialog.setOnOKBtnClickListener(this);
            cancelDownloadAlertDialog.setOnNotOKBtnClickListener(this);
            cancelDownloadAlertDialog.show();
        }
    }

    @Override
    public void onClick(View v) {

        int viewId = v.getId();
        if (viewId == R.id.bsdkGenericAlertDialogOKBtn)
            cancelDownload();
        else if (viewId == R.id.bsdkGenericAlertDialogNotOKBtn)
            closeAlertDialog();

    }

    private void cancelDownload(){
        headlessFragment.cancelDownload();
        setResult(Activity.RESULT_CANCELED);
        closeAlertDialog();
        closeActivity();

    }

    private void closeAlertDialog() {

        if(null != cancelDownloadAlertDialog)
            cancelDownloadAlertDialog.dismiss();

    }


    @Override
    public void onDownloadComplete(List<File> downloadedFiles, List<File> downloadError, List<File> invalidFiles) {

        Intent output = new Intent();
        output.putExtra(ACTIVITY_RESULT_CC_SUCCESSFULLY_DOWNLOADED_FILES, (Serializable) downloadedFiles);
        output.putExtra(ACTIVITY_RESULT_CC_DOWNLOAD_FAILED_FILES, (Serializable) downloadError);
        output.putExtra(ACTIVITY_RESULT_CC_INVALID_FILES, (Serializable) invalidFiles);
        setResult(Activity.RESULT_OK, output);
        closeActivity();
    }

    @Override
    public void onDownloadError(AdobeAssetException error){
        cancelDownload();
    }
}
