package com.behance.sdk.ui.activities;

import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.widget.Toast;
import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageValidatorFactory;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.fragments.BehanceSDKELWFragment;
import com.behance.sdk.ui.fragments.BehanceSDKImageSelectorDialogFragment;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.behance.sdk.util.BehanceSDKLaunchUtil;
import com.behance.sdk.util.BehanceSDKUtils;

import java.util.List;

public class BehanceSDKCreateProjectWFActivity extends BehanceSDKBasePublishActivity implements BehanceSDKImageSelectorDialogFragment.Callbacks,
        BehanceSDKELWFragment.EnterpriseUserWarningDialogCallbacks {

	private void displayNoInternetConnectivity() {
		Toast.makeText(this, getString(R.string.bsdk_connection_error_msg), Toast.LENGTH_LONG).show();
		finish();
	}

    @Override
    public void continueWithWorkflow() {
        launchImageSelectionDialog();
    }

    protected IBehanceSDKUserCredentials getBehanceSDKUserCredentials() {
        BehanceSDKProjectPublishWFManager workflowManager = BehanceSDKProjectPublishWFManager.getInstance();
        return workflowManager.getUserCredentials();
    }

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		BehanceSDK behanceSDK = BehanceSDK.getInstance();

		setContentView(R.layout.bsdk_activity_publish_project_with_image_selection);

		if(!behanceSDK.useBehanceTheme()  && !getResources().getBoolean(R.bool.isTablet)){
			setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
		}
		if (null == savedInstanceState) {
			// we want to check only the first time because user might have
			// added details and if he rotates his device when network
			// connectivity is lost, we don't want to close this view
			NetworkInfo networkInfo = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
			if (!(networkInfo != null && networkInfo.isConnected())) {
				displayNoInternetConnectivity();
			}
            if(false == CanContinueWithWokflow())
			    launchImageSelectionDialog();
		}else {

            FragmentManager fragmentManager = getSupportFragmentManager();
            Fragment enterpriseWarningFragmentInstance = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ENTERPRISE_USER_WARNING_FRAGMENT);
            if (enterpriseWarningFragmentInstance instanceof BehanceSDKELWFragment) {
                ((BehanceSDKELWFragment) enterpriseWarningFragmentInstance).setCallbacks(this);
            }

            Fragment imageChooserFragmentInstance = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ADD_PROJECT_ALBUM_SELECTION_FRAGMENT);
            if (imageChooserFragmentInstance instanceof BehanceSDKImageSelectorDialogFragment) {
                ((BehanceSDKImageSelectorDialogFragment) imageChooserFragmentInstance).setCallbacks(this);
            }
        }
	}

    public void launchImageSelectionDialog() {
		FragmentManager fragmentManager = getSupportFragmentManager();
		BehanceSDKImageSelectorOptions imageSelectionOptions = BehanceSDKImageSelectorOptions.getUnlimitedImageSelectionOptions();
		imageSelectionOptions.setMaxImageSizeInBytes(BehanceSDKConstants.PROJECT_IMAGE_MODULE_MAX_SIZE_BYTES);
		imageSelectionOptions.setAllowedFileExtensions(BehanceSDKUtils.getAllowedFileExtensionsForProjectImage());
		imageSelectionOptions.setImageValidatorType(BehanceSDKImageValidatorFactory.PUBLISH_PROJECT_IMAGE_VALIDATOR);
		BehanceSDKLaunchUtil.launchAlbumsViewToPickPicture(this, imageSelectionOptions, fragmentManager,
				BehanceSDKFragmentTags.FRAGMENT_TAG_ADD_PROJECT_ALBUM_SELECTION_FRAGMENT);
	}

	@Override
	public void onImageSelectorSelectionFinished(List<ImageModule> selectedImageModulesList) {
		BehanceSDKProjectPublishWFManager instance = BehanceSDKProjectPublishWFManager.getInstance();
		instance.addImagesSelectedForProject(selectedImageModulesList);
		startPublishProjectWorkflow();
	}

	private void startPublishProjectWorkflow() {
		Intent detailsIntent = new Intent(this, BehanceSDKPublishProjectActivity.class);
		startActivity(detailsIntent);
		finish();
	}

	@Override
	public void onImageSelectorSelectionCanceled() {
		startPublishProjectWorkflow();
	}

	@Override
	public void onImageSelectorSelectionBackPressed() {
        AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXCancel,
                AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);
		finish();
	}

    @Override
    public void onImageSelectorOnError(Exception e) {
        finish();
    }
}
