/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.activities;

import java.util.List;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKImageDimensions;
import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.BehanceSDKPublishWIPOptions;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.exception.BehanceSDKInvalidArgumentException;
import com.behance.sdk.exception.BehanceSDKInvalidImageException;
import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.exception.BehanceSDKUserNotEntitledException;
import com.behance.sdk.factory.BehanceSDKImageValidatorFactory;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.AddWIPManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.fragments.BehanceSDKELWFragment;
import com.behance.sdk.ui.fragments.BehanceSDKImageSelectorDialogFragment;
import com.behance.sdk.ui.fragments.BehanceSDKPublishWIPDialogFragment;
import com.behance.sdk.ui.fragments.BehanceSDKPublishWIPDialogFragment.Callbacks;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.behance.sdk.util.BehanceSDKUtils;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKCreateWIPWorkflowActivity extends BehanceSDKBasePublishActivity implements BehanceSDKImageSelectorDialogFragment.Callbacks, Callbacks,
        BehanceSDKELWFragment.EnterpriseUserWarningDialogCallbacks {

	private static final String FRAGMENT_TAG_PUBLISH_WIP = "FRAGMENT_TAG_CREATE_WIP_WORKFLOW_PUBLISH_WIP";
	private static final String FRAGMENT_TAG_SELECT_IMAGE = "FRAGMENT_TAG_CREATE_WIP_WORKFLOW_SELECT_IMAGE";

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKCreateWIPWorkflowActivity.class);

	private void displayNoInternetConnectivity() {
		Toast.makeText(this, getString(R.string.bsdk_add_wip_view_connection_error_msg), Toast.LENGTH_LONG).show();
		closeThisActivity(false);
	}
	private BehanceSDK behanceSDK = BehanceSDK.getInstance();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.bsdk_activity_create_wip_workflow);

		if(!behanceSDK.useBehanceTheme() && !getResources().getBoolean(R.bool.isTablet)){
			setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
		}

		if (savedInstanceState == null) {
            // we want to check only the first time because user might have
            // added details and if he rotates his device when network
            // connectivity is lost, we don't want to close this view
            NetworkInfo networkInfo = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
            if (!(networkInfo != null && networkInfo.isConnected())) {
                displayNoInternetConnectivity();
            }
            if(false == CanContinueWithWokflow())
                 launchAlbumsViewToPickPicture();
        } else {
            FragmentManager fragmentManager = getSupportFragmentManager();
            Fragment enterpriseWarningFragmentInstance = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ENTERPRISE_USER_WARNING_FRAGMENT);
            if (enterpriseWarningFragmentInstance instanceof BehanceSDKELWFragment) {
                ((BehanceSDKELWFragment) enterpriseWarningFragmentInstance).setCallbacks(this);
            }

			Fragment imageChooserFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_SELECT_IMAGE);
			if (imageChooserFragmentInstance instanceof BehanceSDKImageSelectorDialogFragment) {
				((BehanceSDKImageSelectorDialogFragment) imageChooserFragmentInstance).setCallbacks(this);
			}

			Fragment publishWIPFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_PUBLISH_WIP);
			if (publishWIPFragmentInstance instanceof BehanceSDKPublishWIPDialogFragment) {
				((BehanceSDKPublishWIPDialogFragment) publishWIPFragmentInstance).setCallbacks(this);
			}
		}
	}

    @Override
    public void continueWithWorkflow() {
        launchAlbumsViewToPickPicture();
    }

    protected IBehanceSDKUserCredentials getBehanceSDKUserCredentials() {
        AddWIPManager wipManager = AddWIPManager.getInstance();
        return wipManager.getUserCredentials();
    }

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
	}

	private void closeThisActivity(boolean overrideTransition) {
		finish();
		if (overrideTransition) {
			overridePendingTransition(0, R.anim.bsdk_publish_wip_info_view_exit);
		}
	}

	private void launchAlbumsViewToPickPicture() {
		BehanceSDKImageDimensions minRequiredWIPImageDimensions = BehanceSDKImageDimensions.getInstance(BehanceSDKConstants.WIP_IMAGE_MIN_WIDTH,
				BehanceSDKConstants.WIP_IMAGE_MIN_HEIGHT);
		BehanceSDKImageSelectorOptions imageSelectionOptions = BehanceSDKImageSelectorOptions.getSingleImageSelectionOptions(
				minRequiredWIPImageDimensions, null);
		imageSelectionOptions.setMaxImageSizeInBytes(BehanceSDKConstants.WIP_IMAGE_MAX_SIZE_BYTES);
		imageSelectionOptions.setAllowedFileExtensions(BehanceSDKUtils.getAllowedFileExtensionsForWIPImage());
		imageSelectionOptions.setImageValidatorType(BehanceSDKImageValidatorFactory.PUBLISH_WIP_IMAGE_VALIDATOR);

		BehanceSDKImageSelectorDialogFragment selectImageDialog = BehanceSDKImageSelectorDialogFragment.getInstance(this, imageSelectionOptions);
		FragmentManager fragmentManager = getSupportFragmentManager();
		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
		Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_SELECT_IMAGE);
		if (previousInstance != null) {
			fragmentTransaction.remove(previousInstance);
		}
		fragmentTransaction.addToBackStack(null);
		selectImageDialog.show(fragmentTransaction, FRAGMENT_TAG_SELECT_IMAGE);
	}

	private void launchPublishView(ImageModule selectedImageModule) {
		BehanceSDKPublishWIPOptions publishOptions = new BehanceSDKPublishWIPOptions(selectedImageModule);
		AddWIPManager wipManager = AddWIPManager.getInstance();
		publishOptions.setNotificationHandlerActivityClass(wipManager.getNotificationHandlerActivityClass());
		/*BehanceSDKPublishWIPDialogFragment publishWIPDialog = BehanceSDKPublishWIPDialogFragment.getInstance(this, publishOptions);
		FragmentManager fragmentManager = getSupportFragmentManager();
		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
		Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_PUBLISH_WIP);
		if (previousInstance != null) {
			fragmentTransaction.remove(previousInstance);
		}
		fragmentTransaction.addToBackStack(null);
		publishWIPDialog.show(fragmentTransaction, FRAGMENT_TAG_PUBLISH_WIP);*/
		BehanceSDK behanceSDK = BehanceSDK.getInstance();
		try {
			behanceSDK.launchPublishWIPView(this, publishOptions);
			finish();
		} catch (BehanceSDKInvalidImageException e) {
			e.printStackTrace();
		} catch (BehanceSDKInvalidArgumentException e) {
			e.printStackTrace();
		} catch (BehanceSDKUserNotEntitledException e) {
			e.printStackTrace();
		} catch (BehanceSDKUserNotAuthenticatedException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void onImageSelectorSelectionFinished(List<ImageModule> imageModulesList) {
		if (imageModulesList != null && !imageModulesList.isEmpty()) {
			ImageModule imageModule = imageModulesList.get(0);
			logger.debug("Image selected from album. [Name - %s]", imageModule.getName());
			launchPublishView(imageModule);
		}
	}

	@Override
	public void onPublishWIPViewClose() {
		closeThisActivity(true);
	}

	@Override
	public void onImageSelectorSelectionCanceled() {
		closeThisActivity(false);
	}

	@Override
	public void onImageSelectorSelectionBackPressed() {
		closeThisActivity(false);
	}

    @Override
    public void onImageSelectorOnError(Exception e) {
        closeThisActivity(false);
    }

}