package com.behance.sdk.ui.activities;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.R;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserProfileTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKEditProfileTaskResult;
import com.behance.sdk.dto.BehanceSDKImageCropperDTO;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.fragments.headless.BehanceSDKEditProfileHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKEditProfileManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKLocationSelectionDialog;
import com.behance.sdk.ui.fragments.BehanceSDKImageCropperDialogFragment;
import com.behance.sdk.ui.fragments.BehanceSDKImageSelectorDialogFragment;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.behance.sdk.util.BehanceSDKUtils;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

import java.util.List;

/**
 * Created by forta on 9/15/14.
 */
public class BehanceSDKEditProfileActivity extends FragmentActivity implements BehanceSDKImageSelectorDialogFragment.Callbacks, View.OnClickListener,
        TextWatcher, BehanceSDKEditProfileHeadlessFragment.Callbacks, BehanceSDKLocationSelectionDialog.Callbacks, BehanceSDKImageCropperDialogFragment.Callbacks {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKEditProfileActivity.class);

    public static final String INTENT_EXTRA_BOOLEAN_ENABLE_ALTERNATE_IMAGE_SELECTION_SOURCES = "INTENT_EXTRA_BOOLEAN_ENABLE_ALTERNATE_IMAGE_SELECTION_SOURCES";

    private static final String FRAGMENT_TAG_SELECT_IMAGE = "FRAGMENT_TAG_EDIT_PROFILE_SELECT_IMAGE";
    private static final String HEADLESS_FRAGMENT_TAG_EDIT_PROFILE = "EDIT_PROFILE_HEADLESS_FRAGMENT_TAG";
    private static final String FRAGMENT_TAG_CROP_IMAGE = "FRAGMENT_TAG_EDIT_PROFILE_CROP_IMAGE";
    private static final String FRAGMENT_TAG_EDIT_PROFILE_LOCATION = "FRAGMENT_TAG_EDIT_PROFILE_LOCATION";

    private BehanceSDKEditProfileHeadlessFragment editProfileHeadlessFragment;
    private View progressBar;
    private EditText firstNameET;
    private EditText lastNameET;
    private EditText occupationET;
    private EditText companyET;
    private EditText websiteET;
    private ImageView avatar;
    private ImageLoader mLoader;
    private TextView location;
    private TextView saveNavBtn;

    private BehanceSDKGenericAlertDialog unsavedChangesAlertDialog;

    private boolean profileChanged = false;
    private boolean enableAlternateImageSelectionSources = true;
    private BehanceSDK behanceSDK = BehanceSDK.getInstance();

    @Override
    public void onStart() {
        super.onStart();
        NetworkInfo networkInfo = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
        if (!(networkInfo != null && networkInfo.isConnected())) {
            displayNoInternetConnectivity();
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.bsdk_activity_edit_profile);

        if(!behanceSDK.useBehanceTheme() && !getResources().getBoolean(R.bool.isTablet)){
            setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
        }

        Intent intent = getIntent();
        if (intent != null) {
            enableAlternateImageSelectionSources = intent.getBooleanExtra(INTENT_EXTRA_BOOLEAN_ENABLE_ALTERNATE_IMAGE_SELECTION_SOURCES, true);
        } else {
            enableAlternateImageSelectionSources = true;
        }

        View backNavBtn = findViewById(R.id.bsdkEditProfileActionbarBackBtnImageView);
        backNavBtn.setOnClickListener(this);

        saveNavBtn = (TextView) findViewById(R.id.bsdkEditProfileActionbarActionBtnTxtView);
        saveNavBtn.setOnClickListener(this);

        progressBar = findViewById(R.id.bsdkEditProfileLoader);

        firstNameET = (EditText) findViewById(R.id.bsdkEditProfileFirstName);
        lastNameET = (EditText) findViewById(R.id.bsdkEditProfileLastName);
        occupationET = (EditText) findViewById(R.id.bsdkEditProfileOccupation);
        companyET = (EditText) findViewById(R.id.bsdkEditProfileCompany);
        websiteET = (EditText) findViewById(R.id.bsdkEditProfileWebsite);
        avatar = (ImageView) findViewById(R.id.bsdkEditProfileAvatar);
        location = (TextView) findViewById(R.id.bsdkEditProfileLocation);

        mLoader = getImageLoaderInstance();

        editProfileHeadlessFragment = (BehanceSDKEditProfileHeadlessFragment) getSupportFragmentManager().findFragmentByTag(HEADLESS_FRAGMENT_TAG_EDIT_PROFILE);
        if (editProfileHeadlessFragment == null) {
            editProfileHeadlessFragment = new BehanceSDKEditProfileHeadlessFragment();
            getSupportFragmentManager().beginTransaction().add(editProfileHeadlessFragment, HEADLESS_FRAGMENT_TAG_EDIT_PROFILE).commit();

            editProfileHeadlessFragment.loadUserDetailsFromServer();
        } else {
            initializeFields();
            hideProgressBar();
        }
        editProfileHeadlessFragment.setCallbacks(this);

        FragmentManager fragmentManager = getSupportFragmentManager();
        Fragment imageChooserFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_SELECT_IMAGE);
        if (imageChooserFragmentInstance instanceof BehanceSDKImageSelectorDialogFragment) {
            ((BehanceSDKImageSelectorDialogFragment) imageChooserFragmentInstance).setCallbacks(this);
        }
        Fragment imageCropperFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_CROP_IMAGE);
        if (imageCropperFragmentInstance instanceof BehanceSDKImageCropperDialogFragment) {
            ((BehanceSDKImageCropperDialogFragment) imageCropperFragmentInstance).setCallbacks(this);
        }
        Fragment locationFilterFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_EDIT_PROFILE_LOCATION);
        if (locationFilterFragmentInstance instanceof BehanceSDKLocationSelectionDialog) {
            ((BehanceSDKLocationSelectionDialog) locationFilterFragmentInstance).setCallbacks(this);
        }

        if (editProfileHeadlessFragment.isGetUserProfileTaskInProgress() || editProfileHeadlessFragment.isEditProfileTaskInProgress())
            showProgressBar();
    }


    private ImageLoader getImageLoaderInstance() {
        // initialize ImageLoader
        ImageLoader instance = ImageLoader.getInstance();
        if (!instance.isInited()) {
            ImageLoaderConfiguration loaderConfiguration = ImageLoaderConfiguration.createDefault(this);
            instance.init(loaderConfiguration);
        }
        return instance;
    }

    @Override
    public void onBackPressed() {
        if (profileChanged) {
            unsavedChangesAlertDialog = BehanceSDKGenericAlertDialog.getInstance(this,
                    R.string.bsdk_edit_profile_unsaved_changes_title, R.string.bsdk_edit_profile_unsaved_changes_body,
                    R.string.bsdk_generic_alert_dialog_ok_btn_label, R.string.bsdk_generic_alert_dialog_cancel_btn_label);
            unsavedChangesAlertDialog.setOnOKBtnClickListener(this);
            unsavedChangesAlertDialog.setOnNotOKBtnClickListener(this);
            unsavedChangesAlertDialog.show();
        } else {
            super.onBackPressed();
        }
    }

    private void displayNoInternetConnectivity() {
        Toast.makeText(this, getString(R.string.bsdk_add_wip_view_connection_error_msg), Toast.LENGTH_LONG).show();
        closeThisActivity();
    }

    private void closeThisActivity() {
        finish();
    }

    private void showProgressBar() {
        if (progressBar != null)
            progressBar.setVisibility(View.VISIBLE);
    }

    private void hideProgressBar() {
        if (progressBar != null)
            progressBar.setVisibility(View.GONE);
    }

    private void launchAlbumsViewToPickPicture() {
        BehanceSDKImageSelectorOptions imageSelectionOptions = BehanceSDKImageSelectorOptions.getSingleImageSelectionOptions();
        if (!enableAlternateImageSelectionSources) {
            imageSelectionOptions.hideImageSelectionSources();
        }
        imageSelectionOptions.setAllowedFileExtensions(BehanceSDKUtils.getAllowedFileExtensionsForProfileImage());
        BehanceSDKImageSelectorDialogFragment selectImageDialog = BehanceSDKImageSelectorDialogFragment.getInstance(this,
                imageSelectionOptions);
        FragmentManager fragmentManager = getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_SELECT_IMAGE);
        if (previousInstance != null) {
            fragmentTransaction.remove(previousInstance);
        }
        fragmentTransaction.addToBackStack(null);
        selectImageDialog.show(fragmentTransaction, FRAGMENT_TAG_SELECT_IMAGE);
    }

    @Override
    public void onImageSelectorSelectionFinished(List<ImageModule> selectedImageModulesList) {
        if (selectedImageModulesList != null && !selectedImageModulesList.isEmpty()) {
            ImageModule imageModule = selectedImageModulesList.get(0);
            BehanceSDKImageCropperDTO cropperDTO = new BehanceSDKImageCropperDTO();
            cropperDTO.setHeight(276);
            cropperDTO.setWidth(276);
            cropperDTO.setMinSize(276);
            cropperDTO.setImageModule(imageModule);
            cropperDTO.setIncludeControls(true);
            BehanceSDKImageCropperDialogFragment cropImageDialog = BehanceSDKImageCropperDialogFragment.getInstance(cropperDTO, this);
            FragmentManager fragmentManager = getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_CROP_IMAGE);
            if (previousInstance != null) {
                fragmentTransaction.remove(previousInstance);
            }
            fragmentTransaction.addToBackStack(null);
            cropImageDialog.show(fragmentTransaction, FRAGMENT_TAG_CROP_IMAGE);
            editProfileHeadlessFragment.setImage(imageModule);
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        //No call for super(). Bug on API Level > 11.
    }

    @Override
    public void onImageSelectorSelectionCanceled() {

    }

    @Override
    public void onImageSelectorSelectionBackPressed() {

    }

    @Override
    public void onImageSelectorOnError(Exception e) {
    }

    @Override
    public void onClick(View v) {
        int i = v.getId();
        if (i == R.id.bsdkEditProfileActionbarBackBtnImageView)
            onBackPressed();
        else if (i == R.id.bsdkEditProfileActionbarActionBtnTxtView)
            saveChanges();
        else if (i == R.id.bsdkEditProfileAvatar) {
            if(BehanceSDKPermissionHelper.checkPermission(this, BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW)) {
                launchAlbumsViewToPickPicture();
            }
            else{
                if (Build.VERSION.SDK_INT >= 23) {
                    // Marshmallow+
                    requestPermissions(BehanceSDKPermissionHelper.getPermissionFromRequestCode(BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW,this),BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW);
                }
            }
        }
        else if (i == R.id.bsdkGenericAlertDialogOKBtn)
            closeThisActivity();
        else if (i == R.id.bsdkGenericAlertDialogNotOKBtn)
            closeUnsavedChangesAlertDialog();
        else if (i == R.id.bsdkEditProfileLocation)
            handleLocationClick();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode,permissions,grantResults);
        switch (requestCode) {
            case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW : {
                if((grantResults.length == 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
                    launchAlbumsViewToPickPicture();
                } else {
                    int messageId = R.string.behance_sdk_permissions_error_generic;
                    //Permission denied! Show some dialog about how it will limits the functionality!
                    Toast.makeText(this, this.getString(messageId), Toast.LENGTH_LONG).show();
                }
            }
            break;
        }
    }

    private void closeUnsavedChangesAlertDialog() {
        if (unsavedChangesAlertDialog != null)
            unsavedChangesAlertDialog.dismiss();
    }

    private void handleLocationClick() {
        FragmentManager fragmentManager = getSupportFragmentManager();
        BehanceSDKLocationSelectionDialog instance = BehanceSDKLocationSelectionDialog.getInstance();
        Bundle bundle = new Bundle();
        setSelectedLocation(bundle);
        instance.setArguments(bundle);
        instance.setCallbacks(this);
        instance.show(fragmentManager, FRAGMENT_TAG_EDIT_PROFILE_LOCATION);
    }

    private void setSelectedLocation(Bundle bundle) {
        if (null != editProfileHeadlessFragment.getCountry()) {
            bundle.putSerializable(BehanceSDKLocationSelectionDialog.INSTANCE_STATE_KEY_SELECTED_COUNTRY, editProfileHeadlessFragment.getCountry());
            if (null != editProfileHeadlessFragment.getState())
                bundle.putSerializable(BehanceSDKLocationSelectionDialog.INSTANCE_STATE_KEY_SELECTED_STATE, editProfileHeadlessFragment.getState());
            if (null != editProfileHeadlessFragment.getCity())
                bundle.putSerializable(BehanceSDKLocationSelectionDialog.INSTANCE_STATE_KEY_SELECTED_CITY, editProfileHeadlessFragment.getCity());

        }
    }

    private void saveChanges() {
        if (profileChanged) {
            String firstNameStr = firstNameET.getText().toString();
            String lastNameStr = lastNameET.getText().toString();
            String occupationStr = occupationET.getText().toString();
            String companyStr = companyET.getText().toString();
            String websiteStr = websiteET.getText().toString();
            boolean valid_url = true;
            try {
                websiteStr = websiteStr.toLowerCase().replaceFirst("^(http://)?(www\\.)?", "");
                websiteStr = websiteStr.toLowerCase().replaceFirst("^(https://)?(www\\.)?", "");
                websiteStr = websiteStr.toLowerCase().replaceFirst("^(www\\.)?", "");
            } catch (Exception e) {
                valid_url = false;
            }

            editProfileHeadlessFragment.setFirstName(firstNameStr);
            editProfileHeadlessFragment.setLastName(lastNameStr);
            editProfileHeadlessFragment.setOccupation(occupationStr);
            editProfileHeadlessFragment.setCompany(companyStr);
            editProfileHeadlessFragment.setWebsite(websiteStr);

            boolean allFieldsValid = true;
            int msgToDisplayResourceId = R.string.bsdk_edit_profile_avatar_required_fields_missing_msg;

            if (isStringEmpty(firstNameStr) && isStringEmpty(lastNameStr)) {
                allFieldsValid = false;
                msgToDisplayResourceId = R.string.bsdk_edit_profile_avatar_required_fields_missing_msg;
            } else if (isStringEmpty(firstNameStr)) {
                allFieldsValid = false;
                msgToDisplayResourceId = R.string.bsdk_edit_profile_avatar_first_name_field_missing_msg;
            } else if (isStringEmpty(lastNameStr)) {
                allFieldsValid = false;
                msgToDisplayResourceId = R.string.bsdk_edit_profile_avatar_last_name_field_missing_msg;
            } else if (false == editProfileHeadlessFragment.isValidLocation()) {
                allFieldsValid = false;
                msgToDisplayResourceId = R.string.bsdk_location_filter_dialog_city_empty_error;
            } else if (!valid_url) {
                allFieldsValid = false;
                msgToDisplayResourceId = R.string.bsdk_edit_profile_website_url_invalid_msg;
            }

            if (allFieldsValid) {
                showProgressBar();
                saveNavBtn.setTextColor(getResources().getColor(R.color.bsdk_adobe_blue_disabled));
                editProfileHeadlessFragment.saveUserPortfolioChanges();
            } else {
                Toast.makeText(this, msgToDisplayResourceId, Toast.LENGTH_LONG).show();
            }
        }
    }

    private boolean isStringEmpty(String str) {
        if (str != null && str.trim().length() > 0) {
            return false;
        } else {
            return true;
        }
    }

    private void initializeFields() {
        firstNameET.setText(editProfileHeadlessFragment.getFirstName());
        lastNameET.setText(editProfileHeadlessFragment.getLastName());
        occupationET.setText(editProfileHeadlessFragment.getOccupation());
        companyET.setText(editProfileHeadlessFragment.getCompany());
        websiteET.setText(editProfileHeadlessFragment.getWebsite());

        String locationText = "";
        if (editProfileHeadlessFragment.getCountry() != null)
            locationText += editProfileHeadlessFragment.getCountry().getDisplayName();
        if (editProfileHeadlessFragment.getState() != null)
            locationText += "/" + editProfileHeadlessFragment.getState().getId();
        if (editProfileHeadlessFragment.getCity() != null)
            locationText += "/" + editProfileHeadlessFragment.getCity().getDisplayName();
        location.setText(locationText);

        if (editProfileHeadlessFragment.getImageBitmap() == null)
            mLoader.displayImage(editProfileHeadlessFragment.getOriginalImageUrl(), avatar);
        else
            avatar.setImageBitmap(editProfileHeadlessFragment.getImageBitmap());

        firstNameET.addTextChangedListener(this);
        lastNameET.addTextChangedListener(this);
        occupationET.addTextChangedListener(this);
        companyET.addTextChangedListener(this);
        websiteET.addTextChangedListener(this);
        avatar.setOnClickListener(this);
        location.setOnClickListener(this);
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {

    }

    @Override
    public void afterTextChanged(Editable s) {
        if (!profileChanged) {
            profileChanged = true;
            saveNavBtn.setTextColor(getResources().getColor(R.color.bsdk_adobe_blue));
        }
    }

    @Override
    public void onGetUserProfileSuccess() {
        initializeFields();
        hideProgressBar();
    }

    @Override
    public void onGetUserProfileFailure(Exception e, BehanceSDKGetUserProfileTaskParams params) {
        logger.error(e, "Problem retrieving user Profile details");
        Toast.makeText(this, getResources().getString(R.string.bsdk_edit_profile_loading_error), Toast.LENGTH_LONG).show();
        closeThisActivity();
    }

    @Override
    public void onEditProfileSuccess(BehanceSDKEditProfileTaskResult result) {
        BehanceSDKEditProfileManager.getInstance().onEditProfileSuccess();
        closeThisActivity();
    }

    @Override
    public void onEditProfileFailure(BehanceSDKEditProfileTaskResult result) {
        List<Exception> resultExceptions = result.getExceptions();
        if (resultExceptions != null) {
            for (Exception e : resultExceptions) {
                logger.error(e, "Problem updating user profile");
            }
        }
        if (result.isProfileUpdateFailed()) {
            Toast.makeText(this, getResources().getString(R.string.bsdk_edit_profile_editing_error), Toast.LENGTH_LONG).show();
        } else if (result.isProfileAvatarUpdateFailed()) {
            Toast.makeText(this, getResources().getString(R.string.bsdk_edit_profile_avatar_saving_error), Toast.LENGTH_LONG).show();
            // profile details are updated, so need to refresh
            BehanceSDKEditProfileManager.getInstance().onEditProfileSuccess();
        } else {
            // showing default message
            Toast.makeText(this, getResources().getString(R.string.bsdk_edit_profile_editing_error), Toast.LENGTH_LONG).show();
        }
        hideProgressBar();
        closeThisActivity();
    }

    @Override
    public void setProfileEdited(boolean edited) {
        profileChanged = edited;
        saveNavBtn.setTextColor(getResources().getColor(edited ? R.color.bsdk_adobe_blue : R.color.bsdk_adobe_blue_disabled));
    }

    @Override
    public void onImageCropped(Bitmap bitmap) {
        editProfileHeadlessFragment.setImageBitmap(bitmap);
        avatar.setImageBitmap(bitmap);
        if (!profileChanged) {
            profileChanged = true;
            saveNavBtn.setTextColor(getResources().getColor(R.color.bsdk_adobe_blue));
        }
    }

    @Override
    public void onLocationFilterDone(BehanceSDKCountryDTO selectedCountry, BehanceSDKStateDTO selectedState, BehanceSDKCityDTO selectedCity) {
        String locationText = "";
        if (selectedCountry != null) {
            editProfileHeadlessFragment.setCountry(selectedCountry);
            locationText += selectedCountry.getDisplayName();
            if (!profileChanged) {
                profileChanged = true;
                saveNavBtn.setTextColor(getResources().getColor(R.color.bsdk_adobe_blue));
            }
        }
        if (selectedState != null) {
            editProfileHeadlessFragment.setState(selectedState);
            locationText += "/" + selectedState.getId();
        } else {
            // selected state can change to null, if the selected country is not US
            editProfileHeadlessFragment.setState(null);
        }

        if (selectedCity != null) {
            editProfileHeadlessFragment.setCity(selectedCity);
            locationText += "/" + selectedCity.getDisplayName();
        }
        location.setText(locationText);
    }

}
