/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.activities;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.fragments.headless.BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.ui.fragments.BehanceSDKProjectDetailFragment;

public class BehanceSDKProjectDetailActivity extends AppCompatActivity implements BehanceSDKProjectDetailFragment.Callbacks , BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment.Callbacks  {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKProjectDetailActivity.class);


	private static final String FRAGMENT_TAG_PROJECT_DETAIL = "FRAGMENT_PROJECT_DETAIL";
	private BehanceSDKProjectDTO activeProject;
	private BehanceSDKProjectDetailFragment projectDetailFragment;
	private BehanceSDK behanceSDK = BehanceSDK.getInstance();

	private static final String FRAGMENT_TAG_RETRIEVE_BEHANCE_USER_DETAILS_USING_ADOBE_ID = "FRAGMENT_TAG_RETRIEVE_BEHANCE_USER_DETAILS_USING_ADOBE_ID";
	private static BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment retrieveBehanceUserDetailHeadlessFragment;

	String projectId;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.bsdk_activity_project_detail);
		if(!behanceSDK.useBehanceTheme() && !getResources().getBoolean(R.bool.isTablet)){
			setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
		}

		projectId = getIntent().getStringExtra(BehanceSDKProjectDetailFragment.ARG_PROJECT_ID);
		retrieveBehanceUserDetailsUsingAdobeId();
		if (savedInstanceState == null) {
			loadProjectDetailFragment();
		}

	}

	public BehanceSDKProjectDTO getActiveProject() {
		return activeProject;
	}

	public void setActiveProject(BehanceSDKProjectDTO activeProject) {
		this.activeProject = activeProject;
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	public void onFailureToLoadProjectDetails(Exception e) {
		// Might come handy in the future
	}

	@Override
	public void onSuccessfulLoadOfProjectDetails(BehanceSDKProjectDTO activeProject) {
		setActiveProject(activeProject);
	}

	private void loadProjectDetailFragment() {
		if (projectId != null) {
			Bundle arguments = new Bundle();
			arguments.putString(BehanceSDKProjectDetailFragment.ARG_PROJECT_ID, projectId);
			projectDetailFragment = new BehanceSDKProjectDetailFragment();
			projectDetailFragment.setArguments(arguments);
			getSupportFragmentManager().beginTransaction().add(R.id.item_detail_container, projectDetailFragment, FRAGMENT_TAG_PROJECT_DETAIL)
					.commit();
		} else {
			Toast.makeText(this, getResources().getString(R.string.bsdk_project_detail_fragment_problem_loading_project_details_msg), Toast.LENGTH_SHORT).show();
			finish();
		}
	}

	private void removeProjectDetailFragment() {
		FragmentManager supportFragmentManager = getSupportFragmentManager();
		FragmentTransaction fragmentTransaction = supportFragmentManager.beginTransaction();

		Fragment activityFragment = supportFragmentManager.findFragmentByTag(FRAGMENT_TAG_PROJECT_DETAIL);
		if (activityFragment != null) {
			fragmentTransaction.remove(activityFragment);
		}
		fragmentTransaction.commit();
	}

	private void retrieveBehanceUserDetailsUsingAdobeId() {
		retrieveBehanceUserDetailHeadlessFragment = (BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment) getSupportFragmentManager().findFragmentByTag(FRAGMENT_TAG_RETRIEVE_BEHANCE_USER_DETAILS_USING_ADOBE_ID);
		if (retrieveBehanceUserDetailHeadlessFragment == null) {
			retrieveBehanceUserDetailHeadlessFragment = new BehanceSDKRetrieveUserDetailsFromBehanceHeadlessFragment();
			getSupportFragmentManager().beginTransaction().add(retrieveBehanceUserDetailHeadlessFragment, FRAGMENT_TAG_RETRIEVE_BEHANCE_USER_DETAILS_USING_ADOBE_ID).commit();
		}
		retrieveBehanceUserDetailHeadlessFragment.setCallbacks(this);
		BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams params = new BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams();
		retrieveBehanceUserDetailHeadlessFragment.retrieveBehanceUserDetailsUsingAdobeID(params);
	}

	@Override
	public void onRetrieveBehanceUserDetailsSuccess(BehanceSDKUserDTO userDTO) {
		{
			if (userDTO != null && userDTO.getId() > 0) {
				BehanceSDKUserManager.getInstance().setUserDTO(userDTO);
			} else {
				logger.error("Problem retrieving Behance user details using Adobe ID");
			}
		}
	}

	@Override
	public void onRetrieveBehanceUserDetailsFailure(Exception e) {

	}
}
