package com.behance.sdk.ui.activities;

import java.util.List;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.view.View;
import android.widget.Toast;
import android.widget.ViewFlipper;

import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.listeners.IBehanceSDKPublishProjectNavigationListener;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.fragments.BehanceSDKImageSelectorDialogFragment.Callbacks;
import com.behance.sdk.ui.fragments.BehanceSDKPublishProjectCoverFragment;
import com.behance.sdk.ui.fragments.BehanceSDKPublishProjectPreviewFragment;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.behance.sdk.util.BehanceSDKLaunchUtil;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.behance.sdk.util.BehanceSDKUtils;

public class BehanceSDKPublishProjectActivity extends BehanceSDKBasePublishActivity implements IBehanceSDKPublishProjectNavigationListener, Callbacks {

	private static final String BUNDLE_KEY_CURRENT_PAGE_NUMBER = "BUNDLE_KEY_CURRENT_PAGE_NUMBER";

	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_IMAGE_MODULES = "INTENT_SERIALIZABLE_EXTRA_PROJECT_IMAGE_MODULES";
	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_TITLE = "INTENT_SERIALIZABLE_EXTRA_PROJECT_TITLE";
	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_DESC = "INTENT_SERIALIZABLE_EXTRA_PROJECT_DESC";
	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_TAGS = "INTENT_SERIALIZABLE_EXTRA_PROJECT_TAGS";
	public static final String INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED = "INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED";

	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_COPYRIGHT_SETTINGS = "INTENT_SERIALIZABLE_EXTRA_PROJECT_COPYRIGHT_SETTINGS";
	public static final String INTENT_SERIALIZABLE_EXTRA_PROJECT_CONTAINS_ADULT_CONTENT = "INTENT_SERIALIZABLE_EXTRA_PROJECT_CONTAINS_ADULT_CONTENT";


	private ViewFlipper publishProjectScreens;
	private boolean requestingPermission = false;
	private BehanceSDK behanceSDK = BehanceSDK.getInstance();


	@Override
	public void continueWithWorkflow() {
		publishProjectScreens.setVisibility(View.VISIBLE);
	}

	protected IBehanceSDKUserCredentials getBehanceSDKUserCredentials() {
		BehanceSDKProjectPublishWFManager workflowManager = BehanceSDKProjectPublishWFManager.getInstance();
		return workflowManager.getUserCredentials();
	}

	@Override
	protected void onCreate(Bundle arg0) {
		super.onCreate(arg0);
		if(!behanceSDK.useBehanceTheme()  && !getResources().getBoolean(R.bool.isTablet)){
			setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
		}
		addValidImageModulesToWFManager();
		addProjectDetailsToWFManager();
		try {
			setContentView(R.layout.bsdk_activity_publish_project);
		}catch (Exception e)
		{
			finish();
		}
		overridePendingTransition(R.anim.bsdk_publish_wip_info_view_enter, 0);
		publishProjectScreens = (ViewFlipper) findViewById(R.id.bsdkPublishProjectViewFlipper);
		if(null != arg0){
			int currentPage = arg0.getInt(BUNDLE_KEY_CURRENT_PAGE_NUMBER, 0);
			publishProjectScreens.setDisplayedChild(currentPage);
		}
		if(CanContinueWithWokflow()){
			publishProjectScreens.setVisibility(View.INVISIBLE);
		}
	}

	@Override
	protected void onStart() {
		super.onStart();
		getWindow().setBackgroundDrawableResource(R.color.bsdk_publish_project_view_bg_color);
	}

	private void showNextScreen() {
		if(publishProjectScreens.getDisplayedChild() != publishProjectScreens.getChildCount()-1){
			addShowNextAnimations();
			publishProjectScreens.showNext();
		}

	}

	public void closeThisActivity(boolean overrideTransition) {
		finish();
		if (overrideTransition) {
			overridePendingTransition(0, R.anim.bsdk_publish_wip_info_view_exit);
		}
	}



	private void showPreviousScreen() {
		onBackPressed();
	}

	private void addShowNextAnimations() {
		publishProjectScreens.setInAnimation(this, R.anim.bsdk_publish_project_open_enter_anim);
		publishProjectScreens.setOutAnimation(this, R.anim.bsdk_publish_project_open_exit_anim);

	}

	private void addShowPreviousAnimations() {
		publishProjectScreens.setInAnimation(this, R.anim.bsdk_publish_project_close_enter_anim);
		publishProjectScreens.setOutAnimation(this, R.anim.bsdk_publish_project_close_exit_anim);

	}

	@Override
	public void showNext() {
		showNextScreen();
	}

	@Override
	public void showPrevious() {
		showPreviousScreen();
	}

	public void launchImageSelectionDialogForEditScreen() {
		if(BehanceSDKPermissionHelper.checkPermission(this,BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW)) {
			launchImageSelectionDialogForEditScreenCommon();
		}
		else{
			if (Build.VERSION.SDK_INT >= 23) {
				// Marshmallow+
				requestingPermission = true;
				requestPermissions(BehanceSDKPermissionHelper.getPermissionFromRequestCode(BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW,this),BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW);
			}
		}
	}

	public void requestPermisssionsForCamera()
	{
		if (Build.VERSION.SDK_INT >= 23) {
			// Marshmallow+
			requestingPermission = true;
			requestPermissions(BehanceSDKPermissionHelper.getPermissionFromRequestCode(BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA,this), BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA);
		}
	}

	public void requestPermisssionsForAlbumsView()
	{
		if (Build.VERSION.SDK_INT >= 23) {
			// Marshmallow+
			requestingPermission = true;
			requestPermissions(BehanceSDKPermissionHelper.getPermissionFromRequestCode(BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT,this), BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT);
		}
	}

	private void launchImageSelectionDialogForEditScreenCommon()
	{
		FragmentManager fragmentManager = getSupportFragmentManager();
		BehanceSDKImageSelectorOptions imageSelectionOptions = BehanceSDKImageSelectorOptions.getUnlimitedImageSelectionOptions();
		Intent intent = getIntent();
		if(intent.getBooleanExtra(INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED,false)) {
			imageSelectionOptions.setCCAssetBrowserDisabled(true);
		}
		imageSelectionOptions.setMaxImageSizeInBytes(BehanceSDKConstants.PROJECT_IMAGE_MODULE_MAX_SIZE_BYTES);
		imageSelectionOptions.setAllowedFileExtensions(BehanceSDKUtils.getAllowedFileExtensionsForProjectImage());
		BehanceSDKLaunchUtil.launchAlbumsViewToPickPicture(this, imageSelectionOptions, fragmentManager,
				BehanceSDKFragmentTags.FRAGMENT_TAG_ADD_PROJECT_ALBUM_SELECTION_FRAGMENT);
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		switch (requestCode) {
			case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW : {
				if (grantResults.length ==1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
					launchImageSelectionDialogForEditScreenCommon();
					requestingPermission = false;
				} else {
					int messageId = R.string.behance_sdk_permissions_error_generic;
					//Permission denied! Show some dialog about how it will limits the functionality!
					Toast.makeText(this, this.getString(messageId), Toast.LENGTH_LONG).show();
				}
			}
			break;
			case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA: {
				if((grantResults.length == 2 && grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED) || (grantResults.length ==1 && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
					Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.bsdkPublishProjectPreviewFragment);
					if(fragment != null)
					{
						((BehanceSDKPublishProjectPreviewFragment)fragment).launchCameraToCapturePicture();
						requestingPermission = false;

					}
				} else {
					//Permission denied! Show some dialog about how it will limits the functionality!
					int messageId = R.string.behance_sdk_permissions_error_generic;
					//Permission denied! Show some dialog about how it will limits the functionality!
					if(grantResults.length ==2 ){
						if(grantResults[0] != PackageManager.PERMISSION_GRANTED && grantResults[1]!=PackageManager.PERMISSION_GRANTED){
							messageId = R.string.behance_sdk_permissions_error;
						}else if(grantResults[0]!= PackageManager.PERMISSION_GRANTED){
							messageId = R.string.behance_sdk_permissions_error_camera;
						}else{
							messageId = R.string.behance_sdk_permissions_error_generic;
						}
					}else if(grantResults.length ==1){
						messageId = R.string.behance_sdk_permissions_error_generic;
					}
					Toast.makeText(this, this.getString(messageId), Toast.LENGTH_LONG).show();
				}
			}
			break;
			case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT:
			{
				if((grantResults.length ==1 && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
					Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.bsdkPublishProjectCoverFragment);
					if(fragment != null)
					{
						((BehanceSDKPublishProjectCoverFragment)fragment).launchAlbumsViewToPickPictures();
						requestingPermission = false;
					}
				} else {
					//Permission denied! Show some dialog about how it will limits the functionality!
					int messageId = R.string.behance_sdk_permissions_error_generic;
					Toast.makeText(this, this.getString(messageId), Toast.LENGTH_LONG).show();
				}
			}
			break;
		}
	}

	@Override
	public void onImageSelectorSelectionFinished(List<ImageModule> selectedImageModulesList) {
		BehanceSDKProjectPublishWFManager instance = BehanceSDKProjectPublishWFManager.getInstance();
		instance.addImagesSelectedForProject(selectedImageModulesList);
	}

	@Override
	public void onImageSelectorSelectionCanceled() {

	}

	@Override
	public void onImageSelectorSelectionBackPressed() {

	}

	@Override
	public void onImageSelectorOnError(Exception e) {

	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		if(!requestingPermission) {
			super.onSaveInstanceState(outState);
			outState.putInt(BUNDLE_KEY_CURRENT_PAGE_NUMBER, publishProjectScreens.getDisplayedChild());
		}
	}

	@Override
	public void onBackPressed() {
		if (publishProjectScreens.getDisplayedChild() != 0){
			addShowPreviousAnimations();
			publishProjectScreens.showPrevious();
		}else{
			BehanceSDKProjectPublishWFManager instance = BehanceSDKProjectPublishWFManager.getInstance();
			instance.cancelWorkflow();

			AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXCancel,
					AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);

			closeThisActivity(true);
		}
	}

	private void addValidImageModulesToWFManager() {
		Intent intent = getIntent();
		List<ImageModule> projectImageModules = (List<ImageModule>) intent.getSerializableExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_IMAGE_MODULES);
		// Image validation is done at the BehanceSDK API level(BehanceSDK.launchPublishProjectView).
		if(null != projectImageModules && false == projectImageModules.isEmpty()) {
			BehanceSDKProjectPublishWFManager instance = BehanceSDKProjectPublishWFManager.getInstance();
			instance.addImagesSelectedForProject(projectImageModules);
		}
	}

	private void addProjectDetailsToWFManager() {

		Intent intent = getIntent();
		BehanceSDKProjectPublishWFManager instance = BehanceSDKProjectPublishWFManager.getInstance();
		instance.initializeProject();
		instance.setProjectTitle(intent.getStringExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_TITLE));
		instance.setProjectDescription(intent.getStringExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_DESC));
		instance.setProjectTags(intent.getStringExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_TAGS));
		BehanceSDKCopyrightOption copyrightSettings = (BehanceSDKCopyrightOption)intent.getSerializableExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_COPYRIGHT_SETTINGS);
		if(null != copyrightSettings)
			instance.setCopyrightOption(copyrightSettings);
		instance.setContainsAdultContent(intent.getBooleanExtra(INTENT_SERIALIZABLE_EXTRA_PROJECT_CONTAINS_ADULT_CONTENT, false));
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.bsdkPublishProjectPreviewFragment);
		fragment.onActivityResult(requestCode, resultCode, data);

	}

	public boolean getCCAssetBrowserDisabled()
	{
		Intent intent = getIntent();
		return intent != null ? intent.getBooleanExtra(INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED,false) : false;
	}
}
