/**
 * **********************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2013 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 * ************************************************************************
 */

package com.behance.sdk.ui.activities;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

/**
 * @author Sujit Reddy G
 */

public class BehanceSDKPublishResultsActivity extends FragmentActivity implements View.OnClickListener {

    public static final String INTENT_STRING_EXTRA_WORK_URL = "INTENT_STRING_EXTRA_WORK_URL";
    public static final String INTENT_STRING_EXTRA_IMAGE_URL = "INTENT_STRING_EXTRA_IMAGE_URL";

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishResultsActivity.class);
    private BehanceSDK  behanceSDK = BehanceSDK.getInstance();

    private String workUrl;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.bsdk_activity_publish_results);
        if(!behanceSDK.useBehanceTheme() && !getResources().getBoolean(R.bool.isTablet)){
            setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
        }
        // initialize ImageLoader
        if (!ImageLoader.getInstance().isInited()) {
            ImageLoaderConfiguration loaderConfiguration = ImageLoaderConfiguration.createDefault(this);
            ImageLoader.getInstance().init(loaderConfiguration);
        }

        Intent intent = getIntent();
        workUrl = intent.getStringExtra(INTENT_STRING_EXTRA_WORK_URL);
        String imageUrl = intent.getStringExtra(INTENT_STRING_EXTRA_IMAGE_URL);

        View backBtn = findViewById(R.id.bsdkPublishResultsTitlebarBackBtnImageView);
        backBtn.setOnClickListener(this);

        TextView workUrlTxtView = (TextView) findViewById(R.id.bsdkPublishResultsUrlTxtView);
        workUrlTxtView.setOnClickListener(this);
        if (workUrl != null) {
            workUrlTxtView.setText(workUrl);
        }

        ImageView workImageView = (ImageView) findViewById(R.id.bsdkPublishResultsWorkImageView);
        workImageView.setOnClickListener(this);
        if (imageUrl != null) {
            DisplayImageOptions imageLoaderOptions = new DisplayImageOptions.Builder().cacheInMemory(true).resetViewBeforeLoading(false).cacheOnDisk(false)
                    .showImageOnFail(android.R.drawable.stat_notify_error).build();
            ImageLoader imageloader = ImageLoader.getInstance();
            imageloader.displayImage(imageUrl, workImageView, imageLoaderOptions);
        }

        View behanceBtn = findViewById(R.id.bsdkPublishResultsBehanceBtn);
        behanceBtn.setOnClickListener(this);
    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.bsdkPublishResultsTitlebarBackBtnImageView) {
            closeThisView();
        } else if (v.getId() == R.id.bsdkPublishResultsBehanceBtn ||
                v.getId() == R.id.bsdkPublishResultsUrlTxtView ||
                v.getId() == R.id.bsdkPublishResultsWorkImageView) {
            dispatchViewWorkUrlIntent();
        }
    }

    private void dispatchViewWorkUrlIntent() {
        try {
            if (workUrl != null) {
                Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(workUrl));
                startActivity(intent);
            }
        } catch (Throwable t) {
            logger.error(t, "Problem launching URL. [Url - %s]", workUrl);
        }
        closeThisView();
    }

    private void closeThisView() {
        finish();
    }

}