/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.activities;

import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKPublishWIPOptions;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.managers.AddWIPManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.fragments.BehanceSDKPublishWIPDialogFragment;
import com.behance.sdk.ui.fragments.BehanceSDKPublishWIPDialogFragment.Callbacks;
import com.behance.sdk.util.BehanceSDKPublishImageValidationResult;
import com.behance.sdk.util.BehanceSDKUtils;

/**
 * @author Sujit Reddy G
 */

public class BehanceSDKPublishWIPActivity extends BehanceSDKBasePublishActivity implements Callbacks {

    public static final String INTENT_SERIALIZABLE_EXTRA_WIP_REVISION_IMAGE_MODULE = "INTENT_SERIALIZABLE_EXTRA_WIP_REVISION_IMAGE_MODULE";
    public static final String INTENT_STRING_EXTRA_WIP_TITLE = "INTENT_STRING_EXTRA_WIP_TITLE";
    public static final String INTENT_STRING_EXTRA_WIP_DESCRIPTION = "INTENT_STRING_EXTRA_WIP_DESCRIPTION";
    public static final String INTENT_STRING_EXTRA_WIP_TAGS = "INTENT_STRING_EXTRA_WIP_TAGS";
    public static final String INTENT_INT_EXTRA_EXISTING_WIP_ID = "INTENT_INT_EXTRA_EXISTING_WIP_ID";

    private static final String FRAGMENT_TAG_PUBLISH_WIP = "BEHANCE_SDK_PUBLISH_WIP_ACTIVITY_FRAGMENT_TAG_PUBLISH_WIP";

    private BehanceSDK behanceSDK = BehanceSDK.getInstance();

    protected IBehanceSDKUserCredentials getBehanceSDKUserCredentials() {
        AddWIPManager wipManager = AddWIPManager.getInstance();
        return wipManager.getUserCredentials();
    }

    @Override
    public void continueWithWorkflow() {
        loadPublishWIPFragment();
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.bsdk_activity_publish_wip);
        if(!behanceSDK.useBehanceTheme()  && !getResources().getBoolean(R.bool.isTablet)){
            setRequestedOrientation(behanceSDK.getBehanceSDKCustomResourcesOptions().getPhoneOrientation());
        }
        if (savedInstanceState == null) {
            // we want to check only the first time because user might have
            // added details and if he rotates his device when network
            // connectivity is lost, we don't want to close this view
            NetworkInfo networkInfo = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
            if (!(networkInfo != null && networkInfo.isConnected())) {
                displayNoInternetConnectivity();
            }
            if(false == CanContinueWithWokflow())
                loadPublishWIPFragment();
        } else {
            FragmentManager fragmentManager = getSupportFragmentManager();
            Fragment publishWIPFragmentInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_PUBLISH_WIP);
            if (publishWIPFragmentInstance instanceof BehanceSDKPublishWIPDialogFragment) {
                ((BehanceSDKPublishWIPDialogFragment) publishWIPFragmentInstance).setCallbacks(this);
            }
        }
    }

    private void loadPublishWIPFragment() {
        BehanceSDKPublishWIPOptions publishWIPOptions = validateParamsAndGetOptions();
        if (publishWIPOptions != null) {
            BehanceSDKPublishWIPDialogFragment publishWIPFragment = BehanceSDKPublishWIPDialogFragment.getInstance(this, publishWIPOptions);
            FragmentManager fragmentManager = getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_PUBLISH_WIP);
            if (previousInstance != null) {
                fragmentTransaction.remove(previousInstance);
            }
            fragmentTransaction.addToBackStack(null);
            publishWIPFragment.show(fragmentTransaction, FRAGMENT_TAG_PUBLISH_WIP);
        } else {
            closeThisActivity();
        }
    }

    private BehanceSDKPublishWIPOptions validateParamsAndGetOptions() {
        boolean allParamsValid = true;
        Intent intent = getIntent();
        int existingWIPId = intent.getIntExtra(INTENT_INT_EXTRA_EXISTING_WIP_ID, 0);
        String wipDescription = intent.getStringExtra(INTENT_STRING_EXTRA_WIP_DESCRIPTION);
        ImageModule wipRevisionImageModule = (ImageModule) intent.getSerializableExtra(INTENT_SERIALIZABLE_EXTRA_WIP_REVISION_IMAGE_MODULE);
        String wipTitle = intent.getStringExtra(INTENT_STRING_EXTRA_WIP_TITLE);
        String wipTags = intent.getStringExtra(INTENT_STRING_EXTRA_WIP_TAGS);

        if (wipRevisionImageModule != null) {
            if (BehanceSDKPublishImageValidationResult.VALID == BehanceSDKUtils.validateImageForWIP(wipRevisionImageModule)) {
                allParamsValid = true;
            } else {
                displayToastMessage(getString(R.string.bsdk_publish_wip_image_required_error_msg));
                allParamsValid = false;
            }
        } else {
            displayToastMessage(getString(R.string.bsdk_publish_wip_image_required_error_msg));
            allParamsValid = false;
        }

        if (allParamsValid) {
            BehanceSDKPublishWIPOptions publishOptions = new BehanceSDKPublishWIPOptions(wipRevisionImageModule);
            publishOptions.setWipTitle(wipTitle);
            publishOptions.setExistingWIPId(existingWIPId);
            publishOptions.setWipDescription(wipDescription);
            publishOptions.setWipTags(wipTags);
            return publishOptions;
        } else {
            return null;
        }
    }

    private void displayToastMessage(String msgToDisplay) {
        if (msgToDisplay != null) {
            Toast.makeText(this, msgToDisplay, Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onPublishWIPViewClose() {
        closeThisActivity();
    }

    private void closeThisActivity() {
        finish();
    }

    private void displayNoInternetConnectivity() {
        Toast.makeText(this, getString(R.string.bsdk_add_wip_view_connection_error_msg), Toast.LENGTH_LONG).show();
        closeThisActivity();
    }
}