/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.activities;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.view.View.OnClickListener;

import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.R;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKUnlinkSocialAccountActivity extends FragmentActivity implements OnClickListener {

	public static String INTENT_EXTRA_ACCOUNT_TYPE_NAME = "INTENT_PARAMS_ACCOUNT_TYPE_NAME";
	public static int RESULT_CODE_ACCOUNT_UNLINK_SUCCESSFUL = 20;
	public static int RESULT_CODE_ACCOUNT_UNLINK_CANCELLED = 21;
	public static int RESULT_CODE_ACCOUNT_UNLINK_FAILED = 22;

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKUnlinkSocialAccountActivity.class);

	private BehanceSDKGenericAlertDialog confirmationDialog;
	private BehanceSDKSocialAccountType socialAccountType;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.bsdk_activity_unlink_social_account);
		Intent intent = getIntent();
		String accountTypeName = intent.getStringExtra(INTENT_EXTRA_ACCOUNT_TYPE_NAME);
		socialAccountType = BehanceSDKSocialAccountType.fromName(accountTypeName);
		if (socialAccountType != null) {
			displayConfirmationDialog();
		} else {
			closeThisActivity(RESULT_CODE_ACCOUNT_UNLINK_CANCELLED);
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
	}

	private void closeThisActivity(int resultCode) {
		setResult(resultCode, getIntent());
		finish();
	}

	private void displayConfirmationDialog() {

		int confirmationTitleResourceId = R.string.bsdk_social_account_unlink_twitter_account_confirmation_title;
		int confirmationBodyResourceId = R.string.bsdk_social_account_unlink_twitter_account_confirmation_body;
		int confirmationOkBtnLabelResourceId = R.string.bsdk_social_account_unlink_twitter_account_confirmation_ok_btn_label;
		int confirmationCancelBtnLabelResourceId = R.string.bsdk_social_account_unlink_twitter_account_confirmation_cancel_btn_label;
		if (socialAccountType == BehanceSDKSocialAccountType.FACEBOOK) {
			confirmationTitleResourceId = R.string.bsdk_social_account_unlink_facebook_account_confirmation_title;
			confirmationBodyResourceId = R.string.bsdk_social_account_unlink_facebook_account_confirmation_body;
			confirmationOkBtnLabelResourceId = R.string.bsdk_social_account_unlink_facebook_account_confirmation_ok_btn_label;
			confirmationCancelBtnLabelResourceId = R.string.bsdk_social_account_unlink_facebook_account_confirmation_cancel_btn_label;
		}
		confirmationDialog = BehanceSDKGenericAlertDialog.getInstance(this, confirmationTitleResourceId, confirmationBodyResourceId,
				confirmationOkBtnLabelResourceId, confirmationCancelBtnLabelResourceId);
		confirmationDialog.setOnNotOKBtnClickListener(this);
		confirmationDialog.setOnOKBtnClickListener(this);
		confirmationDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				closeThisActivity(RESULT_CODE_ACCOUNT_UNLINK_CANCELLED);
			}
		});
		confirmationDialog.show();
	}

	private void closeConfirmationDialog(int resultCode) {
		if (confirmationDialog != null) {
			confirmationDialog.dismiss();
		}
		closeThisActivity(resultCode);
	}

	@Override
	public void onClick(View v) {
		if (v.getId() == R.id.bsdkGenericAlertDialogOKBtn) {
			boolean removalSuccess = unlinkSocialAccount();
			if (removalSuccess) {
				closeConfirmationDialog(RESULT_CODE_ACCOUNT_UNLINK_SUCCESSFUL);
			} else {
				closeConfirmationDialog(RESULT_CODE_ACCOUNT_UNLINK_FAILED);
			}
		} else if (v.getId() == R.id.bsdkGenericAlertDialogNotOKBtn) {
			closeConfirmationDialog(RESULT_CODE_ACCOUNT_UNLINK_CANCELLED);
		}
	}

	private boolean unlinkSocialAccount() {
		try {
			BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(this);
			socialAccountManager.revokeAccountAccess(socialAccountType);
			logger.debug("Unlinked social account. [Account Type - %s]", socialAccountType.name());
			return true;
		} catch (Throwable e) {
			logger.error(e, "Problem unlinking social account. [Account Type - %s]", socialAccountType.name());
			return false;
		}
	}
}