/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2015 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloud;
import com.behance.sdk.R;

import java.util.List;

/**
 * @author Sujit Reddy G
 */
public class BehanceSDKAdobeCloudSelectionListItemAdapter extends BaseAdapter {

    private LayoutInflater inflater;
    private List<AdobeCloud> cloudsList;
    private AdobeCloud selectedCloud;

    public BehanceSDKAdobeCloudSelectionListItemAdapter(Context context, List<AdobeCloud> cloudsList, AdobeCloud selectedCloud) {
        this.cloudsList = cloudsList;
        this.setSelectedCloud(selectedCloud);
        inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    @Override
    public int getCount() {
        if (cloudsList != null) {
            return cloudsList.size();
        } else {
            return 0;
        }
    }

    @Override
    public AdobeCloud getItem(int arg0) {
        if (cloudsList != null) {
            return cloudsList.get(arg0);
        }
        return null;
    }

    @Override
    public View getView(int arg0, View convertView, ViewGroup parent) {
        View rootView;
        if (convertView == null) {
            rootView = inflater.inflate(R.layout.bsdk_adapter_adobe_cloud_selection_list_item, parent, false);
        } else {
            rootView = convertView;
        }
        AdobeCloud adobeCloud = getItem(arg0);
        if (adobeCloud != null) {
            TextView cloudNameTxtView = (TextView) rootView.findViewById(R.id.bsdkCloudSelectionItemAdapterCloudNameTextView);
            View selectedIndicator = rootView.findViewById(R.id.bsdkCloudSelectionItemAdapterSelectedIndicator);

            cloudNameTxtView.setText(adobeCloud.getName());

            AdobeCloud selectedCloud = getSelectedCloud();
            if (selectedCloud != null && selectedCloud.getName() != null && selectedCloud.getName().equals(adobeCloud.getName())) {
                selectedIndicator.setVisibility(View.VISIBLE);
            } else {
                selectedIndicator.setVisibility(View.INVISIBLE);
            }

        }
        return rootView;
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    public AdobeCloud getSelectedCloud() {
        return selectedCloud;
    }

    public void setSelectedCloud(AdobeCloud selectedCloud) {
        this.selectedCloud = selectedCloud;
    }

}