/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import java.util.List;

import android.content.Context;
import android.content.res.Resources;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKAlbumDTO;
import com.behance.sdk.enums.BehanceSDKAlbumType;

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKAlbumGridArrayAdapter extends ArrayAdapter<BehanceSDKAlbumDTO> {

	private LayoutInflater layoutInflater;
	private int albumCoverImageHeight;
	private int albumNameContainerHeight;

	public BehanceSDKAlbumGridArrayAdapter(Context context, List<BehanceSDKAlbumDTO> objects) {
		super(context, R.layout.bsdk_adapter_album_grid_item, objects);
		layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		albumCoverImageHeight = computeAlbumCoverImageHeight();
		albumNameContainerHeight = (int) (albumCoverImageHeight * .25);
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		View rootView = null;
		boolean viewReused = false;
		if (convertView == null) {
			rootView = layoutInflater.inflate(R.layout.bsdk_adapter_album_grid_item, parent, false);
			viewReused = false;
		} else {
			rootView = convertView;
			viewReused = true;
		}

		BehanceSDKAlbumDTO albumDTO = getItem(position);
		ImageView albumCoverImageView = (ImageView) rootView.findViewById(R.id.bsdkAlbumGridAdapterCoverImageView);
		TextView albumNameTxtView = (TextView) rootView.findViewById(R.id.bsdkAlbumGridAdapterAlbumNameTxtView);
		if (viewReused) {
			// clear image displayed currently
			albumCoverImageView.setImageBitmap(null);
		}

		int finalCoverImageHeight = albumCoverImageHeight;
		int finalNameContainerHeight = albumNameContainerHeight;
		
		if (albumDTO != null) {
			Context context = getContext();
			if (albumDTO.getAlbumType() == BehanceSDKAlbumType.DEVICE_ALBUM) {
				albumCoverImageView.setScaleType(ScaleType.CENTER_CROP);
				albumCoverImageView.setImageBitmap(albumDTO.getCoverBitmap(context));
				albumNameTxtView.setText(albumDTO.getName());
				albumNameTxtView.setVisibility(View.VISIBLE);
			} else if (albumDTO.getAlbumType() == BehanceSDKAlbumType.CAMERA) {
				finalCoverImageHeight = finalCoverImageHeight + finalNameContainerHeight;
				albumCoverImageView.setScaleType(ScaleType.CENTER);
				albumCoverImageView.setImageResource(R.drawable.bsdk_icon_adapter_album_camera_type_indicator);
				albumNameTxtView.setVisibility(View.GONE);
			}else if (albumDTO.getAlbumType() == BehanceSDKAlbumType.CREATIVE_CLOUD) {
				finalCoverImageHeight = finalCoverImageHeight + finalNameContainerHeight;
				albumCoverImageView.setScaleType(ScaleType.CENTER);
				albumCoverImageView.setImageResource(R.drawable.bsdk_icon_adapter_album_cloud_type_indicator);
				albumNameTxtView.setVisibility(View.GONE);
			}
		}
		
		// height of this should be same as grid item column width
		LayoutParams layoutParams = albumCoverImageView.getLayoutParams();
		if (layoutParams != null) {
			layoutParams.height = finalCoverImageHeight;
		} else {
			// parent of this component is RelativeLayout, that's why
			layoutParams = new RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, finalCoverImageHeight);
		}
		albumCoverImageView.setLayoutParams(layoutParams);

		// height of this should x percent of height of image
		LayoutParams albumNameTxtViewLayoutParams = albumNameTxtView.getLayoutParams();
		if (albumNameTxtViewLayoutParams != null) {
			albumNameTxtViewLayoutParams.height = finalNameContainerHeight;
		} else {
			// parent of this component is RelativeLayout, that's why
			albumNameTxtViewLayoutParams = new RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, finalNameContainerHeight);
		}
		albumNameTxtView.setLayoutParams(albumNameTxtViewLayoutParams);		
		return rootView;
	}

	private int computeAlbumCoverImageHeight() {
		// album cover image height should be same as it's width, width of the
		// image is grid column width, so we compute that
		Resources resources = getContext().getResources();
		int gridLeftRightPadding = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_album_grid_left_right_padding);
		int itemMargin = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_album_grid_item_margin);
		int columnCount = resources.getInteger(R.integer.bsdk_album_selector_fragment_grid_col_count);
		int gridItemStrokeWidth = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_adapter_bg_stroke_width);
		DisplayMetrics displayMetrics = resources.getDisplayMetrics();
		int deviceWidth = displayMetrics.widthPixels;
		int albumCoverImageHeight = (deviceWidth - (gridLeftRightPadding * 2) - ((columnCount - 1) * itemMargin)) / columnCount;
		// grid column width includes strokes too, so remove
		albumCoverImageHeight = albumCoverImageHeight - (gridItemStrokeWidth * 2);
		return albumCoverImageHeight;
	}
}