/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.content.res.Resources;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.RelativeLayout;

import com.behance.sdk.R;
import com.behance.sdk.project.modules.ImageModule;

import java.util.List;

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKAlbumImageGridArrayAdapter extends ArrayAdapter<ImageModule> {

	private LayoutInflater layoutInflater;
	private List<ImageModule> selectedImageModulesList;
	private int imageHeight;
	private int selectionIndicatorHeight;

	public BehanceSDKAlbumImageGridArrayAdapter(Context context, List<ImageModule> objects) {
		super(context, R.layout.bsdk_adapter_album_image_view_grid_item, objects);
		layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		imageHeight = computeImageHeight();
		selectionIndicatorHeight = computeSelectionIndicatorHeight();
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		boolean viewReused = false;
		View rootView = null;
		if (convertView == null) {
			rootView = layoutInflater.inflate(R.layout.bsdk_adapter_album_image_view_grid_item, parent, false);
			viewReused = false;
		} else {
			rootView = convertView;
			viewReused = true;
		}

		ImageView thumbnailImageView = (ImageView) rootView.findViewById(R.id.bsdkAlbumImageAdapterThumbnailImageView);
		View imageSelectedIndicatorView = rootView.findViewById(R.id.bsdkAlbumImageAdapterSelectionIndicator);

		if (viewReused) {
			thumbnailImageView.setImageDrawable(null);
		}

		// height of this should be same as grid item column width
		LayoutParams layoutParams = thumbnailImageView.getLayoutParams();
		if (layoutParams != null) {
			layoutParams.height = imageHeight;
		} else {
			// parent of this component is RelativeLayout, that's why
			layoutParams = new RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, imageHeight);
		}
		thumbnailImageView.setLayoutParams(layoutParams);

		// height of this should be same as image height
		LayoutParams indicatorLayoutParams = imageSelectedIndicatorView.getLayoutParams();
		if (indicatorLayoutParams != null) {
			indicatorLayoutParams.height = selectionIndicatorHeight;
		} else {
			// parent of this component is RelativeLayout, that's why
			indicatorLayoutParams = new RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, selectionIndicatorHeight);
		}
		imageSelectedIndicatorView.setLayoutParams(indicatorLayoutParams);

		ImageModule imageModule = getItem(position);
		if (imageModule != null) {
			thumbnailImageView.setImageBitmap(imageModule.getThumbNail(getContext()));

			if (null != selectedImageModulesList && selectedImageModulesList.contains(imageModule)) {
				imageSelectedIndicatorView.setVisibility(View.VISIBLE);
			} else {
				imageSelectedIndicatorView.setVisibility(View.INVISIBLE);
			}
		}
		return rootView;
	}

	public void setSelectedImageModulesList(List<ImageModule> values) {
		this.selectedImageModulesList = values;
	}

	private int computeImageHeight() {
		// image height should be same as width of the image, which is grid
		// column width, so we compute that
		Resources resources = getContext().getResources();
		int gridLeftRightPadding = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_album_image_grid_left_right_padding);
		int itemMargin = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_album_image_grid_item_margin);
		int columnCount = resources.getInteger(R.integer.bsdk_image_selection_fragment_grid_col_count);
		int gridItemStrokeWidth = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_adapter_bg_stroke_width);
		DisplayMetrics displayMetrics = resources.getDisplayMetrics();
		int deviceWidth = displayMetrics.widthPixels;
		int imageHeight = (deviceWidth - (gridLeftRightPadding * 2) - ((columnCount - 1) * itemMargin)) / columnCount;
		// grid column width includes strokes too, so remove
		imageHeight = imageHeight - (gridItemStrokeWidth * 2);
		return imageHeight;
	}

	private int computeSelectionIndicatorHeight() {
		Resources resources = getContext().getResources();
		int gridItemStrokeWidth = resources.getDimensionPixelSize(R.dimen.bsdk_image_selector_view_adapter_bg_stroke_width);
		return imageHeight + (gridItemStrokeWidth * 2);
	}
}
