package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.content.res.Resources;
import android.support.v4.app.FragmentActivity;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;


import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKShareContentAdapterItemDTO;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKBottomSheetItemViewHolder;
import com.behance.sdk.util.BehanceSDKUtils;

import java.util.List;

/**
 * Created by forta on 10/29/15.
 */
public class BehanceSDKBottomSheetRecyclerAdapter extends RecyclerView.Adapter<BehanceSDKBottomSheetItemViewHolder> {
    private Context context;
    private LayoutInflater layoutInflater;
    private List<BehanceSDKShareContentAdapterItemDTO> shareItems;
    private Callbacks callbacks;
    private int firstCardHeight, secondCardHeight, navBarHeight;

    public BehanceSDKBottomSheetRecyclerAdapter(Context context, List<BehanceSDKShareContentAdapterItemDTO> shareItems) {
        this.context = context;
        layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        this.shareItems = shareItems;
        Resources res = context.getResources();
        navBarHeight = BehanceSDKUtils.getTranslucentNavHeight((AppCompatActivity) context);
        int startCount = res.getDisplayMetrics().heightPixels / res.getDimensionPixelSize(R.dimen.bsdk_share_sheet_item_height) / 3;
        if(startCount < 3)
            startCount = 3;
        firstCardHeight = res.getDimensionPixelSize(R.dimen.bsdk_share_sheet_item_height) * startCount + res.getDimensionPixelSize(R.dimen.bsdk_toolbar_height) + navBarHeight + res.getDimensionPixelSize(R.dimen.bsdk_status_bar_spacer);
        secondCardHeight = context.getResources().getDisplayMetrics().heightPixels + navBarHeight - firstCardHeight;

    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    public interface Callbacks {
        public void closeDialog(boolean scroll);
        public void share(BehanceSDKShareContentAdapterItemDTO dto);
    }

    @Override
    public BehanceSDKBottomSheetItemViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        return new BehanceSDKBottomSheetItemViewHolder(layoutInflater.inflate(R.layout.bsdk_adapter_bottom_share_sheet_item, parent, false));
    }

    @Override
    public void onBindViewHolder(BehanceSDKBottomSheetItemViewHolder holder, int position) {
        if(position == 0) {
            holder.itemView.getLayoutParams().height = firstCardHeight;
            holder.contentContainer.setVisibility(View.GONE);
            holder.itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (callbacks != null)
                        callbacks.closeDialog(false);
                }
            });
            holder.itemView.setAlpha(0.f);
        } else if (position == 1) {
            holder.itemView.getLayoutParams().height = secondCardHeight;
            holder.contentContainer.setVisibility(View.GONE);
            holder.itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (callbacks != null)
                        callbacks.closeDialog(true);
                }
            });
            holder.itemView.setAlpha(0.f);
        } else if (position == 2) {
            holder.itemView.getLayoutParams().height = context.getResources().getDimensionPixelSize(R.dimen.bsdk_toolbar_height);
            holder.contentContainer.setVisibility(View.GONE);
            holder.contentContainer.setOnClickListener(null);
            holder.contentContainer.setAlpha(1.f);
        } else if (position == getItemCount()-1) {
            holder.itemView.getLayoutParams().height = (BehanceSDKUtils.getTranslucentNavHeight((FragmentActivity)context));
            holder.contentContainer.setVisibility(View.GONE);
            holder.contentContainer.setOnClickListener(null);
            holder.thumbnail.setImageDrawable(null);
            holder.title.setText(null);
            holder.itemView.setAlpha(1.f);
        } else {
            holder.itemView.getLayoutParams().height = RecyclerView.LayoutParams.MATCH_PARENT;
            holder.contentContainer.setVisibility(View.VISIBLE);
            final BehanceSDKShareContentAdapterItemDTO dto = shareItems.get(position-3);
            holder.contentContainer.setOnClickListener(new View.OnClickListener() {
               @Override
               public void onClick(View v) {
                   if (callbacks != null)
                       callbacks.share(dto);
               }
            });
            holder.thumbnail.setImageDrawable(dto.getDisplayIcon());
            holder.title.setText(dto.getDisplayLabel());
            holder.itemView.setAlpha(1.f);
        }
    }

            @Override
    public int getItemCount() {
        return shareItems.size()+4;
    }
}
