package com.behance.sdk.ui.adapters;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.google.listview.SectionalBaseAdapter;

public class BehanceSDKCopyrightSettingsAdapter extends SectionalBaseAdapter {

	Map<String, List<BehanceSDKCopyrightOption>> all;
	private final Context context;

	private int pinnedHeaderViewLeftPadding;
	private BehanceSDKCopyrightOption selectedCopyRight = BehanceSDKCopyrightOption.getDefaultValue();

	public BehanceSDKCopyrightSettingsAdapter(Context context, BehanceSDKCopyrightOption selectedCopyRight) {
		this.context = context;
		this.selectedCopyRight = selectedCopyRight;
		initializeHeaderViewPadding();
		all = BehanceSDKCopyrightOption.getAllCopyrightOptions(context);
	}

	private void initializeHeaderViewPadding() {
		pinnedHeaderViewLeftPadding = context.getResources().getDimensionPixelSize(R.dimen.bsdk_search_filter_creative_field_left_padding);
	}

	@Override
	public int getCount() {
		int res = 0;
		Set<String> keySet = all.keySet();
		for (String key : keySet) {
			res += all.get(key).size();
		}
		return res;
	}

	@Override
	public BehanceSDKCopyrightOption getItem(int position) {
		int c = 0;
		Set<String> keySet = all.keySet();
		for (String key : keySet) {
			List<BehanceSDKCopyrightOption> list = all.get(key);
			if (position >= c && position < c + list.size()) {
				return list.get(position - c);
			}
			c += list.size();
		}
		return null;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	protected void onNextPageRequested(int page) {
	}

	@Override
	protected void bindSectionHeader(View view, int position, boolean displaySectionHeader) {
		View groupHeaderDividerView = view.findViewById(R.id.bsdkCopyrightSettingsGroupHeader);
		View topDividerView = view.findViewById(R.id.bsdkCopyrightSettingsGroupItemDivider);
		
		if (displaySectionHeader) {
			// if group header is displayed, hide the divider
			groupHeaderDividerView.setVisibility(View.VISIBLE);
			topDividerView.setVisibility(View.INVISIBLE);
			TextView lSectionTitle = (TextView) groupHeaderDividerView;
			lSectionTitle.setText(getSections()[getSectionForPosition(position)]);
			
		} else {
			topDividerView.setVisibility(View.VISIBLE);
			groupHeaderDividerView.setVisibility(View.GONE);
		}
	}

	@Override
	public View getAmazingView(int position, View convertView, ViewGroup parent) {
		View rootView = convertView;
		LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		if (rootView == null)
			rootView = inflater.inflate(R.layout.bsdk_adapter_publish_project_copyright_settings_item, parent, false);

		TextView creativeFieldTxtView = (TextView) rootView.findViewById(R.id.bsdkCopyrightSettingsDialogItemTxtView);
		ImageView selectIndicatorImageView = (ImageView) rootView.findViewById(R.id.bsdkCopyrightSettingsDialogItemSelectedIdicatorImageView);
		BehanceSDKCopyrightOption copyRightSetting = getItem(position);
		creativeFieldTxtView.setText(copyRightSetting.getDescription(context));
		
		if (selectedCopyRight.equals(copyRightSetting)) {
			setSelected(creativeFieldTxtView, selectIndicatorImageView);
		} else {
			setUnselected(creativeFieldTxtView, selectIndicatorImageView);
		}

		return rootView;
	}

	/**
	 * @param creativeFieldTxtView
	 * @param selectIndicatorImageView
	 */
	private void setUnselected(TextView creativeFieldTxtView, ImageView selectIndicatorImageView) {
		int color = context.getResources().getColor(R.color.bsdk_search_filter_creative_fields_dialog_text_color);
		creativeFieldTxtView.setTextColor(color);
		selectIndicatorImageView.setVisibility(View.GONE);
	}

	/**
	 * @param creativeFieldTxtView
	 * @param selectIndicatorImageView
	 */
	private void setSelected(TextView creativeFieldTxtView, ImageView selectIndicatorImageView) {
		selectIndicatorImageView.setVisibility(View.VISIBLE);
		int color = context.getResources().getColor(R.color.bsdk_search_filter_creative_fields_dialog_selected_text_color);
		creativeFieldTxtView.setTextColor(color);
	}

	@Override
	public void configurePinnedHeader(View header, int position, int alpha) {
		TextView lSectionHeader = (TextView) header;
		lSectionHeader.setText(getSections()[getSectionForPosition(position)]);
		lSectionHeader.setPadding(pinnedHeaderViewLeftPadding, lSectionHeader.getPaddingTop(), lSectionHeader.getPaddingRight(),
				lSectionHeader.getPaddingBottom());
	}

	@Override
	public int getPositionForSection(int section) {
		if (section < 0)
			section = 0;
		Set<String> keySet = all.keySet();
		if (section >= keySet.size())
			section = keySet.size() - 1;
		int c = 0;
		Iterator<String> iterator = keySet.iterator();
		for (int i = 0; i < keySet.size(); i++) {
			if (section == i) {
				return c;
			}
			String next = iterator.next();
			c += all.get(next).size();
		}
		return 0;
	}

	@Override
	public int getSectionForPosition(int position) {
		int c = 0;
		Set<String> keySet = all.keySet();
		Iterator<String> iterator = keySet.iterator();
		for (int i = 0; i < keySet.size(); i++) {
			String next = iterator.next();
			List<BehanceSDKCopyrightOption> list = all.get(next);
			if (position >= c && position < c + list.size()) {
				return i;
			}
			c += list.size();
		}
		return -1;
	}

	@Override
	public String[] getSections() {
		Set<String> keySet = all.keySet();
		String[] res = new String[keySet.size()];
		return keySet.toArray(res);
	}

	/**
	 * @param selectedCopyrightSetting
	 */
	public void setSelectedCopyRight(BehanceSDKCopyrightOption selectedCopyrightSetting) {
		this.selectedCopyRight = selectedCopyrightSetting;
	}
}