package com.behance.sdk.ui.adapters;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.enums.BehanceSDKCreativeFieldCategory;
import com.behance.sdk.google.listview.SectionalBaseAdapter;

public class BehanceSDKCreativeFieldFilterAdapter extends SectionalBaseAdapter {

	Map<String, List<BehanceSDKCreativeFieldDTO>> all;
	private final Context context;
	private List<BehanceSDKCreativeFieldDTO> selectedField;

	private int pinnedHeaderViewLeftPadding;

	public BehanceSDKCreativeFieldFilterAdapter(Context context, List<BehanceSDKCreativeFieldDTO> data, List<BehanceSDKCreativeFieldDTO> selectedField) {
		this.context = context;
		this.selectedField = selectedField;
		initializeHeaderViewPadding();
		initalizeData(data, selectedField);
	}

	private void initializeHeaderViewPadding() {
		pinnedHeaderViewLeftPadding = context.getResources().getDimensionPixelSize(R.dimen.bsdk_search_filter_creative_field_left_padding);
	}

	/**
	 * @param data
	 * @param selectedField || selectedField.isEmpty()
	 */
	private void initalizeData(List<BehanceSDKCreativeFieldDTO> data, List<BehanceSDKCreativeFieldDTO> selectedField ) {
		all = new LinkedHashMap<String, List<BehanceSDKCreativeFieldDTO>>();
		all.put(BehanceSDKCreativeFieldCategory.POPULAR.getString(context), new ArrayList<BehanceSDKCreativeFieldDTO>());
		for (BehanceSDKCreativeFieldDTO creativeFieldDTO : data) {
			String name = creativeFieldDTO.getName();
			String start = getStartingAlphabet(name);
			List<BehanceSDKCreativeFieldDTO> list = all.get(start);
			if (null == list) {
				list = new ArrayList<BehanceSDKCreativeFieldDTO>();
				all.put(start, list);
			}
			list.add(creativeFieldDTO);
			BehanceSDKCreativeFieldCategory category = creativeFieldDTO.getCategory();
			if (category != BehanceSDKCreativeFieldCategory.NONE) {
				String categoryStr = category.getString(context);
				List<BehanceSDKCreativeFieldDTO> catrgoryList = all.get(categoryStr);
				if (null == catrgoryList) {
					catrgoryList = new ArrayList<BehanceSDKCreativeFieldDTO>();
					all.put(categoryStr, catrgoryList);
				}
				catrgoryList.add(creativeFieldDTO);
			}
		}
	}

	private String getStartingAlphabet(String name) {
		if (false == TextUtils.isEmpty(name)) {
			return String.valueOf(name.charAt(0));
		}
		return null;
	}

	@Override
	public int getCount() {
		int res = 0;
		Set<String> keySet = all.keySet();
		for (String key : keySet) {
			res += all.get(key).size();
		}
		return res;
	}

	@Override
	public BehanceSDKCreativeFieldDTO getItem(int position) {
		int c = 0;
		Set<String> keySet = all.keySet();
		for (String key : keySet) {
			List<BehanceSDKCreativeFieldDTO> list = all.get(key);
			if (position >= c && position < c + list.size()) {
				return list.get(position - c);
			}
			c += list.size();
		}
		return null;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	protected void onNextPageRequested(int page) {
	}

	@Override
	protected void bindSectionHeader(View view, int position, boolean displaySectionHeader) {
		View groupHeader = view.findViewById(R.id.bsdkCreativeFieldGroupHeader);
		View topDividerView = view.findViewById(R.id.bsdkCreativeFieldGroupItemDivider);
		
		if (displaySectionHeader) {
			// if group header is displayed, hide the divider
			groupHeader.setVisibility(View.VISIBLE);
			topDividerView.setVisibility(View.INVISIBLE);
			TextView lSectionTitle = (TextView) groupHeader;
			lSectionTitle.setText(getSections()[getSectionForPosition(position)]);
		} else {
			topDividerView.setVisibility(View.VISIBLE);
			groupHeader.setVisibility(View.GONE);
		}
	}

	@Override
	public View getAmazingView(int position, View convertView, ViewGroup parent) {
		View rootView = convertView;
		LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		if (rootView == null)
			rootView = inflater.inflate(R.layout.bsdk_adapter_publish_project_creative_field_item, parent, false);

		TextView creativeFieldTxtView = (TextView) rootView.findViewById(R.id.bsdkCreativeFieldFilterDialogItemTxtView);
		ImageView selectIndicatorImageView = (ImageView) rootView.findViewById(R.id.bsdkCreativeFieldFilterDialogItemSelectedIdicatorImageView);
		View topDividerView = rootView.findViewById(R.id.bsdkCreativeFieldGroupItemDivider);

		BehanceSDKCreativeFieldDTO creativeFieldDTO = getItem(position);
		creativeFieldTxtView.setText(creativeFieldDTO.getName());
		String creativeFieldId = creativeFieldDTO.getId();
		
		if (selectedField.contains(creativeFieldDTO)) {
			setSelected(creativeFieldTxtView, selectIndicatorImageView);
		} else {
			setUnselected(creativeFieldTxtView, selectIndicatorImageView);
		}
		if (creativeFieldId.equals(BehanceSDKCreativeFieldDTO.ALL_CREATIVE_FIELDS_ID)) {
			topDividerView.setVisibility(View.GONE);
			if(selectedField.isEmpty())
				setSelected(creativeFieldTxtView, selectIndicatorImageView);
		} else {
			topDividerView.setVisibility(View.VISIBLE);
		}

		return rootView;
	}

	/**
	 * @param creativeFieldTxtView
	 * @param selectIndicatorImageView
	 */
	private void setUnselected(TextView creativeFieldTxtView, ImageView selectIndicatorImageView) {
		int color = context.getResources().getColor(R.color.bsdk_search_filter_creative_fields_dialog_text_color);
		creativeFieldTxtView.setTextColor(color);
		selectIndicatorImageView.setVisibility(View.GONE);
	}

	/**
	 * @param creativeFieldTxtView
	 * @param selectIndicatorImageView
	 */
	private void setSelected(TextView creativeFieldTxtView, ImageView selectIndicatorImageView) {
		selectIndicatorImageView.setVisibility(View.VISIBLE);
		int color = context.getResources().getColor(R.color.bsdk_search_filter_creative_fields_dialog_selected_text_color);
		creativeFieldTxtView.setTextColor(color);
	}

	@Override
	public void configurePinnedHeader(View header, int position, int alpha) {
		TextView lSectionHeader = (TextView) header;
		lSectionHeader.setText(getSections()[getSectionForPosition(position)]);
		lSectionHeader.setPadding(pinnedHeaderViewLeftPadding, lSectionHeader.getPaddingTop(), lSectionHeader.getPaddingRight(),
				lSectionHeader.getPaddingBottom());
	}

	@Override
	public int getPositionForSection(int section) {
		if (section < 0)
			section = 0;
		Set<String> keySet = all.keySet();
		if (section >= keySet.size())
			section = keySet.size() - 1;
		int c = 0;
		Iterator<String> iterator = keySet.iterator();
		for (int i = 0; i < keySet.size(); i++) {
			if (section == i) {
				return c;
			}
			String next = iterator.next();
			c += all.get(next).size();
		}
		return 0;
	}

	@Override
	public int getSectionForPosition(int position) {
		int c = 0;
		Set<String> keySet = all.keySet();
		Iterator<String> iterator = keySet.iterator();
		for (int i = 0; i < keySet.size(); i++) {
			String next = iterator.next();
			List<BehanceSDKCreativeFieldDTO> list = all.get(next);
			if (position >= c && position < c + list.size()) {
				return i;
			}
			c += list.size();
		}
		return -1;
	}

	@Override
	public String[] getSections() {
		Set<String> keySet = all.keySet();
		String[] res = new String[keySet.size()];
		return keySet.toArray(res);
	}
}