package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.PorterDuff;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.CardView;
import android.support.v7.widget.RecyclerView;
import android.util.SparseArray;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;
import com.behance.sdk.dto.project.BehanceSDKAbstractCommentDTO;
import com.behance.sdk.dto.project.BehanceSDKAbstractProjectModuleDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCommentDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleAudioDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleEmbedDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleImageDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleTextDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleVideoDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectStylesDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectDetailAppreciateViewHolder;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectDetailCommentViewHolder;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectDetailCommentsHeaderViewHolder;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectDetailInfoShareViewHolder;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectDetailItemViewHolder;
import com.behance.sdk.util.BehanceSDKUtils;
import com.facebook.drawee.backends.pipeline.Fresco;
import com.facebook.drawee.interfaces.DraweeController;
import com.facebook.drawee.view.SimpleDraweeView;
import com.facebook.imagepipeline.request.ImageRequest;
import com.facebook.imagepipeline.request.ImageRequestBuilder;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * Created by forta on 12/7/15.
 */
public class BehanceSDKProjectDetailRecyclerAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private static final double PROJECT_TEXT_MAX_SCALE = 1.5;

    private static final int ITEM_TYPE_HEADER_PADDING = 0;
    private static final int ITEM_TYPE_PROJECT_MODULE = 1;
    private static final int ITEM_TYPE_APPRECIATE_VIEW = 2;
    private static final int ITEM_TYPE_COMMENT_HEADER = 3;
    private static final int ITEM_TYPE_INFO_SHARE_VIEW = 4;
    private static final int ITEM_TYPE_COMMENT_PLACEHOLDER = 5;
    private static final int ITEM_TYPE_COMMENT = 6;
    private static final int ITEM_TYPE_LOADER = 7;

    private static final int PROJECT_CAPTION_TOP_SPACING = 16;

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKProjectDetailRecyclerAdapter.class);

    private Context context;
    private BehanceSDK behanceSDK;
    private List<BehanceSDKAbstractProjectModuleDTO> projectModules;
    private List<BehanceSDKProjectCommentDTO> projectComments;
    private BehanceSDKProjectStylesDTO styles;
    private double displayWidth, scaleFactor;
    private int headerHeight;
    private boolean moreToLoad = true;
    private int loggedInUserId;
    private boolean ownedByLoggedInUser = false;
    private int commentsCount;
    private int bgColor;
    private SparseArray<Integer> webViewHeights;
    private boolean projectAppreciated;
    private boolean commentsAllowed;


    private Callbacks callbacks;

    public interface Callbacks {
        public void onDeleteCommentRequested(BehanceSDKAbstractCommentDTO commentDTO);
        public void onProjectAppreciated();
        public void onProjectInfoClicked();
        public void onProjectShareClick();
        public void onImageClicked(int pos);
    }

    public BehanceSDKProjectDetailRecyclerAdapter(Context context, BehanceSDKProjectDTO projectDTO) {
        this.context = context;
        this.projectModules = projectDTO.getModules();
        this.styles = projectDTO.getProjectStyles();
        BehanceSDKUserManager userManager = BehanceSDKUserManager.getInstance();
        if(userManager.isUserLoggedIn() && userManager.getUserDTO()!= null)
            loggedInUserId = userManager.getUserDTO().getId();
        else
            loggedInUserId = -1;
        for(BehanceSDKUserDTO owner : projectDTO.getOwners()) {
            if(owner.getId() == loggedInUserId) {
                ownedByLoggedInUser = true;
                break;
            }
        }
        displayWidth = context.getResources().getDisplayMetrics().widthPixels;
        scaleFactor = displayWidth * 0.85 / 725;
        headerHeight = 100;
        projectComments = new ArrayList<>();
        commentsCount = projectDTO.getStats().getCommentsCount();
        commentsAllowed = projectDTO.isAllowComments();
        behanceSDK =  BehanceSDK.getInstance();

        //backgrounds being drawn at behind the recycler
        bgColor = context.getResources().getColor(R.color.bsdk_transparent);

        projectAppreciated = projectDTO.isAppreciatedByUser();

        //prevent choppy scrolling by storing height of webview once rendered and using when bound
        webViewHeights = new SparseArray<>(projectModules.size());
        for(int i = 0; i < projectModules.size(); i++) {
            webViewHeights.put(i, -1);
        }
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        switch (viewType) {
            case ITEM_TYPE_LOADER:
                return new BehanceSDKProjectDetailItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_loader, parent, false));
            case ITEM_TYPE_INFO_SHARE_VIEW:
                return new BehanceSDKProjectDetailInfoShareViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_info_share,parent,false));
            case ITEM_TYPE_COMMENT:
                return new BehanceSDKProjectDetailCommentViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_comment, parent, false));
            case ITEM_TYPE_COMMENT_PLACEHOLDER:
                return new BehanceSDKProjectDetailItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_comments_placeholder, parent, false));
            case ITEM_TYPE_COMMENT_HEADER:
                return new BehanceSDKProjectDetailCommentsHeaderViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_comments_header, parent, false));
            case ITEM_TYPE_APPRECIATE_VIEW:
                return new BehanceSDKProjectDetailAppreciateViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_appreciate, parent, false));
            case ITEM_TYPE_PROJECT_MODULE:
            case ITEM_TYPE_HEADER_PADDING:
            default:
                return new BehanceSDKProjectDetailItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project_detail_module, parent, false));
        }

    }

    @Override
    public void onBindViewHolder(final RecyclerView.ViewHolder viewHolder, int position) {

        if(position < projectModules.size()+1) {

            BehanceSDKProjectDetailItemViewHolder holder = (BehanceSDKProjectDetailItemViewHolder) viewHolder;

            holder.container.setBackgroundColor(bgColor);

            holder.container.removeAllViews();

            if (position == 0) {
                holder.container.getLayoutParams().height = headerHeight;
            } else {

                int pos = position - 1;

                if (pos == 0 && styles.getProjectMarginTop() > 0) {
                    View spacer = new View(context);
                    holder.container.addView(spacer);
                    LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) spacer.getLayoutParams();
                    params.height = (int) (styles.getProjectMarginTop() * scaleFactor);
                    params.width = LinearLayout.LayoutParams.MATCH_PARENT;
                    spacer.setLayoutParams(params);
                }

                BehanceSDKAbstractProjectModuleDTO module = projectModules.get(pos);

                if (module instanceof BehanceSDKProjectModuleTextDTO) {
                    BehanceSDKProjectModuleTextDTO textModule = (BehanceSDKProjectModuleTextDTO) module;
                    makeTextModule(textModule.getFormattedText(), holder.container, styles.getCustomCSS(), displayWidth, pos);
                } else if (module instanceof BehanceSDKProjectModuleImageDTO) {
                    makeImageModule((BehanceSDKProjectModuleImageDTO) module, holder.container, pos);
                } else {
                    makeWVModule(module, holder.container, pos);
                }

                if (pos < projectModules.size() - 1) {
                    addDividerSpacer(holder, pos);
                }

            }
        } else if(viewHolder instanceof BehanceSDKProjectDetailAppreciateViewHolder) {
            setUpAppreciateItem((BehanceSDKProjectDetailAppreciateViewHolder) viewHolder);
        } else if (viewHolder instanceof BehanceSDKProjectDetailInfoShareViewHolder){
            setupShareInfoItem((BehanceSDKProjectDetailInfoShareViewHolder) viewHolder);
        }else if (viewHolder instanceof BehanceSDKProjectDetailCommentViewHolder) {
            populateComment((BehanceSDKProjectDetailCommentViewHolder) viewHolder, position - projectModules.size() - 4);
        } else if(viewHolder instanceof BehanceSDKProjectDetailCommentsHeaderViewHolder) {
            makeCommentsHeader((BehanceSDKProjectDetailCommentsHeaderViewHolder) viewHolder);
        } else if(position == getItemCount()-1) {
            viewHolder.itemView.findViewById(R.id.bsdk_card_loader_progress_bar).setVisibility(moreToLoad ? View.VISIBLE : View.INVISIBLE);
            ((CardView)viewHolder.itemView).setCardBackgroundColor(context.getResources().getColor(R.color.bsdk_card_color_off_white));
            ((CardView)viewHolder.itemView).setContentPadding(0, 0, 0, BehanceSDKUtils.getTranslucentNavHeight((AppCompatActivity) context));
        }
    }

    private void setupShareInfoItem(BehanceSDKProjectDetailInfoShareViewHolder viewHolder){
        viewHolder.infoView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(callbacks!= null){
                    callbacks.onProjectInfoClicked();
                }

            }
        });
        viewHolder.shareText.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(callbacks!= null){
                    callbacks.onProjectShareClick();
                }

           }
        });

    }

    private void makeImageModule(BehanceSDKProjectModuleImageDTO imageModule, LinearLayout container, final int pos) {
        SimpleDraweeView imageView = new SimpleDraweeView(context);
        imageView.setScaleType(ImageView.ScaleType.FIT_XY);
        container.addView(imageView);
        LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) imageView.getLayoutParams();
        boolean fullBleed = imageModule.isFullBleed();
        double dispW = fullBleed ? displayWidth : displayWidth * imageModule.getWidth() / 725,
                imW = imageModule.getWidth(),
                imH = imageModule.getHeight();
        params.height = (int) Math.round((imH / imW) * dispW);
        params.width = (int) Math.round(dispW);
      //  imageView.getHierarchy().setProgressBarImage(new BehanceSDKCustomProgressDrawable());
        imageView.getHierarchy().setPlaceholderImage(new ColorDrawable(context.getResources().getColor(R.color.bsdk_project_image_bg_overlay)));
        ImageRequest request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(imageModule.getHDSrcUrl())).setProgressiveRenderingEnabled(true).build();
        ImageRequest requestLowRes = ImageRequestBuilder.newBuilderWithSource(Uri.parse(imageModule.getSrcUrl())).setProgressiveRenderingEnabled(true).build();
        DraweeController controller = Fresco.newDraweeControllerBuilder().setImageRequest(request).setLowResImageRequest(requestLowRes).setOldController(imageView.getController()).setAutoPlayAnimations(true).build();
        imageView.setController(controller);

        imageView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
              //  if(callbacks != null)
                //    callbacks.onImageClicked(findImageNumFromModuleNum(pos));
            }
        });

        if (imageModule.getCaptionFormattedText() != null && !imageModule.getCaptionFormattedText().isEmpty()) {
            addCaptionSpacer(container);
            makeTextModule(imageModule.getCaptionFormattedText(), container, styles.getCustomCSS(), displayWidth, pos);
        }

    }

    private int findImageNumFromModuleNum(int moduleNum) {
        int imageModuleCount = 0;
        for(int i = 0; i < projectModules.size(); i++) {
            BehanceSDKAbstractProjectModuleDTO abstractProjectModule = projectModules.get(i);
            if(abstractProjectModule instanceof BehanceSDKProjectModuleImageDTO) {
                if(i == moduleNum)
                    return imageModuleCount;
                else
                    imageModuleCount++;
            }
        }
        return 0;
    }

    private void makeTextModule(String html, LinearLayout container, String css, double displayWidth, final int pos) {
        final WebView webView = new WebView(context);

        WebSettings webViewsettings = webView.getSettings();
        webViewsettings.setLoadWithOverviewMode(true);
        webViewsettings.setUseWideViewPort(true);
        webViewsettings.setJavaScriptEnabled(false);
        webViewsettings.setSupportZoom(false);
        webViewsettings.setDefaultTextEncodingName("UTF-8");
        webViewsettings.setTextZoom((int) (100 * Math.min(scaleFactor, PROJECT_TEXT_MAX_SCALE)));
        webView.setVerticalScrollBarEnabled(false);
        webView.setHorizontalScrollBarEnabled(false);


        String content = "<html><head><meta name=\"viewport\" content=\"width=725, maximum-scale=3.059\"/>"
                + "<style type=\"text/css\">" + css + "</style>"
                + "</head><body>" + html + "</body></html>";

        webView.setBackgroundColor(bgColor);

        container.addView(webView);
        LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) webView.getLayoutParams();
        params.height=LinearLayout.LayoutParams.WRAP_CONTENT;
        params.width = (int) (displayWidth * 0.85);
        webView.setLayoutParams(params);

        if(webViewHeights.get(pos)<=0) {
            webView.setWebViewClient(new WebViewClient() {

                @Override
                public boolean shouldOverrideUrlLoading(WebView view, String url) {
                    return super.shouldOverrideUrlLoading(view, url);
                }

                @Override
                public void onPageStarted(WebView view, String url, Bitmap favicon) {
                    super.onPageStarted(view, url, favicon);
                }

                public void onPageFinished(WebView webView, String url) {
                    super.onPageFinished(webView, url);
                    webViewHeights.put(pos, webView.getMeasuredHeight());
                    webView.setWebViewClient(null);
                }

            });
        } else {
            container.setMinimumHeight(webViewHeights.get(pos));
        }

        webView.loadDataWithBaseURL(null, content, "text/html", "utf-8", null);
    }

    private void makeWVModule(BehanceSDKAbstractProjectModuleDTO module, LinearLayout container, final int pos) {
        String html = null;
        if (module instanceof BehanceSDKProjectModuleEmbedDTO)
            html = ((BehanceSDKProjectModuleEmbedDTO) module).getEmbedHTML();
        else if (module instanceof BehanceSDKProjectModuleAudioDTO)
            html = ((BehanceSDKProjectModuleAudioDTO) module).getEmbedHTML();
        else if (module instanceof BehanceSDKProjectModuleVideoDTO)
            html = ((BehanceSDKProjectModuleVideoDTO) module).getEmbedHTML();
        WebView webView = new WebView(context);

        WebSettings webViewSettings = webView.getSettings();
        webViewSettings.setLoadWithOverviewMode(true);
        webViewSettings.setUseWideViewPort(true);
        webViewSettings.setJavaScriptEnabled(true);
        webViewSettings.setSupportZoom(false);
        webViewSettings.setDefaultTextEncodingName("UTF-8");


        String content = "<html><head><meta name=\"viewport\" content=\"width=725, maximum-scale=3.059\"/>"
                + "</head><body>" + html + "</body></html>";

        webView.setBackgroundColor(bgColor);

        container.addView(webView);
        LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) webView.getLayoutParams();
        params.height = LinearLayout.LayoutParams.WRAP_CONTENT;
        params.width = (int) (displayWidth * 0.85);
        webView.setLayoutParams(params);

        if(webViewHeights.get(pos)<=0) {
            webView.setWebViewClient(new WebViewClient() {

                @Override
                public boolean shouldOverrideUrlLoading(WebView view, String url) {
                    return super.shouldOverrideUrlLoading(view, url);
                }

                @Override
                public void onPageStarted(WebView view, String url, Bitmap favicon) {
                    super.onPageStarted(view, url, favicon);
                }

                public void onPageFinished(WebView webView, String url) {
                    super.onPageFinished(webView, url);
                    webViewHeights.put(pos, webView.getMeasuredHeight());
                    webView.setWebViewClient(null);
                }

            });
        } else {
            container.setMinimumHeight(webViewHeights.get(pos));
        }

        webView.loadDataWithBaseURL(null, content, "text/html", "utf-8", null);

        if(module instanceof BehanceSDKProjectModuleEmbedDTO) {
            BehanceSDKProjectModuleEmbedDTO embedModule = (BehanceSDKProjectModuleEmbedDTO) module;
            if (embedModule.getCaptionFormattedText() != null && !embedModule.getCaptionFormattedText().isEmpty()) {
                addCaptionSpacer(container);
                makeTextModule(embedModule.getCaptionFormattedText(), container, styles.getCustomCSS(), displayWidth, pos);
            }
        }
    }

    private void addCaptionSpacer(LinearLayout container){
        View spacer = new View(context);
        container.addView(spacer);
        LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) spacer.getLayoutParams();
        params.height = (int) (PROJECT_CAPTION_TOP_SPACING * scaleFactor);
        params.width = LinearLayout.LayoutParams.MATCH_PARENT;
        spacer.setLayoutParams(params);
    }

    private void addDividerSpacer(BehanceSDKProjectDetailItemViewHolder holder, int pos) {
        if (styles.getDividerCSS() == null) {
            if (styles.getModuleMarginBottom() > 0) {
                View spacer = new View(context);
                holder.container.addView(spacer);
                LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) spacer.getLayoutParams();
                params.height = (int) (styles.getModuleMarginBottom() * scaleFactor);
                params.width = LinearLayout.LayoutParams.MATCH_PARENT;
                spacer.setLayoutParams(params);
            }
        } else {
            LinearLayout spacer = new LinearLayout(context);
            spacer.setOrientation(LinearLayout.VERTICAL);
            spacer.setGravity(Gravity.CENTER);
            holder.container.addView(spacer);
            LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) spacer.getLayoutParams();
            params.height = (int) (styles.getModuleMarginBottom() * scaleFactor);
            params.width = LinearLayout.LayoutParams.MATCH_PARENT;
            spacer.setLayoutParams(params);
            makeTextModule("<p class=\"divider\"></p>", spacer, styles.getDividerCSS(), displayWidth, pos);
        }
    }

    private void populateComment(BehanceSDKProjectDetailCommentViewHolder holder, int commentNum) {
        final BehanceSDKProjectCommentDTO commentDTO = projectComments.get(commentNum);
        holder.comment.setText(commentDTO.getContent());
        holder.name.setText(commentDTO.getUser().getDisplayName());
        holder.date.setText(getFormattedDate(commentDTO.getCreatedDate()));
        try {
            ImageRequest request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(commentDTO.getUser().findProfileImageInDecreasingSizeOrder(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_115).getUrl())).setProgressiveRenderingEnabled(true).build();
            final DraweeController controller = Fresco.newDraweeControllerBuilder().setImageRequest(request).setOldController(holder.avatar.getController()).setAutoPlayAnimations(true).build();
            holder.avatar.setController(controller);
        } catch (NullPointerException e) {
            holder.avatar.setImageBitmap(null);
        }
        if(ownedByLoggedInUser || commentDTO.getUser().getId() == loggedInUserId)
            holder.itemView.findViewById(R.id.bsdk_card_project_comment_container).setOnCreateContextMenuListener(new View.OnCreateContextMenuListener() {
                @Override
                public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
                    MenuItem menuItem = menu.add(1, 1, 0, context.getResources().getString(R.string.bsdk_menu_item_label_delete_comment));
                    menuItem.setOnMenuItemClickListener(new MenuItem.OnMenuItemClickListener() {
                        @Override
                        public boolean onMenuItemClick(MenuItem item) {
                            if(callbacks != null) {
                                callbacks.onDeleteCommentRequested(commentDTO);
                                int i = projectComments.indexOf(commentDTO);
                                projectComments.remove(i);
                                if(commentsCount == 1)
                                    notifyItemChanged(i + projectModules.size() + 4);
                                else
                                    notifyItemRemoved(i + projectModules.size() + 4);
                                commentsCount -= 1;
                                notifyItemChanged(projectModules.size() + 3);
                            }
                            return true;
                        }
                    });
                }
            });
    }

    private void makeCommentsHeader(BehanceSDKProjectDetailCommentsHeaderViewHolder holder) {
        if(commentsAllowed) {
            holder.commentHeaderContainer.setVisibility(View.VISIBLE);
            NumberFormat formatter = new DecimalFormat("###,###,###,###");
            String commentsHeaderText = context.getResources().getString(R.string.bsdk_project_detail_fragment_comments_header_text,
                    formatter.format((double) commentsCount));
            holder.commentsHeader.setText(commentsHeaderText);
        } else {
            holder.commentHeaderContainer.setVisibility(View.GONE);
        }

    }

    private void setUpAppreciateItem(final BehanceSDKProjectDetailAppreciateViewHolder holder) {
        holder.itemView.setBackgroundColor(bgColor);
        if (projectAppreciated) {
            holder.appreciateView.getDrawable().setColorFilter(context.getResources().getColor(R.color.bsdk_project_detail_appreciated_badge_color), PorterDuff.Mode.SRC_ATOP);
            holder.appreciateThumb.setVisibility(View.GONE);
            holder.thankYouText.setVisibility(View.VISIBLE);
        } else {
            int color = R.color.bsdk_behance_blue;
           // int color = behanceSDK.useBehanceTheme() ? R.color.bsdk_behance_blue : behanceSDK.getBehanceSDKCustomResourcesOptions().getAppColor();
            //noinspection ResourceType
            holder.appreciateView.getDrawable().setColorFilter(context.getResources().getColor(color), PorterDuff.Mode.SRC_ATOP);
            holder.appreciateView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if(callbacks != null) {
                        holder.appreciateView.setOnClickListener(null);
                        holder.appreciateView.getDrawable().setColorFilter(context.getResources().getColor(R.color.bsdk_project_detail_appreciated_badge_color), PorterDuff.Mode.SRC_ATOP);
                        holder.appreciateThumb.setVisibility(View.GONE);
                        holder.thankYouText.setVisibility(View.VISIBLE);
                        projectAppreciated = true;
                        callbacks.onProjectAppreciated();
                    }
                }
            });
        }
    }

    @Override
    public int getItemCount() {
        return projectModules.size() + Math.max(projectComments.size(),commentsAllowed ?1 :0) + 5;//5 = header, appreciate,share_info, comments header, loader
    }

    @Override
    public int getItemViewType(int position) {
        if(position == 0)
            return ITEM_TYPE_HEADER_PADDING;
        else if(position <= projectModules.size())
            return ITEM_TYPE_PROJECT_MODULE;
        else if(position == projectModules.size() + 1)
            return ITEM_TYPE_APPRECIATE_VIEW;
        else if(position == projectModules.size() + 2)
            return ITEM_TYPE_INFO_SHARE_VIEW;
        else if(position == projectModules.size() + 3)
            return ITEM_TYPE_COMMENT_HEADER;
        else if(position < getItemCount()-1)
            return projectComments.size() == 0 ? ITEM_TYPE_COMMENT_PLACEHOLDER : ITEM_TYPE_COMMENT;
        else
            return ITEM_TYPE_LOADER;
    }

    public void setHeaderHeight(int headerHeight) {
        this.headerHeight = headerHeight;
    }

    public void addComments(List<BehanceSDKProjectCommentDTO> comments) {
        projectComments.addAll(comments);
        notifyDataSetChanged();
    }

    public void newCommentPosted(BehanceSDKProjectCommentDTO commentDTO) {
        projectComments.add(0, commentDTO);
        if(commentsCount == 0)
            notifyItemChanged(projectModules.size()+4);
        else
            notifyItemInserted(projectModules.size()+4);
        commentsCount += 1;
        notifyItemChanged(projectModules.size() + 3);
    }

    private CharSequence getFormattedDate(long time) {
        String dateFormatString;
        Locale currentLocale;
        dateFormatString = context.getResources().getString(R.string.bsdk_wip_details_view_posted_on_date_format);
        currentLocale = context.getResources().getConfiguration().locale;
        if (currentLocale == null) {
            currentLocale = Locale.ENGLISH;
        }
        SimpleDateFormat dateFormat;
        try {
            dateFormat = new SimpleDateFormat(dateFormatString, currentLocale);
            return dateFormat.format(time * 1000);
        } catch (IllegalArgumentException e) {
            try{
                dateFormatString = "MMM d hh:mm aaa";
                dateFormat = new SimpleDateFormat(dateFormatString,currentLocale);
                return dateFormat.format(time * 1000);
            } catch(IllegalArgumentException error){
                logger.error(e, "Comments View date formatting error with locale %s",currentLocale);
            }
        }
        return null;
    }

    public void setMoreToLoad(boolean moreToLoad) {
        this.moreToLoad = moreToLoad;
        notifyItemChanged(getItemCount() - 1);
    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    public List<BehanceSDKProjectCommentDTO> getProjectComments() {
        return projectComments;
    }

}
