package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.net.Uri;
import android.support.v4.app.FragmentActivity;
import android.support.v7.widget.RecyclerView;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;

import com.behance.sdk.dto.project.BehanceSDKProjectCoversDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.ui.adapters.viewholders.BehanceSDKProjectViewHolder;
import com.behance.sdk.ui.animations.BehanceSDKRecyclerItemAnimator;
import com.behance.sdk.util.BehanceSDKColumnCountUtil;
import com.facebook.drawee.backends.pipeline.Fresco;
import com.facebook.drawee.interfaces.DraweeController;
import com.facebook.imagepipeline.request.ImageRequest;
import com.facebook.imagepipeline.request.ImageRequestBuilder;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.List;

/**
 * Created by forta on 7/23/15.
 */
public class BehanceSDKProjectRecyclerViewAdapter extends RecyclerView.Adapter<BehanceSDKProjectViewHolder> {
    private static final String SHARE_PROJECT_ITEM_DIALOG_FRAGMENT_TAG = "SHARE_PROJECT_ITEM_DIALOG_FRAGMENT_TAG";
    private Context context;
    private List<BehanceSDKProjectDTO> projects;
    private BehanceSDK behanceSDK = BehanceSDK.getInstance();
    private LayoutInflater layoutInflater;
    private DisplayMetrics displayMetrics;
    private boolean moreToLoad = true;
    private int columns;
    private int loggedInUserId = -1;
    private boolean cleanFeed;
    private BehanceSDKRecyclerItemAnimator itemAnimator;

    public BehanceSDKProjectRecyclerViewAdapter(Context context, List<BehanceSDKProjectDTO> projects) {
        this.context = context;
        this.projects = projects;
        layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        displayMetrics = context.getResources().getDisplayMetrics();
        columns = BehanceSDKColumnCountUtil.getActivityColumnCount(context);
        BehanceSDKUserManager userManager = BehanceSDKUserManager.getInstance();
        if(userManager != null) {
            BehanceSDKUserDTO userDTO = userManager.getUserDTO();
            if(userDTO != null)
                loggedInUserId = userDTO.getId();
        }
       // cleanFeed = BehanceSDKPreferencesManager.getInstance(context).getBooleanPreference(BehanceAppBooleanPreferenceType.ENABLE_CLEAN_FEED, false);
        itemAnimator = BehanceSDKRecyclerItemAnimator.getInstance(context, columns);
    }

    @Override
    public BehanceSDKProjectViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (viewType == 1) { //loader
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_loader, parent, false);
            return new BehanceSDKProjectViewHolder(v);
        } else { //regular view
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.bsdk_card_view_project, parent, false);
            return new BehanceSDKProjectViewHolder(v);
        }
    }

    @Override
    public void onBindViewHolder(final BehanceSDKProjectViewHolder holder, int position) {
        if (position == getBasicItemCount() && holder.getItemViewType() == 1) {
            holder.card.setVisibility(moreToLoad ? View.VISIBLE : View.INVISIBLE);
            itemAnimator.setAnimation(holder.card, position, true);
        } else {
            if(cleanFeed) {
                holder.title.setLines(1);
                holder.appreciationsContainer.setVisibility(View.GONE);
                holder.artist.setVisibility(View.GONE);
            } else {
                holder.title.setLines(2);
                holder.appreciationsContainer.setVisibility(View.VISIBLE);
                holder.artist.setVisibility(View.VISIBLE);
            }
            final BehanceSDKProjectDTO dto = projects.get(position);
            boolean loggedInUserIsOwner = false;
            if(dto.getOwners() != null) {
                for (BehanceSDKUserDTO userDTO : dto.getOwners()) {
                    if (userDTO.getId() == loggedInUserId) {
                        loggedInUserIsOwner = true;
                        break;
                    }
                }
            }
//            if(loggedInUserIsOwner) {
//                holder.ownerMore.setVisibility(View.VISIBLE);
//                holder.ownerMore.setOnClickListener(new View.OnClickListener() {
//                    @Override
//                    public void onClick(View v) {
//                        PopupMenu popupMenu = new PopupMenu(context, v);
//                        popupMenu.getMenuInflater().inflate(R.menu.project_owner_actions, popupMenu.getMenu());
//                        popupMenu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
//                            @Override
//                            public boolean onMenuItemClick(MenuItem item) {
//                                switch (item.getItemId()) {
//                                    case R.id.project_owner_action_edit: editProject(dto);
//                                        return true;
//                                    case R.id.project_owner_action_clone: cloneProject(dto);
//                                        return true;
//                                    case R.id.project_owner_action_unpublish: unpublishProject(dto);
//                                        return true;
//                                    case R.id.project_owner_action_delete: deleteProject(dto);
//                                        return true;
//                                    default:
//                                        return false;
//                                }
//                            }
//                        });
//                        popupMenu.show();
//                    }
//                });
//            } else {
//                holder.ownerMore.setVisibility(View.GONE);
//            }

            holder.title.setText(dto.getName());
            if(!dto.getOwners().isEmpty())
                holder.artist.setText(dto.getOwners().get(0).getDisplayName());
            holder.cover.setImageDrawable(null);
            if(dto.getCovers() != null && dto.getCovers().findCoverImageInIncreasingSizeOrderAndFallback(BehanceSDKProjectCoversDTO.SIZE_404) != null) {
                ImageRequest request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(dto.getCovers().findCoverImageInIncreasingSizeOrderAndFallback(BehanceSDKProjectCoversDTO.SIZE_404).getUrl())).setProgressiveRenderingEnabled(true).build();
                DraweeController controller = Fresco.newDraweeControllerBuilder().setImageRequest(request).setOldController(holder.cover.getController()).build();
                holder.cover.setController(controller);
            }

            NumberFormat formatter = new DecimalFormat("###,###,###,###");
            if(dto.getStats() != null) {
                holder.appreciations.setText(formatter.format((double) dto.getStats().getAppreciationsCount()));
                holder.views.setText(formatter.format((double) dto.getStats().getViewsCount()));
            }

            holder.card.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    try {
                        behanceSDK.launchProjectDetailsActivity(context, dto.getId());
                    }catch (Exception e){

                    }
                }
            });
            holder.card.setOnLongClickListener(new View.OnLongClickListener() {
                @Override
                public boolean onLongClick(View v) {
                    displayShareProjectView(dto);
                    return true;
                }
            });
            itemAnimator.setAnimation(holder.card, position, false);
        }
    }

    private int getBasicItemCount() {
        return projects.size();
    }

    @Override
    public int getItemCount() {
        return getBasicItemCount() + 1;
    }

    @Override
    public int getItemViewType(int position) {
        if (position == getBasicItemCount()) {
            return 1;
        }
        return 0;
    }

    @Override
    public void onViewDetachedFromWindow(BehanceSDKProjectViewHolder viewHolder) {
        if(viewHolder != null && viewHolder.card != null) {
            viewHolder.card.clearAnimation();
            viewHolder.card.setTranslationY(0.f);
        }
    }

    public void setProjects(List<BehanceSDKProjectDTO> projects) {
        this.projects = projects;
        notifyDataSetChanged();
    }

    public void addProjectsItems(List<BehanceSDKProjectDTO> newProjects) {
        projects.addAll(newProjects);
    }

    public BehanceSDKProjectDTO getProjectItem(int position) {
        if(projects != null && position < projects.size())
            return projects.get(position);
        else
            return null;
    }

    private void displayShareProjectView(BehanceSDKProjectDTO dto) {
      //  BehanceSDKShareContentDialogLauncher.launchProjectShareContentDialog(dto, (FragmentActivity) context, SHARE_PROJECT_ITEM_DIALOG_FRAGMENT_TAG);
    }

    public void setMoreToLoad(boolean moreToLoad) {
        this.moreToLoad = moreToLoad;
        notifyItemChanged(getBasicItemCount());
    }

    private void editProject(BehanceSDKProjectDTO projectDTO) {
        Toast.makeText(context,"pretend it's been renamed...",Toast.LENGTH_SHORT).show();
    }

    private void cloneProject(BehanceSDKProjectDTO projectDTO) {
        Toast.makeText(context,"pretend it's now private...", Toast.LENGTH_SHORT).show();
    }

    private void unpublishProject(BehanceSDKProjectDTO projectDTO) {
        Toast.makeText(context,"pretend project unpublished...",Toast.LENGTH_SHORT).show();
    }

    private void deleteProject(BehanceSDKProjectDTO projectDTO) {
        Toast.makeText(context,"pretend project has been deleted...",Toast.LENGTH_SHORT).show();
    }

    public Context getContext() {
        return context;
    }

}
