/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.project.modules.EmbedModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;

import java.util.List;

/**
 * @author ugs
 *
 */
public class BehanceSDKPublishProjectEditAdapter extends ArrayAdapter<ProjectModule> {

	private LayoutInflater layoutInflater;
	private Callbacks callback;

	public interface Callbacks {
		public void onRotate(ImageModule projectModule);

	}

	public BehanceSDKPublishProjectEditAdapter(Context context, int resource, List<ProjectModule> projectModules) {
		super(context, resource, projectModules);
		layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		final int itemPosition = position;
		ProjectModule projectModule = getItem(itemPosition);
		if (convertView == null) { 
			// if it's not recycled, initialize some attributes
			convertView = layoutInflater.inflate(
					R.layout.bsdk_adapter_publish_project_edit_fragment_grid_item, parent, false);
		}
		
		final ImageView imageView = (ImageView) convertView.findViewById(R.id.add_project_edit_fragment_image_view);
        final ImageView imageViewDuplicate = (ImageView) convertView.findViewById(R.id.add_project_edit_fragment_image_view_duplicate);
		
		View embedView = convertView.findViewById(R.id.add_project_edit_fragment_embed_view);
		final View rotateImage = convertView.findViewById(R.id.add_project_edit_fragment_rotate_icon);
		ProjectModuleTypes moduleType = projectModule.getType();
		if(ProjectModuleTypes.IMAGE.equals(moduleType) || ProjectModuleTypes.CREATIVECLOUD_ASSET.equals(moduleType)){
			final ImageModule image= (ImageModule) projectModule;
			final Context context = getContext();
			embedView.setVisibility(View.GONE);
			imageView.setVisibility(View.VISIBLE);
			rotateImage.setVisibility(View.VISIBLE);
			imageView.setImageBitmap(image.getThumbNail(context));
			rotateImage.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(final View v) {
                    Animation fadeIn = AnimationUtils.loadAnimation(context, R.anim.bsdk_fade_in);
                    rotateImage.startAnimation(fadeIn);
                    imageViewDuplicate.setImageBitmap(image.getThumbNail(context));
                    imageViewDuplicate.setVisibility(View.VISIBLE);
                    imageView.setVisibility(View.INVISIBLE);
                    Animation anim = AnimationUtils.loadAnimation(context, R.anim.bsdk_thumbail_rotate);
                    anim.setAnimationListener(new Animation.AnimationListener() {
                        @Override
                        public void onAnimationStart(Animation animation) {

                        }

                        @Override
                        public void onAnimationEnd(Animation animation) {
                            if(null != callback){
                                ImageModule item = (ImageModule) getItem(itemPosition);
                                callback.onRotate(item);
                                imageView.setImageBitmap(image.getThumbNail(context));
                                imageView.setVisibility(View.VISIBLE);
                                imageViewDuplicate.setVisibility(View.INVISIBLE);
                                imageViewDuplicate.setImageBitmap(null);

                            }
                        }

                        @Override
                        public void onAnimationRepeat(Animation animation) {

                        }
                    });
                    imageViewDuplicate.startAnimation(anim);

				}
			});
		}else if(ProjectModuleTypes.EMBED.equals(moduleType)){
			imageView.setVisibility(View.GONE);
			embedView.setVisibility(View.VISIBLE);
			rotateImage.setVisibility(View.GONE);
			String description = ((EmbedModule)projectModule).getDescription();
			TextView embedDesc = (TextView) embedView.findViewById(R.id.bsdkPublishProjectEditFragmentEmbedDesc);
			embedDesc.setText(description);
		}
		return convertView;
	}

	public void setCallBack(Callbacks callback) {
		this.callback = callback;
	}
}
