/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.project.modules.EmbedModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener;

/**
 * @author ugs
 *
 */
public class BehanceSDKPublishProjectPreviewAdapter extends ArrayAdapter<ProjectModule> implements ImageLoadingListener  {

	private LayoutInflater layoutInflater;
	
	private Callbacks callbacks;
	
	private int numOfModulesLoaded = 0;
	
	private int numOfModulesLoading = 0;
	
	 public interface Callbacks {
	        public void onViewLoadingComplete();
	    }
	 
	 public void setCallbacks(Callbacks callbacks) {
	        this.callbacks = callbacks;
	    }
	
	public BehanceSDKPublishProjectPreviewAdapter(Context context, int resource, List<ProjectModule> projectModules) {
		super(context, resource, projectModules);
		layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}

	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		if (convertView == null) { 
			// if it's not recycled, initialize some attributes
			convertView = layoutInflater.inflate(R.layout.bsdk_adapter_publish_project_preview_project_fragment_list_item, parent, false);
		}
		ProjectModule projectModule = getItem(position);
		final ImageView imageView = (ImageView) convertView.findViewById(R.id.add_project_preview_project_fragment_image_view);
		imageView.setImageBitmap(null);
		View embedContentLayout = convertView.findViewById(R.id.add_project_preview_fragment_embed_view);
		ProjectModuleTypes type = projectModule.getType();
		if(ProjectModuleTypes.IMAGE.equals(type) || ProjectModuleTypes.CREATIVECLOUD_ASSET.equals(type)){
			embedContentLayout.setVisibility(View.GONE);
			imageView.setVisibility(View.VISIBLE);
			ImageModule image= (ImageModule) projectModule;
			numOfModulesLoading++;
			image.displayImage(imageView, this);
		}else if(ProjectModuleTypes.EMBED.equals(type)){
			embedContentLayout.setVisibility(View.VISIBLE);
			imageView.setVisibility(View.GONE);
			String description = ((EmbedModule)projectModule).getDescription();
			String subDescription = ((EmbedModule)projectModule).getSubDescription();
			TextView embedContentDecs = (TextView) embedContentLayout.findViewById(R.id.bsdkPublishProjectPreviewFragmentEmbedDesc);
			embedContentDecs.setText(description);
			TextView embedContentDecssub = (TextView) embedContentLayout.findViewById(R.id.bsdkPublishProjectPreviewFragmentEmbedSubDesc);
			embedContentDecssub.setText(subDescription);
			checkForLoadingComplete();
		}
		return convertView;
	}


	@Override
	public void onLoadingCancelled(String arg0, View arg1) {
		 checkForLoadingComplete();
	}


	@Override
	public void onLoadingComplete(String arg0, View view, Bitmap loadedBitmap) {
        checkForLoadingComplete();
	}


	@Override
	public void onLoadingFailed(String arg0, View arg1, FailReason arg2) {
		 checkForLoadingComplete();
	}


	@Override
	public void onLoadingStarted(String arg0, View arg1) {
		
	}
	
	private void checkForLoadingComplete() {
		if(++numOfModulesLoaded >= numOfModulesLoading){
            if(callbacks != null)
                callbacks.onViewLoadingComplete();
            numOfModulesLoading =0;
            numOfModulesLoaded = 0;
        }
	}
}
