/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.adapters;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKWIPDTO;
import com.behance.sdk.dto.BehanceSDKWIPDTO.WIPRevisionImageType;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKWIPListAdapter extends BaseAdapter {

	private LayoutInflater inflater;
	private List<BehanceSDKWIPDTO> wipsList;
	private DisplayImageOptions imageLoaderOptions;
	private ImageLoader imageloader;
	private int selectedWIPId;

	public BehanceSDKWIPListAdapter(Context context, List<BehanceSDKWIPDTO> wipsList, int selectedWIPId) {
		this.wipsList = wipsList;
		this.setSelectedWIPId(selectedWIPId);
		inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		imageLoaderOptions = new DisplayImageOptions.Builder().cacheInMemory(true).resetViewBeforeLoading(false).cacheOnDisk(false)
				.showImageOnFail(android.R.drawable.stat_notify_error).build();
		imageloader = ImageLoader.getInstance();
	}

	@Override
	public int getCount() {
		if (wipsList != null) {
			return wipsList.size();
		} else {
			return 0;
		}
	}

	@Override
	public BehanceSDKWIPDTO getItem(int arg0) {
		if (wipsList != null) {
			return wipsList.get(arg0);
		}
		return null;
	}

	@Override
	public View getView(int arg0, View convertView, ViewGroup parent) {
		View rootView;
		boolean viewReused = false;
		if (convertView == null) {
			rootView = inflater.inflate(R.layout.bsdk_adapter_wip_list_view_item, parent, false);
		} else {
			rootView = convertView;
			viewReused = true;
		}
		BehanceSDKWIPDTO wipDTO = getItem(arg0);
		if (wipDTO != null) {
			TextView wipNameTxtView = (TextView) rootView.findViewById(R.id.bsdkWIPItemAdapterWIPNameTextView);
			ImageView wipThumbnailImageView = (ImageView) rootView.findViewById(R.id.bsdkWIPItemAdapterWIPThumbnailImageView);
			View selectedIndicator = rootView.findViewById(R.id.bsdkWIPItemAdapterWIPSelectedIndicator);

			if (viewReused) {
				// clearing to remove old image
				wipThumbnailImageView.setImageDrawable(null);
			}
			wipNameTxtView.setText(wipDTO.getTitle());

			String latestRevisionImageUrl = wipDTO.getLatestRevisionImageUrl(WIPRevisionImageType.THUMBNAIL_SM);
			if (latestRevisionImageUrl != null && latestRevisionImageUrl.length() > 0) {
				imageloader.displayImage(latestRevisionImageUrl, wipThumbnailImageView, imageLoaderOptions);
			}

			if (wipDTO.getId() > 0 && wipDTO.getId() == getSelectedWIPId()) {
				selectedIndicator.setVisibility(View.VISIBLE);
			} else {
				selectedIndicator.setVisibility(View.INVISIBLE);
			}

		}
		return rootView;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	public int getSelectedWIPId() {
		return selectedWIPId;
	}

	public void setSelectedWIPId(int selectedWIPId) {
		this.selectedWIPId = selectedWIPId;
	}

}