package com.behance.sdk.ui.animations;

import android.animation.Animator;
import android.animation.ArgbEvaluator;
import android.animation.ValueAnimator;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.res.ColorStateList;
import android.os.Build;
import android.support.design.widget.FloatingActionButton;
import android.view.View;
import android.view.ViewAnimationUtils;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.DecelerateInterpolator;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;

/**
 * Created by forta on 10/8/15.
 */
public class BehanceSDKFabToBarAnimator {
    private static int f = 1;
    private static  final int animDurationOne = f*150, animDurationTwo = f*225, animDurationThree = f*300;
    private Context context;
    private FloatingActionButton fab;
    private View bar;
    private boolean open = false;

    public static BehanceSDKFabToBarAnimator getInstance(Context context, FloatingActionButton fab, View bar) {
        BehanceSDKFabToBarAnimator animator = new BehanceSDKFabToBarAnimator();
        animator.setBar(bar);
        animator.setFab(fab);
        animator.setContext(context);
        return animator;
    }

    public void fabToBar() {
        if(!open)
            animateFabToBar();
    }

    public void barToFab() {
        if(open)
            animateBarToFab();
    }

    public void setContext(Context context) {
        this.context = context;
    }

    public void setFab(FloatingActionButton fab) {
        this.fab = fab;
    }

    public void setBar(View bar) {
        this.bar = bar;
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private void animateFabToBar() {
        fab.setEnabled(false);
        Integer colorFrom;
        colorFrom = context.getResources().getColor(R.color.bsdk_behance_blue);
/*
        BehanceSDK behanceSDK = BehanceSDK.getInstance();
        if(behanceSDK.useBehanceTheme()){
            colorFrom = context.getResources().getColor(R.color.bsdk_behance_blue);
        }
        else{
            colorFrom = context.getResources().getColor(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppColor());
        }
*/
        Integer colorTo = context.getResources().getColor(R.color.bsdk_card_color);
        ValueAnimator colorAnimation = ValueAnimator.ofObject(new ArgbEvaluator(), colorFrom, colorTo);
        colorAnimation.setDuration(animDurationOne);
        colorAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animator) {
                fab.setBackgroundTintList(ColorStateList.valueOf((Integer) animator.getAnimatedValue()));
                bar.setBackgroundColor((Integer) animator.getAnimatedValue());
            }
        });
        fab.animate().translationY(context.getResources().getDimensionPixelSize(R.dimen.bsdk_fab_to_bar_anim_y_translation)).setDuration(animDurationTwo).setStartDelay(0).setInterpolator(new AccelerateDecelerateInterpolator()).start();
        fab.animate().setDuration(animDurationTwo).setStartDelay(0).translationX(-bar.getWidth() / 3).setInterpolator(new AccelerateInterpolator(2.f)).setUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            boolean b = false;

            @Override
            public void onAnimationUpdate(final ValueAnimator animation) {
                if (!b && fab.getX() < bar.getWidth() * 3 / 4) {
                    b = true;
                    Animator animator = ViewAnimationUtils.createCircularReveal(
                            bar, bar.getWidth() / 2, bar.getHeight() / 2, 0, (float) Math.hypot(bar.getWidth() / 2, bar.getHeight()));
                    bar.setVisibility(View.VISIBLE);
                    animator.setInterpolator(new DecelerateInterpolator(1.5f));
                    if (bar.getVisibility() == View.VISIBLE) {
                        animator.setDuration(animDurationThree);
                        animator.addListener(new Animator.AnimatorListener() {
                            @Override
                            public void onAnimationStart(Animator animation) {

                            }

                            @Override
                            public void onAnimationEnd(Animator animation) {
                                open = true;
                            }

                            @Override
                            public void onAnimationCancel(Animator animation) {

                            }

                            @Override
                            public void onAnimationRepeat(Animator animation) {

                            }
                        });
                        bar.setTranslationX(fab.getX() + fab.getWidth() / 2 - bar.getWidth() / 2);
                        bar.setScaleX(0.5f);
                        bar.animate().translationX(0).setDuration(animDurationThree).setInterpolator(new DecelerateInterpolator(1.8f)).start();
                        bar.animate().setDuration(animDurationThree).scaleX(1.f).setInterpolator(new DecelerateInterpolator(1.2f)).start();
                        animator.start();
                        bar.setEnabled(true);
                    }
                }
            }
        }).start();
        colorAnimation.start();
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private void animateBarToFab() {
        bar.setEnabled(false);
        Integer colorTo;
        colorTo = context.getResources().getColor(R.color.bsdk_behance_blue);
/*
        BehanceSDK behanceSDK = BehanceSDK.getInstance();
        if(behanceSDK.useBehanceTheme()){
        colorTo = context.getResources().getColor(R.color.bsdk_behance_blue);
        }
        else{
            colorTo = context.getResources().getColor(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppColor());
        }
        */
        Integer colorFrom = context.getResources().getColor(R.color.bsdk_card_color);

        final ValueAnimator colorAnimation = ValueAnimator.ofObject(new ArgbEvaluator(), colorFrom, colorTo);
        colorAnimation.setDuration(animDurationOne);
        colorAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animator) {
                fab.setBackgroundTintList(ColorStateList.valueOf((Integer) animator.getAnimatedValue()));
            }
        });
        Animator animator = ViewAnimationUtils.createCircularReveal(
                bar, bar.getWidth() / 2, bar.getHeight() / 2, (float) Math.hypot(bar.getWidth() / 2, bar.getHeight()), 0);
        bar.setVisibility(View.VISIBLE);
        animator.setInterpolator(new AccelerateInterpolator(1.5f));
        if (bar.getVisibility() == View.VISIBLE) {
            animator.setDuration(animDurationThree);
            animator.addListener(new Animator.AnimatorListener() {
                @Override
                public void onAnimationStart(Animator animation) {

                }

                @Override
                public void onAnimationEnd(Animator animation) {
                    bar.setVisibility(View.INVISIBLE);
                    colorAnimation.start();
                    fab.setEnabled(true);
                }

                @Override
                public void onAnimationCancel(Animator animation) {

                }

                @Override
                public void onAnimationRepeat(Animator animation) {

                }
            });
            animator.start();
            bar.animate().translationX(bar.getWidth()/3).setDuration(animDurationThree).setInterpolator(new AccelerateInterpolator(1.8f)).start();
            bar.animate().setDuration(animDurationThree).scaleX(0.5f).setInterpolator(new AccelerateInterpolator(1.2f)).start();
            fab.animate().setStartDelay(animDurationTwo).setDuration(animDurationTwo).translationY(0).setInterpolator(new AccelerateDecelerateInterpolator()).start();
            fab.animate().setStartDelay(animDurationTwo).setDuration(animDurationTwo).translationX(0).setInterpolator(new DecelerateInterpolator(2.f)).setListener(new Animator.AnimatorListener() {
                @Override
                public void onAnimationStart(Animator animation) {

                }

                @Override
                public void onAnimationEnd(Animator animation) {
                    open = false;
                }

                @Override
                public void onAnimationCancel(Animator animation) {

                }

                @Override
                public void onAnimationRepeat(Animator animation) {

                }
            }).start();
        }
    }
}
