package com.behance.sdk.ui.components;

import android.content.Context;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.os.Build;
import android.util.AttributeSet;
import android.widget.Button;

import com.behance.sdk.R;

/**
 * Created by forta on 1/7/16.
 */
public class BehanceSDKFollowUnFollowButton extends Button {

    private Context context;
    private boolean usesFollowAll = false;

    public BehanceSDKFollowUnFollowButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        initialize(context, attrs);
        this.context = context;
        layout();
    }

    public BehanceSDKFollowUnFollowButton(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initialize(context, attrs);
        this.context = context;
        layout();
    }

    private void initialize(Context context, AttributeSet attrs) {
        TypedArray values = context.obtainStyledAttributes(attrs, R.styleable.BehanceSDKFollowButton);
        usesFollowAll = values.getBoolean(R.styleable.BehanceSDKFollowButton_bsdk_uses_follow_all, false);
        values.recycle();
    }

    //set view width to width of larger of follow/unfollow text
    private void layout() {
        Resources res = context.getResources();
        Paint paint = new Paint();
        Rect bounds = new Rect();

        paint.setTextSize(getTextSize());
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            paint.setTypeface(Typeface.create("sans-serif", Typeface.NORMAL));

        String followText = res.getString(R.string.bsdk_follow);
        paint.getTextBounds(followText, 0, followText.length(), bounds);
        int followTextWidth = (int) paint.measureText(followText);

        String unFollowText = res.getString(R.string.bsdk_unfollow);
        paint.getTextBounds(unFollowText, 0, unFollowText.length(), bounds);
        int unFollowTextWidth = (int) paint.measureText(unFollowText);

        int textWidth = Math.max(followTextWidth, unFollowTextWidth);

        if(usesFollowAll) {
            String followAllText = res.getString(R.string.bsdk_follow_all);
            paint.getTextBounds(unFollowText, 0, unFollowText.length(), bounds);
            int followAllTextWidth = (int) paint.measureText(followAllText);

            textWidth = Math.max(textWidth, followAllTextWidth);
        }

        setWidth(textWidth + 2 * res.getDimensionPixelSize(R.dimen.bsdk_follow_button_horizontal_padding));
    }

    @Override
    protected void onMeasure(int width, int height) {
        super.onMeasure(width, height);
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
    }

}
