package com.behance.sdk.ui.components;

import android.content.Context;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.AttributeSet;
import android.view.MotionEvent;

import com.behance.sdk.listeners.IBehanceSDKSnappingLayoutManager;


/**
 * Created by forta on 10/29/15.
 */
public class BehanceSDKSnappingRecyclerView extends RecyclerView {
    public BehanceSDKSnappingRecyclerView(Context context) {
        super(context);
    }

    public BehanceSDKSnappingRecyclerView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public BehanceSDKSnappingRecyclerView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    @Override
    public boolean fling(int velocityX, int velocityY) {
        final LayoutManager lm = getLayoutManager();

        if (lm instanceof IBehanceSDKSnappingLayoutManager) {
            int posForVel = ((IBehanceSDKSnappingLayoutManager) getLayoutManager()).getPositionForVelocity(velocityX, velocityY);
            int curFirstVis = ((LinearLayoutManager) super.getLayoutManager()).findFirstVisibleItemPosition();
            if(posForVel < 3 || curFirstVis <= 1) {
                if (posForVel <= 1 && curFirstVis > 2)
                    posForVel = 2;
                else if (posForVel == 0 && curFirstVis > 1)
                    posForVel = 1;
                else if(posForVel > 1 && curFirstVis == 0)
                    posForVel = 1;
                else if(posForVel > 1 && curFirstVis == 1)
                    posForVel = 2;
                else if(posForVel == 1 && curFirstVis == 0 && velocityY <= 0)
                    posForVel = 0;
                super.smoothScrollToPosition(posForVel);
                return true;
            }
        }
        return super.fling(velocityX, velocityY);
    }

    @Override
    public boolean onTouchEvent(MotionEvent e) {
        // We want the parent to handle all touch events--there's a lot going on there,
        // and there is no reason to overwrite that functionality--bad things will happen.
        final boolean ret = super.onTouchEvent(e);
        final LayoutManager lm = getLayoutManager();

        if (lm instanceof IBehanceSDKSnappingLayoutManager
                && (e.getAction() == MotionEvent.ACTION_UP ||
                e.getAction() == MotionEvent.ACTION_CANCEL)
                && getScrollState() == SCROLL_STATE_IDLE) {
            // The layout manager is a SnappyLayoutManager, which means that the
            // children should be snapped to a grid at the end of a drag or
            // fling. The motion event is either a user lifting their finger or
            // the cancellation of a motion events, so this is the time to take
            // over the scrolling to perform our own functionality.
            // Finally, the scroll state is idle--meaning that the resultant
            // velocity after the user's gesture was below the threshold, and
            // no fling was performed, so the view may be in an unaligned state
            // and will not be flung to a proper state.
            int fixedPos = ((IBehanceSDKSnappingLayoutManager) lm).getFixScrollPos();
            if(fixedPos < 3)
                smoothScrollToPosition(fixedPos);
        }

        return ret;
    }
}
