package com.behance.sdk.ui.components;

import android.content.Context;
import android.graphics.Matrix;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.widget.ImageView;

/**
 * Created by forta on 9/17/15.
 */
public class BehanceSDKTopCropImageView extends ImageView {

    public BehanceSDKTopCropImageView(Context context) {
        super(context);
        setScale();
    }

    public BehanceSDKTopCropImageView(Context context, AttributeSet attrs) {
        super(context, attrs);
        setScale();
    }

    public BehanceSDKTopCropImageView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        setScale();
    }

    private void setScale() {
        setScaleType(ScaleType.MATRIX);
    }

    @Override
    public boolean setFrame(int left, int top, int right, int bottom) {
        computeImageMatrix();
        return super.setFrame(left, top, right, bottom);
    }

    private void computeImageMatrix() {
        if(getDrawable() != null) {
            Matrix matrix = getImageMatrix();

            float scale;
            int imageViewWidth = getWidth() - getPaddingLeft() - getPaddingRight();
            int imageViewHeight = getHeight() - getPaddingTop() - getPaddingBottom();
            int drawableWidth = getDrawable().getIntrinsicWidth();
            int drawableHeight = getDrawable().getIntrinsicHeight();

            if (drawableWidth * imageViewHeight > drawableHeight * imageViewWidth) {
                scale = (float) imageViewHeight / (float) drawableHeight;
            } else {
                scale = (float) imageViewWidth / (float) drawableWidth;
            }

            RectF drawableRect = new RectF(0, drawableHeight - (imageViewHeight / scale), drawableWidth, drawableHeight);
            RectF viewRect = new RectF(0, 0, imageViewWidth, imageViewHeight);
            matrix.setRectToRect(drawableRect, viewRect, Matrix.ScaleToFit.FILL);

            setImageMatrix(matrix);
        }
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        computeVerticalScrollOffset();
    }
}
