/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.customviews;

import android.content.Context;
import android.util.AttributeSet;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;

import com.behance.sdk.dto.project.BehanceSDKAbstractCommentDTO;
import com.behance.sdk.dto.search.BehanceSDKImageDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.util.BehanceSDKImageLoaderUtils;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKCommentsView extends LinearLayout implements View.OnClickListener, View.OnCreateContextMenuListener, MenuItem.OnMenuItemClickListener {

	private ImageLoader mloader;
	private DisplayImageOptions options;
	private BehanceSDKAbstractCommentDTO contextMenuActiveCommentDTO;
	private Callbacks callbacks;
	private View contextMenuActiveCommentView;
	private ILogger logger = LoggerFactory.getLogger(BehanceSDKCommentsView.class);
	private boolean ownedByUser = false;

	public interface Callbacks {
		public void onDeleteCommentRequested(BehanceSDKAbstractCommentDTO commentDTO);
	}

	public BehanceSDKCommentsView(Context context) {
		super(context);
		initialize();
	}

	public BehanceSDKCommentsView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initialize();
	}

	public BehanceSDKCommentsView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		initialize();
	}

	private void initialize() {
		mloader = ImageLoader.getInstance();
		options = BehanceSDKImageLoaderUtils.getDefaultImageLoaderOptions();
		this.setOrientation(LinearLayout.VERTICAL);
	}

	public void displayCommentsInLayout(List<? extends BehanceSDKAbstractCommentDTO> commentsList, BehanceSDKUserDTO ownerUserDTO, Callbacks callbacks) {
		List<BehanceSDKUserDTO> usersList = new ArrayList<BehanceSDKUserDTO>(1);
		usersList.add(ownerUserDTO);
		displayCommentsInLayout(commentsList, usersList, callbacks);
	}

	public void displayCommentsInLayout(List<? extends BehanceSDKAbstractCommentDTO> commentsList, List<BehanceSDKUserDTO> ownerUserDTOList, Callbacks callbacks) {

		this.callbacks = callbacks;

		LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

		final List<View> commentViewsList = new ArrayList<View>();

		Set<Integer> ownerIdsSet = new HashSet<Integer>();
		for (BehanceSDKUserDTO ownerDTO : ownerUserDTOList) {
			ownerIdsSet.add(ownerDTO.getId());
		}

		for (BehanceSDKAbstractCommentDTO abstractCommentDTO : commentsList) {
			View commentRootView = createCommentView(inflater, ownerIdsSet, abstractCommentDTO);
			commentViewsList.add(commentRootView);
		}

		this.post(new Runnable() {
			@Override
			public void run() {
				// remove old comments
				clearComments();

				LayoutParams defaultLayoutParams = generateDefaultLayoutParams();
				for (View commentView : commentViewsList) {
					addViewInLayout(commentView, -1, defaultLayoutParams, true);
				}
				commentViewsList.clear();
				setVisibility(View.VISIBLE);
				requestLayout();
			}
		});
	}

	private View createCommentView(LayoutInflater inflater, Set<Integer> ownerIdsSet, BehanceSDKAbstractCommentDTO abstractCommentDTO) {
		View commentRootView = inflater.inflate(R.layout.bsdk_view_comments_comment_item_adapter, null);
		TextView commentUserNameTxtView = (TextView) commentRootView.findViewById(R.id.bsdk_commentViewUserNameTxtView);
		TextView commentContentTxtView = (TextView) commentRootView.findViewById(R.id.bsdk_commentViewCommentContentTxtView);
		TextView commentDateTxtView = (TextView) commentRootView.findViewById(R.id.bsdk_commentViewCommentDateTxtView);
		ImageView commenterImageView = (ImageView) commentRootView.findViewById(R.id.bsdk_commentViewUserImageView);

		commentRootView.setOnCreateContextMenuListener(this);
		commentRootView.setTag(abstractCommentDTO);

		BehanceSDKUserManager behanceUserManager = BehanceSDKUserManager.getInstance();
		BehanceSDKUserDTO loggedInUserDTO = behanceUserManager.getUserDTO();
		if(loggedInUserDTO != null)
			ownedByUser = ownerIdsSet.contains(loggedInUserDTO.getId());

		BehanceSDKUserDTO commenterUserDTO = abstractCommentDTO.getUser();

		if (commenterUserDTO != null) {

			// listen for click
			commenterImageView.setOnClickListener(this);
			commenterImageView.setTag(commenterUserDTO);
			commentUserNameTxtView.setOnClickListener(this);
			commentUserNameTxtView.setTag(commenterUserDTO);

			commentUserNameTxtView.setText(commenterUserDTO.getDisplayName());

			// display user profile image
			BehanceSDKImageDTO profileImageDTO = commenterUserDTO.findProfileImageInIncreasingSizeOrder(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_50);
			if (profileImageDTO != null) {
				mloader.displayImage(profileImageDTO.getUrl(), commenterImageView, options);
			}

		}

		commentDateTxtView.setText(getFormattedDate(abstractCommentDTO.getCreatedDate()));
		String commentContent = abstractCommentDTO.getContent();
		if (commentContent != null) {
			commentContentTxtView.setText(commentContent.trim());
		}
		return commentRootView;
	}

	public void addCommentsToLayout(List<? extends BehanceSDKAbstractCommentDTO> commentsList, List<BehanceSDKUserDTO> ownerUserDTOList, Callbacks callbacks) {

		this.callbacks = callbacks;

		LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

		final List<View> commentViewsList = new ArrayList<View>();

		Set<Integer> ownerIdsSet = new HashSet<Integer>();
		for (BehanceSDKUserDTO ownerDTO : ownerUserDTOList) {
			ownerIdsSet.add(ownerDTO.getId());
		}

		for (BehanceSDKAbstractCommentDTO abstractCommentDTO : commentsList) {
			View commentRootView = createCommentView(inflater, ownerIdsSet, abstractCommentDTO);
			commentViewsList.add(commentRootView);
		}

		this.post(new Runnable() {
			@Override
			public void run() {

				LayoutParams defaultLayoutParams = generateDefaultLayoutParams();
				for (View commentView : commentViewsList) {
					addViewInLayout(commentView, -1, defaultLayoutParams, true);
				}
				commentViewsList.clear();
				setVisibility(View.VISIBLE);
				requestLayout();
			}
		});
	}

	public void clearComments() {
		this.removeAllViews();
	}

	private CharSequence getFormattedDate(long time) {
		String dateFormatString;
		Locale currentLocale;
		dateFormatString = getResources().getString(R.string.bsdk_wip_details_view_posted_on_date_format);
		currentLocale = getResources().getConfiguration().locale;
		if (dateFormatString == null) {
			dateFormatString = new String("MMM d \'at\' hh:mm aaa");
		}

		if (currentLocale == null) {
			currentLocale = Locale.ENGLISH;
		}
		SimpleDateFormat dateFormat = null;
		try {
			dateFormat = new SimpleDateFormat(dateFormatString, currentLocale);
			return dateFormat.format(time * 1000);
		} catch (IllegalArgumentException e) {
			try{
				dateFormatString = new String("MMM d hh:mm aaa");
				dateFormat = new SimpleDateFormat(dateFormatString,currentLocale);
				return dateFormat.format(time * 1000);
			} catch(IllegalArgumentException error){
				logger.error(e, "Comments View date formatting error with locale %s", currentLocale);
			}
		}
		return null;
	}



	@Override
	public void onClick(View v) {
		// all clicks in this are performing same action, so not checking for
		// target view
		if (v.getTag() instanceof BehanceSDKUserDTO) {
		//	BehanceSDK.launchUserProfileActivity(getContext(), (BehanceSDKUserDTO) v.getTag());
		}
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
		BehanceSDKUserManager behanceUserManager = BehanceSDKUserManager.getInstance();
		BehanceSDKUserDTO loggedInUserDTO = behanceUserManager.getUserDTO();
		if(loggedInUserDTO != null)
		if (behanceUserManager.isUserLoggedIn() && loggedInUserDTO != null && callbacks != null) {
			if (v.getTag() instanceof BehanceSDKAbstractCommentDTO) {
				BehanceSDKAbstractCommentDTO commentDTO = (BehanceSDKAbstractCommentDTO) v.getTag();
				BehanceSDKUserDTO commentUserDTO = commentDTO.getUser();
				if (commentUserDTO != null && (commentUserDTO.getId() == loggedInUserDTO.getId() || ownedByUser)) {
					contextMenuActiveCommentDTO = commentDTO;
					contextMenuActiveCommentView = v;
					MenuItem menuItem = menu.add(1, 1, 0, getResources().getString(R.string.bsdk_menu_item_label_delete_comment));
					menuItem.setOnMenuItemClickListener(this);
				}
			}
		}
	}

	@Override
	public boolean onMenuItemClick(MenuItem item) {
		if (contextMenuActiveCommentDTO != null && contextMenuActiveCommentView != null && callbacks != null) {
			callbacks.onDeleteCommentRequested(contextMenuActiveCommentDTO);
			removeView(contextMenuActiveCommentView);
		}
		contextMenuActiveCommentDTO = null;
		contextMenuActiveCommentView = null;
		return true;
	}
}