package com.behance.sdk.ui.customviews;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import com.behance.sdk.R;

/**
 * Created by forta on 12/9/15.
 */
public class BehanceSDKEmptyStatesView extends FrameLayout {

    public BehanceSDKEmptyStatesView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initialize(context, attrs);
    }

    public BehanceSDKEmptyStatesView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initialize(context, attrs);
    }

    private void initialize(Context context, AttributeSet attrs) {
        this.addView(LayoutInflater.from(context).inflate(R.layout.bsdk_custom_view_empty_state, this, false));

        TypedArray values = context.obtainStyledAttributes(attrs, R.styleable.BehanceSDKEmptyStatesView);

        setIcon(values.getDrawable(R.styleable.BehanceSDKEmptyStatesView_bsdk_image_res));
        setTitleText(values.getString(R.styleable.BehanceSDKEmptyStatesView_bsdk_title_res));
        setDescriptionText(values.getString(R.styleable.BehanceSDKEmptyStatesView_bsdk_description_res));
        setActionText(values.getString(R.styleable.BehanceSDKEmptyStatesView_bsdk_action_res));

        values.recycle();
    }


    //Set text to null to hide view
    public void setTitleText(@Nullable String title) {
        if(title == null || title.isEmpty()) {
            findViewById(R.id.bsdk_empty_state_title).setVisibility(GONE);
        } else {
            findViewById(R.id.bsdk_empty_state_title).setVisibility(VISIBLE);
            ((TextView) findViewById(R.id.bsdk_empty_state_title)).setText(title);
        }
    }

    public void setDescriptionText(@Nullable String description) {
        if(description == null || description.isEmpty()) {
            findViewById(R.id.bsdk_empty_state_description).setVisibility(GONE);
        } else {
            findViewById(R.id.bsdk_empty_state_description).setVisibility(VISIBLE);
            ((TextView) findViewById(R.id.bsdk_empty_state_description)).setText(description);
        }
    }

    public void setActionText(@Nullable String action) {
        if(action == null || action.isEmpty()) {
            findViewById(R.id.bsdk_empty_state_action).setVisibility(GONE);
        } else {
            findViewById(R.id.bsdk_empty_state_action).setVisibility(VISIBLE);
            ((TextView) findViewById(R.id.bsdk_empty_state_action)).setText(action);
        }
    }

    public void setIcon(@Nullable Drawable icon) {
        if(icon == null) {
            findViewById(R.id.bsdk_empty_state_icon).setVisibility(GONE);
        } else {
            findViewById(R.id.bsdk_empty_state_icon).setVisibility(VISIBLE);
            ((ImageView) findViewById(R.id.bsdk_empty_state_icon)).setImageDrawable(icon);
        }
    }

    public void setActionViewOnClickListener(OnClickListener onClickListener) {
        findViewById(R.id.bsdk_empty_state_action).setOnClickListener(onClickListener);
    }

}
