package com.behance.sdk.ui.dialogs;

import android.animation.Animator;
import android.annotation.TargetApi;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.DialogFragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.TextView;
import android.widget.Toast;

import com.behance.sdk.R;
import com.behance.sdk.asynctask.params.BehanceSDKDownloadAndSaveImageAsyncTaskParams;
import com.behance.sdk.asynctasks.BehanceSDKDownloadAndSaveImageAsyncTask;
import com.behance.sdk.dto.BehanceSDKShareContentAdapterItemDTO;
import com.behance.sdk.dto.BehanceSDKShareContentDTO;
import com.behance.sdk.enums.BehanceSDKShareContentType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.adapters.BehanceSDKBottomSheetRecyclerAdapter;
import com.behance.sdk.ui.components.BehanceSDKSnappingLinearLayoutManager;
import com.behance.sdk.ui.customviews.BehanceSDKSpaceItemDecorationList;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.behance.sdk.util.BehanceSDKUtils;


import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by forta on 10/29/15.
 */
public class BehanceSDKBottomSheetShareDialog extends DialogFragment implements BehanceSDKBottomSheetRecyclerAdapter.Callbacks, DialogInterface.OnKeyListener {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKBottomSheetShareDialog.class);

    private static final String PACKAGE_NAME_PINTEREST = "pinterest.act";
    private static final String PACKAGE_NAME_FACEBOOK = "facebook.katana";
    private static final String PACKAGE_NAME_TWITTER = "twitter.android";

    private RecyclerView sheetRecycler;
    private View header;
    private View contentOverlay;
    private BehanceSDKShareContentType shareContentType;
    private BehanceSDKShareContentDTO shareContent;

    public BehanceSDKBottomSheetShareDialog() {
    }

    public static BehanceSDKBottomSheetShareDialog getInstance(BehanceSDKShareContentDTO shareContent, BehanceSDKShareContentType shareContentType) {
        BehanceSDKBottomSheetShareDialog dialog = new BehanceSDKBottomSheetShareDialog();
        dialog.shareContent = shareContent;
        dialog.shareContentType = shareContentType;
        return dialog;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        if(savedInstanceState != null)
            this.dismiss(); //close on orientation change
        super.onCreate(savedInstanceState);
        setStyle(DialogFragment.STYLE_NORMAL, R.style.BsdkBottomSheetDialogTheme);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        Dialog addToCollectionDialog = super.onCreateDialog(savedInstanceState);
        addToCollectionDialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        addToCollectionDialog.setOnKeyListener(this);
        return addToCollectionDialog;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.bsdk_dialog_fragment_bottom_sheet, container, false);
        sheetRecycler = (RecyclerView) view.findViewById(R.id.bsdk_bottom_sheet_recycler);
        sheetRecycler.setLayoutManager(new BehanceSDKSnappingLinearLayoutManager(getActivity(), LinearLayoutManager.VERTICAL, false));
        sheetRecycler.addItemDecoration(new BehanceSDKSpaceItemDecorationList(0));
        List<BehanceSDKShareContentAdapterItemDTO> shareList = getShareContentItemsList(shareContentType);
        BehanceSDKBottomSheetRecyclerAdapter adapter = new BehanceSDKBottomSheetRecyclerAdapter(getActivity(), shareList);
        adapter.setCallbacks(this);
        sheetRecycler.setAdapter(adapter);
        header = view.findViewById(R.id.bsdk_bottom_sheet_header);
        final int transY  = getResources().getDisplayMetrics().heightPixels + BehanceSDKUtils.getTranslucentNavHeight(getActivity());
        header.setTranslationY(transY);
        sheetRecycler.postDelayed(new Runnable() {
            @Override
            public void run() {
                sheetRecycler.smoothScrollToPosition(1);
                //must be set after scroll so closeDialog when view is scrolled to top is not called immediately
                sheetRecycler.addOnScrollListener(new RecyclerView.OnScrollListener() {
                    int y = (int) header.getTranslationY();

                    @Override
                    public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                        super.onScrollStateChanged(recyclerView, newState);
                        if (newState == RecyclerView.SCROLL_STATE_IDLE && ((LinearLayoutManager) recyclerView.getLayoutManager()).findFirstCompletelyVisibleItemPosition() == 0)
                            closeDialog(false);
                    }
                    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
                    @Override
                    public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                        super.onScrolled(recyclerView, dx, dy);
                        try {
                            y -= dy;
                            header.setTranslationY(y > 0 ? y : 0);
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
                                header.animate().translationZ(y < 0 ? getResources().getDimensionPixelSize(R.dimen.bsdk_toolbar_elevation) : 0).setDuration(30).start();
                            if (y == transY)
                                closeDialog(false);
                        }catch (NoSuchMethodError e){
                            logger.error(e,"No such Method found error : TranslationZ");
                        }
                    }
                });
            }
        }, 10);
        contentOverlay = view.findViewById(R.id.bsdk_bottom_sheet_content_overlay);
        contentOverlay.animate().alpha(1.f);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                closeDialog(true);
            }
        });
        setTitle(((TextView)view.findViewById(R.id.bsdk_bottom_sheet_header_textview)));
        return view;
    }

    private List<BehanceSDKShareContentAdapterItemDTO> getShareContentItemsList(BehanceSDKShareContentType shareContentType) {

        List<BehanceSDKShareContentAdapterItemDTO> shareContentItemsList = new ArrayList<BehanceSDKShareContentAdapterItemDTO>();
        List<BehanceSDKShareContentAdapterItemDTO> nonDefaultContentItemsList = new ArrayList<BehanceSDKShareContentAdapterItemDTO>();

        PackageManager packageManager = getActivity().getPackageManager();
        Resources resources = getActivity().getResources();

        // Download image
        if(shareContentType == BehanceSDKShareContentType.IMAGE) {
            BehanceSDKShareContentAdapterItemDTO downloadImageItem = new BehanceSDKShareContentAdapterItemDTO();
            downloadImageItem.setPackageName(BehanceSDKShareContentAdapterItemDTO.DOWNLOAD_IMAGE_KEY);
            downloadImageItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_savetodevicce));
            downloadImageItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_download_image));
            shareContentItemsList.add(downloadImageItem);
        }

        // Copy as Link
        BehanceSDKShareContentAdapterItemDTO copyLinkShareItem = new BehanceSDKShareContentAdapterItemDTO();
        copyLinkShareItem.setPackageName(BehanceSDKShareContentAdapterItemDTO.COPY_LINK_SHARING_PACKAGE_NAME);
        copyLinkShareItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_link));
        copyLinkShareItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_copy_link_label));
        shareContentItemsList.add(copyLinkShareItem);

        // Share using Email
        BehanceSDKShareContentAdapterItemDTO emailShareItem = new BehanceSDKShareContentAdapterItemDTO();
        emailShareItem.setPackageName(BehanceSDKShareContentAdapterItemDTO.EMAIL_SHARING_PACKAGE_NAME);
        emailShareItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_email));
        emailShareItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_email_label));
        shareContentItemsList.add(emailShareItem);

        Intent queryIntent = getDefaultShareIntent();

        List<ResolveInfo> resInfo = packageManager.queryIntentActivities(queryIntent, 0);

        if (!resInfo.isEmpty()) {
            Set<String> appNames = new HashSet<String>();
            for (final ResolveInfo info : resInfo) {

                try {
                    ActivityInfo activityInfo = info.activityInfo;
                    ApplicationInfo applicationInfo = activityInfo.applicationInfo;

                    if (applicationInfo != null) {

                        String activityPackageName = activityInfo.packageName;
                        String activityName = activityInfo.name;

                        BehanceSDKShareContentAdapterItemDTO shareItem;

                        // Pinterest App
                        if (containsIgnoreCase(activityPackageName, PACKAGE_NAME_PINTEREST)
                                || containsIgnoreCase(activityName, PACKAGE_NAME_PINTEREST)) {

                            // don't show Pinterest for User and Collection
                            if (shareContentType != BehanceSDKShareContentType.COLLECTION && shareContentType != BehanceSDKShareContentType.USER) {
                                shareItem = new BehanceSDKShareContentAdapterItemDTO();
                                shareItem.setPackageName(PACKAGE_NAME_PINTEREST);
                                shareItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_pinterest));
                                shareItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_pinterest_label));
                                shareContentItemsList.add(shareItem);
                            }
                        } else if (containsIgnoreCase(activityPackageName, PACKAGE_NAME_FACEBOOK)
                                || containsIgnoreCase(activityName, PACKAGE_NAME_FACEBOOK)) {
                            shareItem = new BehanceSDKShareContentAdapterItemDTO();
                            shareItem.setPackageName(PACKAGE_NAME_FACEBOOK);
                            shareItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_facebook));
                            shareItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_facebook_label));
                            shareContentItemsList.add(shareItem);
                        } else if (containsIgnoreCase(activityPackageName, PACKAGE_NAME_TWITTER)
                                || containsIgnoreCase(activityName, PACKAGE_NAME_TWITTER)) {
                            if (!appNames.contains(activityPackageName)) {
                                appNames.add(activityPackageName);
                                shareItem = new BehanceSDKShareContentAdapterItemDTO();
                                shareItem.setPackageName(PACKAGE_NAME_TWITTER);
                                shareItem.setDisplayIcon(resources.getDrawable(R.drawable.bsdk_share_bottom_sheet_icon_twitter));
                                shareItem.setDisplayLabel(resources.getString(R.string.bsdk_share_content_dialog_twitter_label));
                                shareContentItemsList.add(shareItem);
                            } else {
                                // Twitter has two activities registered, one for Compose and other for DM
                                logger.debug("Ignoring duplicate Twitter activity");
                            }
                        } else {
                            if (!appNames.contains(applicationInfo.packageName)) {
                                appNames.add(applicationInfo.packageName);
                                shareItem = new BehanceSDKShareContentAdapterItemDTO();
                                shareItem.setPackageName(activityPackageName);
                                shareItem.setDisplayLabel(applicationInfo.loadLabel(packageManager).toString());
                                shareItem.setDisplayIcon(applicationInfo.loadIcon(packageManager));
                                nonDefaultContentItemsList.add(shareItem);
                            } else {
                                logger.debug("Ignoring duplicate Share option [app name - %s] [activity name - %s]", applicationInfo.packageName,
                                        activityName);
                            }
                        }
                    }
                } catch (Resources.NotFoundException e) {
                    logger.error(e, "Problem getting label or icon for Sharing");
                }
            }
        }


        if (!nonDefaultContentItemsList.isEmpty()) {
            shareContentItemsList.addAll(nonDefaultContentItemsList);
        }

        return shareContentItemsList;
    }

    private Intent getDefaultShareIntent() {
        Intent queryIntent = new Intent(Intent.ACTION_SEND);
//        queryIntent.setType("image/jpeg");
        queryIntent.setType("text/plain");
        return queryIntent;
    }

    private boolean containsIgnoreCase(String sourceStr, String targetStr) {
        return StringUtils.containsIgnoreCase(sourceStr, targetStr);
    }

    private void setTitle(TextView titleView) {
        int title;
        if(shareContentType != null) {
            switch (shareContentType) {
                case PROJECT:
                    title = R.string.bsdk_share_content_dialog_title_project;
                    break;
                case IMAGE:
                    title = R.string.bsdk_share_content_dialog_title_image;
                    break;
                case WIP:
                    title = R.string.bsdk_share_content_dialog_title_wip;
                    break;
                case COLLECTION:
                    title = R.string.bsdk_share_content_dialog_title_collection;
                    break;
                case CURATED_GALLERY:
                    title = R.string.bsdk_share_content_dialog_title_gallery;
                    break;
                case USER:
                    title = R.string.bsdk_share_content_dialog_title_user;
                    break;
                case TEAM:
                    title = R.string.bsdk_share_content_dialog_title_team;
                    break;
                default:
                    title = R.string.bsdk_share_content_dialog_title;
                    break;
            }
            titleView.setText(getResources().getString(title));
        }
    }

    @Override
    public void closeDialog(boolean scroll) {
        if(scroll)
            sheetRecycler.smoothScrollToPosition(0);
        else
            contentOverlay.animate().alpha(0.f).setListener(new Animator.AnimatorListener() {
                @Override
                public void onAnimationStart(Animator animation) {

                }

                @Override
                public void onAnimationEnd(Animator animation) {
                    dismiss();
                }

                @Override
                public void onAnimationCancel(Animator animation) {

                }

                @Override
                public void onAnimationRepeat(Animator animation) {

                }
            }).start();
    }

    @Override
    public void share(BehanceSDKShareContentAdapterItemDTO dto) {
        if (PACKAGE_NAME_PINTEREST.equals(dto.getPackageName())) {
            dispatchShareIntent(PACKAGE_NAME_PINTEREST);
        } else if (PACKAGE_NAME_FACEBOOK.equals(dto.getPackageName())) {
            dispatchShareIntent(PACKAGE_NAME_FACEBOOK);
        } else if (PACKAGE_NAME_TWITTER.equals(dto.getPackageName())) {
            dispatchShareIntent(PACKAGE_NAME_TWITTER);
        } else if (BehanceSDKShareContentAdapterItemDTO.EMAIL_SHARING_PACKAGE_NAME.equals(dto.getPackageName())) {
            handleShareUsingEmailBtnClick();
        } else if (BehanceSDKShareContentAdapterItemDTO.COPY_LINK_SHARING_PACKAGE_NAME.equals(dto.getPackageName())) {
            handleCopyLinkBtnClick();
        } else if (BehanceSDKShareContentAdapterItemDTO.DOWNLOAD_IMAGE_KEY.equals(dto.getPackageName())) {
            saveImageToDevice();
        } else {
            dispatchShareIntent(dto.getPackageName());
        }
    }

    @Override
    public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP) {
            closeDialog(true);
            return true;
        }
        return false;
    }


    private void dispatchShareIntent(String appPackageName) {
        boolean foundMatchingIntent = false;
        Intent shareIntent = getDefaultShareIntent();

        // gets the list of intents that can be loaded.
        List<ResolveInfo> resInfo = getActivity().getPackageManager().queryIntentActivities(shareIntent, 0);
        if (!resInfo.isEmpty()) {
            for (ResolveInfo info : resInfo) {
                if (containsIgnoreCase(info.activityInfo.packageName, appPackageName) || containsIgnoreCase(info.activityInfo.name, appPackageName)) {
                    if (appPackageName.equals(PACKAGE_NAME_TWITTER)) {
                        if (shareContentType == BehanceSDKShareContentType.USER) {
                            shareIntent.putExtra(
                                    Intent.EXTRA_TEXT,
                                    getString(R.string.bsdk_share_content_dialog_twitter_share_body, shareContent.getOwnerName(),
                                            shareContent.getShortUrlToShare()));
                        } else {
                            shareIntent.putExtra(
                                    Intent.EXTRA_TEXT,
                                    getString(R.string.bsdk_share_content_dialog_twitter_share_body, shareContent.getTitle(),
                                            shareContent.getShortUrlToShare()));
                        }

                    } else {
                        shareIntent.putExtra(Intent.EXTRA_TEXT, shareContent.getLongUrlToShare());
                    }

                    shareIntent.setType("text/plain");
                    shareIntent.setPackage(info.activityInfo.packageName);
                    foundMatchingIntent = true;
                    break;
                }
            }
            if (foundMatchingIntent) {
                startActivity(Intent.createChooser(shareIntent, getString(R.string.bsdk_share_content_dialog_share_intent_title)));
            }
        }
        if (!foundMatchingIntent) {
            Toast.makeText(getActivity(), R.string.bsdk_share_content_dialog_app_not_found_msg, Toast.LENGTH_LONG).show();
            return;
        }
        closeDialog(true);
    }

    private void handleShareUsingEmailBtnClick() {
        Intent emailIntent = new Intent(Intent.ACTION_SENDTO, Uri.fromParts("mailto", "", null));
        emailIntent.putExtra(Intent.EXTRA_SUBJECT, shareContent.getTitle());
        switch (shareContentType) {
            case WIP:
                emailIntent.putExtra(Intent.EXTRA_TEXT, getString(R.string.bsdk_share_content_dialog_share_wip_email_body, shareContent.getLongUrlToShare()));
                break;
            case USER:
                emailIntent.putExtra(Intent.EXTRA_SUBJECT, shareContent.getOwnerName());
                emailIntent.putExtra(Intent.EXTRA_TEXT,
                        getString(R.string.bsdk_share_content_dialog_share_user_email_body, shareContent.getOwnerName(), shareContent.getLongUrlToShare()));
                break;
            case COLLECTION:
                emailIntent.putExtra(Intent.EXTRA_TEXT,
                        getString(R.string.bsdk_share_content_dialog_share_collection_email_body, shareContent.getLongUrlToShare()));
                break;
            case CURATED_GALLERY:
                emailIntent.putExtra(Intent.EXTRA_TEXT,
                        getString(R.string.bsdk_share_content_dialog_share_curated_gallery_email_body, shareContent.getLongUrlToShare()));
                break;
            case IMAGE:
                emailIntent
                        .putExtra(Intent.EXTRA_TEXT, getString(R.string.bsdk_share_content_dialog_share_image_email_body, shareContent.getLongUrlToShare()));
                break;
            case TEAM:
                emailIntent.putExtra(Intent.EXTRA_TEXT,
                        getString(R.string.bsdk_share_content_dialog_share_team_email_body, shareContent.getTitle(), shareContent.getLongUrlToShare()));
                break;
            default:
                emailIntent.putExtra(Intent.EXTRA_TEXT,
                        getString(R.string.bsdk_share_content_dialog_share_project_email_body, shareContent.getLongUrlToShare()));
                break;
        }
        startActivity(Intent.createChooser(emailIntent, getString(R.string.bsdk_share_content_dialog_email_a_link_intent_title)));

        closeDialog(true);
    }

    private void handleCopyLinkBtnClick() {
        android.content.ClipboardManager clipboard = (android.content.ClipboardManager) getActivity().getSystemService(Context.CLIPBOARD_SERVICE);
        android.content.ClipData clip = android.content.ClipData.newPlainText("url", shareContent.getLongUrlToShare());
        clipboard.setPrimaryClip(clip);

        Toast toast = Toast.makeText(getActivity(), getString(R.string.bsdk_share_content_dialog_link_copied_msg), Toast.LENGTH_SHORT);
        toast.show();

        closeDialog(true);
    }

    public void saveImageToDevice() {
        if(BehanceSDKPermissionHelper.checkPermissionAndRequest(getActivity(), BehanceSDKPermissionHelper.REQUEST_CODE_STORAGE_DOWNLOAD_IMAGE)) {
            BehanceSDKDownloadAndSaveImageAsyncTaskParams params = new BehanceSDKDownloadAndSaveImageAsyncTaskParams();
            params.setContext(getActivity());
            params.setUrl(shareContent.getLongUrlToShare());
            BehanceSDKDownloadAndSaveImageAsyncTask asyncTask = new BehanceSDKDownloadAndSaveImageAsyncTask();
            asyncTask.execute(params);
            closeDialog(true);
        }
    }
}
