/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.dialogs;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.Window;
import android.widget.Button;
import android.widget.TextView;

import com.behance.sdk.R;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKGenericAlertDialog extends Dialog implements DialogInterface.OnDismissListener {

	private Button okButton;
	private Button notOKButton;
	private android.view.View.OnClickListener okBtnClickListener;
	private android.view.View.OnClickListener notOKBtnClickListener;
	private int titleResourceId;
	private int contentResourceId;
	private int okBtnLabelResourceId;
	private int notOKBtnLabelResourceId;

	public static BehanceSDKGenericAlertDialog getInstance(Context context, int titleResourceId, int contentResourceId, int okBtnLabelResourceId,
			int notOKBtnLabelResourceId) {
		BehanceSDKGenericAlertDialog dialog = new BehanceSDKGenericAlertDialog(context);
		dialog.setTitleResourceId(titleResourceId);
		dialog.setContentResourceId(contentResourceId);
		dialog.setOkBtnLabelResourceId(okBtnLabelResourceId);
		dialog.setNotOKBtnLabelResourceId(notOKBtnLabelResourceId);
		return dialog;
	}

	public BehanceSDKGenericAlertDialog(Context context) {
		super(context);
	}

	public BehanceSDKGenericAlertDialog(Context context, int theme) {
		super(context, theme);
	}

	public BehanceSDKGenericAlertDialog(Context context, boolean cancelable, OnCancelListener cancelListener) {
		super(context, cancelable, cancelListener);
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		setContentView(R.layout.bsdk_dialog_generic_alert);

		TextView titleTextView = (TextView) findViewById(R.id.bsdkGenericAlertDialogTitleTxtView);
		if (titleResourceId > 0) {
			titleTextView.setText(titleResourceId);
		}
		TextView contentTextView = (TextView) findViewById(R.id.bsdkGenericAlertDialogBodyTxtView);
		if (contentResourceId > 0) {
			contentTextView.setText(contentResourceId);
		}

		notOKButton = (Button) findViewById(R.id.bsdkGenericAlertDialogNotOKBtn);
		if (notOKBtnLabelResourceId > 0) {
			notOKButton.setText(notOKBtnLabelResourceId);
		}
		if (notOKBtnClickListener != null) {
			notOKButton.setOnClickListener(notOKBtnClickListener);
		}
		okButton = (Button) findViewById(R.id.bsdkGenericAlertDialogOKBtn);
		if (okBtnLabelResourceId > 0) {
			okButton.setText(okBtnLabelResourceId);
		}
		if (okBtnClickListener != null) {
			okButton.setOnClickListener(okBtnClickListener);
		}
		this.setOnDismissListener(this);
	}

	public void setOnOKBtnClickListener(android.view.View.OnClickListener listener) {
		if (okButton != null) {
			okButton.setOnClickListener(listener);
		} else {
			this.okBtnClickListener = listener;
		}
	}

	public void setOnNotOKBtnClickListener(android.view.View.OnClickListener listener) {
		if (notOKButton != null) {
			notOKButton.setOnClickListener(listener);
		} else {
			this.notOKBtnClickListener = listener;
		}
	}

	@Override
	public void onDismiss(DialogInterface dialog) {
		notOKBtnClickListener = null;
		okBtnClickListener = null;
	}

	public int getTitleResourceId() {
		return titleResourceId;
	}

	public void setTitleResourceId(int titleResourceId) {
		this.titleResourceId = titleResourceId;
	}

	public int getContentResourceId() {
		return contentResourceId;
	}

	public void setContentResourceId(int contentResourceId) {
		this.contentResourceId = contentResourceId;
	}

	public int getOkBtnLabelResourceId() {
		return okBtnLabelResourceId;
	}

	public void setOkBtnLabelResourceId(int okBtnLabelResourceId) {
		this.okBtnLabelResourceId = okBtnLabelResourceId;
	}

	public int getNotOKBtnLabelResourceId() {
		return notOKBtnLabelResourceId;
	}

	public void setNotOKBtnLabelResourceId(int notOKBtnLabelResourceId) {
		this.notOKBtnLabelResourceId = notOKBtnLabelResourceId;
	}
}
