/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.dialogs;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.Window;
import android.widget.Button;
import android.widget.TextView;

import com.behance.sdk.R;

/**
 * @author Sujit Reddy G
 */

public class BehanceSDKGenericAlertWithSingleBtnDialog extends Dialog implements DialogInterface.OnDismissListener {

    private Button okButton;
    private android.view.View.OnClickListener btnClickListener;
    private int titleResourceId;
    private int contentResourceId;
    private int btnLabelResourceId;

    public static BehanceSDKGenericAlertWithSingleBtnDialog getInstance(Context context, int titleResourceId, int contentResourceId, int btnLabelResourceId) {
        BehanceSDKGenericAlertWithSingleBtnDialog dialog = new BehanceSDKGenericAlertWithSingleBtnDialog(context);
        dialog.setTitleResourceId(titleResourceId);
        dialog.setContentResourceId(contentResourceId);
        dialog.setBtnLabelResourceId(btnLabelResourceId);
        return dialog;
    }

    public BehanceSDKGenericAlertWithSingleBtnDialog(Context context) {
        super(context);
    }

    public BehanceSDKGenericAlertWithSingleBtnDialog(Context context, int theme) {
        super(context, theme);
    }

    public BehanceSDKGenericAlertWithSingleBtnDialog(Context context, boolean cancelable, OnCancelListener cancelListener) {
        super(context, cancelable, cancelListener);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        setContentView(R.layout.bsdk_dialog_generic_alert_with_single_btn);

        TextView titleTextView = (TextView) findViewById(R.id.genericAlertSingleBtnDialogTitleTxtView);
        if (titleResourceId > 0) {
            titleTextView.setText(titleResourceId);
        }
        TextView contentTextView = (TextView) findViewById(R.id.genericAlertSingleBtnDialogBodyTxtView);
        if (contentResourceId > 0) {
            contentTextView.setText(contentResourceId);
        }
        okButton = (Button) findViewById(R.id.genericAlertSingleBtnDialogOKBtn);
        if (btnLabelResourceId > 0) {
            okButton.setText(btnLabelResourceId);
        }
        if (btnClickListener != null) {
            okButton.setOnClickListener(btnClickListener);
        }

        this.setOnDismissListener(this);
    }

    public void setBtnClickListener(android.view.View.OnClickListener listener) {
        if (okButton != null) {
            okButton.setOnClickListener(listener);
        } else {
            this.btnClickListener = listener;
        }
    }

    @Override
    public void onDismiss(DialogInterface dialog) {
        btnClickListener = null;
    }

    public int getTitleResourceId() {
        return titleResourceId;
    }

    public void setTitleResourceId(int titleResourceId) {
        this.titleResourceId = titleResourceId;
    }

    public int getContentResourceId() {
        return contentResourceId;
    }

    public void setContentResourceId(int contentResourceId) {
        this.contentResourceId = contentResourceId;
    }

    public int getBtnLabelResourceId() {
        return btnLabelResourceId;
    }

    public void setBtnLabelResourceId(int btnLabelResourceId) {
        this.btnLabelResourceId = btnLabelResourceId;
    }
}
