/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.dialogs;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.EditText;

import com.behance.sdk.R;

/**
 * @author Sujit Reddy G
 * 
 */
public class BehanceSDKNewWIPTitleDialog extends Dialog implements OnClickListener {

	private EditText wipTitleEditText;
	private Callbacks callbacks;

	public interface Callbacks {
		public void onNewWIPTitleDialogOKBtnClicked(String newWIPTitle);
	}

	public static BehanceSDKNewWIPTitleDialog getInstance(Context context, Callbacks callbacks) {
		BehanceSDKNewWIPTitleDialog dialogInstance = new BehanceSDKNewWIPTitleDialog(context);
		dialogInstance.setCallbacks(callbacks);
		return dialogInstance;
	}

	public BehanceSDKNewWIPTitleDialog(Context context) {
		super(context);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.bsdk_dialog_fragment_new_wip_title);
		setCancelable(true);
		wipTitleEditText = (EditText) findViewById(R.id.bsdkNewWIPTitleDialogContentEditTxt);

		View cancelButton = findViewById(R.id.bsdkNewWIPTitleDialogNotOKBtn);
		cancelButton.setOnClickListener(this);
		View okButton = findViewById(R.id.bsdkNewWIPTitleDialogOKBtn);
		okButton.setOnClickListener(this);
	}

	@Override
	public void onClick(View v) {
		int viewID = v.getId();
		if (viewID == R.id.bsdkNewWIPTitleDialogNotOKBtn) {
			dismiss();
		} else if (viewID == R.id.bsdkNewWIPTitleDialogOKBtn) {
			handleOKButtonClick();
		}
	}

	private void handleOKButtonClick() {
		String wipTitle = wipTitleEditText.getText().toString().trim();
		if (wipTitle.length() > 0) {
			if (callbacks != null) {
				callbacks.onNewWIPTitleDialogOKBtnClicked(wipTitle);
			}
			dismiss();
		}
	}

	public Callbacks getCallbacks() {
		return callbacks;
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}
}