package com.behance.sdk.ui.dialogs;

import android.app.Dialog;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.text.Html;
import android.text.SpannableStringBuilder;
import android.text.TextPaint;
import android.text.method.LinkMovementMethod;
import android.text.style.ClickableSpan;
import android.text.style.URLSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.behance.sdk.BehanceSDKHeadlessFragmentTags;
import com.behance.sdk.R;
import com.behance.sdk.dto.project.BehanceSDKProjectCopyrightDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectToolDTO;
import com.behance.sdk.fragments.headless.BehanceSDKGetProjectDetailsHeadlessFragment;
import com.behance.sdk.ui.components.BehanceSDKOverFlowLayout;
import com.behance.sdk.util.BehanceSDKConstants;
import com.facebook.drawee.backends.pipeline.Fresco;
import com.facebook.drawee.interfaces.DraweeController;
import com.facebook.drawee.view.DraweeView;
import com.facebook.imagepipeline.request.ImageRequest;
import com.facebook.imagepipeline.request.ImageRequestBuilder;


import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class BehanceSDKProjectInfoDialogFragment extends DialogFragment implements OnClickListener {

	private BehanceSDKProjectDTO mActiveProject;
	private View projectInfoDialogProjectInfoView;
	private static final String DATE_DISPLAY_FORMAT = "MM/dd/yy";
	SimpleDateFormat displayDateFormat = new SimpleDateFormat(DATE_DISPLAY_FORMAT, Locale.ENGLISH);

	public static final String ACTIVE_PROJECT = "active_project";
	private BehanceSDKOverFlowLayout projectTagsOverFlowContainer;
	private TextView copyrightTxtView;
	private LinearLayout projectCopyrightParentContainer , projectToolsContainer,projectToolsThumbnailContainer;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkprojectInfoDialogTheme);
	}

	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState) {
		Dialog projectInfoDialog = super.onCreateDialog(savedInstanceState);
		projectInfoDialog.requestWindowFeature(Window.FEATURE_NO_TITLE);

		projectInfoDialog.setCanceledOnTouchOutside(true);
		return projectInfoDialog;
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		LayoutInflater layoutInflater = getActivity().getLayoutInflater();
		if (getActivity() != null) {
			BehanceSDKGetProjectDetailsHeadlessFragment getProjectDetailsHeadlessFragment;
			Bundle args = getArguments();
			if (args != null) {
				if (args.containsKey("frag_num"))
					getProjectDetailsHeadlessFragment = (BehanceSDKGetProjectDetailsHeadlessFragment) getActivity().getSupportFragmentManager()
							.findFragmentByTag(BehanceSDKHeadlessFragmentTags.GET_PROJECT_DETAILS + args.getInt("frag_num"));
				else
					getProjectDetailsHeadlessFragment = (BehanceSDKGetProjectDetailsHeadlessFragment) getActivity().getSupportFragmentManager()
							.findFragmentByTag(BehanceSDKHeadlessFragmentTags.GET_PROJECT_DETAILS);
			} else
				getProjectDetailsHeadlessFragment = (BehanceSDKGetProjectDetailsHeadlessFragment) getActivity().getSupportFragmentManager().findFragmentByTag(
						BehanceSDKHeadlessFragmentTags.GET_PROJECT_DETAILS);
			if (getProjectDetailsHeadlessFragment != null) {
				mActiveProject = getProjectDetailsHeadlessFragment.getActiveProject();
			}
			if (mActiveProject == null) {
				Toast.makeText(getActivity(), R.string.bsdk_project_info_dialog_problem_loading_project_info, Toast.LENGTH_SHORT).show();
				dismiss();
			}
			projectInfoDialogProjectInfoView = layoutInflater.inflate(R.layout.bsdk_dialog_fragment_project_info, null);

			// Statistics
			TextView publishedDateTxtView = (TextView) projectInfoDialogProjectInfoView
					.findViewById(R.id.bsdk_projectInfoDialogProjectPublishedDateTxtView);
			Date publishedDate = new Date(mActiveProject.getCreatedDate() * 1000);
			publishedDateTxtView.setText(displayDateFormat.format(publishedDate));
			TextView lastEditedDateTxtView = (TextView) projectInfoDialogProjectInfoView
					.findViewById(R.id.bsdk_projectInfoDialogProjectLastEditedDateTxtView);
			Date lastEditedDate = new Date(mActiveProject.getModifiedDate() * 1000);
			lastEditedDateTxtView.setText(displayDateFormat.format(lastEditedDate));

			// Project Description
			BehanceSDKOverFlowLayout projectDescription = (BehanceSDKOverFlowLayout) projectInfoDialogProjectInfoView
					.findViewById(R.id.bsdk_projectInfoDialogProjectAboutOverFlowContainer);
			TextView descriptionTxtView = new TextView(getActivity());
			descriptionTxtView.setText(Html.fromHtml("<b><font color=#595959>" + getResources().getString(R.string.bsdk_project_info_dialog_title_about)
					+ "</font></b> <font color=#626365>" + mActiveProject.getDescription() + "</font>"));
			descriptionTxtView.setLineSpacing(1f, 1.1f);
			projectDescription.addView(descriptionTxtView);

			// Project Tags
			projectTagsOverFlowContainer = (BehanceSDKOverFlowLayout) projectInfoDialogProjectInfoView
					.findViewById(R.id.bsdk_projectInfoDialogProjectTagOverFlowContainer);
			populateProjectTagsList(mActiveProject);

			//Tools
			projectToolsContainer = (LinearLayout) projectInfoDialogProjectInfoView.findViewById(R.id.bsdk_projectInfoDialogToolsContainer);
			projectToolsThumbnailContainer = (LinearLayout) projectInfoDialogProjectInfoView.findViewById(R.id.bsdkprojectInfoDialogToolsThumbnailContainer);
			populateTools();

			// Project Copyright
			projectCopyrightParentContainer = (LinearLayout) projectInfoDialogProjectInfoView.findViewById(R.id.bsdk_projectInfoDialogCopyrightContainer);
			copyrightTxtView = (TextView) projectInfoDialogProjectInfoView.findViewById(R.id.bsdk_projectInfoDialogProjectCopyrightTxtView);
			populateCopyright();

			// Report spam
			TextView reportSpam = (TextView) projectInfoDialogProjectInfoView.findViewById(R.id.bsdk_projectInfoDialogReportSpam);
			reportSpam.setOnClickListener(this);
		}
		return projectInfoDialogProjectInfoView;
	}

	private void populateProjectTagsList(BehanceSDKProjectDTO activeProject) {
		List<String> projectTags = activeProject.getTags();
		if (projectTags.size() > 0) {
			String html = "<b><font color=#595959>" + getResources().getString(R.string.bsdk_project_info_dialog_tags_text) + "</font></b> ";
			for (int i = 0; i < projectTags.size() - 1; i++) {
				String projectTag = projectTags.get(i);
				html += "<a href='" + projectTag + "' style='text-decoration: none'>" + projectTag + "</a>, ";

			}
			String projectTag = projectTags.get(projectTags.size() - 1);
			html += "<a href='" + projectTag + "' style='text-decoration: none'>" + projectTag + "</a>";

			CharSequence sequence = Html.fromHtml(html);

			SpannableStringBuilder strBuilder = new SpannableStringBuilder(sequence);
			URLSpan[] urls = strBuilder.getSpans(0, sequence.length(), URLSpan.class);
			for (URLSpan span : urls) {
				makeLinkClickable(strBuilder, span);
			}

			TextView projectTagsTextView = new TextView(getActivity());
			projectTagsTextView.setText(strBuilder);
			projectTagsTextView.setLinksClickable(true);
			projectTagsTextView.setMovementMethod(LinkMovementMethod.getInstance());
			projectTagsTextView.setLineSpacing(1f, 1.1f);
			projectTagsOverFlowContainer.addView(projectTagsTextView);
		}
	}

	private class CustomClickableSpan extends ClickableSpan {

		String tag;

		public CustomClickableSpan(String tag) {
			super();
			this.tag = tag;
		}

		@Override
		public void onClick(View widget) {
			handleTagClick(tag);
		}

		public void updateDrawState(TextPaint ds) {
			ds.setUnderlineText(false);
			ds.setColor(getResources().getColor(R.color.bsdk_behance_blue));
		}
	}

	protected void makeLinkClickable(SpannableStringBuilder strBuilder, final URLSpan span) {
		int start = strBuilder.getSpanStart(span);
		int end = strBuilder.getSpanEnd(span);
		int flags = strBuilder.getSpanFlags(span);
		ClickableSpan clickable = new CustomClickableSpan(span.getURL());
		strBuilder.setSpan(clickable, start, end, flags);
		strBuilder.removeSpan(span);
	}


	private void handleTagClick(String tag) {
		String url = "http://www.behance.net/search?content=project&sort=featured_date&time=week&search=" + tag;
		launchUrlIntent(url);
	}


	private void launchUrlIntent(String url){
		if(url == null)
			return;
		Intent i = new Intent(Intent.ACTION_VIEW , Uri.parse(url));
		startActivity(i);
		dismiss();
		}

	private void populateTools(){
		List<BehanceSDKProjectToolDTO> tools = mActiveProject.getTools();
		if(tools != null && tools.size() > 0) {
			LayoutInflater inflater = LayoutInflater.from(getActivity());
			for(BehanceSDKProjectToolDTO tool : tools) {
				if(tool.getIconUrl() != null) {
					DraweeView draweeView = (DraweeView) inflater.inflate(R.layout.bsdk_adapter_project_info_tool_view, projectToolsThumbnailContainer, false);
					projectToolsThumbnailContainer.addView(draweeView);
					ImageRequest request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(tool.getIconUrl())).setProgressiveRenderingEnabled(true).build();
					DraweeController controller = Fresco.newDraweeControllerBuilder().setImageRequest(request).setOldController(draweeView.getController()).build();
					draweeView.setController(controller);
					draweeView.setTag(tool.getUrl());
					draweeView.setOnClickListener(this);
				}
			}
			if(projectToolsThumbnailContainer.getChildCount() == 0)
				projectToolsContainer.setVisibility(View.GONE);
		} else {
			projectToolsContainer.setVisibility(View.GONE);
		}
	}

	private void populateCopyright() {
		BehanceSDKProjectCopyrightDTO projectCopyrightDTO = mActiveProject.getCopyright();
		if (projectCopyrightDTO != null) {
			int displayNameResourceId = projectCopyrightDTO.getDisplayNameResourceId();
			if (displayNameResourceId > 0) {
				copyrightTxtView.setText(displayNameResourceId);
			} else {
				String licenseDescription = projectCopyrightDTO.getLicenseDescription();
				if (licenseDescription.length() > 0) {
					licenseDescription = licenseDescription.replace("-", "  ");
					copyrightTxtView.setText(licenseDescription);
				} else {
					projectCopyrightParentContainer.setVisibility(View.GONE);
				}
			}
		} else {
			projectCopyrightParentContainer.setVisibility(View.GONE);
		}
	}

	@Override
	public void onDestroyView() {
		Dialog projectInfoDialog = getDialog();

		// Work around bug:
		// http://code.google.com/p/android/issues/detail?id=17423
		if ((projectInfoDialog != null) && getRetainInstance())
			projectInfoDialog.setDismissMessage(null);

		super.onDestroyView();
	}

	@Override
	public void onClick(View clickedView) {
		int id = (clickedView.getId());
		if (id == R.id.bsdk_projectInfoDialogReportSpam) {
			handleReportSpam();

		} else {
			if (clickedView.getTag() instanceof String) {
				Intent i = new Intent(Intent.ACTION_VIEW);
				i.setData(Uri.parse((String) clickedView.getTag()));
				startActivity(i);
			}

		}
	}

	private void handleReportSpam() {
		Intent emailIntent = new Intent(Intent.ACTION_SENDTO, Uri.fromParts("mailto", BehanceSDKConstants.REPORT_PROJECT_SPAM_EMAIL_ID, null));
		emailIntent.putExtra(Intent.EXTRA_SUBJECT, getString(R.string.bsdk_project_info_dialog_report_project_spam_email_subject));
		emailIntent.putExtra(Intent.EXTRA_TEXT, mActiveProject.getProjectUrl());
		startActivity(Intent.createChooser(emailIntent, getString(R.string.bsdk_project_info_dialog_report_project_spam_chooser_title)));
	}
}
