/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;

import com.behance.sdk.R;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.google.listview.SectionalListView;
import com.behance.sdk.ui.adapters.BehanceSDKCopyrightSettingsAdapter;
import com.behance.sdk.util.BehanceSDKConstants;

/**
 * @author ugs
 *
 */
public class BehanceSDKCopyrightSettingsDialog  extends DialogFragment {

	private static final String BUNDLE_KEY_COPYRIGHT_ID = "BUNDLE_KEY_COPYRIGHT_ID";
	
	private View rootView;
	private Context context;
	private Callbacks callbacks;
	private BehanceSDKCopyrightOption selectedCopyrightSetting;
	private BehanceSDKCopyrightSettingsAdapter sectionalAdapter;
	
	
	public interface Callbacks {
		public void onCopyrightSettingSelected(BehanceSDKCopyrightOption copyRightSetting);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkFilterDialogTheme);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		
		if(savedInstanceState != null) {
			String string = savedInstanceState.getString(BUNDLE_KEY_COPYRIGHT_ID, BehanceSDKConstants.EMPTY_STRING);
			selectedCopyrightSetting = BehanceSDKCopyrightOption.getCopyrightOptionFromValue(string);
			if(null == selectedCopyrightSetting)
				selectedCopyrightSetting = BehanceSDKCopyrightOption.getDefaultValue();
		}
		rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_copyright_settings, container, false);
		View titleLayout = rootView.findViewById(R.id.bsdkPublishProjectCopyrightSettingsDialogHeaderLayout);
		titleLayout.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dismiss();
				
			}
		});
		SectionalListView lsComposer = (SectionalListView) rootView.findViewById(R.id.bsdkPublishProjectCopyrightSettingsDialogSectionalListView);
		lsComposer.setPinnedHeaderView(inflater.inflate(R.layout.bsdk_adapter_publish_project_copyright_settings_item_header, lsComposer, false));
		sectionalAdapter = new BehanceSDKCopyrightSettingsAdapter(context,selectedCopyrightSetting);
		lsComposer.setAdapter(sectionalAdapter);
		lsComposer.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				handleCopyrightSettingSelection(parent, position);
			}

		});
		return rootView;
	}
	
	
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.DialogFragment#onAttach(android.app.Activity)
	 */
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		context =activity;
	}

	private void handleCopyrightSettingSelection(AdapterView<?> parent, int position) {
		BehanceSDKCopyrightOption itemAtPosition = (BehanceSDKCopyrightOption) parent.getItemAtPosition(position);
		selectedCopyrightSetting = itemAtPosition;
		sectionalAdapter.setSelectedCopyRight(selectedCopyrightSetting);
		sectionalAdapter.notifyDataSetChanged();
		notifyListenersAndCloseDialog();
	}
	
	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}
	private void notifyListenersAndCloseDialog() {
		if (callbacks != null) {
			callbacks.onCopyrightSettingSelected(selectedCopyrightSetting);
		}
		getDialog().dismiss();
	}

	/**
	 * @param selectedField
	 */
	public void setSelectedCopyright(BehanceSDKCopyrightOption selectedCopyright) {
			this.selectedCopyrightSetting = selectedCopyright;
	}

	@Override
	public void onSaveInstanceState(Bundle arg0) {
		super.onSaveInstanceState(arg0);
		if(selectedCopyrightSetting != null) {
			arg0.putString(BUNDLE_KEY_COPYRIGHT_ID, selectedCopyrightSetting.getValue());
		}
	}
	
	@Override
	public void onStart() {
		super.onStart();
		if (getDialog() != null && getActivity() != null) {
			if (getActivity().getResources().getBoolean(R.bool.bsdk_big_screen)) {
				int windowWidth = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_width);
				int windowHeight = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_height);
				getDialog().getWindow().setLayout(windowWidth, windowHeight);
			}
		}
	}
}
