/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;

import com.behance.sdk.R;
import com.behance.sdk.datamanager.FiltersDataManager;
import com.behance.sdk.datamanager.listeners.interfaces.IFiltersDataManagerListener;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.google.listview.SectionalListView;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.ui.adapters.BehanceSDKCreativeFieldFilterAdapter;

/**
 * @author ugs
 *
 */
public class BehanceSDKCreativeFieldsFilterDialog  extends DialogFragment  implements IFiltersDataManagerListener{

	private static final int UNLIMITED = -1;
	private static final String BUNDLE_KEY_FIELD_ID = "BUNDLE_KEY_FIELD_ID";
	private static final String BUNDLE_KEY_SELECTABLE_FIELD_COUNT = "BUNDLE_KEY_SELECTABLE_FIELD_COUNT";
	
	private  int selectable_field_count = UNLIMITED;
	private View rootView;
	private Context context;
	private View progressSpinner;
	private Callbacks callbacks;
	private List<BehanceSDKCreativeFieldDTO> selectedFields;
	private BehanceSDKCreativeFieldFilterAdapter sectionalAdapter;
	
	public BehanceSDKCreativeFieldsFilterDialog() {
		FiltersDataManager.getInstance().addListener(this);
	}
	
	public interface Callbacks {
		public void onCreativeFieldsSelected(List<BehanceSDKCreativeFieldDTO> selectedField);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkFilterDialogTheme);
	}

	@SuppressWarnings("unchecked")
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		
		if(savedInstanceState != null) {
			selectedFields = (List<BehanceSDKCreativeFieldDTO>) savedInstanceState.getSerializable(BUNDLE_KEY_FIELD_ID);
			selectable_field_count = savedInstanceState.getInt(BUNDLE_KEY_SELECTABLE_FIELD_COUNT, UNLIMITED);
		}
		if(null == selectedFields){
			selectedFields = new ArrayList<BehanceSDKCreativeFieldDTO>();
		}
		rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_creative_field_filter, container, false);
		View titleLayout = rootView.findViewById(R.id.bsdkPublishProjectcreativeFieldFilterDialogHeaderLayout);
		titleLayout.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				dismiss();
				
			}
		});
		progressSpinner = rootView.findViewById(R.id.bsdkPublishProjectcreativeFieldFilterDialogProgressSpinner);
		FiltersDataManager instance = FiltersDataManager.getInstance();
		if(false == instance.isLoadCreativeFieldsTaskInProgress()){
			showProgressSpinner();
			instance.loadCreativeFields(BehanceSDKProjectPublishWFManager.getInstance().getUserCredentials());
		}
		View doneButton = rootView.findViewById(R.id.bsdkPublishProjectcreativeFieldFilterDialogCloseDialogBtnBottom);
		doneButton.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				notifyListenersAndCloseDialog();
				dismiss();
			}
		});
		return rootView;
	}
	
	
	private void showProgressSpinner() {
		progressSpinner.setVisibility(View.VISIBLE);
	}

	private void hideProgressSpinner() {
		progressSpinner.setVisibility(View.INVISIBLE);
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.DialogFragment#onAttach(android.app.Activity)
	 */
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		context =activity;
	}
	/* (non-Javadoc)
	 * @see com.behance.network.interfaces.listeners.IFiltersDataManagerListener#onCreativeFieldsLoadingSuccess(java.util.List)
	 */
	@Override
	public void onCreativeFieldsLoadingSuccess(List<BehanceSDKCreativeFieldDTO> creativeFields) {
			populateListView(creativeFields, LayoutInflater.from(context));
	}
	/**
	 * @param creativeFields
	 */
	private void populateListView(List<BehanceSDKCreativeFieldDTO> creativeFields, LayoutInflater inflater) {
		if(null != rootView){
			hideProgressSpinner();
			SectionalListView lsComposer = (SectionalListView) rootView.findViewById(R.id.bsdkPublishProjectcreativeFieldFilterDialogSectionalListView);
			lsComposer.setPinnedHeaderView(inflater.inflate(R.layout.bsdk_adapter_publish_project_creative_field_item_header, lsComposer, false));
			sectionalAdapter = new BehanceSDKCreativeFieldFilterAdapter(context,creativeFields, selectedFields);
			lsComposer.setAdapter(sectionalAdapter);
			lsComposer.setOnItemClickListener(new AdapterView.OnItemClickListener() {

				@Override
				public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
					handleCreativeFieldSelection(parent, position);
				}

			});
		}
	}
	
	
	private void handleCreativeFieldSelection(AdapterView<?> parent, int position) {
		Object itemAtPosition = parent.getItemAtPosition(position);
		if (itemAtPosition instanceof BehanceSDKCreativeFieldDTO) {
			BehanceSDKCreativeFieldDTO selectedField = (BehanceSDKCreativeFieldDTO) itemAtPosition;
			if (this.selectedFields.contains(selectedField)) {
				selectedFields.remove(selectedField);
			} else {
				if (selectable_field_count < 0 || selectedFields.size() < selectable_field_count) {
					selectedFields.add(selectedField);
					if (selectedFields.size() == selectable_field_count) {
						notifyListenersAndCloseDialog();
					}
				}
			}
			sectionalAdapter.notifyDataSetChanged();
		}
	}
	
	@Override
	public void onCreativeFieldsLoadingFailure(Exception e) {
		
	}
	
	public Callbacks getCallbacks() {
		return callbacks;
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}
	private void notifyListenersAndCloseDialog() {
		if (callbacks != null) {
			callbacks.onCreativeFieldsSelected(selectedFields);
		}
		getDialog().dismiss();
	}

	/**
	 * @param selectedField
	 */
	public void setSelectedCreativeField(List<BehanceSDKCreativeFieldDTO> selectedFields) {
		if(null != selectedFields){
			this.selectedFields = new ArrayList<BehanceSDKCreativeFieldDTO>();
			this.selectedFields.addAll(selectedFields);
		}
	}

	@Override
	public void onSaveInstanceState(Bundle arg0) {
		super.onSaveInstanceState(arg0);
		if(selectedFields != null) {
			arg0.putSerializable(BUNDLE_KEY_FIELD_ID, (Serializable) selectedFields);
		}
		arg0.putInt(BUNDLE_KEY_SELECTABLE_FIELD_COUNT, selectable_field_count);
	}
	
	@Override
	public void onStart() {
		super.onStart();
		if (getDialog() != null && getActivity() != null) {
			
			if (getActivity().getResources().getBoolean(R.bool.bsdk_big_screen)) {
				int windowWidth = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_width);
				int windowHeight = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_height);
				getDialog().getWindow().setLayout(windowWidth, windowHeight);
			} else {
				int dimensionPixelVertical = getResources().getDimensionPixelOffset(R.dimen.bsdk_creative_field_dialog_vertical_padding);
				int dimensionPixelHorizontal = getResources().getDimensionPixelOffset(R.dimen.bsdk_creative_field_dialog_horizontal_padding);
				DisplayMetrics displayMetrics = getResources().getDisplayMetrics();
				int heightPixels = displayMetrics.heightPixels;
				int widthPixels = displayMetrics.widthPixels;
				getDialog().getWindow().setLayout(widthPixels - dimensionPixelHorizontal, heightPixels - dimensionPixelVertical);
			}
		}
	}

	/**
	 * @param selectable_field_count the selectable_field_count to set
	 */
	public void setSelectableFieldCount(int selectable_field_count) {
		this.selectable_field_count = selectable_field_count;
	}
}
