package com.behance.sdk.ui.fragments;

import android.content.ContentResolver;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.Point;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKImageCropperDTO;
import com.behance.sdk.edmodo.cropper.CropImageView;
import com.behance.sdk.exception.NullReferenceException;
import com.behance.sdk.project.modules.ImageModule;

import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * Created by forta on 9/23/14.
 */
public class BehanceSDKImageCropperDialogFragment extends DialogFragment implements View.OnClickListener {
    private static final String ARGS_KEY_IMAGE_CROPPER_DTO = "ARGS_KEY_IMAGE_CROPPER_DTO";

    private Callbacks callbacks;

    private CropImageView cropImageView;

    private int imageWidth, imageHeight,
            minLength;//minimum length of the shorter side

    public interface Callbacks {
        public void onImageCropped(Bitmap bitmap);
    }

    public static BehanceSDKImageCropperDialogFragment getInstance(BehanceSDKImageCropperDTO cropperDTO, Callbacks callbacks) {//int width, int height, int minSize, Callbacks callbacks, ImageModule imageModule, boolean includeControls) {
        BehanceSDKImageCropperDialogFragment instance = new BehanceSDKImageCropperDialogFragment();
        Bundle args = new Bundle();
        args.putSerializable(ARGS_KEY_IMAGE_CROPPER_DTO, cropperDTO);
        instance.setArguments(args);
        instance.setCallbacks(callbacks);
        return instance;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(android.support.v4.app.DialogFragment.STYLE_NO_FRAME, R.style.BsdkPublishWIPToBehanceViewTheme);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_image_cropper, container, true);
        cropImageView = (CropImageView) rootView.findViewById(R.id.crop_image_view);
        cropImageView.setMinSize(200);

        Bundle args = getArguments();
        if (args != null && args.get(ARGS_KEY_IMAGE_CROPPER_DTO) != null){
            BehanceSDKImageCropperDTO cropperDTO = (BehanceSDKImageCropperDTO) args.get(ARGS_KEY_IMAGE_CROPPER_DTO);

            if (cropperDTO.isIncludeControls())
                (rootView.findViewById(R.id.bsdkCropImageControlsWrapper)).setVisibility(View.VISIBLE);

            imageHeight = cropperDTO.getHeight();
            imageWidth = cropperDTO.getWidth();
            minLength = cropperDTO.getMinSize();

            setImage(cropperDTO.getImageModule());
        } else {
            Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_image_cropper_failed_to_open_msg), Toast.LENGTH_SHORT).show();
            close();
        }

        View ok = rootView.findViewById(R.id.bsdkCropImageOk);
        ok.setOnClickListener(this);
        View cancel = rootView.findViewById(R.id.bsdkCropImageCancel);
        cancel.setOnClickListener(this);

        return rootView;
    }

    public void setImage(ImageModule imageModule) {
        //If dimensions are not included, allow any size
        if(imageHeight != -1 && imageWidth != -1) {
            cropImageView.setAspectRatio(imageWidth, imageHeight);
            cropImageView.setFixedAspectRatio(true);
        } else
            cropImageView.setFixedAspectRatio(false);
        if(minLength != -1)
            cropImageView.setMinSize(minLength);

        ContentResolver cr = getActivity().getContentResolver();
        Uri contentURI = Uri.parse("file:///" + String.valueOf(imageModule.getFile()));
        InputStream in = null;
        try {
            in = cr.openInputStream(contentURI);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        BitmapFactory.Options options = new BitmapFactory.Options();
        Display display = getActivity().getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        options.inSampleSize = imageModule.getWidth() / size.x;
        Bitmap thumb = BitmapFactory.decodeStream(in, null, options);

        int rotation = imageModule.getRotation();
        if(rotation > 0){
            cropImageView.setImageBitmap(null);
            Matrix matrix = new Matrix();
            // setup rotation degree
            matrix.postRotate(90 * rotation);
            Bitmap updatedImage = Bitmap.createBitmap(thumb, 0, 0, thumb.getWidth(), thumb.getHeight(), matrix, true);
            cropImageView.setImageBitmap(updatedImage);
        } else
            cropImageView.setImageBitmap(thumb);

        cropImageView.setGuidelines(0);//0 = off, 1 = on touch, 2 = on
    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    @Override
    public void onClick(View v) {
        int i = v.getId();
        if (i == R.id.bsdkCropImageOk)
            cropSaveAndClose();
        else if (i == R.id.bsdkCropImageCancel)
            close();
    }

    //Call externally if using controls in parent activity
    public void cropSaveAndClose() {
        if(callbacks != null) {
            Bitmap bm = null;
            try {
                bm = cropImageView.getCroppedImage();
                Bitmap covertedFile =bm;
                if(imageHeight != -1 && imageWidth != -1) {
                    covertedFile = Bitmap.createScaledBitmap(bm,imageWidth,imageWidth,false);
                    bm.recycle();
                }
                if (covertedFile != null)
                    callbacks.onImageCropped(covertedFile);
                else
                    Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_image_cropper_crop_failed_msg), Toast.LENGTH_SHORT).show();
            } catch (NullReferenceException e) {
                e.printStackTrace();
                Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_image_cropper_crop_failed_msg), Toast.LENGTH_SHORT).show();
            }
        }
        close();
    }

    //Call externally if using controls in parent activity
    public void close() {
        this.dismiss();
    }
}
