/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import android.app.Dialog;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;

import com.behance.sdk.R;
import com.behance.sdk.ui.adapters.BehanceSDKDisplayImagePagerAdapter;
import com.behance.sdk.ui.animations.BehanceSDKZoomOutPageTransformer;
import com.behance.sdk.ui.components.BehanceSDKHackyViewPager;

public class BehanceSDKImageDisplayDialogFragment extends DialogFragment {

	private static final String ARG_STARTING_IMAGE_NUMBER = "ARG_STARTING_IMAGE_NUMBER";
	private static final String ARG_IMAGES_URLS_ARRAY = "ARG_IMAGES_URLS_ARRAY";

	public static BehanceSDKImageDisplayDialogFragment getInstance(String[] imageUrlsArray, int startingImageNumber) {
		BehanceSDKImageDisplayDialogFragment dialogFragment = new BehanceSDKImageDisplayDialogFragment();
		Bundle args = new Bundle();
		args.putStringArray(ARG_IMAGES_URLS_ARRAY, imageUrlsArray);
		args.putInt(ARG_STARTING_IMAGE_NUMBER, startingImageNumber);
		dialogFragment.setArguments(args);
		return dialogFragment;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NORMAL, R.style.BsdkImageDisplayDialogTheme);
	}

	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState) {
		Dialog imageDialog = super.onCreateDialog(savedInstanceState);
		imageDialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		return imageDialog;
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		BehanceSDKHackyViewPager mImageDisplayViewPager = (BehanceSDKHackyViewPager) inflater.inflate(R.layout.bsdk_fragment_image_display, container, false);
		int imageNumber = getArguments().getInt(ARG_STARTING_IMAGE_NUMBER);
		String[] URLs = getArguments().getStringArray(ARG_IMAGES_URLS_ARRAY);
		BehanceSDKDisplayImagePagerAdapter mPagerAdapter = new BehanceSDKDisplayImagePagerAdapter(getChildFragmentManager(), URLs);
		mImageDisplayViewPager.setAdapter(mPagerAdapter);
		mImageDisplayViewPager.setPageTransformer(false, new BehanceSDKZoomOutPageTransformer());
		mImageDisplayViewPager.setCurrentItem(imageNumber, false);

		return mImageDisplayViewPager;
	}

	@Override
	public void onDestroyView() {
		Dialog Imagedialog = getDialog();

		// Work around bug:
		// http://code.google.com/p/android/issues/detail?id=17423
		if ((Imagedialog != null) && getRetainInstance())
			Imagedialog.setDismissMessage(null);

		super.onDestroyView();
	}

}