/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import android.content.Context;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.behance.sdk.R;

import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.util.BehanceSDKImageLoaderUtils;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener;

/**
 * Created by forta on 7/9/14.
 */

public class BehanceSDKImageDisplayPageItemFragment extends Fragment implements ImageLoadingListener {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKImageDisplayPageItemFragment.class);

	private static final int HARDWARE_RENDERING_MAX_SIZE = 2048;

	private static final String ARG_URL = "ARG_URL";
    private static final String ARG_ZOOMABLE = "ARG_ZOOMABLE";

	private Context mContext;
	private View progressSpinner;
	private ImageView mImageView;
    private boolean zoomable;

	public static BehanceSDKImageDisplayPageItemFragment newInstance(String url) {
		return newInstance(url, true);
	}

    public static BehanceSDKImageDisplayPageItemFragment newInstance(String url, boolean zoomable) {
        BehanceSDKImageDisplayPageItemFragment imageFragment = new BehanceSDKImageDisplayPageItemFragment();
        Bundle args = new Bundle();
        args.putString(ARG_URL, url);
        args.putBoolean(ARG_ZOOMABLE, zoomable);
        imageFragment.setArguments(args);
        return imageFragment;
    }

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		ViewGroup rootView = (ViewGroup) inflater.inflate(R.layout.bsdk_adapter_image_display_item, container, false);
		mContext = getActivity();
        Bundle args = getArguments();
		String imageUrl = args.getString(ARG_URL);
        zoomable = args.getBoolean(ARG_ZOOMABLE);//TODO disable zoom when false
		try {
			mImageView = (ImageView) rootView.findViewById(R.id.bsdk_imageDisplayActivityImageView);
			progressSpinner = rootView.findViewById(R.id.bsdk_imageDisplayActivityProgressSpinner);
			showProgressSpinner();
			BehanceSDKImageLoaderUtils.displayImageFromCacheOrLoadFromServer(imageUrl, mImageView, this);
		} catch (Exception ex) {
			logger.error(ex, "Unknown error while loading image bitmap [exception is %s]", ex.getMessage());
			displayDecodingErrorMessage(null);
		}
		return rootView;
	}

	private void displayDecodingErrorMessage(FailReason failReason) {
		if (getActivity() != null) {
			Toast toast = Toast.makeText(mContext, getResources().getString(R.string.bsdk_ida_image_decoding_error), Toast.LENGTH_LONG);
			toast.show();
		}
	}

	private void hideProgressSpinner() {
		progressSpinner.setVisibility(View.INVISIBLE);
	}

	private void showProgressSpinner() {
		progressSpinner.setVisibility(View.VISIBLE);
	}

	public void onImageLoadSuccess(Bitmap imageBitmap) {
		setImageViewWithBitmap(imageBitmap);
		hideProgressSpinner();
	}

	@Override
	public void onDestroyView() {

		super.onDestroyView();
	}

	private void setImageViewWithBitmap(Bitmap imageBitmap) {
		try {
			DisplayMetrics displayMetrics = mContext.getResources().getDisplayMetrics();
			// When hardware acceleration is enabled, images beyond
			// 2048x2048 cannot be rendered
			if (imageBitmap.getWidth() > HARDWARE_RENDERING_MAX_SIZE || imageBitmap.getHeight() > HARDWARE_RENDERING_MAX_SIZE) {
				mImageView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
			} else {
				mImageView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
			}

			// We are changing the layout parameters based on the image
			// size so that when user zooms the image occupies complete
			// screen
			if (imageBitmap.getWidth() < displayMetrics.widthPixels || imageBitmap.getHeight() < displayMetrics.heightPixels) {
				RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT,
						RelativeLayout.LayoutParams.MATCH_PARENT);
				layoutParams.addRule(RelativeLayout.CENTER_IN_PARENT);
				mImageView.setLayoutParams(layoutParams);
			} else {
				RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,
						RelativeLayout.LayoutParams.WRAP_CONTENT);
				layoutParams.addRule(RelativeLayout.CENTER_IN_PARENT);
				mImageView.setLayoutParams(layoutParams);
			}
			mImageView.setImageBitmap(imageBitmap);
		} catch (Exception e) {
			logger.error(e, "Unknown error while displaying image bitmap");
			displayDecodingErrorMessage(null);
		}
	}

	public void onImageLoadFailure(Exception e) {
		logger.error(e, "Unable to load image bitmap");
		hideProgressSpinner();
		displayDecodingErrorMessage(null);
	}

	@Override
	public void onLoadingCancelled(String arg0, View arg1) {
		onImageLoadFailure(new BehanceSDKException("Image loading was cancelled"));
	}

	@Override
	public void onLoadingComplete(String arg0, View arg1, Bitmap imageBitmap) {
		onImageLoadSuccess(imageBitmap);
	}

	@Override
	public void onLoadingFailed(String arg0, View arg1, FailReason arg2) {
		onImageLoadFailure(new BehanceSDKException("Image loading failed"));
	}

	@Override
	public void onLoadingStarted(String arg0, View arg1) {
	}
}