/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.fragments;

import android.annotation.SuppressLint;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.behance.sdk.BehanceSDKHeadlessFragmentTags;
import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.R;
import com.behance.sdk.OAuth.TwitterClient;
import com.behance.sdk.asynctask.params.BehanceSDKGetAuthURLFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKGetAuthUrlFromTwitterTaskResult;
import com.behance.sdk.asynctasks.result.BehanceSDKRetrieveAccessTokenFromTwitterTaskResult;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.fragments.headless.BehanceSDKLoginToTwitterHeadlessFragment;
import com.behance.sdk.fragments.headless.BehanceSDKLoginToTwitterHeadlessFragment.Callbacks;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.activities.BehanceSDKLoginToTwitterActivity;

public class BehanceSDKLoginToTwitterFragment extends Fragment implements Callbacks {

	public static final String FRAGMENT_TAG_LOGIN_TO_SOCIAL_CLIENT = "FRAGMENT_TAG_LOGIN_TO_SOCIAL_CLIENT";
	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKLoginToTwitterFragment.class);

	private Callbacks callbacks;
	private WebView webview = null;
	private WebSettings webviewSettings = null;
	private Toast toastMessage;
	private View rootView;
	private ProgressBar progressBar;
	private BehanceSDKLoginToTwitterHeadlessFragment shareProjectOnTwitterHeadlessFragment;
	private BehanceSDKSocialAccountDTO twitterAccount;

	public interface Callbacks {
		public void onTwitterAuthenticationSuccess(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result);

		public void onTwitterAuthenticationFailure(Exception e);
	}

	public void setCallbacks(Callbacks listener) {
		this.callbacks = listener;
	}

	public BehanceSDKLoginToTwitterFragment() {
		// Also need to check if this must become a dialog fragment?
		// Need to change constructor to take a social client as parameter.
		// Currently fragment deals only with twitter
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	@SuppressLint("SetJavaScriptEnabled")
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_login_user_to_social_client, container, false);

		progressBar = (ProgressBar) rootView.findViewById(R.id.bsdkLoginUserToSocialClientProgressBar);
		boolean isNetworkAvailable = true;
		shareProjectOnTwitterHeadlessFragment = (BehanceSDKLoginToTwitterHeadlessFragment) getActivity().getSupportFragmentManager()
				.findFragmentByTag(BehanceSDKHeadlessFragmentTags.SHARE_PROJECT_ON_TWITTER);

		if (shareProjectOnTwitterHeadlessFragment == null) {
			shareProjectOnTwitterHeadlessFragment = new BehanceSDKLoginToTwitterHeadlessFragment();
			getActivity().getSupportFragmentManager().beginTransaction()
					.add(shareProjectOnTwitterHeadlessFragment, BehanceSDKHeadlessFragmentTags.SHARE_PROJECT_ON_TWITTER).commit();
		}
		shareProjectOnTwitterHeadlessFragment.setCallbacks(this);

		NetworkInfo networkInfo = ((ConnectivityManager) getActivity().getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
		if (!(networkInfo != null && networkInfo.isConnected())) {
			isNetworkAvailable = false;
			displayNoInternetConnectivity();
		}

		BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(getActivity());
		twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);

		webview = (WebView) rootView.findViewById(R.id.bsdkLoginUserToSocialClientWebView);
		webviewSettings = webview.getSettings();
		webviewSettings.setSupportZoom(true);
		webviewSettings.setBuiltInZoomControls(true);
		webviewSettings.setUseWideViewPort(true);
		webviewSettings.setJavaScriptEnabled(true);
		webviewSettings.setLoadWithOverviewMode(true);
		if (isNetworkAvailable) {
			getAuthURLFromTwitter();
		}
		return rootView;
	}

	private void getAuthURLFromTwitter() {
		String consumerKey = twitterAccount.getAccountClientId();
		String consumerSecret = twitterAccount.getAccountClientSecret();
		if ((consumerKey != null && !consumerKey.isEmpty()) && (consumerSecret != null && !consumerSecret.isEmpty())) {
			BehanceSDKGetAuthURLFromTwitterAsyncTaskParams params = new BehanceSDKGetAuthURLFromTwitterAsyncTaskParams();
			params.setConsumerKey(consumerKey);
			params.setConsumerSecret(consumerSecret);
			shareProjectOnTwitterHeadlessFragment.getAuthURLFromTwitter(params);
		}
	}

	private void displayNoInternetConnectivity() {
		if (getActivity() != null) {
			toastMessage = Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_connection_error_msg), Toast.LENGTH_LONG);
			toastMessage.setGravity(Gravity.CENTER, 0, 0);
			toastMessage.show();
			getActivity().setResult(BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_NO_NETWORK_CONNECTIVITY);
			getActivity().finishActivity(BehanceSDKPublishWIPDialogFragment.TWITTER_LOGIN_ACTIVITY_REQUEST_CODE);
		}
	}

	public void setWebViewforAuthorization(String url) {
		hideProgressBar();
		// make sure url doesn't open in external browser
		webview.setWebViewClient(new WebViewClient() {

			@Override
			public boolean shouldOverrideUrlLoading(WebView webView, String url) {
				if (url != null && url.startsWith(TwitterClient.TWITTER_OAUTH_CALLBACK_URI)) {
					retrieveTokenFromURLCallback(url);
					webView.setVisibility(View.GONE);
					return true;
				} else
					return false;
			}

			@Override
			public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
				super.onReceivedError(view, errorCode, description, failingUrl);
				displayNoInternetConnectivity();
			}

		});

		webview.loadUrl(url);
	}

	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			onBackPressed();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	public void onBackPressed() {
		if (getActivity() != null) {
			getActivity().finish();
		}
	}

	private void retrieveTokenFromURLCallback(String url) {
		String consumerKey = twitterAccount.getAccountClientId();
		String consumerSecret = twitterAccount.getAccountClientSecret();
		if ((consumerKey != null && !consumerKey.isEmpty()) && (consumerSecret != null && !consumerSecret.isEmpty())) {
			BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams params = new BehanceSDKRetrieveAccessTokenFromTwitterAsyncTaskParams();
			params.setConsumerKey(consumerKey);
			params.setConsumerSecret(consumerSecret);
			params.setAuthURL(url);
			shareProjectOnTwitterHeadlessFragment.retrieveAccessTokenFromTwitter(params);
		}
	}

	private void hideProgressBar() {
		if (progressBar != null) {
			progressBar.setVisibility(View.INVISIBLE);
		}
	}

	@Override
	public void onGetAuthURLFromTwitterSuccess(BehanceSDKGetAuthUrlFromTwitterTaskResult result) {
		if (getActivity() != null) {
			if (result.getAuthURL() != null && result.getAuthURL().length() > 0) {
				setWebViewforAuthorization(result.getAuthURL());
			} else {
				logger.error("Problem retrieving auth URL. AUTH URL retrieved is null");
			}
		}
	}

	@Override
	public void onGetAuthURLFromTwitterFailure(Exception e) {
		if (getActivity() != null) {
			logger.error(e, "Problem retrieving auth URL from Twitter. [Error - %s]", e.getMessage());
			if (callbacks != null) {
				callbacks.onTwitterAuthenticationFailure(e);
			}
		}
	}

	@Override
	public void onRetrieveAccessTokenFromTwitterSuccess(BehanceSDKRetrieveAccessTokenFromTwitterTaskResult result) {
		if (getActivity() != null) {
			if (callbacks != null) {
				callbacks.onTwitterAuthenticationSuccess(result);
			}
		}
	}

	@Override
	public void onRetrieveAccessTokenFromTwitterFailure(Exception e) {
		if (getActivity() != null) {
			logger.error(e, "Problem retrieving access token and secret from Twitter. [Error - %s]", e.getMessage());
			if (callbacks != null) {
				callbacks.onTwitterAuthenticationFailure(e);
			}
		}
	}
}