/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2013 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.fragments;

import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Point;
import android.graphics.SurfaceTexture;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.TransitionDrawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.ListPopupWindow;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.Surface;
import android.view.TextureView;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;

import android.view.animation.AccelerateInterpolator;
import android.view.animation.DecelerateInterpolator;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKHeadlessFragmentTags;
import com.behance.sdk.R;

import com.behance.sdk.asynctask.params.BehanceSDKDeleteProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKFollowUnFollowUserAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectCommentsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectDetailsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKPostProjectCommentAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKPostUserSettingsOnServerAsyncTaskParams;
import com.behance.sdk.dto.BehanceSDKShareContentDTO;
import com.behance.sdk.dto.project.BehanceSDKAbstractCommentDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCommentDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCoversDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectFeaturedDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectStatsDTO;
import com.behance.sdk.dto.search.BehanceSDKImageDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.dto.search.BehanceSDKUserSettingsOnServerDTO;
import com.behance.sdk.enums.BehanceSDKShareContentType;
import com.behance.sdk.fragments.headless.BehanceSDKGetProjectDetailsHeadlessFragment;
import com.behance.sdk.listeners.IBehanceSDKEndlessScrollRecyclerViewListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.ui.activities.BehanceSDKProjectDetailActivity;
import com.behance.sdk.ui.adapters.BehanceSDKProjectDetailRecyclerAdapter;
import com.behance.sdk.ui.adapters.BehanceSDKProjectOwnersListArrayAdapter;
import com.behance.sdk.ui.animations.BehanceSDKFabToBarAnimator;
import com.behance.sdk.ui.components.BehanceSDKEndlessScrollRecyclerView;
import com.behance.sdk.ui.components.BehanceSDKPreCachingLinearLayoutManager;
import com.behance.sdk.ui.customviews.BehanceSDKCommentsView;
import com.behance.sdk.ui.dialogs.BehanceSDKBottomSheetShareDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertWithSingleBtnDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKLoginToProceedUserDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKProjectInfoDialogFragment;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKImageLoaderUtils;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.SimpleImageLoadingListener;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.List;


public class BehanceSDKProjectDetailFragment extends Fragment implements BehanceSDKGetProjectDetailsHeadlessFragment.Callbacks, OnClickListener,
        BehanceSDKCommentsView.Callbacks, IBehanceSDKEndlessScrollRecyclerViewListener, BehanceSDKProjectDetailRecyclerAdapter.Callbacks {

    private static final String FRAGMENT_TAG_PROJECT_INFO_DIALOG = "FRAGMENT_TAG_PROJECT_INFO_DIALOG";
    private static final String FRAGMENT_TAG_PROJECT_SHARE_DIALOG = "PROJECT_DETAILS_FRAGMENT_FRAGMENT_TAG_PROJECT_SHARE_DIALOG";
    private static final String FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG = "FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG";

    public static final String ARG_PROJECT_ID = "ARG_PROJECT_ID";

    private BehanceSDKProjectDTO mActiveProject;
//    private BehanceSDK behanceSDK;
//    private BehanceSDKCustomResourcesOptions options = null;
    private View rootView;
    private BehanceSDKGetProjectDetailsHeadlessFragment getProjectDetailsHeadlessFragment;
    private View projectDetailsProgressSpinner;

    private ImageLoader mloader;
    private android.support.v7.app.ActionBar actionBar;

    private BehanceSDKEndlessScrollRecyclerView projectRecycler;
    private TextureView projectBG;
    private ScrollView projectBGImageScrollContainer;

    private View postCommentContainer;
    private EditText postCommentEditText;
    private ImageView postCommentBtn;
    private FloatingActionButton newCommentButton;
    private TransitionDrawable commentCrossfader;

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKProjectDetailFragment.class);
    private boolean commentsContainerOpen = false;

    private AlertDialog unFollowUserDialog;
    private BehanceSDKGenericAlertDialog matureContentDialog;
    private BehanceSDKGenericAlertWithSingleBtnDialog restrictedAgeDialog;
    private View matureContentBackgroundLayer;
    private BehanceSDKLoginToProceedUserDialog loginToProceedDialog;
    private Button followButton;

    private Toolbar toolbar;
    private View headerContainer, headerStatsFeaturedContainer, toolBarContainer, toolBarView;

    private String projectId;

    private int fragNum = -1;

    private int headerOffset = 0;
    private int headerMaxScroll = -1;
    private int toolBarMaxScroll = -1;
    private int toolBarOffset = 0;

    private boolean isLoggedInUserOnlyProjectOwner = false;

    private BehanceSDKFabToBarAnimator fabToBarAnimator;

    public interface Callbacks {
        public void onSuccessfulLoadOfProjectDetails(BehanceSDKProjectDTO activeProject);

        public void onFailureToLoadProjectDetails(Exception e);
    }

    public BehanceSDKProjectDetailFragment() {
    }

    public static BehanceSDKProjectDetailFragment newInstance(String projectId, int fragNum) {
        BehanceSDKProjectDetailFragment projectDetailFragment = new BehanceSDKProjectDetailFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PROJECT_ID, projectId);
        args.putInt("frag_num", fragNum);
        projectDetailFragment.setArguments(args);
        return projectDetailFragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mloader = ImageLoader.getInstance();
 //       behanceSDK = BehanceSDK.getInstance();
        setHasOptionsMenu(true);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);

        rootView = inflater.inflate(R.layout.bsdk_fragment_project_detail, container, false);
        headerContainer = rootView.findViewById(R.id.bsdk_projectDetailsHeaderContainer);
        toolbar = (Toolbar) rootView.findViewById(R.id.bsdk_projectDetailsToolbar);
        toolbar.setNavigationIcon(R.drawable.bsdk_icon_actionbar_back_button);
        ((AppCompatActivity) getActivity()).setSupportActionBar(toolbar);
        actionBar = ((AppCompatActivity) getActivity()).getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayShowTitleEnabled(false);

        }

        toolBarContainer = rootView.findViewById(R.id.bsdk_projectDetailsToolBarContainer);
        toolBarView = rootView.findViewById(R.id.bsdk_view_toolbar);

        projectDetailsProgressSpinner = rootView.findViewById(R.id.bsdk_projectDetailFragmentProgressSpinner);

        Drawable backgrounds[] = new Drawable[2];
        final Resources res = getResources();
        backgrounds[0] = res.getDrawable(R.drawable.bsdk_icon_float_comment);
        backgrounds[1] = new ColorDrawable(getResources().getColor(R.color.bsdk_behance_blue));

        commentCrossfader = new TransitionDrawable(backgrounds);
        commentCrossfader.setCrossFadeEnabled(true);

        newCommentButton = (FloatingActionButton) rootView.findViewById(R.id.bsdk_projectDetailFragmentCommentFAB);
        postCommentContainer = rootView.findViewById(R.id.bsdk_projectDetailsFragmentPostCommentContainer);
        postCommentEditText = (EditText) rootView.findViewById(R.id.bsdk_projectDetailsFragmentPostCommentET);
        postCommentBtn = (ImageView) rootView.findViewById(R.id.bsdk_projectDetailsFragmentPostCommentSend);
        postCommentBtn.setOnClickListener(this);
        postCommentEditText.addTextChangedListener(new TextWatcher() {

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (s.length() == 0) {
                    postCommentBtn.setEnabled(false);
                    postCommentBtn.animate().alpha(0.3f).start();
                } else {
                    postCommentBtn.setEnabled(true);
                    postCommentBtn.animate().alpha(1f).start();
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
        postCommentBtn.setEnabled(false);
        postCommentBtn.animate().alpha(0.5f).start();
        postCommentContainer.setVisibility(View.INVISIBLE);
        newCommentButton.setOnClickListener(this);

        fabToBarAnimator = BehanceSDKFabToBarAnimator.getInstance(getActivity(), newCommentButton, postCommentContainer);


        projectRecycler = (BehanceSDKEndlessScrollRecyclerView) rootView.findViewById(R.id.bsdk_projectDetailsRecycler);
        projectRecycler.setLayoutManager(new BehanceSDKPreCachingLinearLayoutManager(getActivity(), LinearLayoutManager.VERTICAL, false));
        projectRecycler.setCallbackListener(this);
        projectRecycler.initializeScrollListener((LinearLayoutManager) projectRecycler.getLayoutManager());

        projectBG = (TextureView) rootView.findViewById(R.id.bsdk_projectDetailsBackground);
        projectBG.setSurfaceTextureListener(new TextureView.SurfaceTextureListener() {
            @Override
            public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {
                setupBG();
            }

            @Override
            public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {

            }

            @Override
            public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
                return false;
            }

            @Override
            public void onSurfaceTextureUpdated(SurfaceTexture surface) {

            }
        });
        projectBGImageScrollContainer = (ScrollView) rootView.findViewById(R.id.bsdk_projectDetailsBackgroundImageScrollContainer);

        rootView.findViewById(R.id.bsdk_projectDetailsHeaderArtistContainer).setOnClickListener(this);

        // Mature Content Background View
        matureContentBackgroundLayer = rootView.findViewById(R.id.bsdk_projectDetailFragmentMatureContentBackgroundLayer);

        projectId = "";
        Bundle arguments = getArguments();
        if (arguments == null) {
            arguments = savedInstanceState;
        }
        if (arguments != null) {
            projectId = arguments.getString(ARG_PROJECT_ID);
            fragNum = arguments.getInt("frag_num", -1);
        }

        getProjectDetailsHeadlessFragment = (BehanceSDKGetProjectDetailsHeadlessFragment) getActivity().getSupportFragmentManager().findFragmentByTag(
                BehanceSDKHeadlessFragmentTags.GET_PROJECT_DETAILS + ((fragNum == -1) ? "" : fragNum));

        if (getProjectDetailsHeadlessFragment == null) {
            getProjectDetailsHeadlessFragment = new BehanceSDKGetProjectDetailsHeadlessFragment();
            getActivity().getSupportFragmentManager().beginTransaction()
                    .add(getProjectDetailsHeadlessFragment, BehanceSDKHeadlessFragmentTags.GET_PROJECT_DETAILS + ((fragNum == -1) ? "" : fragNum)).commit();
        }

        getProjectDetailsHeadlessFragment.setCallbacks(this);

        View toolbarTitle = rootView.findViewById(R.id.bsdk_projectDetailsToolbarTitle);
        toolbarTitle.setOnClickListener(this);
        headerStatsFeaturedContainer = rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsFeaturedContainer);

        followButton = (Button) rootView.findViewById(R.id.bsdk_projectDetailsHeaderFollowButton);
        followButton.setOnClickListener(this);


        if (getProjectDetailsHeadlessFragment.isGetProjectDetailsAsyncTaskInProgress()) {
            showProjectLoadingProgressBar();
        } else {
            loadProjectDetails(projectId);
        }

        /*
        if (!behanceSDK.useBehanceTheme()) {
            options = behanceSDK.getBehanceSDKCustomResourcesOptions();
            setUpCustomAppTheme();
        }
        */
        return rootView;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.bsdk_project_details_view_actions, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                getActivity().onBackPressed();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void revealCommentsView() {
        if (!BehanceSDKUserManager.getInstance().isUserLoggedIn()) {
            String ownerName = getActiveProject().getOwners().get(0).getDisplayName();
            String messageText = getResources().getString(R.string.bsdk_login_to_proceed_comment_user_msg, ownerName);
            displayLoginToProceedAlertDialog(messageText);
        } else {
            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                if (fabToBarAnimator != null)
                    fabToBarAnimator.fabToBar();
            } else {
                if (!commentsContainerOpen) {
                    newCommentButton.animate().alpha(0.f).setDuration(150).start();
                    newCommentButton.setEnabled(false);
                    postCommentContainer.setBackgroundColor(getResources().getColor(R.color.bsdk_card_color));
                    postCommentContainer.setVisibility(View.VISIBLE);
                    postCommentContainer.animate().alpha(1.f).setDuration(150).setStartDelay(150).start();
                    postCommentContainer.setEnabled(true);
                    commentsContainerOpen = true;
                }
            }
        }
    }

    private void concealCommentsView() {
        InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(postCommentEditText.getWindowToken(), 0);
        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (fabToBarAnimator != null)
                fabToBarAnimator.barToFab();
        } else {
            if (commentsContainerOpen) {
                newCommentButton.animate().alpha(1.f).setDuration(150).setStartDelay(150).start();
                newCommentButton.setEnabled(true);
                postCommentContainer.animate().alpha(0.f).setDuration(150).start();
                postCommentContainer.setEnabled(false);
                commentsContainerOpen = false;
            }
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        animateToolbar(true);
    }

    private void loadProjectDetails(String projectId) {
        setActiveProject(getProjectDetailsHeadlessFragment.getActiveProject());
        if (getActiveProject() != null && projectId.equals(getProjectDetailsHeadlessFragment.getActiveProject().getId())) {
            displayProjectDetails(projectId);
        } else {
            loadProjectFromServer(projectId, false);
        }
    }

    private void displayProjectDetails(String projectId) {
        // we will check if the logged in user is an owner of this project
        // if yes, we will display unpublish option in actionbar menu options
        isLoggedInUserOnlyProjectOwner = false;
        if (BehanceSDKUserManager.getInstance().isUserLoggedIn()) {
            BehanceSDKUserDTO loggedInUserDTO = BehanceSDKUserManager.getInstance().getUserDTO();
            if (loggedInUserDTO != null) {
                List<BehanceSDKUserDTO> owners = getActiveProject().getOwners();
                if (owners.size() == 1) {
                    if (owners.get(0).getId() == loggedInUserDTO.getId()) {
                        isLoggedInUserOnlyProjectOwner = true;
                        getActivity().invalidateOptionsMenu();
                    }
                }
            }
        }
        if (isLoggedInUserOnlyProjectOwner)
            followButton.setVisibility(View.GONE);


        if (!checkForMatureContent(getActiveProject())) {
            matureContentBackgroundLayer.setVisibility(View.INVISIBLE);
        }

        newCommentButton.setVisibility(mActiveProject.isAllowComments() ? View.VISIBLE : View.GONE);

        displayProjectModules();
        displayCommentCount();
        loadProjectComments(projectId, false);
    }

    private void incrementAndDisplayCommentsCount() {
        BehanceSDKProjectStatsDTO projectStatsDTO = mActiveProject.getStats();
        int updatedCommentsCount = projectStatsDTO.getCommentsCount();
        updatedCommentsCount = updatedCommentsCount + 1;
        if (updatedCommentsCount < 0) {
            updatedCommentsCount = 0;
        }
        projectStatsDTO.setCommentsCount(updatedCommentsCount);
        displayCommentCount();
    }

    private void decrementAndDisplayCommentsCount() {
        BehanceSDKProjectStatsDTO projectStatsDTO = mActiveProject.getStats();
        int updatedCommentsCount = projectStatsDTO.getCommentsCount();
        updatedCommentsCount = updatedCommentsCount - 1;
        if (updatedCommentsCount < 0) {
            updatedCommentsCount = 0;
        }
        projectStatsDTO.setCommentsCount(updatedCommentsCount);
        displayCommentCount();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == 1) {
            if (resultCode == 1) {
                displayFollowUserBtn();
                for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
                    if (owner.isCurrentUserFollowing())
                        getProjectDetailsHeadlessFragment.unFollowUser(owner);
                }
            }
        }
    }

    private void displayCommentCount() {
        int commentsCount = mActiveProject.getStats().getCommentsCount();

        NumberFormat formatter = new DecimalFormat("###,###,###,###");

        // update the comment count displayed in Project header view
        TextView commentsCountTxtView = (TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsComments);
        commentsCountTxtView.setText(formatter.format((double) commentsCount));
    }

    int bgImageHeight = -1;

    private void displayProjectModules() {
        final BehanceSDKProjectDTO project = getActiveProject();
        if (project != null) {
            populateProjectInfoView(project);
            hideProjectLoadingProgressBar();
            loadActionBarControls();

            setupBG();
            BehanceSDKProjectDetailRecyclerAdapter adapter = new BehanceSDKProjectDetailRecyclerAdapter(getActivity(), project);
            adapter.addComments(getProjectDetailsHeadlessFragment.getActiveProjectCommentsList());
            adapter.setMoreToLoad(getProjectDetailsHeadlessFragment.isMoreCommentsAvailable());
            if (headerContainer.getHeight() > 0)
                adapter.setHeaderHeight(headerContainer.getHeight());
            adapter.setCallbacks(this);
            projectRecycler.setAdapter(adapter);
            toolBarView.setAlpha(1.f);
            //handles header on orientation change when not at top of project
            if (((LinearLayoutManager) projectRecycler.getLayoutManager()).findFirstVisibleItemPosition() != 0) {
                toolBarContainer.setAlpha(1.f);
                //header container will not have rendered yet so translation can not be computed here
            }


            projectRecycler.addOnScrollListener(new RecyclerView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                    super.onScrollStateChanged(recyclerView, newState);
                    if(newState == RecyclerView.SCROLL_STATE_IDLE) {
                        if ((recyclerView.computeVerticalScrollOffset() < toolBarScrollDist)) {
                            animateToolbar(true);
                        } else {
                            if (2 * toolBarOffset < toolBarScrollDist) {
                                animateToolbar(true);  // Show toolbar
                            } else {
                                animateToolbar(false);  // Hide toolbar
                            }
                        }
                    }
                }

                int scroll = 0;

                @Override
                public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                    super.onScrolled(recyclerView, dx, dy);

                    toolBarOffset += dy;
                    if (toolBarOffset > toolBarScrollDist)
                        toolBarOffset = toolBarScrollDist;
                    else if (toolBarOffset < 0)
                        toolBarOffset = 0;
                    float  alpha = toolBarOffset / (toolBarContainer.getHeight() * 0.75f) ;
                    if(1 - alpha < 0)
                        toolBarContainer.setAlpha(0);
                    else
                         toolBarContainer.setAlpha(1- alpha);
                    headerContainer.animate().translationY(-toolBarOffset).setDuration(0).start();

                    scroll -= dy;
                    if (bgImageHeight > 0) {
                        projectBG.setTranslationY(scroll % bgImageHeight + bgImageHeight);
                    } else if (bgImageHeight == 0) {
                        projectBGImageScrollContainer.scrollTo(0, -scroll);
                    }
                    concealCommentsView();

                }
            });
        }
    }

    private void animateToolbar(boolean show) {
        if (show) {
            headerContainer.clearAnimation();
            headerContainer.animate().translationY(0).setDuration((long) (Math.abs(headerContainer.getTranslationY()) * 3)).setInterpolator(new DecelerateInterpolator(2)).start();
            toolBarContainer.animate().alpha(1f).setDuration((long) (Math.abs(headerContainer.getTranslationY()) * 3)).setInterpolator(new DecelerateInterpolator(1)).start();
            toolBarOffset = 0;
        } else {
            toolBarOffset = toolBarScrollDist;
            headerContainer.clearAnimation();
            headerContainer.animate().translationY(-toolBarOffset).setDuration((long) (Math.abs(headerContainer.getTranslationY()) * 3)).setInterpolator(new AccelerateInterpolator(2)).start();
            toolBarContainer.animate().alpha(0).setDuration((long) (Math.abs(headerContainer.getTranslationY()) * 3)).setInterpolator(new AccelerateInterpolator(1)).start();

        }
    }


    private void setupBG() {
        if (mActiveProject != null) {
            projectBG.setBackgroundColor(mActiveProject.getProjectStyles().getBgColor());
            Canvas c = projectBG.lockCanvas();
            if (c != null) {
                c.drawColor(mActiveProject.getProjectStyles().getBgColor());
                projectBG.unlockCanvasAndPost(c);
            }

            if (mActiveProject.getProjectStyles().getBgImageUrl() != null && !mActiveProject.getProjectStyles().getBgImageUrl().isEmpty()) {//&& project.getProjectStyles().isBgImageRepeat()) {
                if (mActiveProject.getProjectStyles().isBgImageRepeat()) {
                    BehanceSDKImageLoaderUtils.displayImageFromCacheOrLoadFromServer(mActiveProject.getProjectStyles().getBgImageUrl(), new ImageView(getActivity()), new SimpleImageLoadingListener() {

                        @Override
                        public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
                            if (loadedImage.isRecycled()) {
                                //sometimes returns recycled image from cache, so force reload
                                setupBG();
                            } else {
                                int bmw = (int) (getResources().getDisplayMetrics().widthPixels / 8.0);
                                Bitmap bitmap = getResizedBitmap(loadedImage, bmw, (int) ((1.0 * loadedImage.getHeight() / loadedImage.getWidth()) * bmw));
                                Canvas c = projectBG.lockCanvas();
                                if (c != null) {
                                    c.drawColor(mActiveProject.getProjectStyles().getBgColor());
                                    for (int y = 0; y < projectBG.getHeight(); y += bitmap.getHeight()) {
                                        for (int x = 0; x < projectBG.getWidth(); x += bmw) {
                                            c.drawBitmap(bitmap, x, y, null);
                                        }
                                    }
                                    bgImageHeight = bitmap.getHeight();
                                }
                                projectBG.unlockCanvasAndPost(c);
                            }
                        }

                        @Override
                        public void onLoadingCancelled(String imageUri, View view) {
                            //TODO handle failure
                        }
                    });
                } else {
                    //don't save position on projects with single image backgrounds, it's a pain in the ass and we don't care about them anyway
                    projectRecycler.setSaveEnabled(false);
                    loadSingleImageBG();
                    bgImageHeight = 0;
                }
            }
        }
    }

    private void loadSingleImageBG() {
        BehanceSDKImageLoaderUtils.displayImageFromCacheOrLoadFromServer(mActiveProject.getProjectStyles().getBgImageUrl(), new ImageView(getActivity()), new SimpleImageLoadingListener() {
            boolean firstAttempt = true;

            @Override
            public void onLoadingFailed(String imageUri, View view, FailReason failReason) {

            }

            @Override
            public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
                LinearLayout container = (LinearLayout) rootView.findViewById(R.id.bsdk_projectDetailsBackgroundImageContainer);
                container.setPadding(0, headerContainer.getHeight(), 0, getResources().getDisplayMetrics().heightPixels);

                int widthPixels = getResources().getDisplayMetrics().widthPixels;

                for (int i = 0; i < loadedImage.getHeight(); i += 1000) {
                    int h = (i + 1000 > loadedImage.getHeight()) ? (loadedImage.getHeight() - i) : 1000;
                    Bitmap bm = Bitmap.createBitmap(loadedImage, 0, i, loadedImage.getWidth(), h);
                    ImageView iv = new ImageView(getActivity());
                    LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(widthPixels, (int) (1.0 * h * widthPixels / loadedImage.getWidth()));
                    iv.setLayoutParams(params);
                    iv.setImageBitmap(bm);
                    iv.setScaleType(ImageView.ScaleType.FIT_XY);
                    container.addView(iv);
                }
            }

            @Override
            public void onLoadingCancelled(String imageUri, View view) {
                //often fails the first time, not sure why
                if (firstAttempt) {
                    firstAttempt = false;
                    loadSingleImageBG();
                } else {
                    //handle
                }
            }

        });
    }

    private Bitmap getResizedBitmap(Bitmap bm, int newWidth, int newHeight) {
        int width = bm.getWidth();
        int height = bm.getHeight();
        float scaleWidth = ((float) newWidth) / width;
        float scaleHeight = ((float) newHeight) / height;
        Matrix matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);
        Bitmap resizedBitmap = Bitmap.createBitmap(bm, 0, 0, width, height, matrix, false);
        bm.recycle();
        return resizedBitmap;
    }

    private boolean checkForMatureContent(BehanceSDKProjectDTO project) {
        if (project != null) {
            if (project.isMatureContent()) {
                if (!BehanceSDKProjectDTO.MATURE_CONTENT_ACCESS_ALLOWED.equals(project.getMatureAccess())) {
                    displayMatureContentDialog();
                    return true;
                }
            }
        } else {
            closeProjectActivity();
        }
        return false;
    }

    private void displayMatureContentDialog() {
        String matureAccess = mActiveProject.getMatureAccess();
        if (BehanceSDKProjectDTO.MATURE_CONTENT_ACCESS_RESTRICTED_LOGGED_OUT.equals(matureAccess)) {
            matureContentDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(), R.string.bsdk_login_to_proceed_dialog_title,
                    R.string.bsdk_mature_content_dialog_user_not_logged_in_message_text, R.string.bsdk_mature_content_dialog_log_in_button_text,
                    R.string.bsdk_mature_content_alert_go_back_button_text);
            matureContentDialog.setCancelable(false);
            matureContentDialog.setOnOKBtnClickListener(this);
            matureContentDialog.setOnNotOKBtnClickListener(this);
            matureContentDialog.show();
        } else if (BehanceSDKProjectDTO.MATURE_CONTENT_ACCESS_RESTRICTED_AGE.equals(matureAccess)) {
            restrictedAgeDialog = BehanceSDKGenericAlertWithSingleBtnDialog.getInstance(getActivity(), R.string.bsdk_mature_content_alert_title_text,
                    R.string.bsdk_mature_content_res_age_alert_msg_text, R.string.bsdk_mature_content_res_age_back_btn_lbl);
            restrictedAgeDialog.setCancelable(false);
            restrictedAgeDialog.setBtnClickListener(this);
            restrictedAgeDialog.show();
        } else {
            matureContentDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(), R.string.bsdk_mature_content_alert_title_text,
                    R.string.bsdk_mature_content_alert_msg_text, R.string.bsdk_mature_content_alert_turn_off_button_text,
                    R.string.bsdk_mature_content_alert_go_back_button_text);
            matureContentDialog.setCancelable(false);
            matureContentDialog.setOnOKBtnClickListener(this);
            matureContentDialog.setOnNotOKBtnClickListener(this);
            matureContentDialog.show();
        }
    }

    private void loadActionBarControls() {
        if (getActivity() instanceof BehanceSDKProjectDetailActivity) {
            BehanceSDKProjectDetailActivity projectDetailActivity = (BehanceSDKProjectDetailActivity) getActivity();
            projectDetailActivity.onSuccessfulLoadOfProjectDetails(getActiveProject());
        }

    }

    private void loadProjectFromServer(String projectId, boolean forceLoad) {
        loadProjectDetailsFromServer(projectId);
    }

    private void loadProjectDetailsFromServer(String projectId) {
        if (getProjectDetailsHeadlessFragment.isGetProjectDetailsAsyncTaskInProgress()) {
            showProjectLoadingProgressBar();
        } else {
            BehanceSDKGetProjectDetailsAsyncTaskParams params = new BehanceSDKGetProjectDetailsAsyncTaskParams();
            boolean downloadHD = shouldDownloadHDImage();
            boolean largeTablet = this.getResources().getBoolean(R.bool.bsdk_large_tablet);
            params.setProjectID(projectId);
            params.setDownloadHD(downloadHD);
            params.setFollowHTMLButtonText(getActivity().getResources().getString(R.string.bsdk_project_detail_fragment_follow_user_html_button_text));
            params.setFollowingHTMLButtonText(getActivity().getResources()
                    .getString(R.string.bsdk_project_detail_fragment_following_user_html_button_text));
            params.setLargeTablet(largeTablet);
            getProjectDetailsHeadlessFragment.loadProjectDetailsFromServer(params);
        }
    }

    private boolean shouldDownloadHDImage() {
        return (this.getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_LARGE;
    }

    private void loadProjectComments(String projectId, boolean forceLoadFromServer) {
        if (mActiveProject.isAllowComments() && !getProjectDetailsHeadlessFragment.isGetProjectCommentsAsyncTaskInProgress()) {
            if (forceLoadFromServer || getProjectDetailsHeadlessFragment.getActiveProjectCommentsList() == null
                    || getProjectDetailsHeadlessFragment.getActiveProjectCommentsList().size() <= 0) {
                BehanceSDKGetProjectCommentsAsyncTaskParams params = new BehanceSDKGetProjectCommentsAsyncTaskParams();
                params.setProjectId(projectId);
                getProjectDetailsHeadlessFragment.loadProjectCommentsFromServer(params);
            }
        }
    }

    private void loadProjectCommentsNextPageFromServer() {
        if (mActiveProject.isAllowComments() && getProjectDetailsHeadlessFragment.isMoreCommentsAvailable()
                && !getProjectDetailsHeadlessFragment.isGetProjectCommentsAsyncTaskInProgress()
                && getProjectDetailsHeadlessFragment.getActiveProjectCommentsList() != null
                && getProjectDetailsHeadlessFragment.getActiveProjectCommentsList().size() > 0) {
            BehanceSDKGetProjectCommentsAsyncTaskParams params = new BehanceSDKGetProjectCommentsAsyncTaskParams();
            params.setProjectId(projectId);
            getProjectDetailsHeadlessFragment.loadProjectCommentsNextPageFromServer(params);
        }
    }

    private void showProjectLoadingProgressBar() {
        if (projectDetailsProgressSpinner != null) {
            projectDetailsProgressSpinner.setVisibility(View.VISIBLE);
        }
    }

    private void hideProjectLoadingProgressBar() {
        if (projectDetailsProgressSpinner != null) {
            projectDetailsProgressSpinner.setVisibility(View.INVISIBLE);
        }
    }

    @Override
    public void onGetProjectDetailsSuccess(BehanceSDKProjectDTO activeProject) {
        if (getActivity() != null) {
            if (activeProject != null) {
                long publishedDate = activeProject.getPublishedDate();
                if (publishedDate > 0) {
                    this.setActiveProject(activeProject);
                    hideProjectLoadingProgressBar();
                    displayProjectDetails(activeProject.getId());
                } else {
                    handleProjectLoadingFailure(null, R.string.bsdk_project_detail_fragment_problem_loading_project_details_msg);
                }
            } else {
                handleProjectLoadingFailure(null, R.string.bsdk_project_detail_fragment_problem_loading_project_details_msg);
            }
        }
    }

    @Override
    public void onGetProjectDetailsFailure(Exception e) {
        handleProjectLoadingFailure(e, R.string.bsdk_project_detail_fragment_problem_loading_project_details_msg);
    }

    private void handleProjectLoadingFailure(Exception e, int messageToDisplay) {
        if (e != null) {
            logger.error(e, "Problem loading project details from server");
        } else {
            logger.error("Problem loading project details from server");
        }

        if (getActivity() != null) {
            hideProjectLoadingProgressBar();
            setActiveProject(null);
            if (getProjectDetailsHeadlessFragment != null) {
                getProjectDetailsHeadlessFragment.setActiveProject(null);
            }
            Toast.makeText(getActivity(), messageToDisplay, Toast.LENGTH_LONG).show();

            getActivity().finish();
        }
    }

    @Override
    public void onDetach() {
        dismissMatureContentDialog();
        dismissLoginToProceedAlertDialog();
        unlockScreenOrientation();
        super.onDetach();
    }

    private void populateProjectInfoView(final BehanceSDKProjectDTO activeProject) {
        if (activeProject != null) {

            if (activeProject.getOwners().size() > 1) {
                int ownersFollowedCount = 0;
                int loggedInUserId = -1;
                BehanceSDKUserDTO loggedInUserDTO = BehanceSDKUserManager.getInstance().getUserDTO();
                if (loggedInUserDTO != null)
                    loggedInUserId = loggedInUserDTO.getId();
                for (BehanceSDKUserDTO owner : activeProject.getOwners()) {
                    if (owner.isCurrentUserFollowing() || owner.getId() == loggedInUserId)
                        ownersFollowedCount++;
                }
                followButton.setText(ownersFollowedCount == activeProject.getOwners().size() ? R.string.bsdk_unfollow : R.string.bsdk_follow_all);
            } else {
                followButton.setText(activeProject.getOwners().get(0).isCurrentUserFollowing() ? R.string.bsdk_unfollow : R.string.bsdk_follow);
            }

            NumberFormat formatter = new DecimalFormat("###,###,###,###");

            TextView title = ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsToolbarTitle));
            int padding = getResources().getDimensionPixelSize(R.dimen.bsdk_adapter_project_info_featured_list_container_padding_right) +
                    Math.min(activeProject.getFeatures().size(), 3) *
                    (getResources().getDimensionPixelSize(R.dimen.bsdk_adapter_project_info_featured_list_item_ribbon_image_width_size) +
                    2 * getResources().getDimensionPixelSize(R.dimen.bsdk_adapter_project_info_featured_list_item_ribbon_image_horizontal_padding));
            title.setPadding(0, 0, padding, 0);
            title.setText(activeProject.getName());

            String artistName = activeProject.getOwners().get(0).getDisplayName();
            if (activeProject.getOwners().size() == 2)
                artistName += getResources().getString(R.string.bsdk_project_detail_fragment_multiple_owners_more_one, String.valueOf(activeProject.getOwners().size() - 1));
            else if (activeProject.getOwners().size() > 2)
                artistName += getResources().getString(R.string.bsdk_project_detail_fragment_multiple_owners_more, String.valueOf(activeProject.getOwners().size() - 1));
            ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderArtistName)).setText(artistName);
            ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsViews)).setText(String.valueOf(formatter.format((double) (activeProject.getStats().getViewsCount()))));
            ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsAppreciations)).setText(String.valueOf(formatter.format((double) (activeProject.getStats().getAppreciationsCount()))));
            ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsComments)).setText(String.valueOf(formatter.format((double) (activeProject.getStats().getCommentsCount()))));
            ImageView avatar = (ImageView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderAvatar);
            mloader.displayImage(activeProject.getOwners().get(0).findProfileImageInIncreasingSizeOrder(250).getUrl(), avatar);

            LinearLayout ribbonsContainer = (LinearLayout) rootView.findViewById(R.id.bsdk_projectDetailsHeaderFeaturedRibbonsContainer);
            ribbonsContainer.removeAllViews();
            LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            for (BehanceSDKProjectFeaturedDTO featuredDTO : activeProject.getFeatures()) {
                View featureRootView = inflater.inflate(R.layout.bsdk_adapter_project_info_featured_list_item, ribbonsContainer, false);
                final ImageView featureIconImgView = (ImageView) featureRootView.findViewById(R.id.bsdk_projectInfoFeaturedIconImgView);
                mloader.loadImage(featuredDTO.getSiteRibbonBiggerImage(), new SimpleImageLoadingListener() {
                    @Override
                    public void onLoadingComplete(String imageUri, View view, Bitmap loadedImage) {
                        featureIconImgView.setImageBitmap(loadedImage);
                    }
                });
                featureRootView.setTag(featuredDTO);
                featureRootView.setOnClickListener(this);
                ribbonsContainer.addView(featureRootView);
            }

            projectRecycler.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
                @Override
                public void onGlobalLayout() {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                        projectRecycler.getViewTreeObserver().removeOnGlobalLayoutListener(this);
                    } else {
                        projectRecycler.getViewTreeObserver().removeGlobalOnLayoutListener(this);
                    }
                    if (projectRecycler.getAdapter() != null) {
                        ((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).setHeaderHeight(headerContainer.getHeight());
                        projectRecycler.getAdapter().notifyItemChanged(0);

                        checkAndSetHeaderMaxScroll();

                        if (!activeProject.getProjectStyles().isBgImageRepeat()) {
                            LinearLayout container = (LinearLayout) rootView.findViewById(R.id.bsdk_projectDetailsBackgroundImageContainer);
                            container.setPadding(0, headerContainer.getHeight(), 0, getResources().getDisplayMetrics().heightPixels);
                        }

                        //on orientation change the recycler retains the first fully visible item position and scrolls to it


                    }


                }
            });

        }
    }

    private int toolBarScrollDist;

    private void checkAndSetHeaderMaxScroll() {


        if (toolBarMaxScroll < 0) {
            Resources res = getResources();
            toolBarMaxScroll = toolBarContainer.getHeight() - res.getDimensionPixelSize(R.dimen.bsdk_status_bar_spacer);
            toolBarScrollDist = res.getDimensionPixelSize(R.dimen.bsdk_toolbar_container_height);
        }

        if (headerMaxScroll < 0) {
            Resources res = getResources();
            headerMaxScroll = headerContainer.getHeight() - (res.getDimensionPixelSize(R.dimen.bsdk_toolbar_container_height) + res.getDimensionPixelSize(R.dimen.bsdk_status_bar_spacer));
        }

    }

    @Override
    public void onGetProjectCommentsSuccess(List<BehanceSDKProjectCommentDTO> commentList, boolean moreCommentsAvailable,
                                            BehanceSDKGetProjectCommentsAsyncTaskParams params) {
        if (getActivity() != null && projectRecycler.getAdapter() != null) {
            ((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).addComments(commentList);
            if (!moreCommentsAvailable)
                ((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).setMoreToLoad(false);
        }
    }

    @Override
    public void onGetProjectCommentsFailure(Exception e, BehanceSDKGetProjectCommentsAsyncTaskParams params) {
        if (getActivity() != null) {
            Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_problem_loading_project_comments_msg, Toast.LENGTH_LONG).show();
        }
    }

    public BehanceSDKProjectDTO getActiveProject() {
        return mActiveProject;
    }

    public void setActiveProject(BehanceSDKProjectDTO mActiveProject) {
        this.mActiveProject = mActiveProject;
    }

    @Override
    public void onClick(View clickedView) {
        int i = clickedView.getId();
        if (i == R.id.bsdk_projectDetailFragmentCommentFAB) {
            revealCommentsView();

        } else if (i == R.id.bsdk_projectDetailsHeaderArtistContainer) {
            handleOwnerViewClick(clickedView);
        } else if (i == R.id.bsdk_projectDetailsFragmentPostCommentSend) {
            handlePostCommentClick();

        } else if (i == R.id.bsdkGenericAlertDialogNotOKBtn) {
            handleMatureContentGoBackButtonClick();

        } else if (i == R.id.bsdkGenericAlertDialogOKBtn) {
            handleMatureContentTurnOffButtonClick();

        } else if (i == R.id.bsdk_projectDetailsHeaderFollowButton) {
            handleFollowUnFollowUserClick();

        } else if (i == R.id.bsdk_projectDetailsToolbarTitle) {
            if (projectRecycler != null && projectRecycler.getLayoutManager() != null) {
                if (((LinearLayoutManager) projectRecycler.getLayoutManager()).findFirstVisibleItemPosition() > 6)
                    projectRecycler.scrollToPosition(6);
                projectRecycler.smoothScrollToPosition(0);
            }

        } else {
            if (clickedView.getTag() instanceof BehanceSDKProjectDTO) {
                try {
                    BehanceSDKProjectDTO projectDTO = (BehanceSDKProjectDTO) clickedView.getTag();
                    BehanceSDK behanceSDK = BehanceSDK.getInstance();
                    behanceSDK.launchProjectDetailsActivity(getActivity(), projectDTO);
                } catch (Exception e) {

                }
            } else if (clickedView.getTag() instanceof BehanceSDKUserDTO) {
                BehanceSDKUserDTO owner = (BehanceSDKUserDTO) clickedView.getTag();
                launchUserProfile(owner);
            } else if (clickedView.getTag() instanceof BehanceSDKProjectFeaturedDTO) {
                BehanceSDKProjectFeaturedDTO feature = (BehanceSDKProjectFeaturedDTO) clickedView.getTag();
                handleFeatureRibbonClick(feature);
            }
        }
    }


    private void handleOwnerViewClick(View clickedView) {
        if (mActiveProject != null && mActiveProject.getOwners() != null) {
            if (mActiveProject.getOwners().size() == 1) {
                launchUserProfile(mActiveProject.getOwners().get(0));
            } else {
                handleMultipleOwnerClick(clickedView);
            }
        }
    }

    private void launchUserProfile(BehanceSDKUserDTO userDTO) {
        if (userDTO != null) {
            String url = "http://www.behance.net/" + userDTO.getUserName() + '/';
            launchUrlIntent(url);

        }
    }

    private void launchUrlIntent(String url) {
        if (url == null)
            return;
        Intent i = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
        try {
            startActivity(i);
        } catch (ActivityNotFoundException e) {

        }
    }


    private void handleMoreInfoClick() {
        if (getActiveProject() != null) {
            DialogFragment projectDialogFragment = new BehanceSDKProjectInfoDialogFragment();
            FragmentManager fragmentManager = getActivity().getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_PROJECT_INFO_DIALOG);
            if (previousInstance != null) {
                fragmentTransaction.remove(previousInstance);
            }
            fragmentTransaction.addToBackStack(null);
            projectDialogFragment.show(fragmentTransaction, FRAGMENT_TAG_PROJECT_INFO_DIALOG);
        }
    }

    private void handleShareProjectClick() {
        if (mActiveProject != null) {
            launchProjectShareContentDialog(mActiveProject, getActivity(), FRAGMENT_TAG_PROJECT_SHARE_DIALOG);
        }
    }

    private void launchProjectShareContentDialog(BehanceSDKProjectDTO projectDTO, FragmentActivity fragmentActivity, String fragmentTag) {
        if (projectDTO != null) {
            FragmentManager fragmentManager = fragmentActivity.getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            Fragment previousInstance = fragmentManager.findFragmentByTag(fragmentTag);
            if (previousInstance != null) {
                fragmentTransaction.remove(previousInstance).commit();
                fragmentTransaction = fragmentManager.beginTransaction();
            }
            fragmentTransaction.addToBackStack(null);
            BehanceSDKShareContentDTO shareContent = new BehanceSDKShareContentDTO();
            shareContent.setLongUrlToShare(projectDTO.getProjectUrl());
            shareContent.setTitle(projectDTO.getName());
            shareContent.setOwnerName(projectDTO.getOwners().get(0).getDisplayName());
            shareContent.setTags(projectDTO.getTags());
            shareContent.setDescription(projectDTO.getDescription());
            shareContent.setId(projectDTO.getId());
            shareContent.setOwnerProfileURL(projectDTO.getOwners().get(0).getProfileUrl());
            BehanceSDKImageDTO imageDTO = projectDTO.getCovers().findCoverImageInIncreasingSizeOrderAndFallback(BehanceSDKProjectCoversDTO.SIZE_404);
            if (imageDTO != null) {
                shareContent.setCoverImageURL(imageDTO.getUrl());
            }
            shareContent.setShortUrlToShare(projectDTO.getProjectUrl());

            BehanceSDKBottomSheetShareDialog shareDialog = BehanceSDKBottomSheetShareDialog.getInstance(shareContent, BehanceSDKShareContentType.PROJECT);
            shareDialog.show(fragmentTransaction, fragmentTag);
        }
    }

    private void handleFeatureRibbonClick(BehanceSDKProjectFeaturedDTO feature) {
        if (feature != null) {
            //If user clicks Behance ribbon , do nothing
            String siteDomain = feature.getSiteDomain();
            if (siteDomain.startsWith(BehanceSDKConstants.BEHANCE_DOMAIN_NAME))
                return;
            launchUrlIntent(feature.getSiteUrl());
        }

    }


    private void handleMatureContentTurnOffButtonClick() {

        lockScreenOrientation();
        dismissMatureContentDialog();
        showProjectLoadingProgressBar();
        BehanceSDKPostUserSettingsOnServerAsyncTaskParams params = new BehanceSDKPostUserSettingsOnServerAsyncTaskParams();
        BehanceSDKUserSettingsOnServerDTO settingsDTO = new BehanceSDKUserSettingsOnServerDTO();
        settingsDTO.setSafeBrowsingOn(false);
        params.setSettings(settingsDTO);
        getProjectDetailsHeadlessFragment.setMatureContentAccessForUser(params);
    }

    private void handleMatureContentGoBackButtonClick() {
        closeProjectActivity();
    }

    private void dismissMatureContentDialog() {
        if (matureContentDialog != null && matureContentDialog.isShowing()) {
            matureContentDialog.dismiss();
        }
        if (restrictedAgeDialog != null && restrictedAgeDialog.isShowing()) {
            restrictedAgeDialog.dismiss();
        }
        matureContentBackgroundLayer.setVisibility(View.INVISIBLE);
    }

    private void handleMultipleOwnerClick(View clickedView) {
        final ListPopupWindow ownersPopup = new ListPopupWindow(getActivity());
        ownersPopup.setWidth(ListPopupWindow.WRAP_CONTENT);
        ownersPopup.setBackgroundDrawable(new ColorDrawable(getResources().getColor(R.color.bsdk_card_color)));
        ownersPopup.setAnchorView(clickedView);
        ownersPopup.setAdapter(new BehanceSDKProjectOwnersListArrayAdapter(getActivity(), R.layout.bsdk_adapter_project_detail_owner_list_item, getActiveProject().getOwners()));
        ownersPopup.setModal(true);
        ownersPopup.setVerticalOffset(-clickedView.getHeight());
        ownersPopup.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                ownersPopup.dismiss();
                launchUserProfile(getActiveProject().getOwners().get(position));
            }
        });
        ownersPopup.show();
    }

    private void handlePostCommentClick() {
        concealCommentsView();
        String commentContent = postCommentEditText.getText().toString();
        postCommentEditText.setText("");
        commentContent = commentContent.trim();
        if (commentContent.length() > 0) {
            if (mActiveProject != null && !getProjectDetailsHeadlessFragment.isPostProjectCommentTaskInProgress()) {
                getProjectDetailsHeadlessFragment.postComment(commentContent, mActiveProject.getId());
            }
        } else {
            Toast errorToast = Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_post_comment_empty_error_msg, Toast.LENGTH_SHORT);
            errorToast.show();
        }
    }


    @Override
    public void onPause() {
        super.onPause();
    }


    @Override
    public void onPostCommentSuccess(int commentId, BehanceSDKPostProjectCommentAsyncTaskParams params) {
        if (getActivity() != null) {
            if (commentId > 0 && mActiveProject != null) {

                BehanceSDKProjectCommentDTO commentDTO = new BehanceSDKProjectCommentDTO();
                commentDTO.setContent(params.getCommentContent());
                commentDTO.setUser(BehanceSDKUserManager.getInstance().getUserDTO());
                commentDTO.setCreatedDate(System.currentTimeMillis() / 1000);
                commentDTO.setId(String.valueOf(commentId));
                ((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).newCommentPosted(commentDTO);
                getProjectDetailsHeadlessFragment.setActiveProjectComments(((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).getProjectComments());

//				loadProjectComments(mActiveProject.getId(), true);
                // update comments count
                incrementAndDisplayCommentsCount();
            } else {
                if (params.isUserVerified())
                    Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_post_comment_failure_msg, Toast.LENGTH_LONG).show();
                else
                    Toast.makeText(getActivity(), R.string.bsdk_failure_msg_unverifiedUser, Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    public void onPostCommentFailure(Exception e) {
        if (getActivity() != null) {
            logger.error(e, "Problem posting comment");
            Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_post_comment_failure_msg, Toast.LENGTH_LONG).show();
        }
    }

    private void handleFollowUnFollowUserClick() {
        BehanceSDKUserManager userManager = BehanceSDKUserManager.getInstance();
        if (getActiveProject() != null && getActiveProject().getOwners() != null) {
            if (getActiveProject().getOwners().size() == 1) {
                BehanceSDKUserDTO owner = getActiveProject().getOwners().get(0);
                if (getActiveProject() != null && owner != null && !getProjectDetailsHeadlessFragment.isFollowUnFollowTaskInProgress()) {
                    if (userManager.isUserLoggedIn()) {
                        followUnfollowUser(owner);
                    } else {
                        String ownerName = getActiveProject().getOwners().get(0).getDisplayName();
                        if (getActiveProject().getOwners().size() == 2)
                            ownerName += getResources().getString(R.string.bsdk_project_detail_fragment_multiple_owners_more_one, String.valueOf(getActiveProject().getOwners().size() - 1));
                        else if (getActiveProject().getOwners().size() > 2)
                            ownerName += getResources().getString(R.string.bsdk_project_detail_fragment_multiple_owners_more, String.valueOf(getActiveProject().getOwners().size() - 1));
                        String messageText = getResources().getString(R.string.bsdk_login_to_proceed_follow_user_msg, ownerName);
                        displayLoginToProceedAlertDialog(messageText);
                    }
                }
            } else {
                if (userManager.isUserLoggedIn()) {
                    int ownersFollowedCount = 0;
                    int ownerCount = getActiveProject().getOwners().size();
                    int loggedInUserId = -1;
                    BehanceSDKUserDTO loggedInUserDTO = BehanceSDKUserManager.getInstance().getUserDTO();
                    if (loggedInUserDTO != null)
                        loggedInUserId = loggedInUserDTO.getId();
                    for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
                        if (owner.isCurrentUserFollowing())
                            ownersFollowedCount++;
                        else if (owner.getId() == loggedInUserId)
                            ownerCount -= 1;
                    }
                    if (ownersFollowedCount == ownerCount) {
                        String unFollowMessage;
                        if (ownerCount == 1) {//logged in user is one of two owners
                            if (getActiveProject().getOwners().get(0).getId() != loggedInUserId)
                                unFollowMessage = getResources().getString(R.string.bsdk_follow_user_view_unfollow_user_dialog_title, getActiveProject().getOwners().get(0).getDisplayName());
                            else
                                unFollowMessage = getResources().getString(R.string.bsdk_follow_user_view_unfollow_user_dialog_title, getActiveProject().getOwners().get(1).getDisplayName());
                        } else {
                            unFollowMessage = (getResources().getString(R.string.bsdk_follow_user_view_unfollow_multiple_users_dialog_title, String.valueOf(ownerCount)));
                        }
                        createUnFollowUserAlertDialog();
                        unFollowUserDialog.setMessage(unFollowMessage);
                        unFollowUserDialog.show();
                    } else {
                        for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
                            if (getActiveProject() != null && owner != null) {
                                if (!owner.isCurrentUserFollowing() && owner.getId() != loggedInUserId)
                                    getProjectDetailsHeadlessFragment.followUser(owner);
                            }
                        }
                        displayUnFollowUserBtn();
                    }
                } else {
                    String ownerName = getActiveProject().getOwners().get(0).getDisplayName();
                    String messageText = getResources().getString(R.string.bsdk_login_to_proceed_follow_user_msg, ownerName);
                    displayLoginToProceedAlertDialog(messageText);
                }
            }
        }
    }

    private void followUnfollowUser(BehanceSDKUserDTO owner) {
        if (owner.isCurrentUserFollowing()) {
            String unFollowMessage = getResources().getString(R.string.bsdk_follow_user_view_unfollow_user_dialog_title, owner.getDisplayName());
            createUnFollowUserAlertDialog();
            unFollowUserDialog.setMessage(unFollowMessage);
            unFollowUserDialog.show();
        } else {
            displayUnFollowUserBtn();
            getProjectDetailsHeadlessFragment.followUser(owner);
        }
    }

    private void handleUnFollowUserDialogOkClick() {
        displayFollowUserBtn();
        for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
            if (owner.isCurrentUserFollowing())
                getProjectDetailsHeadlessFragment.unFollowUser(owner);
        }
    }


    private void displayFollowUserBtn() {
        followButton.setText(getActiveProject().getOwners().size() == 1 ? R.string.bsdk_follow : R.string.bsdk_follow_all);
    }

    private void displayUnFollowUserBtn() {
        followButton.setText(R.string.bsdk_unfollow);
    }


    private void createUnFollowUserAlertDialog() {
        if (unFollowUserDialog == null) {
            AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), R.style.BsdkDialogTheme);
            builder.setPositiveButton(R.string.bsdk_follow_user_view_unfollow_user_dialog_ok_btn_label, new DialogInterface.OnClickListener() {

                @Override
                public void onClick(DialogInterface dialog, int which) {
                    handleUnFollowUserDialogOkClick();
                }
            });
            builder.setNegativeButton(null, null);
            unFollowUserDialog = builder.create();
        }
    }

    @Override
    public void onFollowUnFollowUserSuccess(boolean result, BehanceSDKFollowUnFollowUserAsyncTaskParams params) {
        if (getActivity() != null) {
            for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
                if (owner != null && owner.getId() == params.getUserDTO().getId()) {
                    String displayString = "";
                    if (result) {
                        if (params.isFollowUser()) {
                            // changing the status for instance in memory
                            owner.setCurrentUserFollowing(true);
                            displayUnFollowUserBtn();
                        } else {
                            owner.setCurrentUserFollowing(false);
                            displayFollowUserBtn();
                        }
                    } else {
                        if (params.isFollowUser()) {
                            displayFollowUserBtn();
                            displayString = getResources().getString(R.string.bsdk_follow_user_view_follow_user_failure_msg, owner.getDisplayName());
                        } else {
                            displayUnFollowUserBtn();
                            displayString = getResources().getString(R.string.bsdk_follow_user_view_unfollow_user_failure_msg, owner.getDisplayName());
                        }
                        Toast.makeText(getActivity(), displayString, Toast.LENGTH_LONG).show();
                    }
                }
            }
        }
    }

    @Override
    public void onFollowUnFollowUserFailure(Exception e, BehanceSDKFollowUnFollowUserAsyncTaskParams params) {
        logger.error(e, "Problem following user [User id - %d]", params.getUserDTO().getId());
        if (getActivity() != null) {
            for (BehanceSDKUserDTO owner : getActiveProject().getOwners()) {
                if (owner != null && owner.getId() == params.getUserDTO().getId()) {
                    String displayErrorString;
                    if (params.isFollowUser()) {
                        displayFollowUserBtn();
                        displayErrorString = getResources().getString(R.string.bsdk_follow_user_view_follow_user_failure_msg, owner.getDisplayName());
                    } else {
                        displayUnFollowUserBtn();
                        displayErrorString = getResources().getString(R.string.bsdk_follow_user_view_unfollow_user_failure_msg, owner.getDisplayName());
                    }
                    Toast.makeText(getActivity(), displayErrorString, Toast.LENGTH_LONG).show();
                }
            }
        }
    }

    private void closeProjectActivity() {
        if (getActivity() != null) {
            getActivity().finish();
        }
    }

    @Override
    public void onPostSettingsTaskSuccess(boolean result, BehanceSDKPostUserSettingsOnServerAsyncTaskParams params) {
        if (getActivity() != null) {
            if (!result) {
                logger.error("Problem saving User settings on server");
                Toast.makeText(getActivity(), R.string.bsdk_app_settings_dialog_save_user_settings_error_msg, Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_successfully_stored_user_settings, Toast.LENGTH_LONG).show();
                hideProjectLoadingProgressBar();
                unlockScreenOrientation();
            }
        }
    }
/*
    private void setUpCustomAppTheme() {
        int color = getResources().getColor(options.getAppColor());
        followButton.setTextColor(color);
        followButton.getBackground().setColorFilter(new PorterDuffColorFilter(color, PorterDuff.Mode.SRC_ATOP));
        newCommentButton.setBackgroundTintList(ColorStateList.valueOf(color));
        postCommentBtn.getDrawable().setColorFilter(new PorterDuffColorFilter(color, PorterDuff.Mode.SRC_ATOP));
        ProgressBar progressBar = (ProgressBar) rootView.findViewById(R.id.bsdk_project_detail_progressBar);
        progressBar.getIndeterminateDrawable().setColorFilter(new PorterDuffColorFilter(color, PorterDuff.Mode.SRC_ATOP));
    }
*/
    @Override
    public void onPostSettingsTaskFailure(Exception e) {
        if (getActivity() != null) {
            logger.error("Problem saving User settings on server");
            Toast.makeText(getActivity(), R.string.bsdk_app_settings_dialog_save_user_settings_error_msg, Toast.LENGTH_LONG).show();
            unlockScreenOrientation();
            closeProjectActivity();
        }
    }

    @Override
    public void onAppreciateProjectAsyncTaskSuccess(boolean result) {
        NumberFormat formatter = new DecimalFormat("###,###,###,###");
        ((TextView) rootView.findViewById(R.id.bsdk_projectDetailsHeaderStatsAppreciations)).setText(String.valueOf(formatter.format((double) (getActiveProject().getStats().getAppreciationsCount()) + 1)));
    }

    @Override
    public void onAppreciateProjectAsyncTaskFailure(Exception e) {

    }

    private void unlockScreenOrientation() {
        if (getActivity() != null) {
            boolean isBigScreen = getResources().getBoolean(R.bool.bsdk_big_screen);
            if (isBigScreen) {
                getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED);
            }
        }
    }

    private void lockScreenOrientation() {
        if (getActivity() != null) {
            boolean isBigScreen = getResources().getBoolean(R.bool.bsdk_big_screen);
            if (isBigScreen) {
                int height;
                int width;
                Display display = getActivity().getWindowManager().getDefaultDisplay();
                int rotation = display.getRotation();
                Point size = getDisplaySize();
                height = size.y;
                width = size.x;
                switch (rotation) {
                    case Surface.ROTATION_90:
                        if (width > height)
                            getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                        else
                            getActivity().setRequestedOrientation(9/* reversePortait */);
                        break;
                    case Surface.ROTATION_180:
                        if (height > width)
                            getActivity().setRequestedOrientation(9/* reversePortait */);
                        else
                            getActivity().setRequestedOrientation(8/* reverseLandscape */);
                        break;
                    case Surface.ROTATION_270:
                        if (width > height)
                            getActivity().setRequestedOrientation(8/* reverseLandscape */);
                        else
                            getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                        break;
                    default:
                        if (height > width)
                            getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                        else
                            getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                }
            }
        }
    }

    @SuppressWarnings("deprecation")
    private Point getDisplaySize() {
        Point size = new Point();
        Display display = getActivity().getWindowManager().getDefaultDisplay();
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB_MR2) {
            size.y = display.getHeight();
            size.x = display.getWidth();
        } else {
            display.getSize(size);
        }
        return size;
    }

    //

    //@Override
    protected void onNetworkConnectivityAvailable() {
        //	super.onNetworkConnectivityAvailable();
        loadProjectDetails(projectId);
    }

    private void displayLoginToProceedAlertDialog(String messageText) {
        loginToProceedDialog = BehanceSDKLoginToProceedUserDialog.getLoginToProceedDialogInstance(getActivity(), messageText);
        loginToProceedDialog.show();
    }

    private void dismissLoginToProceedAlertDialog() {
        if (loginToProceedDialog != null && loginToProceedDialog.isShowing()) {
            loginToProceedDialog.dismiss();
        }
    }

    @Override
    public void onDeleteCommentRequested(BehanceSDKAbstractCommentDTO commentDTO) {
        if (commentDTO != null && mActiveProject != null) {
            // remove comment from local copy
            List<BehanceSDKProjectCommentDTO> projectCommentsList = getProjectDetailsHeadlessFragment.getActiveProjectCommentsList();
            if (projectCommentsList != null && !projectCommentsList.isEmpty()) {
                projectCommentsList.remove(commentDTO);
            }

            // decrement comment count
            decrementAndDisplayCommentsCount();

            getProjectDetailsHeadlessFragment.deleteComment(mActiveProject.getId(), commentDTO.getId());
        }
    }

    @Override
    public void onProjectAppreciated() {
        handleAppreciateButtonClick();


    }

    private void handleAppreciateButtonClick() {
        if (getProjectDetailsHeadlessFragment != null) {
            getProjectDetailsHeadlessFragment.appreciateProject(mActiveProject.getId());
        }
    }


    @Override
    public void onProjectInfoClicked() {
        handleMoreInfoClick();

    }

    @Override
    public void onProjectShareClick() {
        handleShareProjectClick();

    }

    @Override
    public void onImageClicked(int pos) {
        DialogFragment imageDisplayFragment = BehanceSDKImageDisplayDialogFragment.getInstance(getProjectDetailsHeadlessFragment.getImageURLsArray(), pos);
        FragmentManager fragmentManager = getActivity().getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG);
        if (previousInstance != null) {
            fragmentTransaction.remove(previousInstance);
        }
        fragmentTransaction.addToBackStack(null);
        imageDisplayFragment.show(fragmentTransaction, FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG);
    }

    @Override
    public void onDeleteCommentSuccess(boolean result, BehanceSDKDeleteProjectCommentAsyncTaskParams params) {
        if (getActivity() != null) {
            if (result && mActiveProject != null) {
                getProjectDetailsHeadlessFragment.setActiveProjectComments(((BehanceSDKProjectDetailRecyclerAdapter) projectRecycler.getAdapter()).getProjectComments());
            } else {
                Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_delete_comment_failure_msg, Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    public void onDeleteCommentFailure(Exception e, BehanceSDKDeleteProjectCommentAsyncTaskParams params) {
        if (getActivity() != null) {
            logger.error(e, "Problem deleting comment");
            Toast.makeText(getActivity(), R.string.bsdk_project_detail_fragment_delete_comment_failure_msg, Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void loadNextPage() {
        loadProjectCommentsNextPageFromServer();
    }
}
