/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.fragments;

/**
 * @author Sujit Reddy G
 *
 */

import android.content.Context;
import android.content.res.Resources;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import android.widget.Toast;

import com.behance.sdk.BehanceSDKHeadlessFragmentTags;
import com.behance.sdk.R;

import com.behance.sdk.asynctask.params.BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectsAsyncTaskParams;
import com.behance.sdk.datamanager.BehanceSDKProjectsDataManager;

import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.search.BehanceSDKProjectFiltersSelected;
import com.behance.sdk.enums.BehanceSDKCuratedGalleryType;
import com.behance.sdk.fragments.headless.BehanceSDKGetProjectsHeadlessFragment;
import com.behance.sdk.listeners.IBehanceSDKEndlessScrollRecyclerViewListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.adapters.BehanceSDKProjectRecyclerViewAdapter;
import com.behance.sdk.ui.components.BehanceSDKEndlessScrollRecyclerView;
import com.behance.sdk.ui.customviews.BehanceSDKEmptyStatesView;
import com.behance.sdk.ui.customviews.BehanceSDKSpaceItemDecorationGrid;
import com.behance.sdk.util.BehanceSDKColumnCountUtil;
import com.behance.sdk.util.BehanceSDKUtils;

import java.util.List;


public class BehanceSDKProjectsFeedFragment extends Fragment implements BehanceSDKGetProjectsHeadlessFragment.Callbacks,
		SwipeRefreshLayout.OnRefreshListener, IBehanceSDKEndlessScrollRecyclerViewListener {

	public static final String DISCOVER_PROJECTS_KEY_SELECTED_FILTERS = "DISCOVER_PROJECTS_KEY_SELECTED_FILTERS";
	public static final String PROJECT_FEED_GALLERY_TYPE = "PROJECT_FEED_GALLERY_TYPE";
	public static final String PROJECT_FEED_GALLERY_ID = "PROJECT_FEED_GALLERY_ID";
	public static final String PROJECT_FEED_TYPE = "PROJECT_FEED_TYPE";



	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKProjectsFeedFragment.class);

	private static final String STATE_LOADED_PROJECTS_PAGE_NUMBER = "STATE_LOADED_PAGE_NUMBER";

	private int lastLoadedProjectsPageNumber = 0;
	private View rootView;
	private int galleryID;
	private BehanceSDKCuratedGalleryType galleryType;
	private BehanceSDKProjectsDataManager projectsDataManager;
	private BehanceSDKEndlessScrollRecyclerView projectsRecyclerView;
	private SwipeRefreshLayout projectsSwipeRefresh;
	private BehanceSDKProjectFiltersSelected selectedFilters;


	private BehanceSDKGetProjectsHeadlessFragment getProjectsHeadlessFragment;



	private boolean projectsLoadingInProgress;
	private boolean isProjectTypeGallery = false;

	protected BehanceSDKEmptyStatesView emptyStatesView;


	/**
	 * Mandatory empty constructor for the fragment manager to instantiate the
	 * fragment (e.g. upon screen orientation changes).
	 */
	public BehanceSDKProjectsFeedFragment() {
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		super.onCreateView(inflater, container, savedInstanceState);
		rootView = inflater.inflate(R.layout.bsdk_fragment_projects_feed, container, false);

		projectsDataManager = BehanceSDKProjectsDataManager.getInstance();
		getProjectsHeadlessFragment = (BehanceSDKGetProjectsHeadlessFragment) getActivity().getSupportFragmentManager().findFragmentByTag(
				getHeadlessFragmentTag());
		if (getProjectsHeadlessFragment == null) {
			getProjectsHeadlessFragment = getHeadlessFragmentNewInstance();
			getActivity().getSupportFragmentManager().beginTransaction().add(getProjectsHeadlessFragment, getHeadlessFragmentTag()).commit();
		}
		getProjectsHeadlessFragment.setCallbacks(this);

		emptyStatesView = (BehanceSDKEmptyStatesView) rootView.findViewById(R.id.bsdk_discoverProjectsEmptyStatesView);

		Bundle arguments = savedInstanceState;
		if (arguments == null) {
			arguments = getArguments();
		}

	    selectedFilters = new BehanceSDKProjectFiltersSelected();
		if (arguments != null) {
			lastLoadedProjectsPageNumber = arguments.getInt(STATE_LOADED_PROJECTS_PAGE_NUMBER, 0);
			isProjectTypeGallery = arguments.getBoolean(PROJECT_FEED_TYPE,false);
			if(isProjectTypeGallery) {
				galleryType = (BehanceSDKCuratedGalleryType) arguments.get(PROJECT_FEED_GALLERY_TYPE);
				galleryID = arguments.getInt(PROJECT_FEED_GALLERY_ID);
			}else{
				BehanceSDKProjectFiltersSelected filtersSelected = (BehanceSDKProjectFiltersSelected) arguments.getSerializable(DISCOVER_PROJECTS_KEY_SELECTED_FILTERS);
				if(null != filtersSelected)
					selectedFilters = filtersSelected;
			}
		}

		Resources res = getResources();

		projectsSwipeRefresh = (SwipeRefreshLayout) rootView.findViewById(R.id.bsdk_discoverProjectsSwipeRefresh);
		projectsSwipeRefresh.setOnRefreshListener(this);
		int swipeOffset = res.getDimensionPixelSize(R.dimen.bsdk_toolbar_height) + res.getDimensionPixelSize(R.dimen.bsdk_status_bar_spacer) + res.getDimensionPixelSize(R.dimen.bsdk_card_view_vertical_spacing);
		projectsSwipeRefresh.setProgressViewOffset(false, -swipeOffset, swipeOffset);
		projectsSwipeRefresh.setColorSchemeResources(R.color.bsdk_behance_blue);

		projectsRecyclerView = (BehanceSDKEndlessScrollRecyclerView) rootView.findViewById(R.id.bsdk_discoverProjectsRecycler);
		final int columnCount = BehanceSDKColumnCountUtil.getDiscoverProjectColumnCount(getActivity());
		GridLayoutManager layoutManager = new GridLayoutManager(getActivity(), columnCount, GridLayoutManager.VERTICAL, false);
		layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
			@Override
			public int getSpanSize(int position) {
				if (projectsRecyclerView.getAdapter() != null && projectsRecyclerView.getAdapter() instanceof BehanceSDKProjectRecyclerViewAdapter) {
					BehanceSDKProjectRecyclerViewAdapter adapter = (BehanceSDKProjectRecyclerViewAdapter) projectsRecyclerView.getAdapter();
					switch (adapter.getItemViewType(position)) {
						case 0:
							return 1;
						case 1:
							return columnCount;
					}
				}
				return 1;
			}
		});
		projectsRecyclerView.setLayoutManager(layoutManager);
		projectsRecyclerView.addItemDecoration(new BehanceSDKSpaceItemDecorationGrid(getActivity()));
		projectsRecyclerView.setCallbackListener(this);
		projectsRecyclerView.initializeScrollListener(layoutManager);
		projectsRecyclerView.setPadding(projectsRecyclerView.getPaddingLeft(), projectsRecyclerView.getPaddingTop(), projectsRecyclerView.getPaddingRight(), projectsRecyclerView.getPaddingBottom() + BehanceSDKUtils.getTranslucentNavHeight(getActivity()));

		if (getProjectsHeadlessFragment.isGetProjectsAsyncTaskInProgress()) {
			showProgressBar();
			setProjectsRecyclerAdapter(false);
		} else {
			loadProjects();
		}


		return rootView;
	}

	@Override
	public void onResume() {
		super.onResume();
	}

	@Override
	public void onPause() {
		super.onPause();
	}

	public String getHeadlessFragmentTag() {
		return BehanceSDKHeadlessFragmentTags.GET_PROJECTS;
	}

	protected BehanceSDKGetProjectsHeadlessFragment getHeadlessFragment() {
		return getProjectsHeadlessFragment;
	}

	protected BehanceSDKGetProjectsHeadlessFragment getHeadlessFragmentNewInstance() {
		return new BehanceSDKGetProjectsHeadlessFragment();
	}

	@Override
	public void onStop() {
		super.onStop();
		projectsSwipeRefresh.setRefreshing(false);
	}

	protected void loadProjects() {
		if (getProjectsList().size() <= 0) {
			loadProjectsFromServer();
		} else {
			setProjectsRecyclerAdapter(false);
			hideProgressBar(true);
		}
	}

	protected List<BehanceSDKProjectDTO> getProjectsList() {
		return projectsDataManager.getProjects();
	}

	protected boolean isDeviceOnline() {
		if(getActivity() != null) {
			ConnectivityManager cm = (ConnectivityManager) getActivity().getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo netInfo = cm.getActiveNetworkInfo();
			if (netInfo != null && netInfo.isConnected()) {
				return true;
			}
		}
		return false;
	}

	protected void loadProjectsFromServer() {
		if(isProjectTypeGallery)
			loadGalleriesProjectsFromServer(false);
		else{
			loadDiscoverProjectsFromServer(false);
		}
	}

	private void loadDiscoverProjectsFromServer(boolean ignoreCurrentInProgressLoading) {
		if (isDeviceOnline() && (!projectsLoadingInProgress || ignoreCurrentInProgressLoading)) {
			lastLoadedProjectsPageNumber = 1;
			BehanceSDKGetProjectsAsyncTaskParams params = getAsyncTaskParams();
			setFilters(params);
			getProjectsHeadlessFragment.loadProjectsFromServer(params);
			showProgressBar();
		} else {
			hideProgressBar(false);
		}
	}

	private void loadGalleriesProjectsFromServer(boolean ignoreCurrentInProgressLoading) {
		if (isDeviceOnline() && (!projectsLoadingInProgress || ignoreCurrentInProgressLoading)) {
			lastLoadedProjectsPageNumber = 1;
			BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params = getGalleriesAsyncTaskParams();
			params.setCuratedGalleryId(galleryID);
			params.setGalleryType(galleryType);
			getProjectsHeadlessFragment.loadGalleryProjectsFromServer(params);
			showProgressBar();
		} else {
			hideProgressBar(false);
		}
	}


	protected BehanceSDKGetProjectsAsyncTaskParams getAsyncTaskParams() {
		BehanceSDKGetProjectsAsyncTaskParams params = new BehanceSDKGetProjectsAsyncTaskParams();
		params.setPageNumber(lastLoadedProjectsPageNumber);
		params.setPageSize(20);
		return params;
	}

	protected BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams getGalleriesAsyncTaskParams() {
		BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params = new BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams();
		params.setPageNumber(lastLoadedProjectsPageNumber);
		params.setPageSize(20);
		return params;
	}

	private void setFilters(BehanceSDKGetProjectsAsyncTaskParams asyncTaskParams) {
		asyncTaskParams.setSortOption(selectedFilters.getSortOption());
		BehanceSDKCreativeFieldDTO creativeFieldDTO = selectedFilters.getCreativeFieldDTO();
		if (null != creativeFieldDTO && !BehanceSDKCreativeFieldDTO.ALL_CREATIVE_FIELDS_ID.equals(creativeFieldDTO.getId())) {
			asyncTaskParams.setField(creativeFieldDTO.getName());
		}
		asyncTaskParams.setSearchString(selectedFilters.getSearchString());
		setLocationFilters(asyncTaskParams);
	}

	private void setLocationFilters(BehanceSDKGetProjectsAsyncTaskParams asyncTaskParams) {
		BehanceSDKCountryDTO countryDTO = selectedFilters.getCountryDTO();
		if (null != countryDTO && !BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID.equals(countryDTO.getId())) {
			asyncTaskParams.setCountry(countryDTO.getId());
			BehanceSDKStateDTO stateDTO = selectedFilters.getStateDTO();
			if (null != stateDTO && !BehanceSDKStateDTO.ALL_STATES_ID.equals(stateDTO.getId()))
				asyncTaskParams.setState(stateDTO.getDisplayName());
			BehanceSDKCityDTO cityDTO = selectedFilters.getCityDTO();
			if (null != cityDTO && !BehanceSDKCityDTO.ALL_CITIES_ID.equals(cityDTO.getId()))
				asyncTaskParams.setCity(cityDTO.getDisplayName());
		}
	}

	private void loadNextPageProjectsFromServer() {
		if (!projectsLoadingInProgress) {
			lastLoadedProjectsPageNumber++;
			BehanceSDKGetProjectsAsyncTaskParams params = getAsyncTaskParams();
			getProjectsHeadlessFragment.loadProjectsFromServer(params);
//			showProgressBar();
		}
	}

	private void loadNextPageGalleryProjectsFromServer() {
		if (!projectsLoadingInProgress) {
			lastLoadedProjectsPageNumber++;
			BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params = getGalleriesAsyncTaskParams();
			getProjectsHeadlessFragment.loadGalleryProjectsFromServer(params);
//			showProgressBar();
		}
	}

	@Override
	public void onViewCreated(View view, Bundle savedInstanceState) {
		super.onViewCreated(view, savedInstanceState);
	}

	@Override
	public void onDetach() {
		super.onDetach();
		if (getProjectsHeadlessFragment != null) {
			getProjectsHeadlessFragment.setCallbacks(null);
		}
	}

	private void showProgressBar() {
		projectsLoadingInProgress = true;
		projectsSwipeRefresh.setRefreshing(true);
	}

	private void hideProgressBar(boolean loadingSuccessful) {
		projectsLoadingInProgress = false;
		projectsSwipeRefresh.setRefreshing(false);
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		Bundle bundleToSave = getBundleToSave();
		if (null != bundleToSave)
			outState.putAll(bundleToSave);
	}

	public Bundle getBundleToSave() {
		Bundle arguments = new Bundle();
		arguments.putInt(STATE_LOADED_PROJECTS_PAGE_NUMBER, lastLoadedProjectsPageNumber);
		if(isProjectTypeGallery)
		{
			arguments.putInt(PROJECT_FEED_GALLERY_ID,galleryID);
			arguments.putSerializable(PROJECT_FEED_GALLERY_TYPE, galleryType);
		}
		arguments.putSerializable(DISCOVER_PROJECTS_KEY_SELECTED_FILTERS, selectedFilters);
		return arguments;
	}

	@Override
	public void onGetProjectsSuccess(List<BehanceSDKProjectDTO> result,BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params) {
		setProjectsRecyclerAdapter(params.getPageNumber() == 1);

		if (result != null && !result.isEmpty()) {
			hideProgressBar(true);
			emptyStatesView.setVisibility(View.GONE);
		} else {
			RecyclerView.Adapter adapter = projectsRecyclerView.getAdapter();
			if(adapter instanceof BehanceSDKProjectRecyclerViewAdapter)
				((BehanceSDKProjectRecyclerViewAdapter)adapter).setMoreToLoad(false);
			if (params.getPageNumber() == 1) {
				emptyStatesView.setVisibility(View.VISIBLE);
			}
			hideProgressBar(false);
		}
	}

	@Override
	public void onGetProjectsFailure(Exception e, BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams params) {
		logger.error(e, "Problem getting Projects from server");
		hideProgressBar(false);
		if (getActivity() != null) {
			Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_collection_details_view_loading_projects_error_msg), Toast.LENGTH_LONG).show();
		}
	}

	@Override
	public void onGetProjectsSuccess(List<BehanceSDKProjectDTO> result,BehanceSDKGetProjectsAsyncTaskParams params) {
		setProjectsRecyclerAdapter(params.getPageNumber() == 1);

		if (result != null && !result.isEmpty()) {
			hideProgressBar(true);
			emptyStatesView.setVisibility(View.GONE);
		} else {
			RecyclerView.Adapter adapter = projectsRecyclerView.getAdapter();
			if(adapter instanceof BehanceSDKProjectRecyclerViewAdapter)
				((BehanceSDKProjectRecyclerViewAdapter)adapter).setMoreToLoad(false);
			if (params.getPageNumber() == 1) {
				emptyStatesView.setVisibility(View.VISIBLE);
			}
			hideProgressBar(false);
		}
	}

	@Override
	public void onGetProjectsFailure(Exception e, BehanceSDKGetProjectsAsyncTaskParams params) {
		logger.error(e, "Problem getting Projects from server");
		hideProgressBar(false);
		if (getActivity() != null) {
			Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_collection_details_view_loading_projects_error_msg), Toast.LENGTH_LONG).show();
		}
	}


	private void setProjectsRecyclerAdapter(boolean forceCreateAdapter) {
		if (getActivity() != null) {
			List<BehanceSDKProjectDTO> projects = getProjectsList();
			if (projects != null) {
				if(forceCreateAdapter) {
					BehanceSDKProjectRecyclerViewAdapter adapter = new BehanceSDKProjectRecyclerViewAdapter(getActivity(), projects);
					if(projectsRecyclerView.getAdapter() != null)
						projectsRecyclerView.swapAdapter(adapter, false);
					else
						projectsRecyclerView.setAdapter(adapter);
					projectsRecyclerView.resetScrollCount();
				} else {
					RecyclerView.Adapter adapter = projectsRecyclerView.getAdapter();
					if (adapter instanceof BehanceSDKProjectRecyclerViewAdapter) {
						BehanceSDKProjectRecyclerViewAdapter projectsAdapter = (BehanceSDKProjectRecyclerViewAdapter) adapter;
						projectsAdapter.setProjects(projects);
					} else {
						projectsRecyclerView.setAdapter(new BehanceSDKProjectRecyclerViewAdapter(getActivity(), projects));
					}
				}
			}
		}
	}

	@Override
	public void onRefresh() {
		loadProjectsFromServer();
	}

	@Override
	public void loadNextPage() {
		if(isProjectTypeGallery)
			loadNextPageGalleryProjectsFromServer();
		else
			loadNextPageProjectsFromServer();
	}


}