/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.listeners.IBehanceSDKPublishProjectNavigationListener;

/**
 * @author ugs
 * 
 */
public abstract class BehanceSDKPublishProjectBaseFragment extends Fragment {

	protected TextView actionBarForwardButton;
	
	private View actionBarBackButton;
	
	private TextView actionBarTitle;

	private IBehanceSDKPublishProjectNavigationListener navigationListener;
	
	protected View rootView;
	
	protected abstract int getLayout();
	
	protected abstract int getTitle();
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		setHasOptionsMenu(true);
		rootView = inflater.inflate(getLayout(), container, false);
		createActionBar();
		return rootView;
	}

	protected void hideSoftKeyBoardIfActive(View view) {
		InputMethodManager imm = (InputMethodManager)getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
		imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
	}	
	
	protected void createActionBar() {
			updateNavigationControls();
	}
	
	protected void updateNavigationControls() {
		actionBarTitle = (TextView) rootView.findViewById(getTitleViewID());
		actionBarTitle.setText(getTitle());
		actionBarBackButton = rootView.findViewById(getBackButtonID());
		actionBarBackButton.setVisibility(View.VISIBLE);
		actionBarBackButton.setOnClickListener(getLeftNavigationHandler());
		actionBarForwardButton = (TextView) rootView.findViewById(getForwardButtonID());
		actionBarForwardButton.setText(getForwardButtonText());
		actionBarForwardButton.setOnClickListener(getRightNavigationHandler());
	}
	
	protected void backButtonPressed(){
		navigationListener.showPrevious();
	}
	
	protected void nextButtonPressed(){
		navigationListener.showNext();
	}
	
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		navigationListener = (IBehanceSDKPublishProjectNavigationListener) activity;
	}
	
	@Override
	public void onDetach() {
		super.onDetach();
		navigationListener = null;
	}

	private OnClickListener getRightNavigationHandler() {
		return new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				nextButtonPressed();
			}
		};
	}

	private OnClickListener getLeftNavigationHandler() {
		return new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				backButtonPressed();
			}
		};
	}
	
	public boolean closeWorkflow(){
		return false;
	}
	
	
	protected int getForwardButtonText(){
		return R.string.bsdk_addproject_custom_actionbar_forward_nav_default_txt;
	}
	
	protected int getBackButtonID() {
		return R.id.bsdkPublishProjectTitlebarBackBtnLayout;
	}

	protected int getForwardButtonID() {
		return R.id.bsdkPublishProjectTitlebarActionBtnTxtView;
	}

	protected int getTitleViewID() {
		return R.id.bsdkPublishProjectTitlebarTitleTxtView;
	}


	protected void changeTitle(int title) {
		if (null != actionBarTitle)
			actionBarTitle.setText(title);
	}

	protected void changeTitle(String title) {
		if (null != actionBarTitle)
			actionBarTitle.setText(title);
	}
	protected void disableActionBarRightNav() {
		if (null != actionBarForwardButton)
			actionBarForwardButton.setEnabled(false);
	}

	protected void enableActionBarRightNav() {
		if (null != actionBarForwardButton){
			actionBarForwardButton.setEnabled(true);
		}
	}
	
//	public void addFragment(Fragment fragment, String fragmnetTag, int fragmentID, boolean addtoBackStack){
//		FragmentManager fragmentManager = getFragmentManager();
//		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//		fragmentTransaction.setCustomAnimations(R.anim.bsdk_slide_in_from_right, R.anim.bsdk_slide_out_to_left,R.anim.bsdk_slide_in_from_left, R.anim.bsdk_slide_out_to_right );
//		fragmentTransaction.replace(fragmentID, fragment, fragmnetTag);
//		if(addtoBackStack)
//			fragmentTransaction.addToBackStack(null);
//		fragmentTransaction.commit();
//	}
//	
//	public void addFragment(BehanceSDKPublishProjectBaseFragment fragment, int fragmentID, boolean addtoBackStack){
//		FragmentManager fragmentManager = getFragmentManager();
//		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//		fragmentTransaction.replace(fragmentID, fragment, fragment.getFragmentTag());
//		if(addtoBackStack)
//			fragmentTransaction.addToBackStack(null);
//		fragmentTransaction.commit();
//	}
//	
//	
//	public void addFragment(BehanceSDKPublishProjectBaseFragment fragment, int fragmentID, int enterAnimation, int exitAnimation){
//		FragmentManager fragmentManager = getFragmentManager();
//		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//		fragmentTransaction.setCustomAnimations(enterAnimation, exitAnimation, enterAnimation, exitAnimation);
//		fragmentTransaction.replace(fragmentID, fragment, fragment.getFragmentTag());
//		fragmentTransaction.addToBackStack(null);
//		fragmentTransaction.commit();
//	}
//
//
//    public void addFragment(BehanceSDKPublishProjectBaseFragment fragment, int fragmentID, int enterAnimation, int exitAnimation, int reEnterAnimation, int closeAnimation){
//        FragmentManager fragmentManager = getFragmentManager();
//        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//        fragmentTransaction.setCustomAnimations(enterAnimation, exitAnimation, reEnterAnimation, closeAnimation);
//        fragmentTransaction.replace(fragmentID, fragment, fragment.getFragmentTag());
//        fragmentTransaction.addToBackStack(null);
//        fragmentTransaction.commit();
//    }
//	
//	
//	
//	public void addFragmentAfterPopBackStack(BehanceSDKPublishProjectBaseFragment fragment, int fragmentID, boolean addtoBackStack){
//		FragmentManager fragmentManager = getFragmentManager();
//		fragmentManager.popBackStack();
//		FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//		fragmentTransaction.replace(fragmentID, fragment, fragment.getFragmentTag());
//		if(addtoBackStack)
//			fragmentTransaction.addToBackStack(null);
//		fragmentTransaction.commit();
//	}
//	
//	public void removeFragmentFromBackStack(Fragment fragment){
//		FragmentManager fragmentManager = getFragmentManager();
//			FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
//			fragmentTransaction.remove(fragment);
//			fragmentTransaction.commit();
//			fragmentManager.popBackStack();			
//	}
//	
//	protected void showNextScreen() {
////		BehanceSDKPublishProjectBaseFragment nextFragmentObject = getNextFragmentObject();
////		if (null != nextFragmentObject) {
////			String nextFragmentTag = nextFragmentObject.getFragmentTag();
////			addFragment(nextFragmentObject, nextFragmentTag, R.id.grid_frag_view, true);
////		}
//	}
	
}
