/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.R;
import com.behance.sdk.edmodo.cropper.CropImageView;
import com.behance.sdk.exception.NullReferenceException;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager.ProjectModuleChangeListener;
import com.behance.sdk.project.modules.CoverImage;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.ui.activities.BehanceSDKPublishProjectActivity;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.behance.sdk.util.BehanceSDKLaunchUtil;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener;

import java.util.List;

/**
 * @author ugs
 * 
 */
public class BehanceSDKPublishProjectCoverFragment extends BehanceSDKPublishProjectBaseFragment implements OnClickListener 
													,BehanceSDKImageSelectorDialogFragment.Callbacks, ProjectModuleChangeListener{

	private static final String ADD_PROJECT_COVER_IMAGE_KEY = "ADD_PROJECT_COVER_IMAGE_KEY";

	private static final String ADD_PROJECT_COVER_IMAGE_SELECTION_STATE = "ADD_PROJECT_COVER_IMAGE_SELECTION_STATE";
	
	private static final String ADD_PROJECT_CROPPED_COVER_IMAGE_KEY = "ADD_PROJECT_CROPPED_COVER_IMAGE_KEY";
	
	private static final String ADD_PROJECT_CROPPED_COVER_IMAGE_ORIGINAL_KEY = "ADD_PROJECT_CROPPED_COVER_IMAGE_ORIGINAL_KEY";
	
	private static final int FIRST_IMAGE = 0;

	private ImageModule coverImage;
	
	private CoverImage cover;

	private List<ImageModule> coverImageSelection;

	private LinearLayout filmStripLayout;

	private BehanceSDKProjectPublishWFManager addProjectManager =  BehanceSDKProjectPublishWFManager.getInstance();

	private OnClickListener listener;
	
	private int position;

	private CropImageView cropImageView;
	
	private CropMode state;

	private ImageView croppedImage;

	private View coverImageBackground;

	private View progressBar;

	
	public enum CropMode {
		INPROGRESS, DONE
	}
	
	@Override
	protected int getLayout() {
		return R.layout.bsdk_dialog_fragment_publish_project_select_cover;
	}
	
	@Override
	protected int getTitle() {
		return R.string.bsdk_add_content_project_cover_selection_title;
	}
	
	@Override
	protected void backButtonPressed() {
		if(null == cover){
			cover = new CoverImage(coverImage, null);
		}
		addProjectManager.setSelectedCoverImage(cover);
		super.backButtonPressed();
	}
	
	@Override
	protected void nextButtonPressed() {
		if(state == CropMode.DONE){
			addProjectManager.setSelectedCoverImage(cover);
			super.nextButtonPressed();
		}else if(state == CropMode.INPROGRESS){
			state = CropMode.DONE;
			actionBarForwardButton.setText(getResources().getString(R.string.bsdk_addproject_custom_actionbar_forward_nav_default_txt));
			Bitmap croppedImage = null;
			try {
				croppedImage = cropImageView.getCroppedImage();
				cover = new CoverImage(coverImage, croppedImage);
				coverImage = null;
				updateSelectedCoverImage();
			} catch (NullReferenceException e) {
				e.printStackTrace();
				Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_image_cropper_crop_failed_msg), Toast.LENGTH_SHORT).show();
			}
		}
	}

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		addProjectManager.addProjectModuleChangeListener(this);
	}

	@Override
	public void onDetach() {
		super.onDetach();
		addProjectManager.removeProjectModuleChangeListener(this);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		super.onCreateView(inflater, container, savedInstanceState);
		hideSoftKeyBoardIfActive(rootView);
		coverImageBackground = rootView.findViewById(R.id.add_project_cover_selection_fragment_cover_image_layout);
		cropImageView = (CropImageView) rootView.findViewById(R.id.add_project_cover_selection_fragment_cover_image);
		croppedImage = (ImageView) rootView.findViewById(R.id.add_project_cover_selection_fragment_cropped_cover_Image);
		progressBar = rootView.findViewById(R.id.bsdkPublishProjectCoverPrgressBar);
		filmStripLayout = (LinearLayout) rootView.findViewById(R.id.add_project_cover_selection_fragment_selected_images);
		coverImageSelection = addProjectManager.getImagesSelectedForCover();
		if (null != savedInstanceState) {
			coverImage = (ImageModule) savedInstanceState.get(ADD_PROJECT_COVER_IMAGE_KEY);
			state = CropMode.valueOf(savedInstanceState.getString(ADD_PROJECT_COVER_IMAGE_SELECTION_STATE,CropMode.INPROGRESS.name()));
			Bitmap coverImageBitmap = savedInstanceState.getParcelable(ADD_PROJECT_CROPPED_COVER_IMAGE_KEY);
			if(null != coverImageBitmap){
				ImageModule orginalImageModule = (ImageModule) savedInstanceState.getSerializable(ADD_PROJECT_CROPPED_COVER_IMAGE_ORIGINAL_KEY);
				cover = new CoverImage(orginalImageModule, coverImageBitmap);
			}
		}else{
			cover = addProjectManager.getSelectedCoverImage();
		}
		if(null != cover){
			coverImage = cover.getOriginalImage();
			if(!coverImageSelection.contains(coverImage)){
				coverImage = coverImageSelection.get(FIRST_IMAGE);
				position = FIRST_IMAGE;
				cover = null;
				state = CropMode.INPROGRESS;
			}else if(null != cover.getBitMap())
				state = CropMode.DONE;
			else
				state = CropMode.INPROGRESS;
		}else{
			state = CropMode.INPROGRESS;
			if(null == coverImage && !coverImageSelection.isEmpty()){
				coverImage = coverImageSelection.get(FIRST_IMAGE);
				position = FIRST_IMAGE;
			}
		}
		if(state == CropMode.INPROGRESS){
			actionBarForwardButton.setText(getResources().getString(R.string.bsdk_addproject_project_cover_selection_right_nav_crop_txt));
		}
		addImagesToLayout(filmStripLayout);
		updateSelectedCoverImage();
		View inflate =rootView.findViewById(R.id.add_project_cover_selection_fragment_add_btn);
		inflate.setOnClickListener(this);
		if(null != savedInstanceState){
			FragmentManager supportFragmentManager = getActivity().getSupportFragmentManager();
			Fragment imageChooserFragmentInstance = supportFragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_ADD_PROJECT_COVER_NEW_IMAGE_SELECTION_FRAGMENT);
			if (imageChooserFragmentInstance instanceof BehanceSDKImageSelectorDialogFragment) {
				((BehanceSDKImageSelectorDialogFragment) imageChooserFragmentInstance).setCallbacks(this);
			}
		}
		return rootView;
	}

	public void launchAlbumsViewToPickPictures()
	{
		Context context = getActivity();
		if (context != null) {
			Intent intent = getActivity().getIntent();
			BehanceSDKImageSelectorOptions imageSelectionOptions = BehanceSDKImageSelectorOptions.getUnlimitedImageSelectionOptions();
			if(intent.getBooleanExtra(BehanceSDKPublishProjectActivity.INTENT_SERIALIZABLE_EXTRA_CC_ASSET_BROWSER_DISABLED,false)) {
				imageSelectionOptions.setCCAssetBrowserDisabled(true);
			}
			FragmentManager supportFragmentManager = getActivity().getSupportFragmentManager();
			BehanceSDKLaunchUtil.launchAlbumsViewToPickPicture(this,imageSelectionOptions,
					supportFragmentManager, BehanceSDKFragmentTags.FRAGMENT_TAG_ADD_PROJECT_COVER_NEW_IMAGE_SELECTION_FRAGMENT);
		}
	}

	@Override
	public void onClick(View v) {
		int id = v.getId();
		if (id == R.id.add_project_cover_selection_fragment_add_btn) {
			if(BehanceSDKPermissionHelper.checkPermission(getActivity(), BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT)) {
				launchAlbumsViewToPickPictures();
			}
			else
			{
				BehanceSDKPublishProjectActivity activity = (BehanceSDKPublishProjectActivity) getActivity();
				activity.requestPermisssionsForAlbumsView();
			}
		}

	}

	private void addImagesToLayout(LinearLayout layout) {
		for (int i = 0; i < coverImageSelection.size(); i++) {
			addImagetoLayout(i);
		}
	}

	private void addImagetoLayout(int position) {
        ImageModule image = coverImageSelection.get(position);
		if(getActivity()!= null) {
			LayoutInflater layoutInflater = getLayoutInflater(getArguments());
			View view = layoutInflater.inflate(R.layout.bsdk_adapter_publish_project_cover_image_filmstrip_list_item, (ViewGroup) rootView, false);
			ImageView imageView = (ImageView) view.findViewById(R.id.bsdkPublishProjectCoverImageFilmStripImage);
			if (image.equals(coverImage)) {
				this.position = position;
				showSelector(view);
			}
			imageView.setImageBitmap(image.getThumbNail(getActivity()));
			view.setOnClickListener(getClickListenerForImages());
			filmStripLayout.addView(view);
			view.setId(position);
		}
	}

	private void showSelector(View view) {
		View selector = view.findViewById(R.id.bsdkPublishProjectCoverImageFilmStripImageSelector);
		selector.setVisibility(View.VISIBLE);
	}
	
	private void hideSelector(View view) {
		View selector = view.findViewById(R.id.bsdkPublishProjectCoverImageFilmStripImageSelector);
		if(null != selector)
			selector.setVisibility(View.GONE);
	}


	private void hideSelector(int position) {
		View childAt = filmStripLayout.getChildAt(position);
		if(null != childAt) {
			hideSelector(childAt);
		}
	}

	private OnClickListener getClickListenerForImages() {
		if (null == listener) {
			listener = new OnClickListener() {

				@Override
				public void onClick(View v) {
					int id = v.getId();
					cover = null;
					state = CropMode.INPROGRESS;
					actionBarForwardButton.setText(getResources().getString(R.string.bsdk_addproject_project_cover_selection_right_nav_crop_txt));
					showCoverImage(id);

					View childAt = filmStripLayout.getChildAt(position);
					if(null != childAt){
						hideSelector(childAt);
					}
					showSelector(v);
					position = id;
				}
			};
		}
		return listener;
	}

	@Override
	public void onSaveInstanceState(Bundle arg0) {
		super.onSaveInstanceState(arg0);
		if(null != state)
			arg0.putString(ADD_PROJECT_COVER_IMAGE_SELECTION_STATE, state.name());
		if (null != coverImage)
			arg0.putSerializable(ADD_PROJECT_COVER_IMAGE_KEY, coverImage);
		if(null != cover){
			arg0.putParcelable(ADD_PROJECT_CROPPED_COVER_IMAGE_KEY, cover.getBitMap());
			arg0.putSerializable(ADD_PROJECT_CROPPED_COVER_IMAGE_ORIGINAL_KEY, cover.getOriginalImage());
		}
	}

	private void showCoverImage(int id) {
		coverImage = coverImageSelection.get(id);
		updateSelectedCoverImage();
	}

	private void updateSelectedCoverImage() {
		
		if(state == CropMode.DONE){
			croppedImage.setImageBitmap(cover.getBitMap());
			croppedImage.setVisibility(View.VISIBLE);
			cropImageView.setVisibility(View.GONE);
			coverImageBackground.setBackgroundResource(R.color.bsdk_add_project_cover_image_fragment_done_state_background);
		}else{
			if (null != coverImage) {
				coverImage.displayImage(cropImageView.getImageView(), new ImageLoadingListener() {
					
					@Override
					public void onLoadingStarted(String arg0, View arg1) {
						cropImageView.setVisibility(View.GONE);
						progressBar.setVisibility(View.VISIBLE);
					}
					
					@Override
					public void onLoadingFailed(String arg0, View arg1, FailReason arg2) {
						
					}
					
					@Override
					public void onLoadingComplete(String arg0, View arg1, Bitmap bitmap) {
						cropImageView.setImageBitmap(bitmap);
						croppedImage.setVisibility(View.GONE);
						progressBar.setVisibility(View.GONE);
						cropImageView.setVisibility(View.VISIBLE);
						coverImageBackground.setBackgroundResource(R.color.bsdk_add_project_cover_image_fragment_inprogress_state_background);
						enableActionBarRightNav();
					}
					
					@Override
					public void onLoadingCancelled(String arg0, View arg1) {
						
					}
				});
			}else{
				disableActionBarRightNav();
			}
		}
	}

	@Override
	public void onImageSelectorSelectionFinished(List<ImageModule> selectedImageModulesList) {
		BehanceSDKProjectPublishWFManager wfManager = BehanceSDKProjectPublishWFManager.getInstance();
		wfManager.addImagesSelectedForCover(selectedImageModulesList);
		int size = coverImageSelection.size();
        boolean autoSelect = size == 0;
		coverImageSelection.addAll(selectedImageModulesList);
        if(autoSelect) {
			showCoverImage(0);
		}
		else {
			hideSelector(position);
			showCoverImage(size);
		}
		for (int i = size; i < coverImageSelection.size(); i++) {
			addImagetoLayout(i);
		}

	}

	@Override
	public void onImageSelectorSelectionCanceled() {
		
	}

	@Override
	public void onImageSelectorSelectionBackPressed() {
		
	}

    @Override
    public void onImageSelectorOnError(Exception e) {

    }

    @Override
	public void projectModulesChanged(List<ProjectModule> modules) {
			coverImageSelection = addProjectManager.getImagesSelectedForCover();
			if(null != coverImageSelection && !coverImageSelection.isEmpty()){
				if(!coverImageSelection.contains(coverImage)){
					coverImage = coverImageSelection.get(FIRST_IMAGE);
					position = FIRST_IMAGE;
					cover = null;
					state = CropMode.INPROGRESS;
					actionBarForwardButton.setText(getResources().getString(R.string.bsdk_addproject_project_cover_selection_right_nav_crop_txt));
					updateSelectedCoverImage();
				}
				filmStripLayout.removeAllViews();
				for (int i = 0; i < coverImageSelection.size(); i++) {
					addImagetoLayout(i);
				}
		}
	}

	@Override
	public void rotateImage(ImageModule module) {
		filmStripLayout.removeAllViews();
		for (int i = 0; i < coverImageSelection.size(); i++) {
			addImagetoLayout(i);
		}
		if(module.equals(coverImage))
			updateSelectedCoverImage();
	}
	
}
