/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import java.util.Arrays;
import java.util.List;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnFocusChangeListener;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.dto.BehanceSDKPublishProjectServiceParamsDTO;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.fragments.headless.BehanceSDKPublishProjectHeadlessFragment;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager.CoverImageChangeListener;
import com.behance.sdk.project.modules.CoverImage;
import com.behance.sdk.services.BehanceSDKPublishProjectService;
import com.behance.sdk.ui.activities.BehanceSDKLoginToTwitterActivity;
import com.behance.sdk.ui.activities.BehanceSDKPublishProjectActivity;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFragmentTags;
import com.behance.sdk.util.BehanceSDKUtils;
/*import com.facebook.Session;
import com.facebook.SessionDefaultAudience;
import com.facebook.SessionLoginBehavior;
import com.facebook.SessionState;*/
import com.facebook.AccessToken;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.FacebookSdk;
import com.facebook.login.DefaultAudience;
import com.facebook.login.LoginBehavior;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;

/**
 * @author ugs
 * 
 */
public class BehanceSDKPublishProjectDetailsFragment extends BehanceSDKPublishProjectBaseFragment implements TextWatcher,
		BehanceSDKPublishProjectHeadlessFragment.Callbacks, CoverImageChangeListener {

	private static final String COMMA = ",";
	private static final int MAX_NUMBER_OF_CREATIVE_FIELD_ALLOWED = 3;
	private static final String FACEBOOK_PERMISSION_PUBLISH_ACTIONS = "publish_actions";
	private static final String HEADLESS_FRAGMENT_TAG_PUBLISH_PROJECT = "HEADLESS_FRAGMENT_TAG_PUBLISH_PROJECT";
    public static final int TWITTER_LOGIN_ACTIVITY_REQUEST_CODE = 56790;

    private BehanceSDKProjectPublishWFManager workflowManager = BehanceSDKProjectPublishWFManager.getInstance();
	private List<BehanceSDKCreativeFieldDTO> selectedField;
	private BehanceSDKCopyrightOption selectedCopyRight;
	private boolean shareOnFacebook = false;
	private boolean shareOnTwitter = false;
	private BehanceSDKSocialAccountManager socialAccountManager;
	private BehanceSDKSocialAccountDTO facebookAccount;
	private BehanceSDKSocialAccountDTO twitterAccount;

	private TextView projectNameTxtView;
	private EditText projectDescEditTxt;
	private TextView projectFilterFieldsButton;
	private EditText projectTagsEditTxt;
	private TextView projectCopyrightsButton;
	private CheckBox adultContentCheckBox;
	private ImageView shareOnFacebookBtnImageView;
	private ImageView shareOnTwitterBtnImageView;
	private BehanceSDKGenericAlertDialog loginToTwitterConfirmationDialog;
	private BehanceSDKGenericAlertDialog loginToFacebookConfirmationDialog;
	private ImageView coverImageView;

	private View publishProjectProgressbar;

	private BehanceSDKPublishProjectHeadlessFragment headlessFragment;
	private CallbackManager callbackManager;
	LoginResult savedLoginResult = null;

	private void setEnabledInfoScreen(boolean enabled) {
		projectNameTxtView.setEnabled(enabled);
		projectDescEditTxt.setEnabled(enabled);
		projectFilterFieldsButton.setEnabled(enabled);
		projectTagsEditTxt.setEnabled(enabled);
		projectCopyrightsButton.setEnabled(enabled);
		adultContentCheckBox.setEnabled(enabled);
		shareOnFacebookBtnImageView.setEnabled(enabled);
		shareOnTwitterBtnImageView.setEnabled(enabled);
		if (enabled)
			enableActionBarRightNav();
		else
			disableActionBarRightNav();
	}

	@Override
	protected int getLayout() {
		return R.layout.bsdk_dialog_fragment_publish_project_details;
	}

	@Override
	public void onStart() {
		super.onStart();
		workflowManager.addCoverImageChangeListener(this);
	}

	@Override
	public void onStop() {
		super.onStop();
		workflowManager.removeCoverImageChangeListener(this);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		savedLoginResult = null;
		callbackManager = CallbackManager.Factory.create();
		try {
		LoginManager.getInstance().registerCallback(callbackManager, new FacebookCallback<LoginResult>() {
			@Override
			public void onSuccess(LoginResult loginResult) {
				handleFacebookAuthActivityResult(loginResult,null);
			}

			@Override
			public void onCancel() {
				handleFacebookAuthActivityResult(null,null);
			}

			@Override
			public void onError(FacebookException e) {
				handleFacebookAuthActivityResult(null,e);
			}
		});
		}catch (Exception e)
		{
		}
		super.onCreateView(inflater, container, savedInstanceState);

		disableActionBarRightNav();

		socialAccountManager = BehanceSDKSocialAccountManager.getInstance(getActivity());
		twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
		facebookAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.FACEBOOK);

		FragmentManager fragmentManager = ((FragmentActivity) getActivity()).getSupportFragmentManager();
		headlessFragment = (BehanceSDKPublishProjectHeadlessFragment) fragmentManager.findFragmentByTag(HEADLESS_FRAGMENT_TAG_PUBLISH_PROJECT);
		if (headlessFragment == null) {
			headlessFragment = new BehanceSDKPublishProjectHeadlessFragment();
			getActivity().getSupportFragmentManager().beginTransaction().add(headlessFragment, HEADLESS_FRAGMENT_TAG_PUBLISH_PROJECT).commit();
		}
		headlessFragment.setCallbacks(this);

		Fragment creativeFieldsDialog = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_CREATIVE_FIELDS_FILTER_DIALOG);
		if (creativeFieldsDialog instanceof BehanceSDKCreativeFieldsFilterDialog) {
			((BehanceSDKCreativeFieldsFilterDialog) creativeFieldsDialog).setCallbacks(headlessFragment);
		}

		Fragment copyrightSettingsDialog = fragmentManager.findFragmentByTag(BehanceSDKFragmentTags.FRAGMENT_TAG_COPYRIGHT_SETTINGS_DIALOG);
		if (copyrightSettingsDialog instanceof BehanceSDKCopyrightSettingsDialog) {
			((BehanceSDKCopyrightSettingsDialog) copyrightSettingsDialog).setCallbacks(headlessFragment);
		}
		coverImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentCoverImage);
		CoverImage cover = workflowManager.getSelectedCoverImage();
		loadCoverImage(cover);
		View projectNameLayout = rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentProjectNameLayout);
		projectNameTxtView = (TextView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentProjectName);
		projectNameTxtView.addTextChangedListener(this);
		projectNameTxtView.setOnFocusChangeListener(getFocusChangeListerForLayout(projectNameLayout));
		projectDescEditTxt = (EditText) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentProjectDescription);
		projectDescEditTxt.addTextChangedListener(this);
		projectFilterFieldsButton = (TextView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentCreativeFields);
		projectFilterFieldsButton.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Context context = getActivity();
				if (context instanceof FragmentActivity) {
					FragmentManager fragmentManager = ((FragmentActivity) context).getSupportFragmentManager();
					BehanceSDKCreativeFieldsFilterDialog instance = new BehanceSDKCreativeFieldsFilterDialog();
					instance.setSelectableFieldCount(MAX_NUMBER_OF_CREATIVE_FIELD_ALLOWED);
					instance.setSelectedCreativeField(selectedField);
					instance.setCallbacks(headlessFragment);
					instance.show(fragmentManager, BehanceSDKFragmentTags.FRAGMENT_TAG_CREATIVE_FIELDS_FILTER_DIALOG);
				}
			}
		});
		projectTagsEditTxt = (EditText) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentTags);
		projectTagsEditTxt.addTextChangedListener(this);
		projectCopyrightsButton = (TextView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentCopyrightSettings);

		projectCopyrightsButton.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Context context = getActivity();
				if (context instanceof FragmentActivity) {
					FragmentManager fragmentManager = ((FragmentActivity) context).getSupportFragmentManager();
					BehanceSDKCopyrightSettingsDialog instance = new BehanceSDKCopyrightSettingsDialog();
					instance.setSelectedCopyright(selectedCopyRight);
					instance.setCallbacks(headlessFragment);
					instance.show(fragmentManager, BehanceSDKFragmentTags.FRAGMENT_TAG_COPYRIGHT_SETTINGS_DIALOG);
				}
			}
		});
		adultContentCheckBox = (CheckBox) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentAdultContentCheckbox);
		publishProjectProgressbar = rootView.findViewById(R.id.projectPublishProgressBar);

		shareOnTwitterBtnImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentTwitterIcon);
		if (workflowManager.isHideTwitterSharing()) {
			shareOnTwitterBtnImageView.setVisibility(View.GONE);
		}
		shareOnTwitterBtnImageView.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				handleEnableShareOnTwitterBtnClick();
			}
		});

		shareOnFacebookBtnImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentFacebookIcon);
		if (workflowManager.isHideFacebookSharing()) {
			shareOnFacebookBtnImageView.setVisibility(View.GONE);
		}
		shareOnFacebookBtnImageView.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				handleEnableShareOnFacebookBtnClick();
			}
		});

		if (workflowManager.isHideFacebookSharing() && workflowManager.isHideTwitterSharing()) {
			View shareLayout = rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentShareOnLayout);
			shareLayout.setVisibility(View.GONE);
		}

		if (savedInstanceState == null) {
			// loading for first time
			if (facebookAccount != null) {
				shareOnFacebook = facebookAccount.isSharingEnabledLastTime();
			} else {
				shareOnFacebook = false;
			}

			if (twitterAccount != null) {
				shareOnTwitter = twitterAccount.isSharingEnabledLastTime();
			} else {
				shareOnTwitter = false;
			}
		} else {
			shareOnFacebook = workflowManager.getShareOnFaceBook();
			shareOnTwitter = workflowManager.getShareOnTwitter();
		}

		if (shareOnFacebook) {
			if (!isFacebookAccountAuthenticatedToPublish()) {
				shareOnFacebook = false;
			}
		}

		if (shareOnTwitter) {
			if (!isTwitterAccountAuthenticated()) {
				shareOnTwitter = false;
			}
		}

		updateProjectDetailsInUI();
		BehanceSDKUtils.setContentViewDimensions(getActivity(), rootView.findViewById(R.id.bsdkPublishProjectPublishFragmentLayout));
		return rootView;
	}

	private void updateProjectDetailsInUI() {
		String projectName = workflowManager.getProjectTitle();
		if (false == TextUtils.isEmpty(projectName)) {
			projectNameTxtView.setText(projectName);
		}
		String projectDescStr = workflowManager.getProjectDescription();
		if (false == TextUtils.isEmpty(projectDescStr)) {
			this.projectDescEditTxt.setText(projectDescStr);
		}
		selectedField = workflowManager.getProjectFields();
		if (null != selectedField && selectedField.isEmpty() == false)
			setSelectedCreativeFieldsText();
		String tagsString = workflowManager.getProjectTags();
		if (false == TextUtils.isEmpty(tagsString)) {
			projectTagsEditTxt.setText(tagsString);
		}
		selectedCopyRight = workflowManager.getCopyrightOption();
		projectCopyrightsButton.setText(selectedCopyRight.getDescription(getActivity()));
		adultContentCheckBox.setChecked(workflowManager.isProjectContainsAdultContent());
		updateShareOnFacebookBtnImage();
		updateShareOnTwitterBtnImage();
	}

	@Override
	protected void backButtonPressed() {
		populateProjectDetails();
		hideSoftKeyBoardIfActive(rootView);
		super.backButtonPressed();
	}

	@Override
	protected void nextButtonPressed() {
		NetworkInfo networkInfo = ((ConnectivityManager) getActivity().getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
		if (networkInfo != null && networkInfo.isConnected()) {
            invokePublishProjectService();
		} else {
			Toast toast = Toast.makeText(getActivity(), getString(R.string.bsdk_connection_error_msg), Toast.LENGTH_LONG);
			toast.setGravity(Gravity.CENTER, 0, 0);
			toast.show();
		}
	}

	@Override
	public void onSaveInstanceState(Bundle out) {
		super.onSaveInstanceState(out);
		populateProjectDetails();
	}

	@Override
	public void onDetach() {
		super.onDetach();
		if (headlessFragment != null) {
			headlessFragment.setCallbacks(null);
		}
		closeLoginToTwitterConfirmationDialog();
		closeLoginToFacebookConfirmationDialog();
	}

	private OnFocusChangeListener getFocusChangeListerForLayout(final View view) {
		return new OnFocusChangeListener() {

			@Override
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					view.setActivated(true);
				} else
					view.setActivated(false);
			}
		};
	}

	@Override
	protected int getBackButtonID() {
		return R.id.bsdkPublishProjectPublishFragmentTitlebarBackBtnLayout;
	}

	@Override
	protected int getTitleViewID() {
		return R.id.bsdkPublishProjectPublishFragmentTitlebarTitleTxtView;
	}

	@Override
	protected int getForwardButtonID() {
		return R.id.bsdkPublishProjectPublishFragmentTitlebarActionBtnTxtView;
	}

	@Override
	protected int getForwardButtonText() {
		return R.string.bsdk_add_content_project_publish_right_navigation;
	}

	private void handleEnableShareOnFacebookBtnClick() {
		if (facebookAccount != null) {
			if (shareOnFacebook) {
				shareOnFacebook = false;
				updateShareOnFacebookBtnImage();
			} else {
				checkFacebookAuthAndEnableSharing();
			}
		}
	}

	private void updateShareOnFacebookBtnImage() {
		if (shareOnFacebook) {
			shareOnFacebookBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_facebook_on);
		} else {
			shareOnFacebookBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_facebook_off);
		}
	}

	private void checkFacebookAuthAndEnableSharing() {
		if (isFacebookAccountAuthenticatedToPublish()) {
			shareOnFacebook = true;
			updateShareOnFacebookBtnImage();
			updateFacebookAccountAuthStatus();
		} else {
			displayFacebookLoginConfirmationDialog();
		}
	}

	private void displayFacebookLoginConfirmationDialog() {
		loginToFacebookConfirmationDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(),
				R.string.bsdk_social_account_facebook_login_confirmation_title, R.string.bsdk_social_account_facebook_login_confirmation_body,
				R.string.bsdk_social_account_facebook_login_confirmation_ok_btn_label,
				R.string.bsdk_social_account_facebook_login_confirmation_cancel_btn_label);
		loginToFacebookConfirmationDialog.setOnNotOKBtnClickListener(facebookLoginConfirmDialogClickListener);
		loginToFacebookConfirmationDialog.setOnOKBtnClickListener(facebookLoginConfirmDialogClickListener);
		loginToFacebookConfirmationDialog.show();
	}

	private void LoginToFacebook() {
		hideSoftKeyBoardIfActive(rootView);
		List<String> permissions = Arrays.asList(FACEBOOK_PERMISSION_PUBLISH_ACTIONS);
		LoginManager.getInstance().setLoginBehavior(LoginBehavior.NATIVE_WITH_FALLBACK);
		LoginManager.getInstance().setDefaultAudience(DefaultAudience.EVERYONE);
		LoginManager.getInstance().logInWithPublishPermissions(this,permissions);
	}

	private boolean isFacebookAccountAuthenticatedToPublish() {
		//Session activeSession = Session.getActiveSession();
		if(headlessFragment!= null)
		savedLoginResult = headlessFragment.getLoginResult();
		if (savedLoginResult != null && !savedLoginResult.getAccessToken().isExpired()) {
			if (savedLoginResult.getAccessToken().getPermissions().contains(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
				return true;
			}
		}
		return false;
	}

	/*private Session.OpenRequest createFacebookSessionRequest() {
		Session.OpenRequest openRequest = new Session.OpenRequest(this);
		openRequest.setDefaultAudience(SessionDefaultAudience.EVERYONE);
		List<String> permissions = Arrays.asList(FACEBOOK_PERMISSION_PUBLISH_ACTIONS);
		openRequest.setPermissions(permissions);
		openRequest.setLoginBehavior(SessionLoginBehavior.SSO_WITH_FALLBACK);
		return openRequest;
	}*/

	private void updateFacebookAccountAuthStatus() {
		boolean updateFacebookAccount = false;
		boolean facebookAccountAuthenticated = isFacebookAccountAuthenticatedToPublish();
		if (facebookAccountAuthenticated && !facebookAccount.isUserAuthenticated()) {
			facebookAccount.setUserAuthenticated(true);
			updateFacebookAccount = true;
		} else if (!facebookAccountAuthenticated && facebookAccount.isUserAuthenticated()) {
			facebookAccount.setUserAuthenticated(false);
			updateFacebookAccount = true;
		}
		if (updateFacebookAccount) {
			socialAccountManager.updateAccount(facebookAccount);
		}
	}

	private boolean isTwitterAccountAuthenticated() {
		if (twitterAccount != null) {
			String accessToken = twitterAccount.getUserAuthToken();
			String accessTokenSecret = twitterAccount.getAdditionalData();
			if (accessToken == null || accessToken.isEmpty() || accessTokenSecret == null || accessTokenSecret.isEmpty()) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}

	private void handleEnableShareOnTwitterBtnClick() {
		if (twitterAccount != null) {
			if (shareOnTwitter) {
				shareOnTwitter = false;
			} else {
				checkIfUserHasLoggedIntoTwitter();
			}
			updateShareOnTwitterBtnImage();
		}
	}

	private void checkIfUserHasLoggedIntoTwitter() {
		String accessToken = twitterAccount.getUserAuthToken();
		String accessTokenSecret = twitterAccount.getAdditionalData();
		if (accessToken == null || accessToken.isEmpty() || accessTokenSecret == null || accessTokenSecret.isEmpty()) {
			displayTwitterLoginConfirmationDialog();
		} else {
			shareOnTwitter = true;
			updateShareOnTwitterBtnImage();
		}
	}

	private void displayTwitterLoginConfirmationDialog() {
		loginToTwitterConfirmationDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(),
				R.string.bsdk_social_account_twitter_login_confirmation_title, R.string.bsdk_social_account_twitter_login_confirmation_body,
				R.string.bsdk_social_account_twitter_login_confirmation_ok_btn_label,
				R.string.bsdk_social_account_twitter_login_confirmation_cancel_btn_label);
		loginToTwitterConfirmationDialog.setOnNotOKBtnClickListener(twitterLoginConfirmDialogClickListener);
		loginToTwitterConfirmationDialog.setOnOKBtnClickListener(twitterLoginConfirmDialogClickListener);
		loginToTwitterConfirmationDialog.show();
	}

	private void updateShareOnTwitterBtnImage() {
		if (shareOnTwitter) {
			shareOnTwitterBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_twitter_on);
		} else {
			shareOnTwitterBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_twitter_off);
		}
	}

	@Override
	public void coverImageChanges(CoverImage coverImage) {
		loadCoverImage(coverImage);
	}

	private void launchTwitterLoginActivity() {
		Intent detailIntent = new Intent(getActivity(), BehanceSDKLoginToTwitterActivity.class);
		startActivityForResult(detailIntent, TWITTER_LOGIN_ACTIVITY_REQUEST_CODE);
		closeLoginToTwitterConfirmationDialog();
	}

	private void closeLoginToTwitterConfirmationDialog() {
		if (loginToTwitterConfirmationDialog != null) {
			loginToTwitterConfirmationDialog.dismiss();
		}
	}

	private void closeLoginToFacebookConfirmationDialog() {
		if (loginToFacebookConfirmationDialog != null) {
			loginToFacebookConfirmationDialog.dismiss();
		}
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == TWITTER_LOGIN_ACTIVITY_REQUEST_CODE) {
			switch (resultCode) {
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_LOGIN_FAILED:
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_NO_NETWORK_CONNECTIVITY:
				handleTwitterAuthenticationFailure();
				break;
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_LOGIN_SUCCESSFULL:
				handleTwitterAuthenticationSuccess();
				break;
			default:
				break;
			}
		} else {
			callbackManager.onActivityResult(requestCode, resultCode, data);
		}
	}

	private void handleFacebookAuthActivityResult(LoginResult loginResult,FacebookException exception) {
		if (getActivity() != null) {
			//Session activeSession = Session.getActiveSession();

			if (loginResult != null) {
				if (loginResult.getRecentlyGrantedPermissions().contains(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
					showProgressBar();
					savedLoginResult = loginResult;
					headlessFragment.setLoginResult(loginResult);
					headlessFragment.loadFacebookUserId(loginResult);
				} else {
					// user didn't give permissions to publish
					Toast.makeText(getActivity(), R.string.bsdk_social_account_fb_auth_permissions_failure, Toast.LENGTH_LONG).show();
					shareOnFacebook = false;
					updateShareOnFacebookBtnImage();
					updateFacebookAccountAuthStatus();
				}
			} else {
				// auth failed, display a message
				if (exception != null) {
					Toast.makeText(getActivity(), R.string.bsdk_social_account_fb_auth_failure, Toast.LENGTH_LONG).show();
					updateFacebookAccountAuthStatus();
				}
				shareOnFacebook = false;
				updateShareOnFacebookBtnImage();
			}
		}
	}

	@Override
	public void onGetFacebookUserIdComplete(String userName) {
		if (getActivity() != null) {
			shareOnFacebook = true;
			updateShareOnFacebookBtnImage();
			if (facebookAccount != null) {
				facebookAccount.setUserId(userName);
				updateFacebookAccountAuthStatus();
			}
			hideProgressBar();
		}
	}

	public void handleTwitterAuthenticationSuccess() {
		if (getActivity() != null) {
			shareOnTwitter = true;
			updateShareOnTwitterBtnImage();
			// Refresh Twitter account
			twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
			twitterAccount.setUserAuthenticated(true);
			socialAccountManager.updateAccount(twitterAccount);
		}
	}

	public void handleTwitterAuthenticationFailure() {
		if (getActivity() != null) {
			shareOnTwitter = false;
			updateShareOnTwitterBtnImage();
			Toast.makeText(getActivity(), R.string.bsdk_social_account_twitter_login_authentication_failure, Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	public void beforeTextChanged(CharSequence s, int start, int count, int after) {

	}

	@Override
	public void onTextChanged(CharSequence s, int start, int before, int count) {
	}

	@Override
	public void afterTextChanged(Editable s) {
		enableOrDisablePublishButton();
	}

	private void setSelectedCreativeFieldsText() {
		String creativeFieldsString = BehanceSDKUtils.getCreativeFieldsString(selectedField, COMMA);
		if (false == TextUtils.isEmpty(creativeFieldsString))
			projectFilterFieldsButton.setText(creativeFieldsString);
	}

	private void loadCoverImage(CoverImage cover) {
		if (null != cover)
			coverImageView.setImageBitmap(cover.getBitMap());
	}

	private void showProgressBar() {
		if (null != publishProjectProgressbar) {
			publishProjectProgressbar.setVisibility(View.VISIBLE);
		}
	}

	private void hideProgressBar() {
		if (null != publishProjectProgressbar) {
			publishProjectProgressbar.setVisibility(View.GONE);
		}
	}

	private void invokePublishProjectService() {
		populateProjectDetails();

		boolean allTagsValid = true;
		String projectTagsStr = getProjectTagString();
		projectTagsStr = BehanceSDKUtils.cleanUpTagsString(projectTagsStr);
		String[] tagsArray = projectTagsStr.split(BehanceSDKConstants.TAG_CLIENT_SIDE_SEPERATOR);
		StringBuilder finalTagStringForServer = new StringBuilder();
		for (String tagStr : tagsArray) {
			String tag = tagStr.trim();
			if (BehanceSDKUtils.isTagValidForProjectAndWIP(tag)) {
				if (finalTagStringForServer.length() > 0) {
					finalTagStringForServer.append(BehanceSDKConstants.TAG_SERVER_SIDE_SEPERATOR);
				}
				finalTagStringForServer.append(tag);
			} else {
				allTagsValid = false;
				break;
			}
		}

		if (allTagsValid) {
			hideSoftKeyBoardIfActive(rootView);
			showProgressBar();
			setEnabledInfoScreen(false);

			BehanceSDKPublishProjectServiceParamsDTO serviceParams = new BehanceSDKPublishProjectServiceParamsDTO();
			IBehanceSDKUserCredentials userCredentials = workflowManager.getUserCredentials();
			serviceParams.setAppClientId(userCredentials.getClientId());
			serviceParams.setNotificationHandlerActivity(workflowManager.getNotificationHandlerActivityClass());

			serviceParams.setProjectContainsAdultContent(workflowManager.isProjectContainsAdultContent());
			serviceParams.setProjectCopyright(workflowManager.getCopyrightOption().getValue());
			CoverImage selectedCoverImage = workflowManager.getSelectedCoverImage();
			serviceParams.setProjectCoverImageByteArray(selectedCoverImage.getByteArray());
			serviceParams.setProjectCoverImageFileName(selectedCoverImage.getName());
			String projectCreativeFields = BehanceSDKUtils.getCreativeFieldsString(workflowManager.getProjectFields(), "|");
			serviceParams.setProjectCreativeFields(projectCreativeFields);
			serviceParams.setProjectDescription(workflowManager.getProjectDescription());
			serviceParams.setProjectModules(workflowManager.getProjectModules());
			serviceParams.setProjectTags(finalTagStringForServer.toString());
			serviceParams.setProjectTitle(workflowManager.getProjectTitle());

			boolean updateFacebookAccount = false;
			if (shareOnFacebook && !isFacebookAccountAuthenticatedToPublish()) {
				shareOnFacebook = false;
			}

			if (shareOnFacebook) {
				if (!facebookAccount.isSharingEnabledLastTime()) {
					facebookAccount.setSharingEnabledLastTime(true);
					updateFacebookAccount = true;
				}
			} else {
				if (null != facebookAccount && facebookAccount.isSharingEnabledLastTime()) {
					facebookAccount.setSharingEnabledLastTime(false);
					updateFacebookAccount = true;
				}
			}

			if (updateFacebookAccount) {
				socialAccountManager.updateAccount(facebookAccount);
			}

			boolean updateTwitterAccount = false;

			boolean TwitterAccountAuthenticated = isTwitterAccountAuthenticated();
			if (twitterAccount != null) {
				if (TwitterAccountAuthenticated && !twitterAccount.isUserAuthenticated()) {
					twitterAccount.setUserAuthenticated(true);
					updateTwitterAccount = true;
				} else if (!TwitterAccountAuthenticated && twitterAccount.isUserAuthenticated()) {
					twitterAccount.setUserAuthenticated(false);
					updateTwitterAccount = true;
				}
			}
			if (!TwitterAccountAuthenticated && shareOnTwitter) {
				shareOnTwitter = false;
			}

			if (shareOnTwitter) {
				if (!twitterAccount.isSharingEnabledLastTime()) {
					twitterAccount.setSharingEnabledLastTime(true);
					updateTwitterAccount = true;
				}
			} else {
				if (null != twitterAccount && twitterAccount.isSharingEnabledLastTime()) {
					twitterAccount.setSharingEnabledLastTime(false);
					updateTwitterAccount = true;
				}
			}

			if (updateTwitterAccount) {
				socialAccountManager.updateAccount(twitterAccount);
			}

			serviceParams.setShareOnFacebook(shareOnFacebook);
			serviceParams.setShareOnTwitter(shareOnTwitter);
			if (shareOnTwitter) {
				serviceParams.setTwitterUserAccessToken(twitterAccount.getUserAuthToken());
				serviceParams.setTwitterUserAccessTokenSecret(twitterAccount.getAdditionalData());
				serviceParams.setTwitterConsumerKey(twitterAccount.getAccountClientId());
				serviceParams.setTwitterConsumerSecret(twitterAccount.getAccountClientSecret());
			}

			Intent intent = new Intent(getActivity(), BehanceSDKPublishProjectService.class);
			intent.putExtra(BehanceSDKPublishProjectService.INTENT_EXTRA_PARAMS, serviceParams);
			ComponentName publishProjectComponent = getActivity().startService(intent);
			int displayMsgResourceId = R.string.bsdk_add_content_project_publish_service_initialize_error_msg;
			if (publishProjectComponent != null) {
				displayMsgResourceId = R.string.bsdk_add_content_project_publish_initiated_msg;
			} else {
				displayMsgResourceId = R.string.bsdk_add_content_project_publish_service_initialize_error_msg;
			}
			Toast.makeText(getActivity(), displayMsgResourceId, Toast.LENGTH_LONG).show();

			workflowManager.finishWorkflow();
			((BehanceSDKPublishProjectActivity) getActivity()).closeThisActivity(true);
		} else {
			Toast toast = Toast.makeText(getActivity(), R.string.bsdk_add_content_project_publish_invalid_tag_msg, Toast.LENGTH_LONG);
			toast.setGravity(Gravity.CENTER, 0, 0);
			toast.show();
			projectTagsEditTxt.requestFocus();
		}
	}

	private void populateProjectDetails() {
		workflowManager.setProjectTitle(getProjectTitle());
		workflowManager.setProjectTags(getProjectTagString());
		workflowManager.setProjectFields(getProjectFields());
		workflowManager.setProjectDescription(getProjectDesc());
		workflowManager.setCopyrightOption(selectedCopyRight);
		workflowManager.setShareOnFaceBook(shareOnFacebook);
		workflowManager.setShareOnTwitter(shareOnTwitter);
		workflowManager.setContainsAdultContent(adultContentCheckBox.isChecked());
	}

	private String getProjectTitle() {
		return projectNameTxtView.getText().toString();
	}

	private String getProjectDesc() {
		return projectDescEditTxt.getText().toString();
	}

	private List<BehanceSDKCreativeFieldDTO> getProjectFields() {
		return selectedField;
	}


	private String getProjectTagString() {
		return projectTagsEditTxt.getText().toString();
	}

	private void enableOrDisablePublishButton() {
		String projectTitle = getProjectTitle();
		String projectDesc = getProjectDesc();
		List<BehanceSDKCreativeFieldDTO> projectFields = getProjectFields();
		String projectTags = getProjectTagString();
		if (TextUtils.isEmpty(projectTitle) || TextUtils.isEmpty(projectDesc) || null == projectFields || projectFields.isEmpty()
				|| TextUtils.isEmpty(projectTags)) {
			disableActionBarRightNav();
		} else {
			enableActionBarRightNav();
		}
	}

	@Override
	protected int getTitle() {
		return R.string.bsdk_add_content_project_publish_title;
	}

	@Override
	public void onCreativeFieldsSelected(List<BehanceSDKCreativeFieldDTO> selectedField) {
		this.selectedField = selectedField;
		setSelectedCreativeFieldsText();
		enableOrDisablePublishButton();
	}

	@Override
	public void onCopyrightSettingSelected(BehanceSDKCopyrightOption copyRightSetting) {
		if (null != copyRightSetting)
			selectedCopyRight = copyRightSetting;
		projectCopyrightsButton.setText(selectedCopyRight.getDescription(getActivity()));
		enableOrDisablePublishButton();
	}

	private OnClickListener twitterLoginConfirmDialogClickListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			if (v.getId() == R.id.bsdkGenericAlertDialogOKBtn) {
				closeLoginToTwitterConfirmationDialog();
				launchTwitterLoginActivity();
			} else if (v.getId() == R.id.bsdkGenericAlertDialogNotOKBtn) {
				closeLoginToTwitterConfirmationDialog();
			}
		}
	};

	private OnClickListener facebookLoginConfirmDialogClickListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			if (v.getId() == R.id.bsdkGenericAlertDialogOKBtn) {
				closeLoginToFacebookConfirmationDialog();
				LoginToFacebook();
			} else if (v.getId() == R.id.bsdkGenericAlertDialogNotOKBtn) {
				closeLoginToFacebookConfirmationDialog();
			}
		}
	};
}
