/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import java.io.File;
import java.io.IOException;
import java.util.EnumSet;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnKeyListener;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.Toast;
import android.widget.ViewFlipper;

import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloud;
import com.adobe.creativesdk.foundation.adobeinternal.cloud.AdobeCloudManager;
import com.adobe.creativesdk.foundation.storage.AdobeAssetMimeTypes;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserOption;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.R;
import com.behance.sdk.factory.BehanceSDKImageValidatorFactory;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager;
import com.behance.sdk.managers.BehanceSDKProjectPublishWFManager.ProjectModuleChangeListener;
import com.behance.sdk.mobeta.android.dslv.DragSortListView;
import com.behance.sdk.project.modules.CreativeCloudAssetModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.behance.sdk.ui.activities.BehanceSDKCCLauncherActivity;
import com.behance.sdk.ui.activities.BehanceSDKPublishProjectActivity;
import com.behance.sdk.ui.adapters.BehanceSDKPublishProjectEditAdapter;
import com.behance.sdk.ui.adapters.BehanceSDKPublishProjectPreviewAdapter;
import com.behance.sdk.ui.dialogs.BehanceSDKEmbedModuleDialog;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.behance.sdk.util.BehanceSDKPublishImageValidationResult;
import com.behance.sdk.util.BehanceSDKUtils;


/**
 * @author ugs
 *
 */
public class BehanceSDKPublishProjectPreviewFragment extends BehanceSDKPublishProjectBaseFragment implements ProjectModuleChangeListener,  
												BehanceSDKPublishProjectEditAdapter.Callbacks, DragSortListView.RemoveListener, OnKeyListener, 
												BehanceSDKPublishProjectPreviewAdapter.Callbacks {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishProjectPreviewFragment.class);
	
	private static final String BUNDLE_KEY_CAMERA_IMAGE_URI_EDIT_PREVIEW_FRAGMENT = "BUNDLE_KEY_CAMERA_IMAGE_URI_EDIT_FRAGMENT";
	
	private static final String BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX_EDIT_PREVIEW_FRAGMENT = "BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX_EDIT_PREVIEW_FRAGMENT";
	
	private static final int VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX = 0;
	
	private static final int VIEW_FLIPPER_EDIT_MODULES_VIEW_INDEX = 1;
	
	private static final int ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA = 1;

	private static final int ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER = 6787;

	private BehanceSDKProjectPublishWFManager publishProjectManager =  BehanceSDKProjectPublishWFManager.getInstance();

	private ViewFlipper previewEditViewFlipper;
	
	//Preview View
    private ListView previewModulesListView;
    private BehanceSDKPublishProjectPreviewAdapter previewImageAdapter;
    private ProgressBar progressBar;
   
    //Edit View
	private DragSortListView editModulesDragSortListView;
	private View editEmptyProjectMessage;
	private Uri selectedCameraImageUri;
	private BehanceSDKPublishProjectEditAdapter editImageAdapter;
	
	

	@Override
	protected int getLayout() {
		return R.layout.bsdk_dialog_fragment_publish_project_preview;
	}

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		publishProjectManager.addProjectModuleChangeListener(this);
	}

	@Override
	public void onDetach() {
		super.onDetach();
		publishProjectManager.removeProjectModuleChangeListener(this);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
			super.onCreateView(inflater, container, savedInstanceState);
			previewEditViewFlipper = (ViewFlipper) rootView.findViewById(R.id.bsdkPublishProjectPreviewAndEditViewFlipper);
			initializePreviewProjectScreen();
	        initializeEditProjectScreen();
			int childIndexToDisplay = VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX;
			if(null!= savedInstanceState){
				selectedCameraImageUri = savedInstanceState.getParcelable(BUNDLE_KEY_CAMERA_IMAGE_URI_EDIT_PREVIEW_FRAGMENT);
				childIndexToDisplay = savedInstanceState.getInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX_EDIT_PREVIEW_FRAGMENT, 0);
			}else if(isEmptyProject()){
				childIndexToDisplay = VIEW_FLIPPER_EDIT_MODULES_VIEW_INDEX;
			}
	        
			if(childIndexToDisplay == VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX){
				displayPreviewModulesView(false, true);
				enableActionBarRightNav();
			}else{
				displayEditModulesView(false);
			}
			rootView.setFocusableInTouchMode(true);
			rootView.setOnKeyListener(this);
			return rootView;
	}

	
	@Override
	public boolean onKey(View v, int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() != KeyEvent.ACTION_DOWN) {
			backButtonPressed();
			return true;
		}
		return false;
	}
	private void initializePreviewProjectScreen() {
		//Preview View
		previewModulesListView = (ListView) rootView.findViewById(R.id.bsdkPublishProjectPreviewProjectModulesListView);
		progressBar = (ProgressBar)rootView.findViewById(R.id.bsdkPublishProjectPreviewPrgressBar);
	}
	
	@Override
    public void onViewLoadingComplete() {
        progressBar.setVisibility(View.GONE);
        previewModulesListView.setVisibility(View.VISIBLE);
    }

	private void initializeEditProjectScreen() {
		//Edit View
        editModulesDragSortListView = (DragSortListView) rootView.findViewById(R.id.bsdkPublishProjectEditProjectModulesListView);
        editEmptyProjectMessage = rootView.findViewById(R.id.bsdkPublishProjectEditEmptyProjectMessageLayout);
        View albums = rootView.findViewById(R.id.bsdkPublishProjectEditOptionAlbums);
		albums.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				BehanceSDKPublishProjectActivity activity = (BehanceSDKPublishProjectActivity) getActivity();
				activity.launchImageSelectionDialogForEditScreen();

			}
		});
		
		View camera = rootView.findViewById(R.id.bsdkPublishProjectEditOptionCamera);
		camera.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				if(BehanceSDKPermissionHelper.checkPermission(getActivity(),BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA)) {
					launchCameraToCapturePicture();
				}
				else
				{
					BehanceSDKPublishProjectActivity activity = (BehanceSDKPublishProjectActivity) getActivity();
					activity.requestPermisssionsForCamera();
				}
			}});
		
		View embed = rootView.findViewById(R.id.bsdkPublishProjectEditOptionEmbed);
		embed.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Context context = getActivity();
				BehanceSDKEmbedModuleDialog instance = BehanceSDKEmbedModuleDialog.getInstance(context);
				instance.show();
			}
		});

		View creativeCloud = rootView.findViewById(R.id.bsdkPublishProjectEditOptionCreativeCloud);
		AdobeCloudManager sharedCloudManager = AdobeCloudManager.getSharedCloudManager();
		AdobeCloud defaultCloud = sharedCloudManager.getDefaultCloud();

		boolean disableAssetBrowser = false;
		if(getActivity() != null) {
			BehanceSDKPublishProjectActivity activity = (BehanceSDKPublishProjectActivity) getActivity();
			disableAssetBrowser = activity.getCCAssetBrowserDisabled();
		}

		if(null == defaultCloud || disableAssetBrowser){
			creativeCloud.setVisibility(View.GONE);
		} else {
			creativeCloud.setOnClickListener(new View.OnClickListener() {

				@Override
				public void onClick(View v) {
					launchCreativeCloudAssetBrowser();


				}
			});
		}

	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		switch (requestCode) {
			case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_ALBUMS_VIEW : {
				if(grantResults.length > 0 &&
						grantResults[0] == PackageManager.PERMISSION_GRANTED) {
					//launchImageSelectionDialogForEditScreenCommon();
				} else {
					//Permission denied! Show some dialog about how it will limits the functionality!
				}
			}
		}
	}
	
	private void displayPreviewModulesView(boolean animateViewDisplay, boolean forceCreate) {
		List<ProjectModule> projectModules = publishProjectManager.getProjectModules();
		if(null == previewImageAdapter || forceCreate){
			previewImageAdapter = new BehanceSDKPublishProjectPreviewAdapter(getActivity(),R.layout.bsdk_adapter_publish_project_preview_project_fragment_list_item, projectModules);
			previewModulesListView.setAdapter(previewImageAdapter);
			previewImageAdapter.setCallbacks(this);
			float fraction = getResources().getFraction(R.fraction.bsdk_add_project_preview_fragment_screen_width_percentage, 1, 1);
			BehanceSDKUtils.setContentViewPaddings(getActivity(), previewModulesListView, fraction);
			previewModulesListView.setVisibility(View.INVISIBLE);
		}
        if (animateViewDisplay) {
        	previewEditViewFlipper.setInAnimation(getActivity(), R.anim.bsdk_publish_project_preview_enter_anim);
        	previewEditViewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_publish_project_edit_exit_anim);
		} else {
			previewEditViewFlipper.setInAnimation(null);
			previewEditViewFlipper.setOutAnimation(null);
		}
        previewEditViewFlipper.setDisplayedChild(VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX);
        updateNavigationControls();
	}

	private void displayEditModulesView(List<ProjectModule> projectModules, boolean animateViewDisplay) {
		if(null == projectModules || projectModules.isEmpty()){
			showEmptyProjectMessage();
		}
		else{
			editEmptyProjectMessage.setVisibility(View.GONE);
			editModulesDragSortListView.setVisibility(View.VISIBLE);
			initializeListView(projectModules);
		}
		if (animateViewDisplay) {
        	previewEditViewFlipper.setInAnimation(getActivity(), R.anim.bsdk_publish_project_edit_enter_anim);
        	previewEditViewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_publish_project_preview_exit_anim);
		} else {
			previewEditViewFlipper.setInAnimation(null);
			previewEditViewFlipper.setOutAnimation(null);
		}
		previewEditViewFlipper.setDisplayedChild(VIEW_FLIPPER_EDIT_MODULES_VIEW_INDEX);
		updateNavigationControls();
	}

	private void showEmptyProjectMessage() {
		disableActionBarRightNav();
		editEmptyProjectMessage.setVisibility(View.VISIBLE);
		editModulesDragSortListView.setVisibility(View.GONE);
		editImageAdapter = null;
	}
	
	private void displayEditModulesView(boolean animateViewDisplay) {
		List<ProjectModule> projectModules = publishProjectManager.getProjectModules();
		displayEditModulesView(projectModules, animateViewDisplay);
	}
	
	@Override
	protected int getBackButtonID() {
		if(isPreviewScreen())
			return R.id.bsdkPublishProjectPreviewFragmentTitlebarBackBtnLayout;
		else
			return super.getBackButtonID();
	}
	
	@Override
	protected int getForwardButtonID() {
		if(isPreviewScreen())
			return R.id.bsdkPublishProjectPreviewFragmentTitlebarActionBtnTxtView;
		else
			return super.getForwardButtonID();
	}

	private boolean isPreviewScreen() {
		return null != previewEditViewFlipper && previewEditViewFlipper.getDisplayedChild() == VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX;
	}
	
	@Override
	protected int getTitleViewID() {
		if(isPreviewScreen())
			return R.id.bsdkPublishProjectPreviewFragmentTitlebarTitleTxtView;
		else
			return super.getTitleViewID();
	}
	
	@Override
	protected int getForwardButtonText(){
		if(isPreviewScreen())
			return super.getForwardButtonText();
		else
			return R.string.bsdk_add_project_edit_project_fragment_right_navigation;
	}
	
	@Override
	protected int getTitle() {
		if(isPreviewScreen())
			return  R.string.bsdk_add_project_preview_project_fragment_title;
		else
			return  R.string.bsdk_add_project_edit_project_fragment_title;
	}
	
	@Override
	protected void updateNavigationControls() {
		super.updateNavigationControls();
		
		View editProject = rootView.findViewById(R.id.bsdkPublishProjectPreviewFragmentTitlebarEditActionBtnTxtView);
		editProject.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				displayEditModulesView(true);
			}
		});
		if(null != previewEditViewFlipper){
			int displayedChild = previewEditViewFlipper.getDisplayedChild();
			if(displayedChild == VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX){
				editProject.setVisibility(View.VISIBLE);
			}else{
				editProject.setVisibility(View.GONE);
			}
			
		}
	}
	
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode){
			case ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA:
				if (resultCode == Activity.RESULT_OK) {
					logger.debug("Camera Activity returned with ok result. [Photo Uri - %s]", selectedCameraImageUri);
					if (selectedCameraImageUri != null) {
						File cameraImageFile = new File(selectedCameraImageUri.getPath());
						publishProjectManager.addImageSelectedForProject(cameraImageFile);
						//add the new image to the gallery and reload the images.
						BehanceSDKUtils.addImageToGallery(getActivity(), cameraImageFile.getAbsolutePath());
						List<ProjectModule> modules = publishProjectManager.getProjectModules();
						notifyEditViewGridAdapter(modules);
						enableActionBarRightNav();
					}
				} else {
					logger.debug("Camera Activity returned with not ok result. [Result - %s]", resultCode);
				}
				break;
			case ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER:
				if(resultCode == Activity.RESULT_OK) {
					Bundle bundle = data.getExtras();
					List<File> downloadedFiles = (List<File>) bundle.getSerializable(BehanceSDKCCLauncherActivity.ACTIVITY_RESULT_CC_SUCCESSFULLY_DOWNLOADED_FILES);
					if (null != downloadedFiles && false == downloadedFiles.isEmpty()) {
						for (File imageFile : downloadedFiles) {
							ProjectModule module = new CreativeCloudAssetModule(imageFile);
							publishProjectManager.addModuleToCurrentProject(module);
						}
						List<ProjectModule> modules = publishProjectManager.getProjectModules();
						notifyEditViewGridAdapter(modules);
						enableActionBarRightNav();
					}
				}
				break;
			default:
				break;
		}
	}

	@Override
	protected void enableActionBarRightNav() {
		if(false == isEmptyProject())
			super.enableActionBarRightNav();
	}

	
	private boolean isEmptyProject() {
		List<ProjectModule> projectModules = publishProjectManager.getProjectModules();
		if(null == projectModules || projectModules.isEmpty())
			return true;
		return false;
	}
	
	@Override
	protected void nextButtonPressed() {
		int displayedChild = previewEditViewFlipper.getDisplayedChild();
		if(displayedChild == VIEW_FLIPPER_PREVIEW_MODULES_VIEW_INDEX){
			super.nextButtonPressed();
		}else{
			displayPreviewModulesView(true, false);
		}
	}
	
	@Override
	protected void backButtonPressed() {
		int displayedChild = previewEditViewFlipper.getDisplayedChild();
		if(displayedChild == VIEW_FLIPPER_EDIT_MODULES_VIEW_INDEX && false  == isEmptyProject()){
			displayPreviewModulesView(true, false);
		}else{
			super.backButtonPressed();
		}
	}
	
	@Override
	public boolean closeWorkflow() {
		return true;
	}

    @Override
	public void onSaveInstanceState(Bundle arg0) {
		super.onSaveInstanceState(arg0);
		if(null != previewEditViewFlipper)
			arg0.putInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX_EDIT_PREVIEW_FRAGMENT, previewEditViewFlipper.getDisplayedChild());
		if(null != selectedCameraImageUri)
			arg0.putParcelable(BUNDLE_KEY_CAMERA_IMAGE_URI_EDIT_PREVIEW_FRAGMENT, selectedCameraImageUri);
	}
    
    //EDIT Screen

	private void launchCreativeCloudAssetBrowser() {
		Activity context = getActivity();
		Intent detailsIntent = new Intent(context, BehanceSDKCCLauncherActivity.class);
		EnumSet<AdobeAssetMimeTypes> mimeTypes = EnumSet.of(AdobeAssetMimeTypes.MIMETYPE_GIF, AdobeAssetMimeTypes.MIMETYPE_JPEG, AdobeAssetMimeTypes.MIMETYPE_PNG);
		EnumSet<AdobeUXAssetBrowserOption> browserOptions = EnumSet.of(AdobeUXAssetBrowserOption.ENABLE_MULTI_SELECT,AdobeUXAssetBrowserOption.SHOW_MULTI_SELECT_ON_POPUP);
		detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_ALLOWED_MIME_TYPES, mimeTypes);
		detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_ASSET_BROWSER_OPTIONS, browserOptions);
		detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_IMAGE_VALIDATOR_TYPE, BehanceSDKImageValidatorFactory.PUBLISH_PROJECT_IMAGE_VALIDATOR);
		context.startActivityForResult(detailsIntent, ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER);
	}
    
    public void launchCameraToCapturePicture() {
		try {
			File photoFile = BehanceSDKUtils.createTempImageFile();
			Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
			// Ensure that there's a camera activity to handle the intent
			if (takePictureIntent.resolveActivity(getActivity().getPackageManager()) != null) {
				// Continue only if the File was successfully created
				if (photoFile != null) {

					selectedCameraImageUri = Uri.fromFile(photoFile);
					takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, selectedCameraImageUri);
					startActivityForResult(takePictureIntent, ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA);
				}
			} else {
				logger.error("Camera intent not found");
				Toast.makeText(getActivity(), R.string.bsdk_add_wip_pick_source_view_camera_launch_error, Toast.LENGTH_LONG).show();
			}
		} catch (IOException ex) {
			logger.error(ex, "Problem creating temp file to capture image");
			Toast.makeText(getActivity(), R.string.bsdk_add_wip_pick_source_view_camera_launch_error, Toast.LENGTH_LONG).show();
		}
	}

    @Override
	public void projectModulesChanged(List<ProjectModule> modules) {
		if(null == modules || modules.isEmpty()){
			disableActionBarRightNav();
		}else{
			enableActionBarRightNav();
			notifyEditViewGridAdapter(modules);
		}
		notifyPreviewViewGridAdapter();
	}
    
    private void notifyPreviewViewGridAdapter() {
    	if(null != previewImageAdapter)
    		previewImageAdapter.notifyDataSetChanged();
	}

	private void notifyEditViewGridAdapter(List<ProjectModule> modules) {
		if(null != editImageAdapter){
			editImageAdapter.notifyDataSetChanged();
		}
		else{
			displayEditModulesView(modules, false);
		}
	}
    
    private void initializeListView(List<ProjectModule> projectModules) {
    	editImageAdapter = new BehanceSDKPublishProjectEditAdapter(getActivity(),R.layout.bsdk_adapter_publish_project_edit_fragment_grid_item, projectModules);
		editModulesDragSortListView.setAdapter(editImageAdapter);
		editModulesDragSortListView.setRemoveListener(this);
		editImageAdapter.setCallBack(this);
		DragSortListView.DropListener onDrop = new DragSortListView.DropListener() {
			@Override
			public void drop(int from, int to) {
				publishProjectManager.reorderSelectedModules(from, to);
				editModulesDragSortListView.moveCheckState(from, to);
				editImageAdapter.notifyDataSetChanged();
				notifyPreviewViewGridAdapter();
			}
		};
		editModulesDragSortListView.setDropListener(onDrop);
		
	}

	@Override
	public void onRotate(ImageModule projectModule) {
		publishProjectManager.rotateImage(projectModule);
	}
	
	@Override
	public void remove(int which) {
		ProjectModule item = editImageAdapter.getItem(which);
		if (item.getType().equals(ProjectModuleTypes.IMAGE)) {
			ImageModule module = (ImageModule) item;
			module.clearRotation();
		}
		publishProjectManager.removeProjectModule(item);
		if(publishProjectManager.isEmptyProject()){
			showEmptyProjectMessage();
		}else{
			editImageAdapter.notifyDataSetChanged();
		}
	}

	@Override
	public void rotateImage(ImageModule module) {
		notifyPreviewViewGridAdapter();	
	}



	
}
