/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.ui.fragments;

import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.DialogFragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnFocusChangeListener;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.view.inputmethod.InputMethodManager;
import android.widget.*;

import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.BehanceSDKPublishWIPOptions;
import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKPublishWIPServiceParamsDTO;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.dto.BehanceSDKWIPDTO;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.fragments.headless.BehanceSDKPublishWIPHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.AddWIPManager;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.behance.sdk.services.BehanceSDKPublishWIPService;
import com.behance.sdk.ui.activities.BehanceSDKLoginToTwitterActivity;
import com.behance.sdk.ui.adapters.BehanceSDKWIPListAdapter;
import com.behance.sdk.ui.dialogs.BehanceSDKGenericAlertDialog;
import com.behance.sdk.ui.dialogs.BehanceSDKNewWIPTitleDialog;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUtils;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.login.DefaultAudience;
import com.facebook.login.LoginBehavior;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;

import java.util.Arrays;
import java.util.List;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKPublishWIPDialogFragment extends DialogFragment implements OnClickListener, BehanceSDKPublishWIPHeadlessFragment.Callbacks,
		BehanceSDKNewWIPTitleDialog.Callbacks, OnKeyListener {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishWIPDialogFragment.class);

	private static final String FACEBOOK_PERMISSION_PUBLISH_ACTIONS = "publish_actions";
	private static final String HEADLESS_FRAGMENT_TAG_ADD_WIP = "PUBLISH_WIP_TO_BEHANCE_FRAGMENT_HEADLESS_FRAGMENT_TAG_ADD_WIP";
	private static final String BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX = "BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX";
	private static final String BUNDLE_KEY_SELECTED_WIP_TITLE = "BUNDLE_KEY_SELECTED_WIP_TITLE";
	private static final String BUNDLE_KEY_SELECTED_WIP_ID = "BUNDLE_KEY_SELECTED_WIP_ID";
	private static final String BUNDLE_KEY_USER_ENTERED_NEW_WIP_TITLE = "BUNDLE_KEY_NEW_WIP_TITLE_ENTERED_BY_USER";
	private static final String BUNDLE_KEY_SHARE_ON_FACEBOOK_SELECTED = "BUNDLE_KEY_SHARE_ON_FACEBOOK_SELECTED";
	private static final String BUNDLE_KEY_SHARE_ON_TWITTER_SELECTED = "BUNDLE_KEY_SHARE_ON_TWITTER_SELECTED";

	private static final String ARGS_KEY_WIP_IMAGE_MODULE = "ARGS_KEY_WIP_IMAGE_MODULE";
	private static final String ARGS_KEY_WIP_TITLE = "ARGS_KEY_WIP_TITLE";
	private static final String ARGS_KEY_WIP_DESCRIPTION = "ARGS_KEY_WIP_DESCRIPTION";
	private static final String ARGS_KEY_WIP_ID = "ARGS_KEY_WIP_ID";
	private static final String ARGS_KEY_WIP_TAGS = "ARGS_KEY_WIP_TAGS";

	private static final int VIEW_FLIPPER_PUBLISH_VIEW_INDEX = 0;
	private static final int VIEW_FLIPPER_PICK_WIP_VIEW_INDEX = 1;

	public static final int TWITTER_LOGIN_ACTIVITY_REQUEST_CODE = 56789;

	private View rootView;
	private ViewFlipper viewFlipper;
	private TextView publishWIPBtnTxtView;
	private TextView showNewWIPDialogBtnTxtView;
	private TextView titleBarTitleTextView;
	private ImageView wipThumbnailImageView;
	private View publishWipProgressbar;
	private EditText wipDescEditTxt;
	private EditText wipTitleEditTxt;
	private View showPickWIPsViewBtn;
	private EditText wipTagsEditTxt;
	private TextView selectedWIPTitleTxtView;
	private ListView userWIPsListView;
	private BehanceSDKGenericAlertDialog loginToTwitterConfirmationDialog;
	private BehanceSDKGenericAlertDialog loginToFacebookConfirmationDialog;
	private BehanceSDKNewWIPTitleDialog newWIPTitleDialog;
	private View pickWIPRootContainer;
	private View publishWIPInfoRootContainer;
    CheckBox adultContentCheckBox;
	private ImageView shareOnTwitterBtnImageView;
	private ImageView shareOnFacebookBtnImageView;

	private BehanceSDKPublishWIPHeadlessFragment headlessFragment;
	private ImageModule selectedWIPImageModule;
	private int selectedWIPId;
	private String selectedWIPTitle;
	private boolean userEnteredNewWIPTitle;
	private boolean viewPaused = false;
	private boolean closeThisViewOnResume = false;
	private Callbacks callbacks;
	private boolean shareWIPOnFacebook;
	private boolean shareWIPOnTwitter;
	private BehanceSDKSocialAccountManager socialAccountManager;
	private BehanceSDKSocialAccountDTO facebookAccount;
	private BehanceSDKSocialAccountDTO twitterAccount;

	private AddWIPManager wipManager;
	private View shareOptionsView;
	private CallbackManager callbackManager;
	LoginResult savedLoginResult = null;

	public interface Callbacks {

		public void onPublishWIPViewClose();
	}

	public static BehanceSDKPublishWIPDialogFragment getInstance(Callbacks callbacks, BehanceSDKPublishWIPOptions options) {
		BehanceSDKPublishWIPDialogFragment instance = new BehanceSDKPublishWIPDialogFragment();
		Bundle args = new Bundle();
		args.putSerializable(ARGS_KEY_WIP_IMAGE_MODULE, options.getWipRevisionImageModule());
        args.putInt(ARGS_KEY_WIP_ID, options.getExistingWIPId());
        args.putString(ARGS_KEY_WIP_DESCRIPTION, options.getWipDescription());
        args.putString(ARGS_KEY_WIP_TAGS, options.getWipTags());
        args.putString(ARGS_KEY_WIP_TITLE, options.getWipTitle());

		instance.setArguments(args);
		instance.setCallbacks(callbacks);
		return instance;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkPublishWIPToBehanceViewTheme);
		savedLoginResult = null;
		callbackManager = CallbackManager.Factory.create();
		LoginManager.getInstance().registerCallback(callbackManager, new FacebookCallback<LoginResult>() {
			@Override
			public void onSuccess(LoginResult loginResult) {
				handleFacebookAuthActivityResult(loginResult, null);
			}

			@Override
			public void onCancel() {
				handleFacebookAuthActivityResult(null, null);
			}

			@Override
			public void onError(FacebookException e) {
				handleFacebookAuthActivityResult(null, e);
			}
		});

	}

	@Override
	public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_publish_wip_to_behance, container, false);

		Bundle arguments = getArguments();
		if (arguments != null) {
			selectedWIPImageModule = (ImageModule) arguments.getSerializable(ARGS_KEY_WIP_IMAGE_MODULE);
		}

		socialAccountManager = BehanceSDKSocialAccountManager.getInstance(getActivity());
		twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
		facebookAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.FACEBOOK);

		int childIndexToDisplay = 0;
		userEnteredNewWIPTitle = false;
		shareWIPOnFacebook = false;
		shareWIPOnTwitter = false;
		if (savedInstanceState != null) {
			childIndexToDisplay = savedInstanceState.getInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX, 0);
			selectedWIPId = savedInstanceState.getInt(BUNDLE_KEY_SELECTED_WIP_ID, 0);
			selectedWIPTitle = savedInstanceState.getString(BUNDLE_KEY_SELECTED_WIP_TITLE, null);
			userEnteredNewWIPTitle = savedInstanceState.getBoolean(BUNDLE_KEY_USER_ENTERED_NEW_WIP_TITLE, false);
			shareWIPOnTwitter = savedInstanceState.getBoolean(BUNDLE_KEY_SHARE_ON_TWITTER_SELECTED, false);
			shareWIPOnFacebook = savedInstanceState.getBoolean(BUNDLE_KEY_SHARE_ON_FACEBOOK_SELECTED, false);
		} else {
			if (facebookAccount != null) {
				shareWIPOnFacebook = facebookAccount.isSharingEnabledLastTime();
			} else {
				shareWIPOnFacebook = false;
			}

			if (twitterAccount != null) {
				shareWIPOnTwitter = twitterAccount.isSharingEnabledLastTime();
			} else {
				shareWIPOnTwitter = false;
			}
		}

		if (shareWIPOnFacebook) {
			if (!isFacebookAccountAuthenticatedToPublish()) {
				shareWIPOnFacebook = false;
			}
		}

		if (shareWIPOnTwitter) {
			if (!isTwitterAccountAuthenticated()) {
				shareWIPOnTwitter = false;
			}
		}

		headlessFragment = (BehanceSDKPublishWIPHeadlessFragment) getActivity().getSupportFragmentManager().findFragmentByTag(
				HEADLESS_FRAGMENT_TAG_ADD_WIP);
		if (headlessFragment == null) {
			headlessFragment = new BehanceSDKPublishWIPHeadlessFragment();
			getActivity().getSupportFragmentManager().beginTransaction().add(headlessFragment, HEADLESS_FRAGMENT_TAG_ADD_WIP).commit();
		}
		headlessFragment.setCallbacks(this);

		loadUserWIPs();

		View backBtn = rootView.findViewById(R.id.bsdkPublishWipTitlebarBackBtnImageView);
		backBtn.setOnClickListener(this);

		titleBarTitleTextView = (TextView) rootView.findViewById(R.id.bsdkPublishWipTitlebarTitleTxtView);
		publishWIPBtnTxtView = (TextView) rootView.findViewById(R.id.bsdkPublishWipTitlebarPublishBtnTxtView);
		publishWIPBtnTxtView.setOnClickListener(this);
		showNewWIPDialogBtnTxtView = (TextView) rootView.findViewById(R.id.bsdkPublishWipTitlebarNewWIPBtnTxtView);
		showNewWIPDialogBtnTxtView.setOnClickListener(this);

		viewFlipper = (ViewFlipper) rootView.findViewById(R.id.bsdkPublishWipViewFlipper);
		publishWIPInfoRootContainer = rootView.findViewById(R.id.bsdkPublishWipPublishViewRootContainer);
		pickWIPRootContainer = rootView.findViewById(R.id.bsdkPublishWipPickWIPRootContainer);

		wipThumbnailImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishWipPublishViewWIPThumbnailImageView);
		publishWipProgressbar = rootView.findViewById(R.id.bsdkPublishWipPublishViewProgressBar);
		wipTitleEditTxt = (EditText) rootView.findViewById(R.id.bsdkPublishWipPublishViewWIPNameEditTxt);
		wipTitleEditTxt.addTextChangedListener(allTextFieldsTextWatcher);
		wipDescEditTxt = (EditText) rootView.findViewById(R.id.bsdkPublishWipPublishViewWIPDescriptionEditTxt);
		wipDescEditTxt.addTextChangedListener(allTextFieldsTextWatcher);
		wipTagsEditTxt = (EditText) rootView.findViewById(R.id.bsdkPublishWipPublishViewWIPTagsEditTxt);
		wipTagsEditTxt.addTextChangedListener(allTextFieldsTextWatcher);
		selectedWIPTitleTxtView = (TextView) rootView.findViewById(R.id.bsdkPublishWipPublishViewSelectedWIPNameTxtView);
		showPickWIPsViewBtn = rootView.findViewById(R.id.bsdkPublishWipPublishViewShowPickWIPsViewBtn);
		showPickWIPsViewBtn.setOnClickListener(this);
        adultContentCheckBox = (CheckBox) rootView.findViewById(R.id.bsdkPublishWIPPublishViewAdultContentCheckbox);

        // this is to make sure the blue focus background is displayed
		View wipTitleContainer = rootView.findViewById(R.id.bsdkPublishWipPublishViewWIPNameContainer);
		wipTitleEditTxt.setOnFocusChangeListener(getFocusChangeListerForLayout(wipTitleContainer));

		userWIPsListView = (ListView) rootView.findViewById(R.id.bsdkPublishWipPickWIPListView);
		userWIPsListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				onUserWIPsItemClick(parent, view, position, id);
			}
		});

		shareOnTwitterBtnImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishWipPublishViewTwitterShareImageView);
		shareOnTwitterBtnImageView.setOnClickListener(this);

		shareOnFacebookBtnImageView = (ImageView) rootView.findViewById(R.id.bsdkPublishWipPublishViewFacebookShareImageView);
		shareOnFacebookBtnImageView.setOnClickListener(this);

		wipManager = AddWIPManager.getInstance();
		shareOptionsView = rootView.findViewById(R.id.bsdkPublishWipPublishViewShareOptionsContainer);

        if (arguments != null && savedInstanceState == null) {
            // pre-fill WIP details on first launch
            selectedWIPId = arguments.getInt(ARGS_KEY_WIP_ID, 0);
            if(selectedWIPId <= 0) {
                String newWipTitle = arguments.getString(ARGS_KEY_WIP_TITLE, null);
                if(!isStringEmpty(newWipTitle)) {
                    userEnteredNewWIPTitle = true;
                    wipTitleEditTxt.setText(newWipTitle);
                }
            } else {
                userEnteredNewWIPTitle = false;
            }

            wipDescEditTxt.setText(arguments.getString(ARGS_KEY_WIP_DESCRIPTION, ""));
            wipTagsEditTxt.setText(arguments.getString(ARGS_KEY_WIP_TAGS, ""));
        }

		if (wipManager.isHideTwitterSharing()) {
			shareOnTwitterBtnImageView.setVisibility(View.GONE);
		}
		if (wipManager.isHideFacebookSharing()) {
			shareOnFacebookBtnImageView.setVisibility(View.GONE);
		}

		if (wipManager.isHideTwitterSharing() && wipManager.isHideFacebookSharing()) {
			shareOptionsView.setVisibility(View.INVISIBLE);
		}

		if (childIndexToDisplay == VIEW_FLIPPER_PUBLISH_VIEW_INDEX) {
			if (selectedWIPImageModule != null) {
				displayPublishView(false);
			}
		} else if (childIndexToDisplay == VIEW_FLIPPER_PICK_WIP_VIEW_INDEX) {
			displayPickWIPView(false);
		}

		setContentViewDimensions(publishWIPInfoRootContainer);
		setContentViewDimensions(pickWIPRootContainer);
		updateShareOnFacebookBtnImage();
		updateShareOnTwitterBtnImage();

		getDialog().setOnKeyListener(this);
		return rootView;
	}

	private void setContentViewDimensions(View contentView) {
		boolean displayContentViewInFullScreen = getResources().getBoolean(R.bool.bsdk_show_view_full_screen);
		if (!displayContentViewInFullScreen) {
			DisplayMetrics displayMetrics = getResources().getDisplayMetrics();
			int deviceWidth = displayMetrics.widthPixels;
			int deviceHeight = displayMetrics.heightPixels;

			if (deviceWidth > 0 && deviceHeight > 0) {
				int contentContainerWidth = (int) (deviceWidth * BehanceSDKConstants.CONTENT_VIEW_WIDTH_PERCENTAGE);
				LayoutParams layoutParams = contentView.getLayoutParams();
				if (layoutParams != null) {
					layoutParams.width = contentContainerWidth;
					layoutParams.height = LayoutParams.WRAP_CONTENT;
					contentView.setLayoutParams(layoutParams);
				}
			}
		}
	}

	private OnFocusChangeListener getFocusChangeListerForLayout(final View view) {
		return new OnFocusChangeListener() {

			@Override
			public void onFocusChange(View v, boolean hasFocus) {
				if (hasFocus) {
					view.setActivated(true);
				} else
					view.setActivated(false);
			}
		};
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX, viewFlipper.getDisplayedChild());
		outState.putInt(BUNDLE_KEY_SELECTED_WIP_ID, selectedWIPId);
		outState.putString(BUNDLE_KEY_SELECTED_WIP_TITLE, selectedWIPTitle);
		outState.putBoolean(BUNDLE_KEY_USER_ENTERED_NEW_WIP_TITLE, userEnteredNewWIPTitle);
		outState.putBoolean(BUNDLE_KEY_SHARE_ON_TWITTER_SELECTED, shareWIPOnTwitter);
		outState.putBoolean(BUNDLE_KEY_SHARE_ON_FACEBOOK_SELECTED, shareWIPOnFacebook);
	}

	@Override
	public void onPause() {
		super.onPause();
		this.viewPaused = true;
	}

	@Override
	public void onResume() {
		super.onResume();
		this.viewPaused = false;
		if (this.closeThisViewOnResume) {
			this.closeThisViewOnResume = false;
			closeThisView();
		}
	}

	@Override
	public void onDetach() {
		super.onDetach();
		if (headlessFragment != null) {
			headlessFragment.setCallbacks(null);
		}
		closeLoginToFacebookConfirmationDialog();
		closeLoginToTwitterConfirmationDialog();
	}

	@Override
	public void onClick(View v) {
		if (v.getId() == R.id.bsdkPublishWipTitlebarBackBtnImageView) {
			handleBackBtnClick();
		} else if (v.getId() == R.id.bsdkPublishWipTitlebarPublishBtnTxtView) {
			NetworkInfo networkInfo = ((ConnectivityManager) getActivity().getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
			if (networkInfo != null && networkInfo.isConnected()) {
				invokePublishWIPService();
			} else {
				Toast toast = Toast.makeText(getActivity(), getString(R.string.bsdk_add_wip_view_connection_error_msg), Toast.LENGTH_LONG);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
			}
		} else if (v.getId() == R.id.bsdkPublishWipTitlebarNewWIPBtnTxtView) {
			displayNewWIPTitleDialog();
		} else if (v.getId() == R.id.bsdkPublishWipPublishViewShowPickWIPsViewBtn) {
			handleShowPickWIPsViewClick();
		} else if (v.getId() == R.id.bsdkPublishWipPublishViewTwitterShareImageView) {
			handleEnableShareOnTwitterBtnClick();
		} else if (v.getId() == R.id.bsdkPublishWipPublishViewFacebookShareImageView) {
			handleEnableShareOnFacebookBtnClick();
		}
	}

	private void launchTwitterLoginActivity() {
		Intent detailIntent = new Intent(getActivity(), BehanceSDKLoginToTwitterActivity.class);
		startActivityForResult(detailIntent, TWITTER_LOGIN_ACTIVITY_REQUEST_CODE);
		closeLoginToTwitterConfirmationDialog();
	}

	private void closeLoginToTwitterConfirmationDialog() {
		if (loginToTwitterConfirmationDialog != null) {
			loginToTwitterConfirmationDialog.dismiss();
		}
	}

	private void closeLoginToFacebookConfirmationDialog() {
		if (loginToFacebookConfirmationDialog != null) {
			loginToFacebookConfirmationDialog.dismiss();
		}
	}

	private void handleEnableShareOnFacebookBtnClick() {
		if (facebookAccount != null) {
			if (shareWIPOnFacebook) {
				shareWIPOnFacebook = false;
				updateShareOnFacebookBtnImage();
			} else {
				checkFacebookAuthAndEnableSharing();
			}
		}
	}

	private void checkFacebookAuthAndEnableSharing() {
		if (isFacebookAccountAuthenticatedToPublish()) {
			shareWIPOnFacebook = true;
			updateShareOnFacebookBtnImage();
			updateFacebookAccountAuthStatus();
		} else {
			displayFacebookLoginConfirmationDialog();
		}
	}

	private void displayFacebookLoginConfirmationDialog() {
		loginToFacebookConfirmationDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(),
				R.string.bsdk_social_account_facebook_login_confirmation_title, R.string.bsdk_social_account_facebook_login_confirmation_body,
				R.string.bsdk_social_account_facebook_login_confirmation_ok_btn_label,
				R.string.bsdk_social_account_facebook_login_confirmation_cancel_btn_label);
		loginToFacebookConfirmationDialog.setOnNotOKBtnClickListener(facebookLoginConfirmDialogClickListener);
		loginToFacebookConfirmationDialog.setOnOKBtnClickListener(facebookLoginConfirmDialogClickListener);
		loginToFacebookConfirmationDialog.show();
	}

	private void LoginToFacebook() {
		hideKeyboard();

		List<String> permissions = Arrays.asList(FACEBOOK_PERMISSION_PUBLISH_ACTIONS);
		LoginManager.getInstance().setLoginBehavior(LoginBehavior.NATIVE_WITH_FALLBACK);
		LoginManager.getInstance().setDefaultAudience(DefaultAudience.EVERYONE);
		LoginManager.getInstance().logInWithPublishPermissions(this, permissions);
	}

	private boolean isFacebookAccountAuthenticatedToPublish() {
		/*Session activeSession = Session.getActiveSession();
		if (activeSession != null && activeSession.isOpened()) {
			if (activeSession.isPermissionGranted(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
				return true;
			}
		} else {
			Session session = new Session.Builder(getActivity()).setApplicationId(facebookAccount.getAccountClientId()).build();
			Session.setActiveSession(session);
			if (session.isOpened()) {
				if (session.isPermissionGranted(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
					return true;
				}
			} else if (session.getState() == SessionState.CREATED_TOKEN_LOADED) {
				// session is not open, open it
				Session.OpenRequest openRequest = createFacebookSessionRequest();
				session.openForPublish(openRequest);
				if (session.isPermissionGranted(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
					return true;
				}
			}
		}
		return false;*/
		if (savedLoginResult != null && !savedLoginResult.getAccessToken().isExpired()) {
			if (savedLoginResult.getAccessToken().getPermissions().contains(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
				return true;
			}
		}
		return false;
	}

	/*private Session.OpenRequest createFacebookSessionRequest() {
		Session.OpenRequest openRequest = new Session.OpenRequest(this);
		openRequest.setDefaultAudience(SessionDefaultAudience.EVERYONE);
		List<String> permissions = Arrays.asList(FACEBOOK_PERMISSION_PUBLISH_ACTIONS);
		openRequest.setPermissions(permissions);
		openRequest.setLoginBehavior(SessionLoginBehavior.SSO_WITH_FALLBACK);
		return openRequest;
	}*/

	private void updateFacebookAccountAuthStatus() {
		boolean updateFacebookAccount = false;
		boolean facebookAccountAuthenticated = isFacebookAccountAuthenticatedToPublish();
		if (facebookAccountAuthenticated && !facebookAccount.isUserAuthenticated()) {
			facebookAccount.setUserAuthenticated(true);
			updateFacebookAccount = true;
		} else if (!facebookAccountAuthenticated && facebookAccount.isUserAuthenticated()) {
			facebookAccount.setUserAuthenticated(false);
			updateFacebookAccount = true;
		}
		if (updateFacebookAccount) {
			socialAccountManager.updateAccount(facebookAccount);
		}
	}

	private void updateShareOnFacebookBtnImage() {
		if (shareWIPOnFacebook) {
			shareOnFacebookBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_facebook_on);
		} else {
			shareOnFacebookBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_facebook_off);
		}
	}

	private boolean isTwitterAccountAuthenticated() {
		if (twitterAccount != null) {
			String accessToken = twitterAccount.getUserAuthToken();
			String accessTokenSecret = twitterAccount.getAdditionalData();
			if (accessToken == null || accessToken.isEmpty() || accessTokenSecret == null || accessTokenSecret.isEmpty()) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}

	private void handleEnableShareOnTwitterBtnClick() {
		if (twitterAccount != null) {
			if (shareWIPOnTwitter) {
				shareWIPOnTwitter = false;
			} else {
				checkIfUserHasLoggedIntoTwitter();
			}
			updateShareOnTwitterBtnImage();
		}
	}

	private void checkIfUserHasLoggedIntoTwitter() {
		String accessToken = twitterAccount.getUserAuthToken();
		String accessTokenSecret = twitterAccount.getAdditionalData();
		if (accessToken == null || accessToken.isEmpty() || accessTokenSecret == null || accessTokenSecret.isEmpty()) {
			displayTwitterLoginConfirmationDialog();
		} else {
			shareWIPOnTwitter = true;
			updateShareOnTwitterBtnImage();
		}
	}

	private void displayTwitterLoginConfirmationDialog() {
		loginToTwitterConfirmationDialog = BehanceSDKGenericAlertDialog.getInstance(getActivity(),
				R.string.bsdk_social_account_twitter_login_confirmation_title, R.string.bsdk_social_account_twitter_login_confirmation_body,
				R.string.bsdk_social_account_twitter_login_confirmation_ok_btn_label,
				R.string.bsdk_social_account_twitter_login_confirmation_cancel_btn_label);
		loginToTwitterConfirmationDialog.setOnNotOKBtnClickListener(twitterLoginConfirmDialogClickListener);
		loginToTwitterConfirmationDialog.setOnOKBtnClickListener(twitterLoginConfirmDialogClickListener);
		loginToTwitterConfirmationDialog.show();
	}

	private void updateShareOnTwitterBtnImage() {
		if (shareWIPOnTwitter) {
			shareOnTwitterBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_twitter_on);
		} else {
			shareOnTwitterBtnImageView.setImageResource(R.drawable.bsdk_icon_addcontent_publish_shareon_twitter_off);
		}
	}

	@Override
	public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() != KeyEvent.ACTION_DOWN) {
			handleBackBtnClick();
			return true;
		}
		return false;
	}

	private void loadUserWIPs() {
		if (!headlessFragment.isGetUserWIPsTaskInProgress() && headlessFragment.getUserWIPsList() == null) {
			headlessFragment.loadUserWIPsFromServer();
		}
	}

	private void handleShowPickWIPsViewClick() {
		if (!headlessFragment.isGetUserWIPsTaskInProgress()) {
			List<BehanceSDKWIPDTO> userWIPsList = headlessFragment.getUserWIPsList();
			if (userWIPsList != null && !userWIPsList.isEmpty()) {
				displayPickWIPView(true);
			} else {
				showPickWIPsViewBtn.setVisibility(View.GONE);
				wipTitleEditTxt.setVisibility(View.VISIBLE);
			}
		}
	}

	private void handleBackBtnClick() {
		hideKeyboard();
		int displayedChild = viewFlipper.getDisplayedChild();
		if (displayedChild == VIEW_FLIPPER_PICK_WIP_VIEW_INDEX) {
			displayPublishView(true);
		} else {
            AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishUXCancel,
					AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceWIP);
			if(null != selectedWIPImageModule && selectedWIPImageModule.getType() == ProjectModuleTypes.CREATIVECLOUD_ASSET){
				selectedWIPImageModule.deleteFromFileSystem();
			}
			closeThisView();
		}
	}

	private void setUserWIPsListAdapter() {
		if (getActivity() != null) {
			int displayedChildIndex = viewFlipper.getDisplayedChild();
			if (displayedChildIndex == VIEW_FLIPPER_PICK_WIP_VIEW_INDEX) {
				List<BehanceSDKWIPDTO> userWIPsList = headlessFragment.getUserWIPsList();
				if (userWIPsList != null) {
					BehanceSDKWIPListAdapter adapter = (BehanceSDKWIPListAdapter) userWIPsListView.getAdapter();
					if (adapter == null) {
						adapter = new BehanceSDKWIPListAdapter(getActivity(), userWIPsList, selectedWIPId);
						userWIPsListView.setAdapter(adapter);
					} else {
						adapter.setSelectedWIPId(selectedWIPId);
						adapter.notifyDataSetChanged();
					}
				}
			}
		}
	}

	private void displayNewWIPTitleDialog() {
		newWIPTitleDialog = BehanceSDKNewWIPTitleDialog.getInstance(getActivity(), this);
		newWIPTitleDialog.show();
	}

	private void closeThisView() {
		closeThisView(true);
	}

	private void closeThisView(boolean notifyCallbacks) {
		if (this.viewPaused) {
			this.closeThisViewOnResume = true;
		} else {
			if (notifyCallbacks) {
				if (callbacks != null) {
					rootView.post(new Runnable() {

						@Override
						public void run() {
							callbacks.onPublishWIPViewClose();
						}
					});
				}
			}
			dismissAllowingStateLoss();
		}
	}

	private void invokePublishWIPService() {
		String wipDescStr = wipDescEditTxt.getText().toString();
		String wipTitleStr = wipTitleEditTxt.getText().toString();
		String wipTagsStr = wipTagsEditTxt.getText().toString();

		if (allRequiredValuesForPublishAvaiable(wipDescStr, wipTitleStr, wipTagsStr)) {

			// check if all tags are valid
			boolean allTagsValid = true;
			wipTagsStr = BehanceSDKUtils.cleanUpTagsString(wipTagsStr);
			String[] tagsArray = wipTagsStr.split(BehanceSDKConstants.TAG_CLIENT_SIDE_SEPERATOR);
			StringBuilder finalTagStringForServer = new StringBuilder();
			for (String tagStr : tagsArray) {
				String tag = tagStr.trim();
				if (BehanceSDKUtils.isTagValidForProjectAndWIP(tag)) {
					if (finalTagStringForServer.length() > 0) {
						finalTagStringForServer.append(BehanceSDKConstants.TAG_SERVER_SIDE_SEPERATOR);
					}
					finalTagStringForServer.append(tag);
				} else {
					allTagsValid = false;
					break;
				}
			}

			if (allTagsValid) {
				hideKeyboard();
				showMainProgressSpinner();

				BehanceSDKPublishWIPServiceParamsDTO serviceParams = new BehanceSDKPublishWIPServiceParamsDTO();
				IBehanceSDKUserCredentials userCredentials = wipManager.getUserCredentials();
				serviceParams.setAppClientId(userCredentials.getClientId());

				serviceParams.setWipDescription(wipDescStr);

				if (selectedWIPId > 0) {
					serviceParams.setWipId(selectedWIPId);
					serviceParams.setWipTitle(selectedWIPTitle);
				} else {
					serviceParams.setWipTitle(wipTitleStr);
					serviceParams.setWipId(0);
				}

				serviceParams.setWipTags(finalTagStringForServer.toString());
                serviceParams.setContainsAdultContent(adultContentCheckBox.isChecked());

				serviceParams.setWipImageFile(selectedWIPImageModule);
				serviceParams.setNotificationHandlerActivity(wipManager.getNotificationHandlerActivityClass());

				boolean updateFacebookAccount = false;
				
				if (shareWIPOnFacebook && !isFacebookAccountAuthenticatedToPublish()) {
					shareWIPOnFacebook = false;
				}
				if (shareWIPOnFacebook) {
					if (!facebookAccount.isSharingEnabledLastTime()) {
						facebookAccount.setSharingEnabledLastTime(true);
						updateFacebookAccount = true;
					}
				} else {
					if (null!= facebookAccount && facebookAccount.isSharingEnabledLastTime()) {
						facebookAccount.setSharingEnabledLastTime(false);
						updateFacebookAccount = true;
					}
				}

				if (updateFacebookAccount) {
					socialAccountManager.updateAccount(facebookAccount);
				}

				boolean updateTwitterAccount = false;

				boolean TwitterAccountAuthenticated = isTwitterAccountAuthenticated();
				if (twitterAccount != null) {
					if (TwitterAccountAuthenticated && !twitterAccount.isUserAuthenticated()) {
						twitterAccount.setUserAuthenticated(true);
						updateTwitterAccount = true;
					} else if (!TwitterAccountAuthenticated && twitterAccount.isUserAuthenticated()) {
						twitterAccount.setUserAuthenticated(false);
						updateTwitterAccount = true;
					}
				}

				if (!TwitterAccountAuthenticated && shareWIPOnTwitter) {
					shareWIPOnTwitter = false;
				}

				if (shareWIPOnTwitter) {
					if (twitterAccount != null && !twitterAccount.isSharingEnabledLastTime()) {
						twitterAccount.setSharingEnabledLastTime(true);
						updateTwitterAccount = true;
					}
				} else {
					if (twitterAccount != null && twitterAccount.isSharingEnabledLastTime()) {
						twitterAccount.setSharingEnabledLastTime(false);
						updateTwitterAccount = true;
					}
				}

				if (updateTwitterAccount) {
					socialAccountManager.updateAccount(twitterAccount);
				}

				serviceParams.setShareOnFacebook(shareWIPOnFacebook);
				serviceParams.setShareOnTwitter(shareWIPOnTwitter);
				if (shareWIPOnTwitter) {
					serviceParams.setTwitterUserAccessToken(twitterAccount.getUserAuthToken());
					serviceParams.setTwitterUserAccessTokenSecret(twitterAccount.getAdditionalData());
					serviceParams.setTwitterConsumerKey(twitterAccount.getAccountClientId());
					serviceParams.setTwitterConsumerSecret(twitterAccount.getAccountClientSecret());
				}

				Intent intent = new Intent(getActivity(), BehanceSDKPublishWIPService.class);
				intent.putExtra(BehanceSDKPublishWIPService.INTENT_EXTRA_PARAMS, serviceParams);
				ComponentName publishWIPComponent = getActivity().startService(intent);
				int displayMsgResourceId = R.string.bsdk_wip_publish_view_publish_service_initialize_error_msg;
				if (publishWIPComponent != null) {
					displayMsgResourceId = R.string.bsdk_wip_publish_view_publish_initiated_msg;
				} else {
					displayMsgResourceId = R.string.bsdk_wip_publish_view_publish_service_initialize_error_msg;
				}
				Toast.makeText(getActivity(), displayMsgResourceId, Toast.LENGTH_LONG).show();
				closeThisView();
			} else {
				Toast toast = Toast.makeText(getActivity(), R.string.bsdk_wip_publish_view_publish_invalid_tag_msg, Toast.LENGTH_LONG);
				toast.setGravity(Gravity.CENTER, 0, 0);
				toast.show();
				wipTagsEditTxt.requestFocus();
			}
		} else {
			Toast.makeText(getActivity(), R.string.bsdk_wip_publish_view_publish_required_fields_missing_msg, Toast.LENGTH_LONG).show();
		}
	}

	private void updatePublishBtnState() {
		String wipDescStr = wipDescEditTxt.getText().toString();
		String wipTitleStr = wipTitleEditTxt.getText().toString();
		String wipTagsStr = wipTagsEditTxt.getText().toString();
		if (allRequiredValuesForPublishAvaiable(wipDescStr, wipTitleStr, wipTagsStr)) {
			publishWIPBtnTxtView.setEnabled(true);
		} else {
			publishWIPBtnTxtView.setEnabled(false);
		}
	}

	private boolean allRequiredValuesForPublishAvaiable(String wipDescStr, String wipTitleStr, String wipTagsStr) {
		if (selectedWIPImageModule != null && (selectedWIPId > 0 || !isStringEmpty(wipTitleStr))
				&& !isStringEmpty(wipTagsStr)) {
			return true;
		} else {
			return false;
		}
	}

	private boolean isStringEmpty(String str) {
		if (str != null && str.trim().length() > 0) {
			return false;
		} else {
			return true;
		}
	}

	private void showMainProgressSpinner() {
		if (publishWipProgressbar != null) {
			publishWipProgressbar.setVisibility(View.VISIBLE);
		}
	}

	private void hideMainProgressSpinner() {
		if (publishWipProgressbar != null) {
			publishWipProgressbar.setVisibility(View.GONE);
		}
	}

	private void onUserWIPsItemClick(AdapterView<?> parent, View view, int position, long id) {
		Object selectedItemObject = parent.getItemAtPosition(position);
		if (selectedItemObject instanceof BehanceSDKWIPDTO) {
			BehanceSDKWIPDTO selectedWIP = (BehanceSDKWIPDTO) selectedItemObject;
			selectedWIPId = selectedWIP.getId();
			selectedWIPTitle = selectedWIP.getTitle();
			userEnteredNewWIPTitle = false;
			displayPublishView(true);
		}
	}

	private void displayPublishView(boolean animateViewDisplay) {
		if (selectedWIPImageModule != null && wipThumbnailImageView != null) {
			if (animateViewDisplay) {
				viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
				viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
			} else {
				viewFlipper.setInAnimation(null);
				viewFlipper.setOutAnimation(null);
			}

			wipThumbnailImageView.setImageBitmap(selectedWIPImageModule.getThumbNail(getActivity()));

			publishWIPBtnTxtView.setVisibility(View.VISIBLE);
			showNewWIPDialogBtnTxtView.setVisibility(View.GONE);
			updatePublishBtnState();

			titleBarTitleTextView.setText(R.string.bsdk_wip_publish_view_titlebar_title_publish_label);

			if (headlessFragment.isGetUserWIPsTaskInProgress()) {
				showPickWIPsViewBtn.setVisibility(View.VISIBLE);
				selectedWIPTitleTxtView.setText(R.string.bsdk_wip_publish_view_wips_loading_label);
				wipTitleEditTxt.setVisibility(View.INVISIBLE);
			} else {
				populateSelectedWIPInPublishView();
			}
			viewFlipper.setDisplayedChild(VIEW_FLIPPER_PUBLISH_VIEW_INDEX);
		}
	}

	private void populateSelectedWIPInPublishView() {
		if (!userEnteredNewWIPTitle) {
			List<BehanceSDKWIPDTO> userWIPsList = headlessFragment.getUserWIPsList();
			if (userWIPsList == null || userWIPsList.isEmpty()) {
				showPickWIPsViewBtn.setVisibility(View.GONE);
				wipTitleEditTxt.setVisibility(View.VISIBLE);
				selectedWIPId = 0;
			} else {
				BehanceSDKWIPDTO selectedUserWIP = null;
				if (selectedWIPId > 0) {
					for (BehanceSDKWIPDTO behanceSDKWIPDTO : userWIPsList) {
						if (selectedWIPId == behanceSDKWIPDTO.getId()) {
							selectedUserWIP = behanceSDKWIPDTO;
							break;
						}
					}
				}
				if (selectedUserWIP == null) {
					selectedUserWIP = userWIPsList.get(0);
				}
				selectedWIPTitleTxtView.setText(selectedUserWIP.getTitle());
				selectedWIPId = selectedUserWIP.getId();
				selectedWIPTitle = selectedUserWIP.getTitle();
				showPickWIPsViewBtn.setVisibility(View.VISIBLE);
				wipTitleEditTxt.setVisibility(View.INVISIBLE);
			}
		} else {
			showPickWIPsViewBtn.setVisibility(View.GONE);
			wipTitleEditTxt.setVisibility(View.VISIBLE);
			selectedWIPId = 0;
		}
	}

	private void displayPickWIPView(boolean animateViewDisplay) {
		hideKeyboard();
		if (animateViewDisplay) {
			viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
			viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
		} else {
			viewFlipper.setInAnimation(null);
			viewFlipper.setOutAnimation(null);
		}

		publishWIPBtnTxtView.setVisibility(View.GONE);
		showNewWIPDialogBtnTxtView.setVisibility(View.VISIBLE);

		titleBarTitleTextView.setText(R.string.bsdk_add_wip_pick_wip_view_titlebar_title_select_wip_label);
		viewFlipper.setDisplayedChild(VIEW_FLIPPER_PICK_WIP_VIEW_INDEX);

		setUserWIPsListAdapter();
	}

	@Override
	public void onGetUserWIPsSuccess() {
		logger.debug("Retrieved WIPs from server");
		if (getActivity() != null) {
			int displayedChildIndex = viewFlipper.getDisplayedChild();
			if (displayedChildIndex == VIEW_FLIPPER_PUBLISH_VIEW_INDEX) {
				populateSelectedWIPInPublishView();
			} else {
				setUserWIPsListAdapter();
			}
		}
	}

	@Override
	public void onGetUserWIPsFailure(Exception e) {
		logger.error(e, "Problem getting user WIPs");
		if (getActivity() != null) {
			showPickWIPsViewBtn.setVisibility(View.GONE);
			wipTitleEditTxt.setVisibility(View.VISIBLE);
		}
	}

	@Override
	public void onNewWIPTitleDialogOKBtnClicked(String newWIPTitle) {
		userEnteredNewWIPTitle = true;
		selectedWIPId = 0;
		selectedWIPTitle = null;
		displayPublishView(true);
		showPickWIPsViewBtn.setVisibility(View.GONE);
		wipTitleEditTxt.setText(newWIPTitle);
		wipTitleEditTxt.setVisibility(View.VISIBLE);
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}

	private void hideKeyboard() {
		if (rootView != null) {
			InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.hideSoftInputFromWindow(rootView.getWindowToken(), 0);
		}
	}

	private TextWatcher allTextFieldsTextWatcher = new TextWatcher() {

		@Override
		public void onTextChanged(CharSequence s, int start, int before, int count) {
		}

		@Override
		public void beforeTextChanged(CharSequence s, int start, int count, int after) {
		}

		@Override
		public void afterTextChanged(Editable s) {
			updatePublishBtnState();
		}
	};

	public void handleTwitterAuthenticationSuccess() {
		if (getActivity() != null) {
			shareWIPOnTwitter = true;
			updateShareOnTwitterBtnImage();
			// Refresh Twitter account
			twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
			twitterAccount.setUserAuthenticated(true);
			socialAccountManager.updateAccount(twitterAccount);
		}
	}

	public void handleTwitterAuthenticationFailure() {
		if (getActivity() != null) {
			shareWIPOnTwitter = false;
			updateShareOnTwitterBtnImage();
			Toast.makeText(getActivity(), R.string.bsdk_social_account_twitter_login_authentication_failure, Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == TWITTER_LOGIN_ACTIVITY_REQUEST_CODE) {
			switch (resultCode) {
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_LOGIN_FAILED:
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_NO_NETWORK_CONNECTIVITY:
				handleTwitterAuthenticationFailure();
				break;
			case BehanceSDKLoginToTwitterActivity.TWITTER_ACTIVITY_LOGIN_SUCCESSFULL:
				handleTwitterAuthenticationSuccess();
				break;
			default:
				break;
			}
		} else {
			callbackManager.onActivityResult(requestCode, resultCode, data);
		}
	}

	private void handleFacebookAuthActivityResult(LoginResult loginResult,FacebookException exception) {
		if (getActivity() != null) {

			if (loginResult != null) {
				if (loginResult.getRecentlyGrantedPermissions().contains(FACEBOOK_PERMISSION_PUBLISH_ACTIONS)) {
					showMainProgressSpinner();
					savedLoginResult = loginResult;
					headlessFragment.loadFacebookUserId(loginResult);
				} else {
					// user didn't give permissions to publish
					Toast.makeText(getActivity(), R.string.bsdk_social_account_fb_auth_permissions_failure, Toast.LENGTH_LONG).show();
					shareWIPOnFacebook = false;
					updateShareOnFacebookBtnImage();
					updateFacebookAccountAuthStatus();
				}
			} else {
				// auth failed, display a message
				if (exception != null) {
					Toast.makeText(getActivity(), R.string.bsdk_social_account_fb_auth_failure, Toast.LENGTH_LONG).show();
					updateFacebookAccountAuthStatus();
				}
				shareWIPOnFacebook = false;
				updateShareOnFacebookBtnImage();
			}
		}
	}

	private OnClickListener twitterLoginConfirmDialogClickListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			if (v.getId() == R.id.bsdkGenericAlertDialogOKBtn) {
				closeLoginToTwitterConfirmationDialog();
				launchTwitterLoginActivity();
			} else if (v.getId() == R.id.bsdkGenericAlertDialogNotOKBtn) {
				closeLoginToTwitterConfirmationDialog();
			}
		}
	};

	@Override
	public void onGetFacebookUserIdComplete(String userName) {
		if (getActivity() != null) {
			shareWIPOnFacebook = true;
			updateShareOnFacebookBtnImage();
			if (facebookAccount != null) {
				facebookAccount.setUserId(userName);
				updateFacebookAccountAuthStatus();
			}
			hideMainProgressSpinner();
		}
	}

	private OnClickListener facebookLoginConfirmDialogClickListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			if (v.getId() == R.id.bsdkGenericAlertDialogOKBtn) {
				closeLoginToFacebookConfirmationDialog();
				LoginToFacebook();
			} else if (v.getId() == R.id.bsdkGenericAlertDialogNotOKBtn) {
				closeLoginToFacebookConfirmationDialog();
			}
		}
	};
}