/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.util;

import android.content.Context;


import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.exception.BehanceSDKFileUtilsInitializationException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

import org.apache.commons.io.FileUtils;

import java.io.File;

public class BehanceSDKFileUtils {

	private static final ILogger logger = LoggerFactory.getLogger(FileUtils.class);

	private static final String ROOT_TEMP_FOLDER_NAME = "BehanceTemp";
	private static File externalFilesDir;

	public static void initialize(Context context) throws BehanceSDKFileUtilsInitializationException {
		externalFilesDir = context.getExternalFilesDir(null);
		if (externalFilesDir == null) {
			logger.error("Problem detecting storage");
			throw new BehanceSDKFileUtilsInitializationException("Problem detecting storage");
		}
	}

	private static String getExternalStorageRootPath() {
		// we are making sure there are no NPEs, this will help in not crashing App at runtime
		// but at development time this will make it difficult to find cases where FileUtils is used without initializing
		if (externalFilesDir != null) {
			return externalFilesDir.getAbsolutePath();
		} else {
			return "";
		}
	}

	public static String getAppStoragePath() {
		return getExternalStorageRootPath() + File.separator + ROOT_TEMP_FOLDER_NAME + File.separator;
	}

	public static String getNetworkAppProjectHTMLStoragePath(String projectId) {
		return getAppStoragePath() + projectId + BehanceSDKProjectDTO.LAYOUT_HTML_FILE_NAME;
	}
}
