package com.behance.sdk.util;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.Resources.NotFoundException;
import android.graphics.Typeface;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.behance.sdk.R;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

public class BehanceSDKFontUtils {
	
	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKFontUtils.class);
	private static final String RAW = ".raw";
	private static final String TMP = "/tmp_";
	private static Map<BehanceSDKCustomFonts, Map<BehanceSDKFontSyles, Typeface>> fonts;
	
	
	
	
	public enum BehanceSDKFontSyles{
		REGULAR, BOLD ,LIGHT
		
	}
	
	public enum BehanceSDKCustomFonts{
		ADOBECLEAN
	}
	
	public static void setAdobeCleanLightFont(Context context, View topView) {
		setCustomFont(context, topView, BehanceSDKCustomFonts.ADOBECLEAN, BehanceSDKFontSyles.LIGHT);
	}
	
	public static void setCustomFont(Context context, View topView, BehanceSDKCustomFonts font,  BehanceSDKFontSyles style ) {
	    if (null == fonts || fonts.isEmpty()) {
	    	initializeFonts(context);
	    }
	    if (topView instanceof ViewGroup) {
	      setCustomFont((ViewGroup) topView, font, style);
	    } else if (topView instanceof TextView) {
	      setCustomFont((TextView) topView, font, style);
	    }
	  }

	private static void initializeFonts(Context context) {
		fonts = new HashMap<BehanceSDKCustomFonts, Map<BehanceSDKFontSyles,Typeface>>();
		Map<BehanceSDKFontSyles, Typeface> adobeClean = new HashMap<BehanceSDKFontSyles, Typeface>();
		Typeface adobeCleanRegular = getFontFromRes(context, R.raw.adobeclean_regular, true);
		adobeClean.put(BehanceSDKFontSyles.REGULAR, adobeCleanRegular);
		Typeface adobeCleanBold = getFontFromRes(context, R.raw.adobeclean_bold, true);
		adobeClean.put(BehanceSDKFontSyles.BOLD, adobeCleanBold);
		Typeface adobeCleanLight = getFontFromRes(context, R.raw.adobeclean_light, true);
		adobeClean.put(BehanceSDKFontSyles.LIGHT, adobeCleanLight);
		fonts.put(BehanceSDKCustomFonts.ADOBECLEAN, adobeClean);
	}
	
	  private static void processsViewGroup(ViewGroup v, final int len,  BehanceSDKCustomFonts font, BehanceSDKFontSyles style) {

	    for (int i = 0; i < len; i++) {
	      final View c = v.getChildAt(i);
	      if (c instanceof TextView) {
	        setCustomFont((TextView) c, font, style);
	      } else if (c instanceof ViewGroup) {
	        setCustomFont((ViewGroup) c, font, style);
	      }
	    }
	  }

	  private static void setCustomFont(TextView c,  BehanceSDKCustomFonts font, BehanceSDKFontSyles style) {
		  
		  if(null != fonts && false == fonts.isEmpty()){
			  Map<BehanceSDKFontSyles, Typeface> map = fonts.get(font);
			  if(null != map){
				  Typeface typeface = map.get(style);
				  if(null != typeface){
					  c.setTypeface(typeface);
				  }
			  }
		  }
	  }


	  private static void setCustomFont(ViewGroup v, BehanceSDKCustomFonts font, BehanceSDKFontSyles style) {
	    final int len = v.getChildCount();
	    processsViewGroup(v, len, font, style);
	  }
	  
	  
	  public static Typeface getFontFromRes(Context context, int resId, boolean deleteAfterwards){ 

		    String resEntryName = context.getResources().getResourceEntryName(resId);
		    String outPath = context.getCacheDir() + TMP + resEntryName + RAW;

		    //First see if the file already exists in the cachDir
		    FileInputStream fis = null;
		    try {
		        fis = new FileInputStream(new File(outPath));
		    } catch (FileNotFoundException e) {
		        logger.debug("Font "+ resEntryName+ " is not cached at location "+ outPath);
		    }

		    if(fis != null){            
		        try {
		        	logger.debug("Found cached font:"+ resEntryName+ " at location "+ outPath);
		            fis.close();
		        } catch (IOException e) {
		        	logger.debug("Exception while closing the inputstream ",e);
		        }
		        //File is already cached so return it now
		        return Typeface.createFromFile(outPath);
		    }       

		    InputStream is = null;
		    try {
		        is = context.getResources().openRawResource(resId);
		    }catch(NotFoundException e) {
		    	logger.error("Could not find font "+resEntryName+" in resources!" );
		    }

		    try{
		        byte[] buffer = new byte[is.available()];
		        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(outPath));

		        int l = 0;
		        while((l = is.read(buffer)) > 0)
		            bos.write(buffer, 0, l);

		        bos.close();
		    }catch (IOException e){
		    	logger.error("Error reading in font!",e );
		        return null;
		    }           

		    Typeface tf = Typeface.createFromFile(outPath);

		    if(deleteAfterwards){
		        // clean up if desired
		        new File(outPath).delete();
		    }
		    logger.debug("Successfully loaded font.");
		    return tf;      
		}

	@SuppressLint("DefaultLocale") 
	public static void setAdobeCleanFont(Context context, View topView,String fontName) {
			if(BehanceSDKFontSyles.BOLD.toString().toLowerCase().equals(fontName)) 
				setCustomFont(context, topView, BehanceSDKCustomFonts.ADOBECLEAN, BehanceSDKFontSyles.BOLD);
			 else if (BehanceSDKFontSyles.REGULAR.toString().toLowerCase().equals(fontName))
				 setCustomFont(context, topView, BehanceSDKCustomFonts.ADOBECLEAN, BehanceSDKFontSyles.REGULAR);
			else if (BehanceSDKFontSyles.LIGHT.toString().toLowerCase().equals(fontName))
				 setCustomFont(context, topView, BehanceSDKCustomFonts.ADOBECLEAN, BehanceSDKFontSyles.LIGHT);
			else 
				setCustomFont(context, topView, BehanceSDKCustomFonts.ADOBECLEAN, BehanceSDKFontSyles.LIGHT);
		
	}

	
}
