/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.util;

import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.widget.ImageView;

import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.animations.BehanceSDKCustomFadeInBitmapDisplayer;
import com.nostra13.universalimageloader.cache.memory.MemoryCache;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;
import com.nostra13.universalimageloader.core.assist.ImageScaleType;
import com.nostra13.universalimageloader.core.assist.QueueProcessingType;
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener;
import com.nostra13.universalimageloader.utils.MemoryCacheUtils;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKImageLoaderUtils {

	private static final int ANIMATION_DURATION_IN_MILLISECONDS = 300;
	
	private static DisplayImageOptions imageOptions;

	public static ImageLoaderConfiguration getImageLoaderConfig(Context context) {
		ImageLoaderConfiguration.Builder builder = new ImageLoaderConfiguration.Builder(context);
		ImageLoaderConfiguration loaderConfiguration = builder.threadPoolSize(10).tasksProcessingOrder(QueueProcessingType.LIFO).build();
		return loaderConfiguration;
	}
	
	public static DisplayImageOptions getDefaultImageLoaderOptions() {
		if (imageOptions == null) {
			imageOptions = new DisplayImageOptions.Builder()
                    .cacheInMemory(false)
                    .resetViewBeforeLoading(false)
                    .cacheOnDisk(false)
                    .imageScaleType(ImageScaleType.EXACTLY)
                    .displayer(new BehanceSDKCustomFadeInBitmapDisplayer(ANIMATION_DURATION_IN_MILLISECONDS, true, false, false))
                    .showImageOnFail(android.R.drawable.stat_notify_error)
                    .bitmapConfig(Bitmap.Config.RGB_565)
                    .build();
		}
		return imageOptions;
	}
	
	public static void displayImageFromCacheOrLoadFromServer(String imageUrl, ImageView imageView, ImageLoadingListener listener) {
		displayImageFromCacheOrLoadFromServer(imageUrl, imageView, listener, getDefaultImageLoaderOptions());
	}

	public static void displayImageFromCacheOrLoadFromServer(String imageUrl, ImageView imageView, ImageLoadingListener listener, DisplayImageOptions options) {
		ImageLoader imageloader = ImageLoader.getInstance();
		boolean loadedFromCache = false;
		imageloader.cancelDisplayTask(imageView);
		MemoryCache imageLoaderMemoryCache = imageloader.getMemoryCache();
		if (imageLoaderMemoryCache != null) {
			List<Bitmap> cachedBitmapsForImageUri = MemoryCacheUtils.findCachedBitmapsForImageUri(imageUrl, imageLoaderMemoryCache);
			if (cachedBitmapsForImageUri != null && cachedBitmapsForImageUri.size() > 0) {
				Bitmap cachedBitmap = cachedBitmapsForImageUri.get(0);
				if (cachedBitmap != null && !cachedBitmap.isRecycled()) {
					loadedFromCache = true;
					if (listener != null) {
						listener.onLoadingComplete(imageUrl, imageView, cachedBitmap);
					} else {
						if(imageView != null)
							imageView.setImageBitmap(cachedBitmap);
					}
				}
			}
		}
		if (!loadedFromCache) {
			if (listener != null) {
				imageloader.displayImage(imageUrl, imageView, options, listener);
			} else {
				imageloader.displayImage(imageUrl, imageView, options);
			}
		}
	}

	public static void displayImageFromCacheOrLoadFromServer(String imageUrl, ImageView imageView) {
		displayImageFromCacheOrLoadFromServer(imageUrl, imageView, null);
	}
	
	public static Bitmap getImageBitmap(String ImageUrl) {
		ImageLoader imageloader = ImageLoader.getInstance();
		return imageloader.loadImageSync(ImageUrl, getDefaultImageLoaderOptions());
	}
	
	
	public static Bitmap getImageBitmapFromCache(String ImageUrl) {
		ImageLoader imageloader = ImageLoader.getInstance();
		MemoryCache imageLoaderMemoryCache = imageloader.getMemoryCache();
		if (imageLoaderMemoryCache != null) {
			List<Bitmap> cachedBitmapsForImageUri = MemoryCacheUtils.findCachedBitmapsForImageUri(ImageUrl,
							imageLoaderMemoryCache);
			if (cachedBitmapsForImageUri != null
					&& cachedBitmapsForImageUri.size() > 0) {
				Bitmap cachedBitmap = cachedBitmapsForImageUri.get(0);
				if (cachedBitmap != null && !cachedBitmap.isRecycled()) {
					return cachedBitmap;
				}
			}
		}
		return null;
	}
	
	public static Bitmap updateBitmapForRotation(Bitmap bitmap, ImageModule imageModule) {
		int rotation = imageModule.getRotation();
		if(rotation > 0){
            Matrix matrix = new Matrix();
            // setup rotation degree
            matrix.postRotate(90* rotation);
            return Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
        }
		return bitmap;
	}
}