package com.behance.sdk.util;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.widget.Toast;

/**
 * Created by rkumarba on 12/14/2015.
 */
public class BehanceSDKPermissionHelper {
    public static final int REQUEST_CODE_LAUNCH_ALBUMS_VIEW = 1;
    public static final int REQUEST_CODE_LAUNCH_CAMERA = 2;
    public static final int REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT = 3;
    public static final int REQUEST_CODE_STORAGE_DOWNLOAD_IMAGE =4;



    public static boolean hasPermissionInManifest(Context context, String permissionName) {
        final String packageName = context.getPackageName();
        try {
            final PackageInfo packageInfo = context.getPackageManager()
                    .getPackageInfo(packageName, PackageManager.GET_PERMISSIONS);
            final String[] declaredPermisisons = packageInfo.requestedPermissions;
            if (declaredPermisisons != null && declaredPermisisons.length > 0) {
                for (String p : declaredPermisisons) {
                    if (p.equals(permissionName)) {
                        return true;
                    }
                }
            }
        } catch (PackageManager.NameNotFoundException e) {

        }
        return false;
    }

    public static boolean checkPermission(Context context, int requestCode) {
        String [] permissions = getPermissionFromRequestCode(requestCode, context);
        boolean allPermissionGranted = true;
        if(permissions != null && permissions.length > 0) {
            for (String permission : permissions) {
                allPermissionGranted = allPermissionGranted && (ContextCompat.checkSelfPermission(context, permission) == PackageManager.PERMISSION_GRANTED);
            }
        }
        return allPermissionGranted;
    }

    public static String[] getPermissionFromRequestCode(int requestCode,Context context) {
        switch (requestCode) {
            case REQUEST_CODE_LAUNCH_CAMERA:
                if(hasPermissionInManifest(context,"android.permission.CAMERA")){
                    return new String[]{Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE};
                }else{
                    return new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE};
                }
            case REQUEST_CODE_LAUNCH_ALBUMS_VIEW:
            case REQUEST_CODE_STORAGE_DOWNLOAD_IMAGE :
            case REQUEST_CODE_LAUNCH_ALBUMS_VIEW_COVER_FRAGMENT:
                return new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE};
            default:
                return null;
        }
    }

    public static boolean checkPermissionAndRequest(Context context, int requestCode) {
        if (!checkPermission(context,requestCode)) {
            ActivityCompat.requestPermissions((AppCompatActivity) context, getPermissionFromRequestCode(requestCode,context), requestCode);
            return false;
        } else {
            return true;
        }
    }

}
