package com.behance.sdk.util;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.view.WindowManager.LayoutParams;
import android.webkit.JsResult;
import android.webkit.WebChromeClient;
import android.webkit.WebView;

import com.behance.sdk.R;

import com.behance.sdk.enums.BehanceSDKProjectModuleType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.fragments.BehanceSDKImageDisplayDialogFragment;

import java.util.List;

public class BehanceSDKProjectDisplayWebChromeClient extends WebChromeClient {

	public static String EVENT_TYPE_APPRECIATE_PROJECT = "EVENT_TYPE_APPRECIATE_PROJECT";
	public static String EVENT_TYPE_FOLLOW_UNFOLLOW_USER = "EVENT_TYPE_FOLLOW_UNFOLLOW_USER";
	public static String EVENT_TYPE_ADD_TO_COLLECTION = "EVENT_TYPE_ADD_TO_COLLECTION";

	private static final String FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG = "FRAGMENT_TAG_PROJECT_IMAGE_DISPLAY_DIALOG";
	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKProjectDisplayWebChromeClient.class);

	private Context mContext;
	private String[] projectImagesURLsArray;
	private Callbacks callbacks;

	public interface Callbacks {
		public void onChromeClientProjectModuleClicked();

		public void onChromeClientAddToCollectionClicked();
	}

	public BehanceSDKProjectDisplayWebChromeClient(Context context) {
		mContext = context;
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}

	public void setProjectImagesURLs(String[] imageURLs) {
		this.projectImagesURLsArray = imageURLs;
	}

	// @Override
	public boolean onJsAlert(WebView view, String url, String message, final JsResult result) {
		if (message.startsWith(BehanceSDKProjectModuleType.IMAGE.name())) {
			int imageNumber = Integer.parseInt(message.substring(BehanceSDKProjectModuleType.IMAGE.name().length()));
			DialogFragment imageDisplayFragment = BehanceSDKImageDisplayDialogFragment.getInstance(projectImagesURLsArray, imageNumber);

			FragmentManager fragmentManager = null;
			try {
				fragmentManager = ((FragmentActivity) mContext).getSupportFragmentManager();
				FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
				Fragment previousInstance = fragmentManager.findFragmentByTag(FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG);
				if (previousInstance != null) {
					fragmentTransaction.remove(previousInstance);
				}
				imageDisplayFragment.show(fragmentTransaction, FRAGMENT_TAG_IMAGE_DISPLAY_DIALOG);
			} catch (Exception e) {
				logger.error(e, "Can't get fragment manager from context ");
			}
		} else if (message.startsWith(BehanceSDKProjectModuleType.EMBED.name())) {
			String urlToLaunch = message.substring(BehanceSDKProjectModuleType.EMBED.name().length());
			Intent intent = new Intent(Intent.ACTION_VIEW);
			intent.setData(Uri.parse(urlToLaunch));
			Intent launchIntent = Intent.createChooser(intent, mContext.getResources().getString(R.string.bsdk_cwcc_chooser_title_embed_module));
			mContext.startActivity(launchIntent);
		} else if (message.startsWith(BehanceSDKProjectModuleType.VIDEO.name())) {
			// check if there are players that can play this video
			PackageManager packageManager = mContext.getPackageManager();
			Intent videoIntent = new Intent(Intent.ACTION_VIEW);
			videoIntent.setType("video/x-flv");
			List<ResolveInfo> activitiesThanCanHandle = packageManager.queryIntentActivities(videoIntent, 0);

			if (activitiesThanCanHandle.size() > 0) {
				Intent intent = new Intent(Intent.ACTION_VIEW);
				intent.setType("video/x-flv");
				String urlToLaunch = message.substring(BehanceSDKProjectModuleType.VIDEO.name().length());
				intent.setData(Uri.parse(urlToLaunch));
				Intent launchIntent = Intent.createChooser(intent, mContext.getResources().getString(R.string.bsdk_cwcc_chooser_title_video_module));
				mContext.startActivity(launchIntent);
			} else {
				displayCannotPlayMediaMessage(mContext, mContext.getResources().getString(R.string.bsdk_cwcc_video_player_not_found_msg_title), mContext
						.getResources().getString(R.string.bsdk_cwcc_video_player_not_found_msg_body));
			}
		}

		if (this.callbacks != null) {
			// This is required to close the comment box in project view
			callbacks.onChromeClientProjectModuleClicked();

			if (message.startsWith(EVENT_TYPE_ADD_TO_COLLECTION)) {
				callbacks.onChromeClientAddToCollectionClicked();
			}
		}

		result.confirm();
		return true;
	}

	private void displayCannotPlayMediaMessage(Context context, String title, String message) {
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		builder.setTitle(title);
		builder.setMessage(message);
		builder.setPositiveButton(context.getResources().getString(R.string.bsdk_cwcc_player_not_found_ok_btn), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialogInterface, int i) {
			}
		});

		builder.create();
		AlertDialog errorDialog = builder.show();
		errorDialog.getWindow().setLayout(context.getResources().getDimensionPixelSize(R.dimen.bsdk_ed_error_dialog_width), LayoutParams.WRAP_CONTENT);
	}
}