/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.util;

import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKUrlUtil {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKUrlUtil.class);

	public static final String KEY_SERVER_ROOT_URL = "server_root_url";
	public static final String KEY_CLIENT_ID_PARAM = "key_client_id_param";
	public static final String KEY_CLIENT_ID = "clientId";
    public static final String KEY_WIP_ID = "wip_id";
    public static final String KEY_PROJECT_ID = "project_id";
    public static final String KEY_COMMENT_ID = "comment_id";

    public static final String KEY_WIP_REVISION_ID = "wip_revision_id";
	public static final String KEY_USER_ID = "user_id";
    public static final String KEY_COUNTRY_ID = "countryId";
    public static final String KEY_USER_TOKEN = "usertoken";
    public static final String KEY_SITE_ID = "site_id";



    public static final String PARAM_KEY_ACCESS_TOKEN = "access_token";
	public static final String PARAM_KEY_USER_TOKEN = "user_token";
	public static final String PARAM_KEY_PAGE_SIZE = "per_page";
	public static final String PARAM_KEY_PAGE_NUMBER = "page";
    public static final String PARAM_KEY_FIRST_NAME = "first_name";
    public static final String PARAM_KEY_LAST_NAME = "last_name";
    public static final String PARAM_KEY_OCCUPATION = "occupation";
    public static final String PARAM_KEY_COMPANY = "company";
    public static final String PARAM_KEY_WEBSITE = "website";
    public static final String PARAM_KEY_COUNTRY = "country";
    public static final String PARAM_KEY_STATE = "state";
    public static final String PARAM_KEY_CITY = "city";
    public static final String PARAM_KEY_LOCATION_FILTER_COUNTRY = "country";
    public static final String PARAM_KEY_LOCATION_FILTER_STATE = "stateprov";
    public static final String PARAM_KEY_LOCATION_FILTER_CITY = "city";

    public static final String PARAM_SERVED_FIELD = "served";
    public static final String PARAM_NETWORK_FIELD = "network";

    public static final String PARAM_KEY_SORT = "sort";
    public static final String PARAM_KEY_TIME = "time";
    public static final String PARAM_KEY_FIELD = "field";
    public static final String PARAM_FREE_TEXT_QUERY = "q";
    public static final String KEY_X_IMS_CLIENT_ID = "X-IMS-ClientId";
    public static final String PARAM_KEY_CLIENT_ID = "client_id";
    public static final String PARAM_KEY_SEARCH_FREE_TEXT_QUERY = "search";





    public static final String GET_PROJECTS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static final String GET_PROJECT_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/html?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
            + "}";

    public static final String VIEW_PROJECT_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/view?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";


    public static final String GET_CREATIVE_FIELDS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/fields?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

	public static final String GET_WIP_TAGS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/tags/wips?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

	public static final String PROJECT_CREATE_URL =  "{" + KEY_SERVER_ROOT_URL + "}/v2/project/editor?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";
	
	public static final String PROJECT_DELETE_URL =  "{" + KEY_SERVER_ROOT_URL + "}/v2/project/editor/{PROJECTID}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";
	
	public static final String ADD_PROJECT_MODULE_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/project/editor/{PROJECTID}/modules?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";
	
	public static final String PUBLISH_PROJECT_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/project/editor/{PROJECTID}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";
    public static String GET_BEHANCE_USER_PROFILE_WITH_IMS_TOKEN_API_URL = BehanceSDKConstants.ADOBE_API_SERVER_ROOT_URL + "/v2/users/{" + KEY_USER_ID + "}?api_key={"
            + KEY_CLIENT_ID + "}&user_token={" + KEY_USER_TOKEN + "}";
	
	public static final String PUBLISH_NEW_WIP_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/wips/?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";

	public static final String PUBLISH_IMAGE_TO_EXISTING_WIP_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/wips/{" + KEY_WIP_ID + "}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID 
			+ "}";

    public static final String UNPUBLISH_PROJECT_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/project/editor/{" + KEY_PROJECT_ID + "}?{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String GET_CURATED_GALLERIES_PROJECTS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";


    public static final String GET_USER_SETTINGS_ON_SERVER_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/settings/browsing?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static final String POST_PROJECT_COMMENTS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/comments?{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String DELETE_WIP_REVISION_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/wips/{" + KEY_WIP_ID + "}/{" + KEY_WIP_REVISION_ID +"}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
            + "}";

	public static final String GET_USER_WIPS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/users/{" + KEY_USER_ID + "}/wips?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
			+ "}";

    public static final String GET_USER_BASIC_DETAILS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/users/{" + KEY_USER_ID + "}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
            + "}";

    public static final String EDIT_USER_PROFILE_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/profile/editor?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static final String EDIT_USER_PROFILE_AVATAR_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/profile/avatar?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static final String GET_COUNTRIES_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/utilities/location?level=1&{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String GET_STATES_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/utilities/location?level=2&country={" + KEY_COUNTRY_ID + "}&{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String GET_CITIES_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/utilities/location?level=3&{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";
    public static final String GET_USER_PROJECTS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/users/{" + KEY_USER_ID + "}/projects?{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String DELETE_PROJECT_COMMENT_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/comments/{" + KEY_COMMENT_ID + "}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
            + "}";

    public static final String APPRECIATE_PROJECT_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/appreciate?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static final String FOLLOW_USER_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/users/{" + KEY_USER_ID + "}/follow?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID
            + "}";

    public static final String GET_PROJECT_COMMENTS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/projects/{" + KEY_PROJECT_ID + "}/comments?{" + KEY_CLIENT_ID_PARAM + "}={"
            + KEY_CLIENT_ID + "}";

    public static final String CHECK_WIP_CONVERSION_STATUS_API_URL = "{" + KEY_SERVER_ROOT_URL + "}/v2/wips/conversions/{" + KEY_WIP_ID + "}?{" + KEY_CLIENT_ID_PARAM + "}={" + KEY_CLIENT_ID + "}";

    public static String getUrlFromTemplate(String urlTemplate, Map<String, String> urlParams) {
        urlTemplate = urlTemplate.replace("{" + KEY_SERVER_ROOT_URL + "}", BehanceSDKConstants.ADOBE_API_SERVER_ROOT_URL);
        urlTemplate = urlTemplate.replace("{" + KEY_CLIENT_ID_PARAM + "}", BehanceSDKConstants.ADOBE_API_CLIENT_ID_PARAM);

        if (urlParams != null) {
            for (String urlParamKey : urlParams.keySet()) {
                String urlParamValue = urlParams.get(urlParamKey);
                urlTemplate = urlTemplate.replace("{" + urlParamKey + "}", urlParamValue);
            }
        }
        return urlTemplate;
    }

    public static String appendQueryStringParam(String url, String paramKey, Object paramValue) {
        try {
            if (paramKey.equals(PARAM_KEY_ACCESS_TOKEN) || paramKey.equals(PARAM_KEY_USER_TOKEN)) {
                paramKey = PARAM_KEY_USER_TOKEN;
            }

            if (url.contains("?")) {
                return url + "&" + paramKey + "=" + URLEncoder.encode(String.valueOf(paramValue), "UTF-8");
            } else {
                return url + "?" + paramKey + "=" + URLEncoder.encode(String.valueOf(paramValue), "UTF-8");
            }
        } catch (UnsupportedEncodingException e) {
            logger.error(e);
        }
        return url;
    }
}