/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.util;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.os.Build;
import android.os.Environment;
import android.provider.MediaStore;
import android.provider.MediaStore.Images;
import android.util.DisplayMetrics;
import android.view.View;
import android.view.ViewGroup.LayoutParams;

import com.adobe.creativesdk.foundation.storage.AdobeAssetMimeTypes;
import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.project.modules.ImageModule;

/**
 * @author ugs
 * 
 */
public class BehanceSDKUtils {

	
	private static final String IMAGE_JPEG = "image/jpeg";
	private static final String EMPTY_STRING = "";
	private static ArrayList<String> allowedFileExtensionsForProjectImages;
	private static ArrayList<String> allowedFileExtensionsForWIPImage;
	private static ArrayList<String> allowedFileExtensionsForProfileImage;
    private static ArrayList<String> restrictedCharForTagList;


	@SuppressLint("SimpleDateFormat")
	public static File createTempImageFile() throws IOException {
		String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
		String imageFileName = "JPEG_" + timeStamp + "_";
		File storageDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES);
		File image = File.createTempFile(imageFileName, /* prefix */
				".jpg", /* suffix */
				storageDir /* directory */
		);
		return image;
	}

	public static String combineStringsWithSeperateor(List<String> strings, String seperator) {
		if (null != strings && strings.isEmpty() == false) {
			StringBuffer finalString = new StringBuffer();
			for (String tag : strings) {
				if (finalString.length() > 0)
					finalString.append(seperator);
				finalString.append(tag.trim());
			}
			return finalString.toString();
		}
		return EMPTY_STRING;
	}

	public static String getCreativeFieldsString(List<BehanceSDKCreativeFieldDTO> strings, String seperator) {
		if (null != strings && strings.isEmpty() == false) {
			StringBuffer finalString = new StringBuffer();
			for (BehanceSDKCreativeFieldDTO field : strings) {
				if (finalString.length() > 0)
					finalString.append(seperator);
				finalString.append(field.getName());
			}
			return finalString.toString();
		}
		return null;
	}

	public static void setContentViewDimensions(Context context, View contentView) {
		boolean displayContentViewInFullScreen = context.getResources().getBoolean(R.bool.bsdk_show_view_full_screen);
		if (!displayContentViewInFullScreen) {
			DisplayMetrics displayMetrics = context.getResources().getDisplayMetrics();
			int deviceWidth = displayMetrics.widthPixels;
			int deviceHeight = displayMetrics.heightPixels;

			if (deviceWidth > 0 && deviceHeight > 0) {
				int contentContainerWidth = (int) (deviceWidth * BehanceSDKConstants.CONTENT_VIEW_WIDTH_PERCENTAGE);
				LayoutParams layoutParams = contentView.getLayoutParams();
				if (layoutParams != null) {
					layoutParams.width = contentContainerWidth;
					layoutParams.height = LayoutParams.WRAP_CONTENT;
					contentView.setLayoutParams(layoutParams);
				}
			}
		}
	}

	public static void setContentViewPaddings(Context context, View contentView, float precentageScreenWidth) {
		DisplayMetrics displayMetrics = context.getResources().getDisplayMetrics();
		int deviceWidth = displayMetrics.widthPixels;
		int deviceHeight = displayMetrics.heightPixels;

		if (deviceWidth > 0 && deviceHeight > 0) {
			int totalPadding = deviceWidth - (int) (deviceWidth * precentageScreenWidth);
			contentView.setPadding(totalPadding / 2, contentView.getPaddingTop(), totalPadding / 2, contentView.getPaddingBottom());
		}
	}

	public static void addImageToGallery(Context context, String filePath) {

		ContentValues values = new ContentValues();
		values.put(Images.Media.DATE_TAKEN, System.currentTimeMillis());
		values.put(Images.Media.MIME_TYPE, IMAGE_JPEG);
		values.put(MediaStore.MediaColumns.DATA, filePath);
		context.getContentResolver().insert(Images.Media.EXTERNAL_CONTENT_URI, values);
	}

	public static String getFileExtension(String filePath) {
		if (filePath == null) {
			return null;
		}
		int extensionPos = filePath.lastIndexOf('.');
		int lastUnixPos = filePath.lastIndexOf('/');
		int lastWindowsPos = filePath.lastIndexOf('\\');
		int lastSeparator = Math.max(lastUnixPos, lastWindowsPos);
		int extensionIndex = (lastSeparator > extensionPos ? -1 : extensionPos);
		if (extensionIndex == -1) {
			return "";
		} else {
			return filePath.substring(extensionIndex + 1);
		}
	}

	public static ArrayList<String> getAllowedFileExtensionsForProjectImage() {
		// we could have user toLowerCase of file extension while comparing, but
		// was not sure about local related errors
		if (allowedFileExtensionsForProjectImages == null) {
			allowedFileExtensionsForProjectImages = new ArrayList<String>();
			allowedFileExtensionsForProjectImages.add("jpg");
			allowedFileExtensionsForProjectImages.add("JPG");
			allowedFileExtensionsForProjectImages.add("jpeg");
			allowedFileExtensionsForProjectImages.add("JPEG");
			allowedFileExtensionsForProjectImages.add("png");
			allowedFileExtensionsForProjectImages.add("PNG");
			allowedFileExtensionsForProjectImages.add("gif");
			allowedFileExtensionsForProjectImages.add("GIF");
		}
		return allowedFileExtensionsForProjectImages;
	}

	public static ArrayList<String> getAllowedFileExtensionsForWIPImage() {
		// we could have user toLowerCase of file extension while comparing, but
		// was not sure about local related errors
		if (allowedFileExtensionsForWIPImage == null) {
			allowedFileExtensionsForWIPImage = new ArrayList<String>();
			allowedFileExtensionsForWIPImage.add("jpg");
			allowedFileExtensionsForWIPImage.add("JPG");
			allowedFileExtensionsForWIPImage.add("jpeg");
			allowedFileExtensionsForWIPImage.add("JPEG");
			allowedFileExtensionsForWIPImage.add("png");
			allowedFileExtensionsForWIPImage.add("PNG");
			allowedFileExtensionsForWIPImage.add("gif");
			allowedFileExtensionsForWIPImage.add("GIF");
		}
		return allowedFileExtensionsForWIPImage;
	}

    public static ArrayList<String> getAllowedFileExtensionsForProfileImage() {
        // we could have user toLowerCase of file extension while comparing, but
        // was not sure about local related errors
        if (allowedFileExtensionsForProfileImage == null) {
            allowedFileExtensionsForProfileImage = new ArrayList<String>();
            allowedFileExtensionsForProfileImage.add("jpg");
            allowedFileExtensionsForProfileImage.add("JPG");
            allowedFileExtensionsForProfileImage.add("jpeg");
            allowedFileExtensionsForProfileImage.add("JPEG");
            allowedFileExtensionsForProfileImage.add("png");
            allowedFileExtensionsForProfileImage.add("PNG");
            allowedFileExtensionsForProfileImage.add("gif");
            allowedFileExtensionsForProfileImage.add("GIF");
        }
        return allowedFileExtensionsForProfileImage;
    }

	public static EnumSet<AdobeAssetMimeTypes> getAdobeAssetMimeTypes(List<String> fileExtentions){
		if(null != fileExtentions && false == fileExtentions.isEmpty()) {
			Set<AdobeAssetMimeTypes> enumSet = new HashSet<AdobeAssetMimeTypes>(fileExtentions.size());
			for (String extention : fileExtentions) {
				if ("jpg".equalsIgnoreCase(extention) || "jpeg".equalsIgnoreCase(extention)) {
					enumSet.add(AdobeAssetMimeTypes.MIMETYPE_JPEG);
				} else if("png".equalsIgnoreCase(extention)){
					enumSet.add(AdobeAssetMimeTypes.MIMETYPE_PNG);
				} else if("gif".equalsIgnoreCase(extention)) {
					enumSet.add(AdobeAssetMimeTypes.MIMETYPE_GIF);
				}
			}
			return EnumSet.copyOf(enumSet);
		}
		return null;
	}


	public static String cleanUpTagsString(String tagsString) {
		String[] commasArray = { "FF0C" };
		String finalStr = "";
		for (String commaUnicode : commasArray) {
			char commaChar = (char) Integer.parseInt(commaUnicode, 16);
			String commaStr = Character.toString(commaChar);
			finalStr = tagsString.replaceAll(commaStr, BehanceSDKConstants.TAG_CLIENT_SIDE_SEPERATOR);
		}
		return finalStr;
	}

    public static boolean isTagValidForProjectAndWIP(String tagStr) {
        if (tagStr != null && tagStr.length() > 0 && !tagStr.matches("-?\\d+(\\.\\d+)?")) {
            if(restrictedCharForTagList == null) {
                restrictedCharForTagList = new ArrayList<String>();
                restrictedCharForTagList.add("~");
                restrictedCharForTagList.add("`");
                restrictedCharForTagList.add("!");
                restrictedCharForTagList.add("@");
                restrictedCharForTagList.add("#");
                restrictedCharForTagList.add("$");
                restrictedCharForTagList.add("%");
                restrictedCharForTagList.add("^");
                restrictedCharForTagList.add("&");
                restrictedCharForTagList.add("*");
                restrictedCharForTagList.add("(");
                restrictedCharForTagList.add(")");
                restrictedCharForTagList.add("-");
//                restrictedCharForTagList.add("_");// underscore is allowed
                restrictedCharForTagList.add("+");
                restrictedCharForTagList.add("=");
                restrictedCharForTagList.add("[");
                restrictedCharForTagList.add("]");
                restrictedCharForTagList.add("{");
                restrictedCharForTagList.add("}");
                restrictedCharForTagList.add("|");
                restrictedCharForTagList.add("\\");
                restrictedCharForTagList.add(";");
                restrictedCharForTagList.add(":");
                restrictedCharForTagList.add("\"");
                restrictedCharForTagList.add("'");
                restrictedCharForTagList.add("<");
                restrictedCharForTagList.add(">");
                restrictedCharForTagList.add(",");
                restrictedCharForTagList.add(".");
                restrictedCharForTagList.add("?");
                restrictedCharForTagList.add("/");
            }
            for (String restrictedChar : restrictedCharForTagList) {
                if(tagStr.contains(restrictedChar)) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    public static BehanceSDKPublishImageValidationResult validateImageForWIP(File imageFile){
        ImageModule module = new ImageModule(imageFile);
        return validateImageForWIP(module);
    }

    public static BehanceSDKPublishImageValidationResult validateImageForWIP(ImageModule imageModule){
        if(null != imageModule) {
            File imageFile = imageModule.getFile();
            if (imageFile != null && imageFile.exists() && imageFile.isFile()) {
                String fileName = imageFile.getName();
                //validate the image file extension
                String fileExtension = BehanceSDKUtils.getFileExtension(fileName);
                if (fileExtension != null && fileExtension.length() > 0) {
                    ArrayList<String> allowedFileExtensionsList = BehanceSDKUtils.getAllowedFileExtensionsForWIPImage();
                    if (allowedFileExtensionsList != null && !allowedFileExtensionsList.isEmpty()) {
                        if (false == allowedFileExtensionsList.contains(fileExtension)) {
                            return BehanceSDKPublishImageValidationResult.INVALID_FILE_TYPE;
                        }
                    }
                } else {
                    return BehanceSDKPublishImageValidationResult.INVALID_FILE_TYPE;
                }
                //validate the image size requirements for the WIP
                long imageSizeInBytes = imageModule.getSizeInBytes();

                if (imageSizeInBytes <= 0 || imageSizeInBytes > BehanceSDKConstants.WIP_IMAGE_MAX_SIZE_BYTES) {
                    return BehanceSDKPublishImageValidationResult.INVALID_SIZE;
                }
                //validate the image dimensions
                int imageWidth = imageModule.getWidth();
                int imageHeight = imageModule.getHeight();
                    if (imageWidth <= 0 || imageHeight <= 0 || imageWidth < BehanceSDKConstants.WIP_IMAGE_MIN_WIDTH
                            || imageHeight < BehanceSDKConstants.WIP_IMAGE_MIN_HEIGHT) {
                        return BehanceSDKPublishImageValidationResult.INVALID_IMAGE_DIMENSIONS;
                    }
                return BehanceSDKPublishImageValidationResult.VALID;
            }
        }
        return BehanceSDKPublishImageValidationResult.INVALID;
    }



    public static BehanceSDKPublishImageValidationResult validateImageForProject(File imageFile){
        ImageModule imageModule = new ImageModule(imageFile);
        return validateImageForProject(imageModule);

    }

    public static BehanceSDKPublishImageValidationResult validateImageForProject(ImageModule imageModule){
        if(null != imageModule) {
            File imageFile = imageModule.getFile();
            if (imageFile != null && imageFile.exists() && imageFile.isFile()) {
                String fileName = imageFile.getName();
                //validate the image file extension
                String fileExtension = BehanceSDKUtils.getFileExtension(fileName);
                if (fileExtension != null && fileExtension.length() > 0) {
                    ArrayList<String> allowedFileExtensionsList = BehanceSDKUtils.getAllowedFileExtensionsForProjectImage();
                    if (allowedFileExtensionsList != null && !allowedFileExtensionsList.isEmpty()) {
                        if (false == allowedFileExtensionsList.contains(fileExtension)) {
                            return BehanceSDKPublishImageValidationResult.INVALID_FILE_TYPE;
                        }
                    }
                } else {
                    return BehanceSDKPublishImageValidationResult.INVALID_FILE_TYPE;
                }
                //validate the image size requirements for the Project
                long imageSizeInBytes = imageModule.getSizeInBytes();
                if (imageSizeInBytes <= 0 || imageSizeInBytes > BehanceSDKConstants.PROJECT_IMAGE_MODULE_MAX_SIZE_BYTES) {
                    return BehanceSDKPublishImageValidationResult.INVALID_SIZE;
                }
                return BehanceSDKPublishImageValidationResult.VALID;
            }
        }
        return BehanceSDKPublishImageValidationResult.INVALID;
    }


	public static void deleteRecursive(File fileOrDirectory) {
		if(null != fileOrDirectory && fileOrDirectory.exists() ) {
			if (fileOrDirectory.isDirectory()) {
				for (File child : fileOrDirectory.listFiles()) {
					deleteRecursive(child);
				}
			}
			fileOrDirectory.delete();
		}
	}

	public static boolean isBlank(final CharSequence s)
	{
		if (s == null)
		{
			return true;
		}
		for (int i = 0; i < s.length(); i++) {
			if (!Character.isWhitespace(s.charAt(i))) {
				return false;
			}
		}
		return true;
	}

	public static boolean isEmpty(final CharSequence s)
	{
		if (s == null)
		{
			return true;
		}
		return s.length() == 0;
	}

	public static int getSoftButtonsBarSizePort(Activity activity) {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
			DisplayMetrics metrics = new DisplayMetrics();
			activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
			int usableHeight = metrics.heightPixels;
			activity.getWindowManager().getDefaultDisplay().getRealMetrics(metrics);
			int realHeight = metrics.heightPixels;
			if (realHeight > usableHeight)
				return realHeight - usableHeight;
			else
				return 0;
		}
		return 0;
	}

	public static boolean getHasTranslucentNav(Activity activity) {
		return getSoftButtonsBarSizePort(activity) > 0 && activity.getResources().getBoolean(R.bool.bsdk_transparentNavBar);
	}

	public static int getTranslucentNavHeight(Activity activity) {
		return getHasTranslucentNav(activity) ? activity.getResources().getDimensionPixelSize(R.dimen.bsdk_nav_bar_spacer) : 0;
	}

	public static boolean shouldHideStatusBarOnScroll() {
		return Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP;
	}

}